@@ -517,10 +517,6 @@ public interface ManagedCursor {
      */
     void rewind();
 
-    default void rewind(boolean readCompacted) {
-        rewind();
-    }
-
     /**
      * Move the cursor to a different read position.
      *
@@ -658,31 +654,6 @@ public interface ManagedCursor {
     void asyncFindNewestMatching(FindPositionConstraint constraint, Predicate<Entry> condition,
             FindEntryCallback callback, Object ctx, boolean isFindFromLedger);
 
-
-    /**
-     * Find the newest entry that matches the given predicate.
-     *
-     * @param constraint
-     *            search only active entries or all entries
-     * @param condition
-     *            predicate that reads an entry an applies a condition
-     * @param callback
-     *            callback object returning the resultant position
-     * @param startPosition
-     *           start position to search from.
-     * @param endPosition
-     *          end position to search to.
-     * @param ctx
-     *            opaque context
-     * @param isFindFromLedger
-     *            find the newest entry from ledger
-     */
-    default void asyncFindNewestMatching(FindPositionConstraint constraint, Predicate<Entry> condition,
-                                 PositionImpl startPosition, PositionImpl endPosition, FindEntryCallback callback,
-                                 Object ctx, boolean isFindFromLedger) {
-        asyncFindNewestMatching(constraint, condition, callback, ctx, isFindFromLedger);
-    }
-
     /**
      * reset the cursor to specified position to enable replay of messages.
      *
