@@ -33,7 +33,7 @@ import org.apache.bookkeeper.common.annotation.InterfaceStability;
 import org.apache.bookkeeper.mledger.impl.NullLedgerOffloader;
 import org.apache.bookkeeper.mledger.intercept.ManagedLedgerInterceptor;
 import org.apache.commons.collections4.MapUtils;
-import org.apache.pulsar.common.util.collections.OpenLongPairRangeSet;
+import org.apache.pulsar.common.util.collections.ConcurrentOpenLongPairRangeSet;
 
 /**
  * Configuration class for a ManagedLedger.
@@ -79,7 +79,6 @@ public class ManagedLedgerConfig {
     private ManagedLedgerInterceptor managedLedgerInterceptor;
     private Map<String, String> properties;
     private int inactiveLedgerRollOverTimeMs = 0;
-    private long inactiveOffloadedLedgerEvictionTimeMs = 0;
     @Getter
     @Setter
     private boolean cacheEvictionByMarkDeletedPosition = false;
@@ -90,8 +89,6 @@ public class ManagedLedgerConfig {
     @Getter
     @Setter
     private String shadowSourceName;
-    @Getter
-    private boolean persistIndividualAckAsLongArray;
 
     public boolean isCreateIfMissing() {
         return createIfMissing;
@@ -102,11 +99,6 @@ public class ManagedLedgerConfig {
         return this;
     }
 
-    public ManagedLedgerConfig setPersistIndividualAckAsLongArray(boolean persistIndividualAckAsLongArray) {
-        this.persistIndividualAckAsLongArray = persistIndividualAckAsLongArray;
-        return this;
-    }
-
     /**
      * @return the lazyCursorRecovery
      */
@@ -289,7 +281,7 @@ public class ManagedLedgerConfig {
     }
 
     /**
-     * should use {@link OpenLongPairRangeSet} to store unacked ranges.
+     * should use {@link ConcurrentOpenLongPairRangeSet} to store unacked ranges.
      * @return
      */
     public boolean isUnackedRangesOpenCacheSetEnabled() {
@@ -487,16 +479,6 @@ public class ManagedLedgerConfig {
         return maxBatchDeletedIndexToPersist;
     }
 
-    /**
-     * Set max batch deleted index that will be persisted and recovered.
-     *
-     * @param maxBatchDeletedIndexToPersist
-     *            max batch deleted index that will be persisted and recovered.
-     */
-    public void setMaxBatchDeletedIndexToPersist(int maxBatchDeletedIndexToPersist) {
-        this.maxBatchDeletedIndexToPersist = maxBatchDeletedIndexToPersist;
-    }
-
     public boolean isPersistentUnackedRangesWithMultipleEntriesEnabled() {
         return persistentUnackedRangesWithMultipleEntriesEnabled;
     }
@@ -522,10 +504,8 @@ public class ManagedLedgerConfig {
         return maxUnackedRangesToPersistInMetadataStore;
     }
 
-    public ManagedLedgerConfig setMaxUnackedRangesToPersistInMetadataStore(
-            int maxUnackedRangesToPersistInMetadataStore) {
+    public void setMaxUnackedRangesToPersistInMetadataStore(int maxUnackedRangesToPersistInMetadataStore) {
         this.maxUnackedRangesToPersistInMetadataStore = maxUnackedRangesToPersistInMetadataStore;
-        return this;
     }
 
     /**
@@ -711,14 +691,6 @@ public class ManagedLedgerConfig {
         this.inactiveLedgerRollOverTimeMs = (int) unit.toMillis(inactiveLedgerRollOverTimeMs);
     }
 
-    public long getInactiveOffloadedLedgerEvictionTimeMs() {
-        return inactiveOffloadedLedgerEvictionTimeMs;
-    }
-
-    public void setInactiveOffloadedLedgerEvictionTime(long inactiveOffloadedLedgerEvictionTime, TimeUnit unit) {
-        this.inactiveOffloadedLedgerEvictionTimeMs = unit.toMillis(inactiveOffloadedLedgerEvictionTime);
-    }
-
     /**
      * Minimum cursors with backlog after which broker is allowed to cache read entries to reuse them for other cursors'
      * backlog reads. (Default = 0, broker will not cache backlog reads)
