@@ -26,7 +26,6 @@ import java.util.concurrent.TimeUnit;
 import lombok.SneakyThrows;
 import org.apache.bookkeeper.client.PulsarMockBookKeeper;
 import org.apache.bookkeeper.common.util.OrderedScheduler;
-import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.ManagedLedgerFactoryConfig;
 import org.apache.bookkeeper.mledger.impl.ManagedLedgerFactoryImpl;
 import org.apache.pulsar.metadata.api.MetadataStoreConfig;
@@ -71,8 +70,7 @@ public abstract class MockedBookKeeperTestCase {
     public final void setUp(Method method) throws Exception {
         LOG.info(">>>>>> starting {}", method);
         metadataStore = new FaultInjectionMetadataStore(
-                MetadataStoreExtended.create("memory:local",
-                        MetadataStoreConfig.builder().metadataStoreName("metastore-" + method.getName()).build()));
+                MetadataStoreExtended.create("memory:local", MetadataStoreConfig.builder().build()));
 
         try {
             // start bookkeeper service
@@ -83,17 +81,13 @@ public abstract class MockedBookKeeperTestCase {
         }
 
         ManagedLedgerFactoryConfig managedLedgerFactoryConfig = new ManagedLedgerFactoryConfig();
-        initManagedLedgerFactoryConfig(managedLedgerFactoryConfig);
+        // increase default cache eviction interval so that caching could be tested with less flakyness
+        managedLedgerFactoryConfig.setCacheEvictionIntervalMs(200);
         factory = new ManagedLedgerFactoryImpl(metadataStore, bkc);
 
         setUpTestCase();
     }
 
-    protected void initManagedLedgerFactoryConfig(ManagedLedgerFactoryConfig config) {
-        // increase default cache eviction interval so that caching could be tested with less flakyness
-        config.setCacheEvictionIntervalMs(200);
-    }
-
     protected void setUpTestCase() throws Exception {
 
     }
@@ -108,11 +102,7 @@ public abstract class MockedBookKeeperTestCase {
         }
         try {
             LOG.info("@@@@@@@@@ stopping " + method);
-            try {
-                factory.shutdownAsync().get(10, TimeUnit.SECONDS);
-            } catch (ManagedLedgerException.ManagedLedgerFactoryClosedException e) {
-                // ignore
-            }
+            factory.shutdownAsync().get(10, TimeUnit.SECONDS);
             factory = null;
             stopBookKeeper();
             metadataStore.close();
