@@ -49,7 +49,6 @@ import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.impl.auth.AuthenticationSasl;
 import org.apache.pulsar.common.configuration.PulsarConfigurationLoader;
-import org.apache.pulsar.common.util.ObjectMapperFactory;
 import org.apache.pulsar.proxy.server.ProxyConfiguration;
 import org.apache.pulsar.proxy.server.ProxyService;
 import org.slf4j.Logger;
@@ -194,17 +193,15 @@ public class ProxySaslAuthenticationTest extends ProducerConsumerBase {
 		conf.setAuthenticationProviders(providers);
 		conf.setClusterName("test");
 		conf.setSuperUserRoles(ImmutableSet.of("client/" + localHostname + "@" + kdc.getRealm()));
-		// set admin auth, to verify admin web resources
-		Map<String, String> clientSaslConfig = new HashMap<>();
-		clientSaslConfig.put("saslJaasClientSectionName", "PulsarClient");
-		clientSaslConfig.put("serverType", "broker");
-		conf.setBrokerClientAuthenticationPlugin(AuthenticationSasl.class.getName());
-		conf.setBrokerClientAuthenticationParameters(ObjectMapperFactory
-				.getMapper().getObjectMapper().writeValueAsString(clientSaslConfig));
 
 		super.init();
 
 		lookupUrl = new URI(pulsar.getBrokerServiceUrl());
+
+		// set admin auth, to verify admin web resources
+		Map<String, String> clientSaslConfig = new HashMap<>();
+		clientSaslConfig.put("saslJaasClientSectionName", "PulsarClient");
+		clientSaslConfig.put("serverType", "broker");
 		log.info("set client jaas section name: PulsarClient");
 		admin = PulsarAdmin.builder()
 			.serviceHttpUrl(brokerUrl.toString())
@@ -258,11 +255,7 @@ public class ProxySaslAuthenticationTest extends ProducerConsumerBase {
 		proxyConfig.setForwardAuthorizationCredentials(true);
 		AuthenticationService authenticationService = new AuthenticationService(
                         PulsarConfigurationLoader.convertFrom(proxyConfig));
-		@Cleanup
-		final Authentication proxyClientAuthentication = AuthenticationFactory.create(proxyConfig.getBrokerClientAuthenticationPlugin(),
-				proxyConfig.getBrokerClientAuthenticationParameters());
-		proxyClientAuthentication.start();
-		ProxyService proxyService = new ProxyService(proxyConfig, authenticationService, proxyClientAuthentication);
+		ProxyService proxyService = new ProxyService(proxyConfig, authenticationService);
 
 		proxyService.start();
 		final String proxyServiceUrl = "pulsar://localhost:" + proxyService.getListenPort().get();
