@@ -19,7 +19,6 @@
 package org.apache.pulsar.bookie.rackawareness;
 
 import static org.apache.pulsar.bookie.rackawareness.BookieRackAffinityMapping.METADATA_STORE_INSTANCE;
-import com.google.common.annotations.VisibleForTesting;
 import io.netty.util.HashedWheelTimer;
 import java.util.Arrays;
 import java.util.Collections;
@@ -28,6 +27,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
+import java.util.concurrent.CompletableFuture;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.client.BKException.BKNotEnoughBookiesException;
 import org.apache.bookkeeper.client.RackawareEnsemblePlacementPolicy;
@@ -61,7 +61,6 @@ public class IsolatedBookieEnsemblePlacementPolicy extends RackawareEnsemblePlac
 
     private static final String PULSAR_SYSTEM_TOPIC_ISOLATION_GROUP = "*";
 
-    private volatile BookiesRackConfiguration cachedRackConfiguration = null;
 
     public IsolatedBookieEnsemblePlacementPolicy() {
         super();
@@ -73,7 +72,7 @@ public class IsolatedBookieEnsemblePlacementPolicy extends RackawareEnsemblePlac
             StatsLogger statsLogger, BookieAddressResolver bookieAddressResolver) {
         MetadataStore store;
         try {
-            store = BookieRackAffinityMapping.getMetadataStore(conf);
+            store = BookieRackAffinityMapping.createMetadataStore(conf);
         } catch (MetadataException e) {
             throw new RuntimeException(METADATA_STORE_INSTANCE + " failed initialized");
         }
@@ -87,12 +86,7 @@ public class IsolatedBookieEnsemblePlacementPolicy extends RackawareEnsemblePlac
             }
             // Only add the bookieMappingCache if we have defined an isolation group
             bookieMappingCache = store.getMetadataCache(BookiesRackConfiguration.class);
-            bookieMappingCache.get(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH).thenAccept(opt -> opt.ifPresent(
-                            bookiesRackConfiguration -> cachedRackConfiguration = bookiesRackConfiguration))
-                    .exceptionally(e -> {
-                        log.warn("Failed to load bookies rack configuration while initialize the PlacementPolicy.");
-                        return null;
-                    });
+            bookieMappingCache.get(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH).join();
         }
         if (conf.getProperty(SECONDARY_ISOLATION_BOOKIE_GROUPS) != null) {
             String secondaryIsolationGroupsString = ConfigurationStringUtil
@@ -185,30 +179,26 @@ public class IsolatedBookieEnsemblePlacementPolicy extends RackawareEnsemblePlac
         return pair;
     }
 
-    @VisibleForTesting
-    Set<BookieId> getExcludedBookiesWithIsolationGroups(int ensembleSize,
+    private Set<BookieId> getExcludedBookiesWithIsolationGroups(int ensembleSize,
         Pair<Set<String>, Set<String>> isolationGroups) {
         Set<BookieId> excludedBookies = new HashSet<>();
-        if (isolationGroups != null && isolationGroups.getLeft().contains(PULSAR_SYSTEM_TOPIC_ISOLATION_GROUP)) {
+        if (isolationGroups != null && isolationGroups.getLeft().contains(PULSAR_SYSTEM_TOPIC_ISOLATION_GROUP))  {
             return excludedBookies;
         }
         try {
             if (bookieMappingCache != null) {
-                bookieMappingCache.get(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH)
-                        .thenAccept(opt -> cachedRackConfiguration = opt.orElse(null)).exceptionally(e -> {
-                            log.warn("Failed to update the newest bookies rack config.");
-                            return null;
-                        });
+                CompletableFuture<Optional<BookiesRackConfiguration>> future =
+                        bookieMappingCache.get(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH);
 
-                BookiesRackConfiguration allGroupsBookieMapping = cachedRackConfiguration;
-                if (allGroupsBookieMapping == null) {
-                    log.debug("The bookies rack config is not available at now.");
-                    return excludedBookies;
-                }
-                Set<String> allGroups = allGroupsBookieMapping.keySet();
-                if (allGroups.isEmpty()) {
+                Optional<BookiesRackConfiguration> optRes = (future.isDone() && !future.isCompletedExceptionally())
+                        ? future.join() : Optional.empty();
+
+                if (optRes.isEmpty()) {
                     return excludedBookies;
                 }
+
+                BookiesRackConfiguration allGroupsBookieMapping = optRes.get();
+                Set<String> allBookies = allGroupsBookieMapping.keySet();
                 int totalAvailableBookiesInPrimaryGroup = 0;
                 Set<String> primaryIsolationGroup = Collections.emptySet();
                 Set<String> secondaryIsolationGroup = Collections.emptySet();
@@ -217,7 +207,7 @@ public class IsolatedBookieEnsemblePlacementPolicy extends RackawareEnsemblePlac
                     primaryIsolationGroup = isolationGroups.getLeft();
                     secondaryIsolationGroup = isolationGroups.getRight();
                 }
-                for (String group : allGroups) {
+                for (String group : allBookies) {
                     Set<String> bookiesInGroup = allGroupsBookieMapping.get(group).keySet();
                     if (!primaryIsolationGroup.contains(group)) {
                         for (String bookieAddress : bookiesInGroup) {
@@ -225,10 +215,9 @@ public class IsolatedBookieEnsemblePlacementPolicy extends RackawareEnsemblePlac
                         }
                     } else {
                         for (String groupBookie : bookiesInGroup) {
-                            BookieId bookieId = BookieId.parse(groupBookie);
-                            if (primaryGroupBookies.add(bookieId)) {
-                                totalAvailableBookiesInPrimaryGroup += knownBookies.containsKey(bookieId) ? 1 : 0;
-                            }
+                            totalAvailableBookiesInPrimaryGroup += knownBookies
+                                .containsKey(BookieId.parse(groupBookie)) ? 1 : 0;
+                            primaryGroupBookies.add(BookieId.parse(groupBookie));
                         }
                     }
                 }
@@ -260,9 +249,8 @@ public class IsolatedBookieEnsemblePlacementPolicy extends RackawareEnsemblePlac
                         Map<String, BookieInfo> bookieGroup = allGroupsBookieMapping.get(group);
                         if (bookieGroup != null && !bookieGroup.isEmpty()) {
                             for (String bookieAddress : bookieGroup.keySet()) {
-                                if (excludedBookies.remove(BookieId.parse(bookieAddress))) {
-                                    totalAvailableBookiesFromPrimaryAndSecondary += 1;
-                                }
+                                excludedBookies.remove(BookieId.parse(bookieAddress));
+                                totalAvailableBookiesFromPrimaryAndSecondary += 1;
                             }
                         }
                     }
