@@ -19,12 +19,8 @@
 package org.apache.pulsar.bookie.rackawareness;
 
 import static org.apache.bookkeeper.feature.SettableFeatureProvider.DISABLE_ALL;
-import static org.apache.pulsar.bookie.rackawareness.BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH;
-import static org.mockito.Mockito.mock;
-import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNull;
-import static org.testng.Assert.assertTrue;
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.google.common.collect.Lists;
 import com.google.common.util.concurrent.ThreadFactoryBuilder;
@@ -32,7 +28,6 @@ import io.netty.util.HashedWheelTimer;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -40,12 +35,7 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
-import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
-import lombok.Cleanup;
 import org.apache.bookkeeper.client.DefaultBookieAddressResolver;
 import org.apache.bookkeeper.client.EnsemblePlacementPolicy;
 import org.apache.bookkeeper.client.RackawareEnsemblePlacementPolicy;
@@ -56,21 +46,15 @@ import org.apache.bookkeeper.discover.RegistrationClient;
 import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.net.BookieNode;
 import org.apache.bookkeeper.net.BookieSocketAddress;
-import org.apache.bookkeeper.net.NetworkTopology;
 import org.apache.bookkeeper.proto.BookieAddressResolver;
 import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.stats.StatsLogger;
-import org.apache.bookkeeper.versioning.Version;
-import org.apache.bookkeeper.versioning.Versioned;
 import org.apache.pulsar.common.policies.data.BookieInfo;
 import org.apache.pulsar.common.policies.data.BookiesRackConfiguration;
 import org.apache.pulsar.common.util.ObjectMapperFactory;
-import org.apache.pulsar.metadata.api.MetadataCache;
 import org.apache.pulsar.metadata.api.MetadataStore;
 import org.apache.pulsar.metadata.api.MetadataStoreConfig;
 import org.apache.pulsar.metadata.api.MetadataStoreFactory;
-import org.apache.pulsar.metadata.api.Notification;
-import org.apache.pulsar.metadata.api.NotificationType;
 import org.apache.pulsar.metadata.bookkeeper.BookieServiceInfoSerde;
 import org.apache.pulsar.metadata.bookkeeper.PulsarRegistrationClient;
 import org.awaitility.Awaitility;
@@ -105,7 +89,7 @@ public class BookieRackAffinityMappingTest {
         String data = "{\"group1\": {\"" + BOOKIE1
                 + "\": {\"rack\": \"/rack0\", \"hostname\": \"bookie1.example.com\"}, \"" + BOOKIE2
                 + "\": {\"rack\": \"/rack1\", \"hostname\": \"bookie2.example.com\"}}}";
-        store.put(BOOKIE_INFO_ROOT_PATH, data.getBytes(), Optional.empty()).join();
+        store.put(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH, data.getBytes(), Optional.empty()).join();
 
         // Case1: ZKCache is given
         BookieRackAffinityMapping mapping = new BookieRackAffinityMapping();
@@ -140,7 +124,7 @@ public class BookieRackAffinityMappingTest {
                 + "\": {\"rack\": \"/\", \"hostname\": \"bookie1.example.com\"}, \"" + BOOKIE2
                 + "\": {\"rack\": \"\", \"hostname\": \"bookie2.example.com\"}}}";
 
-        store.put(BOOKIE_INFO_ROOT_PATH, data.getBytes(), Optional.empty()).join();
+        store.put(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH, data.getBytes(), Optional.empty()).join();
 
         // Case1: ZKCache is given
         BookieRackAffinityMapping mapping1 = new BookieRackAffinityMapping();
@@ -178,7 +162,7 @@ public class BookieRackAffinityMappingTest {
 
         bookieMapping.put("group1", mainBookieGroup);
 
-        store.put(BOOKIE_INFO_ROOT_PATH, jsonMapper.writeValueAsBytes(bookieMapping),
+        store.put(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH, jsonMapper.writeValueAsBytes(bookieMapping),
                 Optional.empty()).join();
 
         Awaitility.await().untilAsserted(() -> {
@@ -200,7 +184,7 @@ public class BookieRackAffinityMappingTest {
 
         bookieMapping.put("group1", mainBookieGroup);
 
-        store.put(BOOKIE_INFO_ROOT_PATH, jsonMapper.writeValueAsBytes(bookieMapping),
+        store.put(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH, jsonMapper.writeValueAsBytes(bookieMapping),
                 Optional.empty()).join();
 
         BookieRackAffinityMapping mapping = new BookieRackAffinityMapping();
@@ -220,7 +204,7 @@ public class BookieRackAffinityMappingTest {
         secondaryBookieGroup.put(BOOKIE3, BookieInfo.builder().rack("rack0").build());
 
         bookieMapping.put("group2", secondaryBookieGroup);
-        store.put(BOOKIE_INFO_ROOT_PATH, jsonMapper.writeValueAsBytes(bookieMapping),
+        store.put(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH, jsonMapper.writeValueAsBytes(bookieMapping),
                 Optional.empty()).join();
         Awaitility.await().untilAsserted(() -> {
             List<String> r = mapping.resolve(Lists.newArrayList("127.0.0.1", "127.0.0.2", "127.0.0.3"));
@@ -228,7 +212,7 @@ public class BookieRackAffinityMappingTest {
             assertEquals(r.get(1), "/rack1");
             assertEquals(r.get(2), "/rack0");
         });
-        store.put(BOOKIE_INFO_ROOT_PATH, "{}".getBytes(),
+        store.put(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH, "{}".getBytes(),
                 Optional.empty()).join();
 
         Awaitility.await().untilAsserted(() -> {
@@ -244,7 +228,7 @@ public class BookieRackAffinityMappingTest {
         String data = "{\"group1\": {\"" + BOOKIE1
                 + "\": {\"rack\": \"/rack0\", \"hostname\": \"bookie1.example.com\"}, \"" + BOOKIE2
                 + "\": {\"rack\": \"/rack1\", \"hostname\": \"bookie2.example.com\"}}}";
-        store.put(BOOKIE_INFO_ROOT_PATH, data.getBytes(), Optional.empty()).join();
+        store.put(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH, data.getBytes(), Optional.empty()).join();
 
         // Case1: ZKCache is given
         BookieRackAffinityMapping mapping = new BookieRackAffinityMapping();
@@ -264,10 +248,12 @@ public class BookieRackAffinityMappingTest {
                 .stream().filter(Objects::nonNull).toList();
         assertEquals(racks.size(), 0);
 
-        @Cleanup("stop")
-        HashedWheelTimer timer = getTestHashedWheelTimer(bkClientConf);
+        HashedWheelTimer timer = new HashedWheelTimer(
+                new ThreadFactoryBuilder().setNameFormat("TestTimer-%d").build(),
+                bkClientConf.getTimeoutTimerTickDurationMs(), TimeUnit.MILLISECONDS,
+                bkClientConf.getTimeoutTimerNumTicks());
+
         RackawareEnsemblePlacementPolicy repp = new RackawareEnsemblePlacementPolicy();
-        mapping.registerRackChangeListener(repp);
         Class<?> clazz1 = Class.forName("org.apache.bookkeeper.client.TopologyAwareEnsemblePlacementPolicy");
         Field field1 = clazz1.getDeclaredField("knownBookies");
         field1.setAccessible(true);
@@ -337,183 +323,6 @@ public class BookieRackAffinityMappingTest {
         assertEquals(knownBookies.get(BOOKIE2.toBookieId()).getNetworkLocation(), "/rack1");
         assertEquals(knownBookies.get(BOOKIE3.toBookieId()).getNetworkLocation(), "/default-rack");
 
-        //remove bookie2 rack, the bookie2 rack should be /default-rack
-        data = "{\"group1\": {\"" + BOOKIE1
-                + "\": {\"rack\": \"/rack0\", \"hostname\": \"bookie1.example.com\"}}}";
-        store.put(BOOKIE_INFO_ROOT_PATH, data.getBytes(), Optional.empty()).join();
-        Awaitility.await().atMost(30, TimeUnit.SECONDS).until(() -> ((BookiesRackConfiguration)field.get(mapping)).get("group1").size() == 1);
-
-        racks = mapping
-                .resolve(Lists.newArrayList(BOOKIE1.getHostName(), BOOKIE2.getHostName(), BOOKIE3.getHostName()))
-                .stream().filter(Objects::nonNull).toList();
-        assertEquals(racks.size(), 1);
-        assertEquals(racks.get(0), "/rack0");
-        assertEquals(knownBookies.size(), 3);
-        assertEquals(knownBookies.get(BOOKIE1.toBookieId()).getNetworkLocation(), "/rack0");
-        assertEquals(knownBookies.get(BOOKIE2.toBookieId()).getNetworkLocation(), "/default-rack");
-        assertEquals(knownBookies.get(BOOKIE3.toBookieId()).getNetworkLocation(), "/default-rack");
-    }
-
-    @Test
-    public void testNoDeadlockWithRackawarePolicy() throws Exception {
-        ClientConfiguration bkClientConf = new ClientConfiguration();
-        bkClientConf.setProperty(BookieRackAffinityMapping.METADATA_STORE_INSTANCE, store);
-
-        BookieRackAffinityMapping mapping = new BookieRackAffinityMapping();
-        mapping.setBookieAddressResolver(BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
-        mapping.setConf(bkClientConf);
-
-        @Cleanup("stop")
-        HashedWheelTimer timer = getTestHashedWheelTimer(bkClientConf);
-        RackawareEnsemblePlacementPolicy repp = new RackawareEnsemblePlacementPolicy();
-        repp.initialize(bkClientConf, Optional.of(mapping), timer,
-                DISABLE_ALL, NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
-        repp.withDefaultRack(NetworkTopology.DEFAULT_REGION_AND_RACK);
-
-        mapping.registerRackChangeListener(repp);
-
-        @Cleanup("shutdownNow")
-        ExecutorService executor1 = Executors.newSingleThreadExecutor();
-        @Cleanup("shutdownNow")
-        ExecutorService executor2 = Executors.newSingleThreadExecutor();
-
-        CountDownLatch count = new CountDownLatch(2);
-
-        executor1.submit(() -> {
-            try {
-                Method handleUpdates =
-                        BookieRackAffinityMapping.class.getDeclaredMethod("handleUpdates", Notification.class);
-                handleUpdates.setAccessible(true);
-                Notification n =
-                        new Notification(NotificationType.Modified, BOOKIE_INFO_ROOT_PATH);
-                long start = System.currentTimeMillis();
-                while (System.currentTimeMillis() - start < 2_000) {
-                    handleUpdates.invoke(mapping, n);
-                }
-                count.countDown();
-            } catch (Exception e) {
-                throw new RuntimeException(e);
-            }
-        });
-
-        executor2.submit(() -> {
-            Set<BookieId> writableBookies = new HashSet<>();
-            writableBookies.add(BOOKIE1.toBookieId());
-            long start = System.currentTimeMillis();
-            while (System.currentTimeMillis() - start < 2_000) {
-                repp.onClusterChanged(writableBookies, Collections.emptySet());
-                repp.onClusterChanged(Collections.emptySet(), Collections.emptySet());
-            }
-            count.countDown();
-        });
-
-        assertTrue(count.await(3, TimeUnit.SECONDS));
-    }
-
-    @Test
-    public void testZKEventListenersOrdering() throws Exception {
-        @Cleanup
-        PulsarRegistrationClient pulsarRegistrationClient =
-                new PulsarRegistrationClient(store, "/ledgers");
-        DefaultBookieAddressResolver defaultBookieAddressResolver =
-                new DefaultBookieAddressResolver(pulsarRegistrationClient);
-        // Create and configure the mapping
-        BookieRackAffinityMapping mapping = new BookieRackAffinityMapping();
-        ClientConfiguration bkClientConf = new ClientConfiguration();
-        bkClientConf.setProperty(BookieRackAffinityMapping.METADATA_STORE_INSTANCE, store);
-        mapping.setBookieAddressResolver(defaultBookieAddressResolver);
-        mapping.setConf(bkClientConf);
-
-        // Create RackawareEnsemblePlacementPolicy and initialize it
-        @Cleanup("stop")
-        HashedWheelTimer timer = getTestHashedWheelTimer(bkClientConf);
-        RackawareEnsemblePlacementPolicy repp = new RackawareEnsemblePlacementPolicy();
-        repp.initialize(bkClientConf, Optional.of(mapping), timer,
-                DISABLE_ALL, NullStatsLogger.INSTANCE, defaultBookieAddressResolver);
-        mapping.registerRackChangeListener(repp);
-
-        // Create a BookieWatcherImpl instance via reflection
-        Class<?> watcherClazz = Class.forName("org.apache.bookkeeper.client.BookieWatcherImpl");
-        Constructor<?> constructor = watcherClazz.getDeclaredConstructor(
-                ClientConfiguration.class,
-                EnsemblePlacementPolicy.class,
-                RegistrationClient.class,
-                BookieAddressResolver.class,
-                StatsLogger.class);
-        constructor.setAccessible(true);
-        Object watcher = constructor.newInstance(
-                bkClientConf,
-                repp,
-                pulsarRegistrationClient,
-                defaultBookieAddressResolver,
-                NullStatsLogger.INSTANCE
-        );
-        Method initMethod = watcherClazz.getDeclaredMethod("initialBlockingBookieRead");
-        initMethod.setAccessible(true);
-        initMethod.invoke(watcher);
-
-        // Prepare a BookiesRackConfiguration that maps BOOKIE1 -> /rack0
-        BookieInfo bi = BookieInfo.builder().rack("/rack0").build();
-        BookiesRackConfiguration racks = new BookiesRackConfiguration();
-        racks.updateBookie("group1", BOOKIE1.toString(), bi);
-
-        // Create a mock cache for racks /bookies
-        MetadataCache<BookiesRackConfiguration> mockCache = mock(MetadataCache.class);
-        Field f = BookieRackAffinityMapping.class.getDeclaredField("bookieMappingCache");
-        f.setAccessible(true);
-        f.set(mapping, mockCache);
-        when(mockCache.get(BookieRackAffinityMapping.BOOKIE_INFO_ROOT_PATH))
-                .thenReturn(CompletableFuture.completedFuture(Optional.of(racks)));
-
-        // Inject the bookie address list into BookieRackAffinityMapping
-        Field addressListField = BookieRackAffinityMapping.class.getDeclaredField("bookieAddressListLastTime");
-        addressListField.setAccessible(true);
-        addressListField.set(mapping, List.of(BOOKIE1.toBookieId()));
-
-        // Inject the writable bookie into PulsarRegistrationClient
-        Field writableField = PulsarRegistrationClient.class.getDeclaredField("writableBookieInfo");
-        writableField.setAccessible(true);
-        @SuppressWarnings("unchecked")
-        Map<BookieId, Versioned<BookieServiceInfo>> writableBookieInfo =
-                (Map<BookieId, Versioned<BookieServiceInfo>>) writableField.get(pulsarRegistrationClient);
-        writableBookieInfo.put(
-                BOOKIE1.toBookieId(),
-                new Versioned<>(BookieServiceInfoUtils.buildLegacyBookieServiceInfo(BOOKIE1.toString()), Version.NEW)
-        );
-
-        // watcher.processWritableBookiesChanged runs FIRST triggering RackAware ensemble policy listener → incorrect
-        // ordering
-        Method procMethod =
-                watcherClazz.getDeclaredMethod("processWritableBookiesChanged", java.util.Set.class);
-        procMethod.setAccessible(true);
-        Set<BookieId> ids = new HashSet<>();
-        ids.add(BOOKIE1.toBookieId());
-        procMethod.invoke(watcher, ids);
-
-        // BookieRackAffinityMapping rack mapping update runs SECOND → delayed rack info
-        Method processRackUpdateMethod = BookieRackAffinityMapping.class.getDeclaredMethod("processRackUpdate",
-                BookiesRackConfiguration.class);
-        processRackUpdateMethod.setAccessible(true);
-        processRackUpdateMethod.invoke(mapping, racks);
-
-        // -------------------
-        // NOW CHECK REPP INTERNAL STATE
-        // -------------------
-        // BookieNode.getNetworkLocation()
-        Class<?> clazz1 = Class.forName("org.apache.bookkeeper.client.TopologyAwareEnsemblePlacementPolicy");
-        Field field1 = clazz1.getDeclaredField("knownBookies");
-        field1.setAccessible(true);
-        Map<BookieId, BookieNode> knownBookies = (Map<BookieId, BookieNode>) field1.get(repp);
-        BookieNode bn = knownBookies.get(BOOKIE1.toBookieId());
-        // Rack info update is delayed but because of new callback the rack info on ensemble policy should be updated.
-        assertEquals(bn.getNetworkLocation(), "/rack0",
-                "Network location should match /rack0 on bookie");
-    }
-
-    private static HashedWheelTimer getTestHashedWheelTimer(ClientConfiguration bkClientConf) {
-        return new HashedWheelTimer(
-                new ThreadFactoryBuilder().setNameFormat("TestTimer-%d").build(),
-                bkClientConf.getTimeoutTimerTickDurationMs(), TimeUnit.MILLISECONDS,
-                bkClientConf.getTimeoutTimerNumTicks());
+        timer.stop();
     }
 }
