@@ -44,8 +44,8 @@ import org.apache.pulsar.common.partition.PartitionedTopicMetadata;
 import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.Policies;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
-import org.apache.pulsar.common.util.CmdGenerateDocs;
 import org.apache.pulsar.common.util.ShutdownUtil;
+import org.apache.pulsar.docs.tools.CmdGenerateDocs;
 import org.apache.pulsar.functions.worker.WorkerUtils;
 import org.apache.pulsar.metadata.api.MetadataStore;
 import org.apache.pulsar.metadata.api.MetadataStoreConfig;
@@ -94,19 +94,6 @@ public class PulsarClusterMetadataSetup {
                 description = "Broker-service URL for new cluster with TLS encryption", required = false)
         private String clusterBrokerServiceUrlTls;
 
-        @Parameter(names = {"-te",
-                "--tls-enable"},
-                description = "Enable TLS connection for new cluster")
-        private Boolean clusterBrokerClientTlsEnabled;
-
-        @Parameter(names = "--auth-plugin",
-                description = "The authentication plugin for new cluster")
-        protected String clusterAuthenticationPlugin;
-
-        @Parameter(names = "--auth-parameters",
-                description = "The authentication parameters for new cluster")
-        protected String clusterAuthenticationParameters;
-
         @Parameter(names = { "-zk",
                 "--zookeeper" }, description = "Local ZooKeeper quorum connection string",
                 required = false,
@@ -137,17 +124,8 @@ public class PulsarClusterMetadataSetup {
                 hidden = false)
         private String configurationMetadataStore;
 
-        @Parameter(names = {"-mscp",
-                "--metadata-store-config-path"}, description = "Metadata Store config path", hidden = false)
-        private String metadataStoreConfigPath;
-
-        @Parameter(names = {"-cmscp",
-                "--configuration-metadata-store-config-path"}, description = "Configuration Metadata Store config path",
-                hidden = false)
-        private String configurationStoreConfigPath;
-
         @Parameter(names = {
-                "--initial-num-stream-storage-containers"
+            "--initial-num-stream-storage-containers"
         }, description = "Num storage containers of BookKeeper stream storage")
         private int numStreamStorageContainers = 16;
 
@@ -290,11 +268,9 @@ public class PulsarClusterMetadataSetup {
         log.info("Setting up cluster {} with metadata-store={} configuration-metadata-store={}", arguments.cluster,
                 arguments.metadataStoreUrl, arguments.configurationMetadataStore);
 
-        MetadataStoreExtended localStore = initLocalMetadataStore(arguments.metadataStoreUrl,
-                arguments.metadataStoreConfigPath,
-                arguments.zkSessionTimeoutMillis);
+        MetadataStoreExtended localStore =
+                initLocalMetadataStore(arguments.metadataStoreUrl, arguments.zkSessionTimeoutMillis);
         MetadataStoreExtended configStore = initConfigMetadataStore(arguments.configurationMetadataStore,
-                arguments.configurationStoreConfigPath,
                 arguments.zkSessionTimeoutMillis);
 
         final String metadataStoreUrlNoIdentifer = MetadataStoreFactoryImpl
@@ -339,36 +315,14 @@ public class PulsarClusterMetadataSetup {
 
         PulsarResources resources = new PulsarResources(localStore, configStore);
 
-        ClusterData.Builder clusterDataBuilder = ClusterData.builder();
-        if (arguments.clusterWebServiceUrl != null) {
-            clusterDataBuilder.serviceUrl(arguments.clusterWebServiceUrl);
-        }
-        if (arguments.clusterWebServiceUrlTls != null) {
-            clusterDataBuilder.serviceUrlTls(arguments.clusterWebServiceUrlTls);
-        }
-        if (arguments.clusterBrokerServiceUrl != null) {
-            clusterDataBuilder.brokerServiceUrl(arguments.clusterBrokerServiceUrl);
-        }
-        if (arguments.clusterBrokerServiceUrlTls != null) {
-            clusterDataBuilder.brokerServiceUrlTls(arguments.clusterBrokerServiceUrlTls);
-        }
-        if (arguments.clusterBrokerClientTlsEnabled != null) {
-            clusterDataBuilder.brokerClientTlsEnabled(arguments.clusterBrokerClientTlsEnabled);
-        }
-        if (arguments.clusterAuthenticationPlugin != null) {
-            clusterDataBuilder.authenticationPlugin(arguments.clusterAuthenticationPlugin);
-        }
-        if (arguments.clusterAuthenticationParameters != null) {
-            clusterDataBuilder.authenticationParameters(arguments.clusterAuthenticationParameters);
-        }
-        if (arguments.clusterProxyUrl != null) {
-            clusterDataBuilder.proxyServiceUrl(arguments.clusterProxyUrl);
-        }
-        if (arguments.clusterProxyProtocol != null) {
-            clusterDataBuilder.proxyProtocol(arguments.clusterProxyProtocol);
-        }
-
-        ClusterData clusterData = clusterDataBuilder.build();
+        ClusterData clusterData = ClusterData.builder()
+                .serviceUrl(arguments.clusterWebServiceUrl)
+                .serviceUrlTls(arguments.clusterWebServiceUrlTls)
+                .brokerServiceUrl(arguments.clusterBrokerServiceUrl)
+                .brokerServiceUrlTls(arguments.clusterBrokerServiceUrlTls)
+                .proxyServiceUrl(arguments.clusterProxyUrl)
+                .proxyProtocol(arguments.clusterProxyProtocol)
+                .build();
         if (!resources.getClusterResources().clusterExists(arguments.cluster)) {
             resources.getClusterResources().createCluster(arguments.cluster, clusterData);
         }
@@ -419,8 +373,8 @@ public class PulsarClusterMetadataSetup {
         }
     }
 
-    public static void createNamespaceIfAbsent(PulsarResources resources, NamespaceName namespaceName,
-                                               String cluster, int bundleNumber) throws IOException {
+    static void createNamespaceIfAbsent(PulsarResources resources, NamespaceName namespaceName,
+            String cluster, int bundleNumber) throws IOException {
         NamespaceResources namespaceResources = resources.getNamespaceResources();
 
         if (!namespaceResources.namespaceExists(namespaceName)) {
@@ -473,17 +427,9 @@ public class PulsarClusterMetadataSetup {
         }
     }
 
-    public static MetadataStoreExtended initLocalMetadataStore(String connection,
-                                                               int sessionTimeout) throws Exception {
-        return initLocalMetadataStore(connection, null, sessionTimeout);
-    }
-
-    public static MetadataStoreExtended initLocalMetadataStore(String connection,
-                                                               String configPath,
-                                                               int sessionTimeout) throws Exception {
+    public static MetadataStoreExtended initLocalMetadataStore(String connection, int sessionTimeout) throws Exception {
         MetadataStoreExtended store = MetadataStoreExtended.create(connection, MetadataStoreConfig.builder()
                 .sessionTimeoutMillis(sessionTimeout)
-                .configFilePath(configPath)
                 .metadataStoreName(MetadataStoreConfig.METADATA_STORE)
                 .build());
         if (store instanceof MetadataStoreLifecycle) {
@@ -492,19 +438,10 @@ public class PulsarClusterMetadataSetup {
         return store;
     }
 
-    public static MetadataStoreExtended initConfigMetadataStore(String connection,
-                                                                int sessionTimeout)
-            throws Exception {
-        return initConfigMetadataStore(connection, null, sessionTimeout);
-    }
-
-    public static MetadataStoreExtended initConfigMetadataStore(String connection,
-                                                                String configPath,
-                                                                int sessionTimeout)
+    public static MetadataStoreExtended initConfigMetadataStore(String connection, int sessionTimeout)
             throws Exception {
         MetadataStoreExtended store = MetadataStoreExtended.create(connection, MetadataStoreConfig.builder()
                 .sessionTimeoutMillis(sessionTimeout)
-                .configFilePath(configPath)
                 .metadataStoreName(MetadataStoreConfig.CONFIGURATION_METADATA_STORE)
                 .build());
         if (store instanceof MetadataStoreLifecycle) {
