@@ -29,7 +29,6 @@ import io.netty.channel.EventLoopGroup;
 import java.io.IOException;
 import java.util.Map;
 import java.util.Optional;
-import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.TimeUnit;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.client.BKException;
@@ -50,43 +49,39 @@ import org.apache.pulsar.metadata.api.extended.MetadataStoreExtended;
 import org.apache.pulsar.metadata.bookkeeper.AbstractMetadataDriver;
 import org.apache.pulsar.metadata.bookkeeper.PulsarMetadataClientDriver;
 
-@SuppressWarnings("deprecation")
 @Slf4j
 public class BookKeeperClientFactoryImpl implements BookKeeperClientFactory {
 
     @Override
-    public CompletableFuture<BookKeeper> create(ServiceConfiguration conf, MetadataStoreExtended store,
-                                                EventLoopGroup eventLoopGroup,
-                                                Optional<Class<? extends EnsemblePlacementPolicy>> policyClass,
-                                                Map<String, Object> properties) {
-        return create(conf, store, eventLoopGroup, policyClass, properties,
+    public BookKeeper create(ServiceConfiguration conf, MetadataStoreExtended store,
+                             EventLoopGroup eventLoopGroup,
+                             Optional<Class<? extends EnsemblePlacementPolicy>> ensemblePlacementPolicyClass,
+                             Map<String, Object> properties) throws IOException {
+        return create(conf, store, eventLoopGroup, ensemblePlacementPolicyClass, properties,
                 NullStatsLogger.INSTANCE);
     }
 
     @Override
-    public CompletableFuture<BookKeeper> create(ServiceConfiguration conf, MetadataStoreExtended store,
+    public BookKeeper create(ServiceConfiguration conf, MetadataStoreExtended store,
                              EventLoopGroup eventLoopGroup,
                              Optional<Class<? extends EnsemblePlacementPolicy>> ensemblePlacementPolicyClass,
-                             Map<String, Object> properties, StatsLogger statsLogger) {
+                             Map<String, Object> properties, StatsLogger statsLogger) throws IOException {
         PulsarMetadataClientDriver.init();
 
         ClientConfiguration bkConf = createBkClientConfiguration(store, conf);
         if (properties != null) {
-            properties.forEach((key, value) -> bkConf.setProperty(key, value));
+            properties.forEach(bkConf::setProperty);
         }
         if (ensemblePlacementPolicyClass.isPresent()) {
             setEnsemblePlacementPolicy(bkConf, conf, store, ensemblePlacementPolicyClass.get());
         } else {
             setDefaultEnsemblePlacementPolicy(bkConf, conf, store);
         }
-
-        return CompletableFuture.supplyAsync(() -> {
-            try {
-                return getBookKeeperBuilder(conf, eventLoopGroup, statsLogger, bkConf).build();
-            } catch (InterruptedException | BKException | IOException e) {
-                throw new RuntimeException(e);
-            }
-        });
+        try {
+            return getBookKeeperBuilder(conf, eventLoopGroup, statsLogger, bkConf).build();
+        } catch (InterruptedException | BKException e) {
+            throw new IOException(e);
+        }
     }
 
     @VisibleForTesting
@@ -224,7 +219,7 @@ public class BookKeeperClientFactoryImpl implements BookKeeperClientFactory {
         }
     }
 
-    static void setEnsemblePlacementPolicy(ClientConfiguration bkConf, ServiceConfiguration conf, MetadataStore store,
+    private void setEnsemblePlacementPolicy(ClientConfiguration bkConf, ServiceConfiguration conf, MetadataStore store,
                                             Class<? extends EnsemblePlacementPolicy> policyClass) {
         bkConf.setEnsemblePlacementPolicy(policyClass);
         bkConf.setProperty(BookieRackAffinityMapping.METADATA_STORE_INSTANCE, store);
@@ -232,9 +227,6 @@ public class BookKeeperClientFactoryImpl implements BookKeeperClientFactory {
             bkConf.setProperty(REPP_DNS_RESOLVER_CLASS, conf.getProperties().getProperty(REPP_DNS_RESOLVER_CLASS,
                     BookieRackAffinityMapping.class.getName()));
 
-            bkConf.setMinNumRacksPerWriteQuorum(conf.getBookkeeperClientMinNumRacksPerWriteQuorum());
-            bkConf.setEnforceMinNumRacksPerWriteQuorum(conf.isBookkeeperClientEnforceMinNumRacksPerWriteQuorum());
-
             bkConf.setProperty(NET_TOPOLOGY_SCRIPT_FILE_NAME_KEY,
                 conf.getProperties().getProperty(
                     NET_TOPOLOGY_SCRIPT_FILE_NAME_KEY,
