@@ -18,14 +18,12 @@
  */
 package org.apache.pulsar.broker;
 
-import com.github.benmanes.caffeine.cache.AsyncCache;
-import com.github.benmanes.caffeine.cache.Caffeine;
 import com.google.common.annotations.VisibleForTesting;
 import io.netty.channel.EventLoopGroup;
 import java.io.IOException;
 import java.util.Map;
 import java.util.Optional;
-import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.RejectedExecutionException;
 import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.conf.ClientConfiguration;
@@ -50,8 +48,8 @@ public class ManagedLedgerClientFactory implements ManagedLedgerStorage {
 
     private ManagedLedgerFactory managedLedgerFactory;
     private BookKeeper defaultBkClient;
-    private final AsyncCache<EnsemblePlacementPolicyConfig, BookKeeper>
-            bkEnsemblePolicyToBkClientMap = Caffeine.newBuilder().buildAsync();
+    private final Map<EnsemblePlacementPolicyConfig, BookKeeper>
+            bkEnsemblePolicyToBkClientMap = new ConcurrentHashMap<>();
     private StatsProvider statsProvider = new NullStatsProvider();
 
     public void initialize(ServiceConfiguration conf, MetadataStoreExtended metadataStore,
@@ -65,28 +63,19 @@ public class ManagedLedgerClientFactory implements ManagedLedgerStorage {
         managedLedgerFactoryConfig.setCacheEvictionTimeThresholdMillis(
                 conf.getManagedLedgerCacheEvictionTimeThresholdMillis());
         managedLedgerFactoryConfig.setCopyEntriesInCache(conf.isManagedLedgerCacheCopyEntries());
-        long managedLedgerMaxReadsInFlightSizeBytes = conf.getManagedLedgerMaxReadsInFlightSizeInMB() * 1024L * 1024L;
-        if (managedLedgerMaxReadsInFlightSizeBytes > 0 && conf.getDispatcherMaxReadSizeBytes() > 0
-                && managedLedgerMaxReadsInFlightSizeBytes < conf.getDispatcherMaxReadSizeBytes()) {
-            log.warn("Invalid configuration for managedLedgerMaxReadsInFlightSizeInMB: {}, "
-                            + "dispatcherMaxReadSizeBytes: {}. managedLedgerMaxReadsInFlightSizeInMB in bytes should "
-                            + "be greater than dispatcherMaxReadSizeBytes. You should set "
-                            + "managedLedgerMaxReadsInFlightSizeInMB to at least {}",
-                    conf.getManagedLedgerMaxReadsInFlightSizeInMB(), conf.getDispatcherMaxReadSizeBytes(),
-                    (conf.getDispatcherMaxReadSizeBytes() / (1024L * 1024L)) + 1);
-        }
-        managedLedgerFactoryConfig.setManagedLedgerMaxReadsInFlightSize(managedLedgerMaxReadsInFlightSizeBytes);
-        managedLedgerFactoryConfig.setManagedLedgerMaxReadsInFlightPermitsAcquireTimeoutMillis(
-                conf.getManagedLedgerMaxReadsInFlightPermitsAcquireTimeoutMillis());
-        managedLedgerFactoryConfig.setManagedLedgerMaxReadsInFlightPermitsAcquireQueueSize(
-                conf.getManagedLedgerMaxReadsInFlightPermitsAcquireQueueSize());
+        managedLedgerFactoryConfig.setManagedLedgerMaxReadsInFlightSize(
+                conf.getManagedLedgerMaxReadsInFlightSizeInMB() * 1024L * 1024L);
         managedLedgerFactoryConfig.setPrometheusStatsLatencyRolloverSeconds(
                 conf.getManagedLedgerPrometheusStatsLatencyRolloverSeconds());
         managedLedgerFactoryConfig.setTraceTaskExecution(conf.isManagedLedgerTraceTaskExecution());
         managedLedgerFactoryConfig.setCursorPositionFlushSeconds(conf.getManagedLedgerCursorPositionFlushSeconds());
         managedLedgerFactoryConfig.setManagedLedgerInfoCompressionType(conf.getManagedLedgerInfoCompressionType());
+        managedLedgerFactoryConfig.setManagedLedgerInfoCompressionThresholdInBytes(
+                conf.getManagedLedgerInfoCompressionThresholdInBytes());
         managedLedgerFactoryConfig.setStatsPeriodSeconds(conf.getManagedLedgerStatsPeriodSeconds());
         managedLedgerFactoryConfig.setManagedCursorInfoCompressionType(conf.getManagedCursorInfoCompressionType());
+        managedLedgerFactoryConfig.setManagedCursorInfoCompressionThresholdInBytes(
+                conf.getManagedCursorInfoCompressionThresholdInBytes());
 
         Configuration configuration = new ClientConfiguration();
         if (conf.isBookkeeperClientExposeStatsToPrometheus()) {
@@ -100,20 +89,27 @@ public class ManagedLedgerClientFactory implements ManagedLedgerStorage {
         StatsLogger statsLogger = statsProvider.getStatsLogger("pulsar_managedLedger_client");
 
         this.defaultBkClient =
-                bookkeeperProvider.create(conf, metadataStore, eventLoopGroup, Optional.empty(), null, statsLogger)
-                        .get();
+                bookkeeperProvider.create(conf, metadataStore, eventLoopGroup, Optional.empty(), null, statsLogger);
 
         BookkeeperFactoryForCustomEnsemblePlacementPolicy bkFactory = (
                 EnsemblePlacementPolicyConfig ensemblePlacementPolicyConfig) -> {
-            if (ensemblePlacementPolicyConfig == null || ensemblePlacementPolicyConfig.getPolicyClass() == null) {
-                return CompletableFuture.completedFuture(defaultBkClient);
-            }
-
+            BookKeeper bkClient = null;
             // find or create bk-client in cache for a specific ensemblePlacementPolicy
-            return bkEnsemblePolicyToBkClientMap.get(ensemblePlacementPolicyConfig,
-                    (config, executor) -> bookkeeperProvider.create(conf, metadataStore, eventLoopGroup,
-                            Optional.ofNullable(ensemblePlacementPolicyConfig.getPolicyClass()),
-                            ensemblePlacementPolicyConfig.getProperties(), statsLogger));
+            if (ensemblePlacementPolicyConfig != null && ensemblePlacementPolicyConfig.getPolicyClass() != null) {
+                bkClient = bkEnsemblePolicyToBkClientMap.computeIfAbsent(ensemblePlacementPolicyConfig, (key) -> {
+                    try {
+                        return bookkeeperProvider.create(conf, metadataStore, eventLoopGroup,
+                                Optional.ofNullable(ensemblePlacementPolicyConfig.getPolicyClass()),
+                                ensemblePlacementPolicyConfig.getProperties(), statsLogger);
+                    } catch (Exception e) {
+                        log.error("Failed to initialize bk-client for policy {}, properties {}",
+                                ensemblePlacementPolicyConfig.getPolicyClass(),
+                                ensemblePlacementPolicyConfig.getProperties(), e);
+                    }
+                    return this.defaultBkClient;
+                });
+            }
+            return bkClient != null ? bkClient : defaultBkClient;
         };
 
         this.managedLedgerFactory =
@@ -134,7 +130,7 @@ public class ManagedLedgerClientFactory implements ManagedLedgerStorage {
 
     @VisibleForTesting
     public Map<EnsemblePlacementPolicyConfig, BookKeeper> getBkEnsemblePolicyToBookKeeperMap() {
-        return bkEnsemblePolicyToBkClientMap.synchronous().asMap();
+        return bkEnsemblePolicyToBkClientMap;
     }
 
     @Override
@@ -162,7 +158,7 @@ public class ManagedLedgerClientFactory implements ManagedLedgerStorage {
                 // factory, however that might be introducing more unknowns.
                 log.warn("Encountered exceptions on closing bookkeeper client", ree);
             }
-            bkEnsemblePolicyToBkClientMap.synchronous().asMap().forEach((policy, bk) -> {
+            bkEnsemblePolicyToBkClientMap.forEach((policy, bk) -> {
                 try {
                     if (bk != null) {
                         bk.close();
