@@ -50,7 +50,6 @@ import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.commons.collections4.CollectionUtils;
-import org.apache.commons.collections4.MapUtils;
 import org.apache.commons.lang3.mutable.MutableLong;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pulsar.broker.delayed.AbstractDelayedDeliveryTracker;
@@ -105,24 +104,22 @@ public class BucketDelayedDeliveryTracker extends AbstractDelayedDeliveryTracker
     private CompletableFuture<Void> pendingLoad = null;
 
     public BucketDelayedDeliveryTracker(PersistentDispatcherMultipleConsumers dispatcher,
-                                        Timer timer, long tickTimeMillis,
-                                        boolean isDelayedDeliveryDeliverAtTimeStrict,
-                                        BucketSnapshotStorage bucketSnapshotStorage,
-                                        long minIndexCountPerBucket, long timeStepPerBucketSnapshotSegmentInMillis,
-                                        int maxIndexesPerBucketSnapshotSegment, int maxNumBuckets)
-            throws RecoverDelayedDeliveryTrackerException {
+                                 Timer timer, long tickTimeMillis,
+                                 boolean isDelayedDeliveryDeliverAtTimeStrict,
+                                 BucketSnapshotStorage bucketSnapshotStorage,
+                                 long minIndexCountPerBucket, long timeStepPerBucketSnapshotSegmentInMillis,
+                                 int maxIndexesPerBucketSnapshotSegment, int maxNumBuckets) {
         this(dispatcher, timer, tickTimeMillis, Clock.systemUTC(), isDelayedDeliveryDeliverAtTimeStrict,
                 bucketSnapshotStorage, minIndexCountPerBucket, timeStepPerBucketSnapshotSegmentInMillis,
                 maxIndexesPerBucketSnapshotSegment, maxNumBuckets);
     }
 
     public BucketDelayedDeliveryTracker(PersistentDispatcherMultipleConsumers dispatcher,
-                                        Timer timer, long tickTimeMillis, Clock clock,
-                                        boolean isDelayedDeliveryDeliverAtTimeStrict,
-                                        BucketSnapshotStorage bucketSnapshotStorage,
-                                        long minIndexCountPerBucket, long timeStepPerBucketSnapshotSegmentInMillis,
-                                        int maxIndexesPerBucketSnapshotSegment, int maxNumBuckets)
-            throws RecoverDelayedDeliveryTrackerException {
+                                 Timer timer, long tickTimeMillis, Clock clock,
+                                 boolean isDelayedDeliveryDeliverAtTimeStrict,
+                                 BucketSnapshotStorage bucketSnapshotStorage,
+                                 long minIndexCountPerBucket, long timeStepPerBucketSnapshotSegmentInMillis,
+                                 int maxIndexesPerBucketSnapshotSegment, int maxNumBuckets) {
         super(dispatcher, timer, tickTimeMillis, clock, isDelayedDeliveryDeliverAtTimeStrict);
         this.minIndexCountPerBucket = minIndexCountPerBucket;
         this.timeStepPerBucketSnapshotSegmentInMillis = timeStepPerBucketSnapshotSegmentInMillis;
@@ -135,27 +132,14 @@ public class BucketDelayedDeliveryTracker extends AbstractDelayedDeliveryTracker
                 new MutableBucket(dispatcher.getName(), dispatcher.getCursor(), FutureUtil.Sequencer.create(),
                         bucketSnapshotStorage);
         this.stats = new BucketDelayedMessageIndexStats();
-
-        // Close the tracker if failed to recover.
-        try {
-            this.numberDelayedMessages = recoverBucketSnapshot();
-        } catch (RecoverDelayedDeliveryTrackerException e) {
-            close();
-            throw e;
-        }
+        this.numberDelayedMessages = recoverBucketSnapshot();
     }
 
-    private synchronized long recoverBucketSnapshot() throws RecoverDelayedDeliveryTrackerException {
+    private synchronized long recoverBucketSnapshot() throws RuntimeException {
         ManagedCursor cursor = this.lastMutableBucket.getCursor();
-        Map<String, String> cursorProperties = cursor.getCursorProperties();
-        if (MapUtils.isEmpty(cursorProperties)) {
-            log.info("[{}] Recover delayed message index bucket snapshot finish, don't find bucket snapshot",
-                    dispatcher.getName());
-            return 0;
-        }
         FutureUtil.Sequencer<Void> sequencer = this.lastMutableBucket.getSequencer();
         Map<Range<Long>, ImmutableBucket> toBeDeletedBucketMap = new HashMap<>();
-        cursorProperties.keySet().forEach(key -> {
+        cursor.getCursorProperties().keySet().forEach(key -> {
             if (key.startsWith(DELAYED_BUCKET_KEY_PREFIX)) {
                 String[] keys = key.split(DELIMITER);
                 checkArgument(keys.length == 3);
@@ -179,7 +163,8 @@ public class BucketDelayedDeliveryTracker extends AbstractDelayedDeliveryTracker
                 futures = new HashMap<>(immutableBucketMap.size());
         for (Map.Entry<Range<Long>, ImmutableBucket> entry : immutableBucketMap.entrySet()) {
             Range<Long> key = entry.getKey();
-            futures.put(key, handleRecoverBucketSnapshotEntry(entry.getValue()));
+            ImmutableBucket immutableBucket = entry.getValue();
+            futures.put(key, immutableBucket.asyncRecoverBucketSnapshotEntry(this::getCutoffTime));
         }
 
         try {
@@ -189,7 +174,7 @@ public class BucketDelayedDeliveryTracker extends AbstractDelayedDeliveryTracker
             if (e instanceof InterruptedException) {
                 Thread.currentThread().interrupt();
             }
-            throw new RecoverDelayedDeliveryTrackerException(e);
+            throw new RuntimeException(e);
         }
 
         for (Map.Entry<Range<Long>, CompletableFuture<List<DelayedIndex>>> entry : futures.entrySet()) {
@@ -230,33 +215,6 @@ public class BucketDelayedDeliveryTracker extends AbstractDelayedDeliveryTracker
         return numberDelayedMessages.getValue();
     }
 
-    /**
-     * Handle the BucketNotExistException when recover bucket snapshot entry.
-     * The non exist bucket will be added to `toBeDeletedBucketMap` and deleted from `immutableBuckets`
-     * in the next step.
-     *
-     * @param bucket
-     * @return
-     */
-    private CompletableFuture<List<DelayedIndex>> handleRecoverBucketSnapshotEntry(ImmutableBucket bucket) {
-        CompletableFuture<List<DelayedIndex>> f = new CompletableFuture<>();
-        bucket.asyncRecoverBucketSnapshotEntry(this::getCutoffTime)
-                .whenComplete((v, e) -> {
-                    if (e == null) {
-                        f.complete(v);
-                    } else {
-                        if (e instanceof BucketNotExistException) {
-                            // If the bucket does not exist, return an empty list,
-                            // the bucket will be deleted from `immutableBuckets` in the next step.
-                            f.complete(Collections.emptyList());
-                        } else {
-                            f.completeExceptionally(e);
-                        }
-                    }
-                });
-        return f;
-    }
-
     private synchronized void putAndCleanOverlapRange(Range<Long> range, ImmutableBucket immutableBucket,
                                                       Map<Range<Long>, ImmutableBucket> toBeDeletedBucketMap) {
         RangeMap<Long, ImmutableBucket> subRangeMap = immutableBuckets.subRangeMap(range);
@@ -754,7 +712,7 @@ public class BucketDelayedDeliveryTracker extends AbstractDelayedDeliveryTracker
                 .orElse(false);
     }
 
-    public synchronized boolean containsMessage(long ledgerId, long entryId) {
+    public boolean containsMessage(long ledgerId, long entryId) {
         if (lastMutableBucket.containsMessage(ledgerId, entryId)) {
             return true;
         }
