@@ -161,8 +161,7 @@ public class LinuxInfoUtils {
      * </pre>
      * <p>
      * Line is split in "words", filtering the first. The sum of all numbers give the amount of cpu cycles used this
-     * far. Real CPU usage should equal the sum substracting the idle cycles(that is idle+iowait), this would include
-     * cpu, user, nice, system, irq, softirq, steal, guest and guest_nice.
+     * far. Real CPU usage should equal the sum subtracting the idle cycles, this would include iowait, irq and steal.
      */
     public static ResourceUsage getCpuUsageForEntireHost() {
         try (Stream<String> stream = Files.lines(Paths.get(PROC_STAT_PATH))) {
@@ -176,7 +175,7 @@ public class LinuxInfoUtils {
                     .filter(s -> !s.contains("cpu"))
                     .mapToLong(Long::parseLong)
                     .sum();
-            long idle = Long.parseLong(words[4]) + Long.parseLong(words[5]);
+            long idle = Long.parseLong(words[4]);
             return ResourceUsage.builder()
                     .usage(total - idle)
                     .idle(idle)
@@ -198,20 +197,12 @@ public class LinuxInfoUtils {
                 return false;
             }
             // Check the type to make sure it's ethernet (type "1")
-            final Path nicTypePath = nicPath.resolve("type");
-            if (!Files.exists(nicTypePath)) {
-                if (log.isDebugEnabled()) {
-                    log.debug("Failed to read NIC type, the expected linux type file does not exist."
-                              + " nic_type_path={}", nicTypePath);
-                }
-               return false;
-            }
+            String type = readTrimStringFromFile(nicPath.resolve("type"));
             // wireless NICs don't report speed, ignore them.
-            return Integer.parseInt(readTrimStringFromFile(nicTypePath)) == ARPHRD_ETHER;
-        } catch (Exception ex) {
-            if (log.isDebugEnabled()) {
-                log.debug("Failed to read NIC type. nic_path={}", nicPath, ex);
-            }
+            return Integer.parseInt(type) == ARPHRD_ETHER;
+        } catch (Exception e) {
+            log.warn("[LinuxInfo] Failed to read {} NIC type, the detail is: {}", nicPath, e.getMessage());
+            // Read type got error.
             return false;
         }
     }
