@@ -37,10 +37,10 @@ import org.apache.commons.collections4.CollectionUtils;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.broker.intercept.BrokerInterceptor;
-import org.apache.pulsar.broker.service.persistent.CompactorSubscription;
 import org.apache.pulsar.broker.service.persistent.DispatchRateLimiter;
 import org.apache.pulsar.broker.service.persistent.PersistentSubscription;
 import org.apache.pulsar.broker.service.persistent.PersistentTopic;
+import org.apache.pulsar.broker.service.persistent.PulsarCompactorSubscription;
 import org.apache.pulsar.broker.service.plugin.EntryFilter;
 import org.apache.pulsar.broker.transaction.pendingack.impl.PendingAckHandleImpl;
 import org.apache.pulsar.client.api.transaction.TxnID;
@@ -49,7 +49,6 @@ import org.apache.pulsar.common.api.proto.MessageMetadata;
 import org.apache.pulsar.common.api.proto.ReplicatedSubscriptionsSnapshot;
 import org.apache.pulsar.common.protocol.Commands;
 import org.apache.pulsar.common.protocol.Markers;
-import org.apache.pulsar.compaction.Compactor;
 import org.checkerframework.checker.nullness.qual.Nullable;
 
 @Slf4j
@@ -74,6 +73,7 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
         this.dispatchThrottlingOnBatchMessageEnabled = serviceConfig.isDispatchThrottlingOnBatchMessageEnabled();
     }
 
+
     /**
      * Filter messages that are being sent to a consumers.
      * <p>
@@ -174,46 +174,34 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
             if (msgMetadata != null && msgMetadata.hasTxnidMostBits()
                     && msgMetadata.hasTxnidLeastBits()) {
                 if (Markers.isTxnMarker(msgMetadata)) {
-                    if (cursor == null || !cursor.getName().equals(Compactor.COMPACTION_SUBSCRIPTION)) {
-                        // because consumer can receive message is smaller than maxReadPosition,
-                        // so this marker is useless for this subscription
-                        individualAcknowledgeMessageIfNeeded(Collections.singletonList(entry.getPosition()),
-                                Collections.emptyMap());
-                        entries.set(i, null);
-                        entry.release();
-                        continue;
-                    }
+                    // because consumer can receive message is smaller than maxReadPosition,
+                    // so this marker is useless for this subscription
+                    individualAcknowledgeMessageIfNeeded(entry.getPosition(), Collections.emptyMap());
+                    entries.set(i, null);
+                    entry.release();
+                    continue;
                 } else if (((PersistentTopic) subscription.getTopic())
                         .isTxnAborted(new TxnID(msgMetadata.getTxnidMostBits(), msgMetadata.getTxnidLeastBits()),
                                 (PositionImpl) entry.getPosition())) {
-                    individualAcknowledgeMessageIfNeeded(Collections.singletonList(entry.getPosition()),
-                            Collections.emptyMap());
+                    individualAcknowledgeMessageIfNeeded(entry.getPosition(), Collections.emptyMap());
                     entries.set(i, null);
                     entry.release();
                     continue;
                 }
             }
 
-            if (msgMetadata == null || (Markers.isServerOnlyMarker(msgMetadata))) {
+            if (msgMetadata == null || Markers.isServerOnlyMarker(msgMetadata)) {
                 PositionImpl pos = (PositionImpl) entry.getPosition();
                 // Message metadata was corrupted or the messages was a server-only marker
 
                 if (Markers.isReplicatedSubscriptionSnapshotMarker(msgMetadata)) {
-                    final int readerIndex = metadataAndPayload.readerIndex();
                     processReplicatedSubscriptionSnapshot(pos, metadataAndPayload);
-                    metadataAndPayload.readerIndex(readerIndex);
                 }
 
-                // Deliver marker to __compaction cursor to avoid compaction task stuck,
-                // and filter out them when doing topic compaction.
-                if (msgMetadata == null || cursor == null
-                        || !cursor.getName().equals(Compactor.COMPACTION_SUBSCRIPTION)) {
-                    entries.set(i, null);
-                    entry.release();
-                    individualAcknowledgeMessageIfNeeded(Collections.singletonList(pos),
-                            Collections.emptyMap());
-                    continue;
-                }
+                entries.set(i, null);
+                entry.release();
+                individualAcknowledgeMessageIfNeeded(pos, Collections.emptyMap());
+                continue;
             } else if (trackDelayedDelivery(entry.getLedgerId(), entry.getEntryId(), msgMetadata)) {
                 // The message is marked for delayed delivery. Ignore for now.
                 entries.set(i, null);
@@ -225,7 +213,12 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
                 this.filterAcceptedMsgs.add(entryMsgCnt);
             }
 
+            totalEntries++;
             int batchSize = msgMetadata.getNumMessagesInBatch();
+            totalMessages += batchSize;
+            totalBytes += metadataAndPayload.readableBytes();
+            totalChunkedMessages += msgMetadata.hasChunkId() ? 1 : 0;
+            batchSizes.setBatchSize(i, batchSize);
             long[] ackSet = null;
             if (indexesAcks != null && cursor != null) {
                 PositionImpl position = PositionImpl.get(entry.getLedgerId(), entry.getEntryId());
@@ -269,12 +262,6 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
                 }
             }
 
-            totalEntries++;
-            totalMessages += batchSize;
-            totalBytes += metadataAndPayload.readableBytes();
-            totalChunkedMessages += msgMetadata.hasChunkId() ? 1 : 0;
-            batchSizes.setBatchSize(i, batchSize);
-
             BrokerInterceptor interceptor = subscription.interceptor();
             if (null != interceptor) {
                 // keep for compatibility if users has implemented the old interface
@@ -283,7 +270,8 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
             }
         }
         if (CollectionUtils.isNotEmpty(entriesToFiltered)) {
-            individualAcknowledgeMessageIfNeeded(entriesToFiltered, Collections.emptyMap());
+            subscription.acknowledgeMessage(entriesToFiltered, AckType.Individual,
+                    Collections.emptyMap());
 
             int filtered = entriesToFiltered.size();
             Topic topic = subscription.getTopic();
@@ -312,9 +300,9 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
         return totalEntries;
     }
 
-    private void individualAcknowledgeMessageIfNeeded(List<Position> positions, Map<String, Long> properties) {
-        if (!(subscription instanceof CompactorSubscription)) {
-            subscription.acknowledgeMessage(positions, AckType.Individual, properties);
+    private void individualAcknowledgeMessageIfNeeded(Position position, Map<String, Long> properties) {
+        if (!(subscription instanceof PulsarCompactorSubscription)) {
+            subscription.acknowledgeMessage(Collections.singletonList(position), AckType.Individual, properties);
         }
     }
 
@@ -365,8 +353,6 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
 
     protected abstract void reScheduleRead();
 
-    public abstract String getName();
-
     protected boolean reachDispatchRateLimit(DispatchRateLimiter dispatchRateLimiter) {
         if (dispatchRateLimiter.isDispatchRateLimitingEnabled()) {
             if (!dispatchRateLimiter.hasMessageDispatchPermit()) {
