@@ -18,12 +18,10 @@
  */
 package org.apache.pulsar.broker.service;
 
-import static java.util.concurrent.TimeUnit.SECONDS;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.CompletableFuture;
-import lombok.Getter;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedCursor.IndividualDeletedEntries;
@@ -34,7 +32,6 @@ import org.apache.bookkeeper.mledger.proto.MLDataFormats.ManagedLedgerInfo;
 import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.resources.NamespaceResources;
 import org.apache.pulsar.broker.service.persistent.PersistentTopic;
-import org.apache.pulsar.broker.service.persistent.PersistentTopicMetrics.BacklogQuotaMetrics;
 import org.apache.pulsar.common.naming.NamespaceName;
 import org.apache.pulsar.common.policies.data.BacklogQuota;
 import org.apache.pulsar.common.policies.data.BacklogQuota.BacklogQuotaType;
@@ -44,7 +41,6 @@ import org.apache.pulsar.metadata.api.MetadataStoreException;
 
 @Slf4j
 public class BacklogQuotaManager {
-    @Getter
     private final BacklogQuotaImpl defaultQuota;
     private final NamespaceResources namespaceResources;
 
@@ -59,6 +55,10 @@ public class BacklogQuotaManager {
         this.namespaceResources = pulsar.getPulsarResources().getNamespaceResources();
     }
 
+    public BacklogQuotaImpl getDefaultQuota() {
+        return this.defaultQuota;
+    }
+
     public BacklogQuotaImpl getBacklogQuota(NamespaceName namespace, BacklogQuotaType backlogQuotaType) {
         try {
             if (namespaceResources == null) {
@@ -86,34 +86,27 @@ public class BacklogQuotaManager {
     public void handleExceededBacklogQuota(PersistentTopic persistentTopic, BacklogQuotaType backlogQuotaType,
                                            boolean preciseTimeBasedBacklogQuotaCheck) {
         BacklogQuota quota = persistentTopic.getBacklogQuota(backlogQuotaType);
-        BacklogQuotaMetrics topicBacklogQuotaMetrics =
-                persistentTopic.getPersistentTopicMetrics().getBacklogQuotaMetrics();
         log.info("Backlog quota type {} exceeded for topic [{}]. Applying [{}] policy", backlogQuotaType,
                 persistentTopic.getName(), quota.getPolicy());
         switch (quota.getPolicy()) {
-            case consumer_backlog_eviction:
-                switch (backlogQuotaType) {
-                    case destination_storage:
+        case consumer_backlog_eviction:
+            switch (backlogQuotaType) {
+                case destination_storage:
                         dropBacklogForSizeLimit(persistentTopic, quota);
-                        topicBacklogQuotaMetrics.recordSizeBasedBacklogEviction();
                         break;
-                    case message_age:
+                case message_age:
                         dropBacklogForTimeLimit(persistentTopic, quota, preciseTimeBasedBacklogQuotaCheck);
-                        topicBacklogQuotaMetrics.recordTimeBasedBacklogEviction();
                         break;
-                    default:
-                        break;
-                }
-                break;
-            case producer_exception:
-            case producer_request_hold:
-                if (!advanceSlowestSystemCursor(persistentTopic)) {
-                    // The slowest is not a system cursor. Disconnecting producers to put backpressure.
-                    disconnectProducers(persistentTopic);
-                }
-                break;
-            default:
-                break;
+                default:
+                    break;
+            }
+            break;
+        case producer_exception:
+        case producer_request_hold:
+            disconnectProducers(persistentTopic);
+            break;
+        default:
+            break;
         }
     }
 
@@ -214,7 +207,7 @@ public class BacklogQuotaManager {
             );
         } else {
             // If disabled precise time based backlog quota check, will try to remove whole ledger from cursor's backlog
-            long currentMillis = ((ManagedLedgerImpl) persistentTopic.getManagedLedger()).getClock().millis();
+            Long currentMillis = ((ManagedLedgerImpl) persistentTopic.getManagedLedger()).getClock().millis();
             ManagedLedgerImpl mLedger = (ManagedLedgerImpl) persistentTopic.getManagedLedger();
             try {
                 for (; ; ) {
@@ -233,7 +226,7 @@ public class BacklogQuotaManager {
                     }
                     // Timestamp only > 0 if ledger has been closed
                     if (ledgerInfo.getTimestamp() > 0
-                            && currentMillis - ledgerInfo.getTimestamp() > SECONDS.toMillis(quota.getLimitTime())) {
+                            && currentMillis - ledgerInfo.getTimestamp() > quota.getLimitTime() * 1000) {
                         // skip whole ledger for the slowest cursor
                         PositionImpl nextPosition =
                                 PositionImpl.get(mLedger.getNextValidLedger(ledgerInfo.getLedgerId()), -1);
@@ -267,36 +260,12 @@ public class BacklogQuotaManager {
             futures.add(producer.disconnect());
         });
 
-        FutureUtil.waitForAll(futures)
-                .thenRun(() ->
-                        log.info("All producers on topic [{}] are disconnected", persistentTopic.getName()))
-                .exceptionally(exception -> {
-                    log.error("Error in disconnecting producers on topic [{}] [{}]", persistentTopic.getName(),
-                            exception);
-                    return null;
+        FutureUtil.waitForAll(futures).thenRun(() -> {
+            log.info("All producers on topic [{}] are disconnected", persistentTopic.getName());
+        }).exceptionally(exception -> {
+            log.error("Error in disconnecting producers on topic [{}] [{}]", persistentTopic.getName(), exception);
+            return null;
+
         });
     }
-
-    /**
-     * Advances the slowest cursor if that is a system cursor.
-     *
-     * @param persistentTopic Persistent topic
-     * @return true if the slowest cursor is a system cursor
-     */
-    private boolean advanceSlowestSystemCursor(PersistentTopic persistentTopic) {
-
-        ManagedLedgerImpl mLedger = (ManagedLedgerImpl) persistentTopic.getManagedLedger();
-        ManagedCursor slowestConsumer = mLedger.getSlowestConsumer();
-        if (slowestConsumer == null) {
-            return false;
-        }
-
-        if (PersistentTopic.isDedupCursorName(slowestConsumer.getName())) {
-            persistentTopic.getMessageDeduplication().takeSnapshot();
-            return true;
-        }
-
-        // We may need to check other system cursors here : replicator, compaction
-        return false;
-    }
 }
