@@ -27,7 +27,6 @@ import java.util.TreeMap;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.DeleteCursorCallback;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.MarkDeleteCallback;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.OpenCursorCallback;
@@ -56,8 +55,6 @@ public class MessageDeduplication {
     private final ManagedLedger managedLedger;
     private ManagedCursor managedCursor;
 
-    private static final String IS_LAST_CHUNK = "isLastChunk";
-
     enum Status {
 
         // Deduplication is initialized
@@ -133,9 +130,6 @@ public class MessageDeduplication {
 
     private final String replicatorPrefix;
 
-
-    private final AtomicBoolean snapshotTaking = new AtomicBoolean(false);
-
     public MessageDeduplication(PulsarService pulsar, PersistentTopic topic, ManagedLedger managedLedger) {
         this.pulsar = pulsar;
         this.topic = topic;
@@ -157,15 +151,9 @@ public class MessageDeduplication {
 
         // Replay all the entries and apply all the sequence ids updates
         log.info("[{}] Replaying {} entries for deduplication", topic.getName(), managedCursor.getNumberOfEntries());
-        CompletableFuture<Position> future = new CompletableFuture<>();
+        CompletableFuture<Void> future = new CompletableFuture<>();
         replayCursor(future);
-        return future.thenCompose(lastPosition -> {
-            if (lastPosition != null && snapshotCounter >= snapshotInterval) {
-                snapshotCounter = 0;
-                return takeSnapshot(lastPosition);
-            }
-            return CompletableFuture.completedFuture(null);
-        });
+        return future;
     }
 
     /**
@@ -174,11 +162,11 @@ public class MessageDeduplication {
      *
      * @param future future to trigger when the replay is complete
      */
-    private void replayCursor(CompletableFuture<Position> future) {
+    private void replayCursor(CompletableFuture<Void> future) {
         managedCursor.asyncReadEntries(100, new ReadEntriesCallback() {
             @Override
             public void readEntriesComplete(List<Entry> entries, Object ctx) {
-                Position lastPosition = null;
+
                 for (Entry entry : entries) {
                     ByteBuf messageMetadataAndPayload = entry.getDataBuffer();
                     MessageMetadata md = Commands.parseMessageMetadata(messageMetadataAndPayload);
@@ -188,8 +176,7 @@ public class MessageDeduplication {
                     highestSequencedPushed.put(producerName, sequenceId);
                     highestSequencedPersisted.put(producerName, sequenceId);
                     producerRemoved(producerName);
-                    snapshotCounter++;
-                    lastPosition = entry.getPosition();
+
                     entry.release();
                 }
 
@@ -198,7 +185,7 @@ public class MessageDeduplication {
                     pulsar.getExecutor().execute(() -> replayCursor(future));
                 } else {
                     // Done replaying
-                    future.complete(lastPosition);
+                    future.complete(null);
                 }
             }
 
@@ -218,7 +205,7 @@ public class MessageDeduplication {
      * returning a future to track the completion of the task
      */
     public CompletableFuture<Void> checkStatus() {
-        boolean shouldBeEnabled = topic.isDeduplicationEnabled();
+        boolean shouldBeEnabled = isDeduplicationEnabled();
         synchronized (this) {
             if (status == Status.Recovering || status == Status.Removing) {
                 // If there's already a transition happening, check later for status
@@ -337,12 +324,11 @@ public class MessageDeduplication {
         String producerName = publishContext.getProducerName();
         long sequenceId = publishContext.getSequenceId();
         long highestSequenceId = Math.max(publishContext.getHighestSequenceId(), sequenceId);
-        MessageMetadata md = null;
         if (producerName.startsWith(replicatorPrefix)) {
             // Message is coming from replication, we need to use the original producer name and sequence id
             // for the purpose of deduplication and not rely on the "replicator" name.
             int readerIndex = headersAndPayload.readerIndex();
-            md = Commands.parseMessageMetadata(headersAndPayload);
+            MessageMetadata md = Commands.parseMessageMetadata(headersAndPayload);
             producerName = md.getProducerName();
             sequenceId = md.getSequenceId();
             highestSequenceId = Math.max(md.getHighestSequenceId(), sequenceId);
@@ -351,23 +337,7 @@ public class MessageDeduplication {
             publishContext.setOriginalHighestSequenceId(highestSequenceId);
             headersAndPayload.readerIndex(readerIndex);
         }
-        long chunkID = -1;
-        long totalChunk = -1;
-        if (publishContext.isChunked()) {
-            if (md == null) {
-                int readerIndex = headersAndPayload.readerIndex();
-                md = Commands.parseMessageMetadata(headersAndPayload);
-                headersAndPayload.readerIndex(readerIndex);
-            }
-            chunkID = md.getChunkId();
-            totalChunk = md.getNumChunksFromMsg();
-        }
-        // All chunks of a message use the same message metadata and sequence ID,
-        // so we only need to check the sequence ID for the last chunk in a chunk message.
-        if (chunkID != -1 && chunkID != totalChunk - 1) {
-            publishContext.setProperty(IS_LAST_CHUNK, Boolean.FALSE);
-            return MessageDupStatus.NotDup;
-        }
+
         // Synchronize the get() and subsequent put() on the map. This would only be relevant if the producer
         // disconnects and re-connects very quickly. At that point the call can be coming from a different thread
         synchronized (highestSequencedPushed) {
@@ -393,11 +363,6 @@ public class MessageDeduplication {
             }
             highestSequencedPushed.put(producerName, highestSequenceId);
         }
-        // Only put sequence ID into highestSequencedPushed and
-        // highestSequencedPersisted until receive and persistent the last chunk.
-        if (chunkID != -1 && chunkID == totalChunk - 1) {
-            publishContext.setProperty(IS_LAST_CHUNK, Boolean.TRUE);
-        }
         return MessageDupStatus.NotDup;
     }
 
@@ -418,10 +383,8 @@ public class MessageDeduplication {
             sequenceId = publishContext.getOriginalSequenceId();
             highestSequenceId = publishContext.getOriginalHighestSequenceId();
         }
-        Boolean isLastChunk = (Boolean) publishContext.getProperty(IS_LAST_CHUNK);
-        if (isLastChunk == null || isLastChunk) {
-            highestSequencedPersisted.put(producerName, Math.max(highestSequenceId, sequenceId));
-        }
+
+        highestSequencedPersisted.put(producerName, Math.max(highestSequenceId, sequenceId));
         if (++snapshotCounter >= snapshotInterval) {
             snapshotCounter = 0;
             takeSnapshot(position);
@@ -439,17 +402,10 @@ public class MessageDeduplication {
         }
     }
 
-    private CompletableFuture<Void> takeSnapshot(Position position) {
-        CompletableFuture<Void> future = new CompletableFuture<>();
+    private void takeSnapshot(Position position) {
         if (log.isDebugEnabled()) {
             log.debug("[{}] Taking snapshot of sequence ids map", topic.getName());
         }
-
-        if (!snapshotTaking.compareAndSet(false, true)) {
-            future.complete(null);
-            return future;
-        }
-
         Map<String, Long> snapshot = new TreeMap<>();
         highestSequencedPersisted.forEach((producerName, sequenceId) -> {
             if (snapshot.size() < maxNumberOfProducers) {
@@ -464,28 +420,23 @@ public class MessageDeduplication {
                     log.debug("[{}] Stored new deduplication snapshot at {}", topic.getName(), position);
                 }
                 lastSnapshotTimestamp = System.currentTimeMillis();
-                snapshotTaking.set(false);
-                future.complete(null);
             }
 
             @Override
             public void markDeleteFailed(ManagedLedgerException exception, Object ctx) {
                 log.warn("[{}] Failed to store new deduplication snapshot at {}", topic.getName(), position);
-                snapshotTaking.set(false);
-                future.completeExceptionally(exception);
             }
         }, null);
-        return future;
+    }
+
+    private boolean isDeduplicationEnabled() {
+        return topic.getHierarchyTopicPolicies().getDeduplicationEnabled().get();
     }
 
     /**
      * Topic will call this method whenever a producer connects.
      */
     public void producerAdded(String producerName) {
-        if (!isEnabled()) {
-            return;
-        }
-
         // Producer is no-longer inactive
         inactiveProducers.remove(producerName);
     }
@@ -494,10 +445,6 @@ public class MessageDeduplication {
      * Topic will call this method whenever a producer disconnects.
      */
     public void producerRemoved(String producerName) {
-        if (!isEnabled()) {
-            return;
-        }
-
         // Producer is no-longer active
         inactiveProducers.put(producerName, System.currentTimeMillis());
     }
@@ -509,14 +456,6 @@ public class MessageDeduplication {
         long minimumActiveTimestamp = System.currentTimeMillis() - TimeUnit.MINUTES
                 .toMillis(pulsar.getConfiguration().getBrokerDeduplicationProducerInactivityTimeoutMinutes());
 
-        // if not enabled just clear all inactive producer record.
-        if (!isEnabled()) {
-            if (!inactiveProducers.isEmpty()) {
-                inactiveProducers.clear();
-            }
-            return;
-        }
-
         Iterator<Map.Entry<String, Long>> mapIterator = inactiveProducers.entrySet().iterator();
         boolean hasInactive = false;
         while (mapIterator.hasNext()) {
@@ -569,10 +508,5 @@ public class MessageDeduplication {
         return managedCursor;
     }
 
-    @VisibleForTesting
-    Map<String, Long> getInactiveProducers() {
-        return inactiveProducers;
-    }
-
     private static final Logger log = LoggerFactory.getLogger(MessageDeduplication.class);
 }
