@@ -29,11 +29,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.TreeMap;
-import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicReference;
@@ -60,7 +57,6 @@ import org.apache.commons.collections4.MapUtils;
 import org.apache.commons.lang3.tuple.MutablePair;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.broker.intercept.BrokerInterceptor;
-import org.apache.pulsar.broker.loadbalance.extensions.ExtensibleLoadManagerImpl;
 import org.apache.pulsar.broker.service.AbstractSubscription;
 import org.apache.pulsar.broker.service.AnalyzeBacklogResult;
 import org.apache.pulsar.broker.service.BrokerServiceException;
@@ -124,24 +120,17 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
     // Map of properties that is used to mark this subscription as "replicated".
     // Since this is the only field at this point, we can just keep a static
     // instance of the map.
-    private static final Map<String, Long> REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES = new TreeMap<>();
-    private static final Map<String, Long> NON_REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES = Collections.emptyMap();
+    private static final Map<String, Long> REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES =
+            Map.of(REPLICATED_SUBSCRIPTION_PROPERTY, 1L);
+    private static final Map<String, Long> NON_REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES = Map.of();
 
     private volatile ReplicatedSubscriptionSnapshotCache replicatedSubscriptionSnapshotCache;
     private final PendingAckHandle pendingAckHandle;
     private volatile Map<String, String> subscriptionProperties;
     private volatile CompletableFuture<Void> fenceFuture;
-    private volatile CompletableFuture<Void> inProgressResetCursorFuture;
-    private volatile Boolean replicatedControlled;
-    private final ServiceConfiguration config;
 
-    static {
-        REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES.put(REPLICATED_SUBSCRIPTION_PROPERTY, 1L);
-    }
-
-    static Map<String, Long> getBaseCursorProperties(Boolean isReplicated) {
-        return isReplicated != null && isReplicated ? REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES :
-                NON_REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES;
+    static Map<String, Long> getBaseCursorProperties(boolean isReplicated) {
+        return isReplicated ? REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES : NON_REPLICATED_SUBSCRIPTION_CURSOR_PROPERTIES;
     }
 
     static boolean isCursorFromReplicatedSubscription(ManagedCursor cursor) {
@@ -149,27 +138,23 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
     }
 
     public PersistentSubscription(PersistentTopic topic, String subscriptionName, ManagedCursor cursor,
-                                  Boolean replicated) {
+                                  boolean replicated) {
         this(topic, subscriptionName, cursor, replicated, Collections.emptyMap());
     }
 
     public PersistentSubscription(PersistentTopic topic, String subscriptionName, ManagedCursor cursor,
-                                  Boolean replicated, Map<String, String> subscriptionProperties) {
+                                  boolean replicated, Map<String, String> subscriptionProperties) {
         this.topic = topic;
-        this.config = topic.getBrokerService().getPulsar().getConfig();
         this.cursor = cursor;
         this.topicName = topic.getName();
         this.subName = subscriptionName;
         this.fullName = MoreObjects.toStringHelper(this).add("topic", topicName).add("name", subName).toString();
-        this.expiryMonitor = new PersistentMessageExpiryMonitor(topicName, subscriptionName, cursor, this);
-        if (replicated != null) {
-            this.setReplicated(replicated);
-        }
+        this.expiryMonitor = new PersistentMessageExpiryMonitor(topic, subscriptionName, cursor, this);
+        this.setReplicated(replicated);
         this.subscriptionProperties = MapUtils.isEmpty(subscriptionProperties)
                 ? Collections.emptyMap() : Collections.unmodifiableMap(subscriptionProperties);
-        if (config.isTransactionCoordinatorEnabled()
-                && !isEventSystemTopic(TopicName.get(topicName))
-                && !ExtensibleLoadManagerImpl.isInternalTopic(topicName)) {
+        if (topic.getBrokerService().getPulsar().getConfig().isTransactionCoordinatorEnabled()
+                && !isEventSystemTopic(TopicName.get(topicName))) {
             this.pendingAckHandle = new PendingAckHandleImpl(this);
         } else {
             this.pendingAckHandle = new PendingAckHandleDisabled();
@@ -203,7 +188,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
     }
 
     public boolean setReplicated(boolean replicated) {
-        replicatedControlled = replicated;
+        ServiceConfiguration config = topic.getBrokerService().getPulsar().getConfig();
 
         if (!replicated || !config.isEnableReplicatedSubscriptions()) {
             this.replicatedSubscriptionSnapshotCache = null;
@@ -214,12 +199,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
 
         if (this.cursor != null) {
             if (replicated) {
-                if (!config.isEnableReplicatedSubscriptions()) {
-                    log.warn("[{}][{}] Failed set replicated subscription status to {}, please enable the "
-                            + "configuration enableReplicatedSubscriptions", topicName, subName, replicated);
-                } else {
-                    return this.cursor.putProperty(REPLICATED_SUBSCRIPTION_PROPERTY, 1L);
-                }
+                return this.cursor.putProperty(REPLICATED_SUBSCRIPTION_PROPERTY, 1L);
             } else {
                 return this.cursor.removeProperty(REPLICATED_SUBSCRIPTION_PROPERTY);
             }
@@ -230,16 +210,6 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
 
     @Override
     public CompletableFuture<Void> addConsumer(Consumer consumer) {
-        CompletableFuture<Void> inProgressResetCursorFuture = this.inProgressResetCursorFuture;
-        if (inProgressResetCursorFuture != null) {
-            return inProgressResetCursorFuture.handle((ignore, ignoreEx) -> null)
-                    .thenCompose(ignore -> addConsumerInternal(consumer));
-        } else {
-            return addConsumerInternal(consumer);
-        }
-    }
-
-    private CompletableFuture<Void> addConsumerInternal(Consumer consumer) {
         return pendingAckHandle.pendingAckHandleFuture().thenCompose(future -> {
             synchronized (PersistentSubscription.this) {
                 cursor.updateLastActive();
@@ -250,26 +220,18 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
 
                 if (dispatcher == null || !dispatcher.isConsumerConnected()) {
                     Dispatcher previousDispatcher = null;
-                    boolean useStreamingDispatcher = topic.getBrokerService().getPulsar()
-                            .getConfiguration().isStreamingDispatch();
                     switch (consumer.subType()) {
                         case Exclusive:
                             if (dispatcher == null || dispatcher.getType() != SubType.Exclusive) {
                                 previousDispatcher = dispatcher;
-                                dispatcher = useStreamingDispatcher
-                                        ? new PersistentStreamingDispatcherSingleActiveConsumer(
-                                        cursor, SubType.Exclusive, 0, topic, this)
-                                        : new PersistentDispatcherSingleActiveConsumer(
+                                dispatcher = new PersistentDispatcherSingleActiveConsumer(
                                         cursor, SubType.Exclusive, 0, topic, this);
                             }
                             break;
                         case Shared:
                             if (dispatcher == null || dispatcher.getType() != SubType.Shared) {
                                 previousDispatcher = dispatcher;
-                                dispatcher = useStreamingDispatcher
-                                        ? new PersistentStreamingDispatcherMultipleConsumers(
-                                        topic, cursor, this)
-                                        : new PersistentDispatcherMultipleConsumers(topic, cursor, this);
+                                dispatcher = new PersistentDispatcherMultipleConsumers(topic, cursor, this);
                             }
                             break;
                         case Failover:
@@ -283,10 +245,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
 
                             if (dispatcher == null || dispatcher.getType() != SubType.Failover) {
                                 previousDispatcher = dispatcher;
-                                dispatcher = useStreamingDispatcher
-                                        ? new PersistentStreamingDispatcherSingleActiveConsumer(
-                                        cursor, SubType.Failover, partitionIndex, topic, this) :
-                                        new PersistentDispatcherSingleActiveConsumer(cursor, SubType.Failover,
+                                dispatcher = new PersistentDispatcherSingleActiveConsumer(cursor, SubType.Failover,
                                                 partitionIndex, topic, this);
                             }
                             break;
@@ -339,15 +298,6 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
 
         if (dispatcher != null && dispatcher.getConsumers().isEmpty()) {
             deactivateCursor();
-            // Remove the cursor from the waiting cursors list.
-            // For durable cursors, we should *not* cancel the pending read with cursor.cancelPendingReadRequest.
-            // This is because internally, in the dispatcher implementations, there is a "havePendingRead" flag
-            // that is not reset. If the pending read is cancelled, the dispatcher will not continue reading from
-            // the managed ledger when a new consumer is added to the dispatcher since based on the "havePendingRead"
-            // state, it will continue to expect that a read is pending and will not submit a new read.
-            // For non-durable cursors, there's no difference since the cursor is not expected to be used again.
-
-            // remove waiting cursor from the managed ledger, this applies to both durable and non-durable cursors.
             topic.getManagedLedger().removeWaitingCursor(cursor);
 
             if (!cursor.isDurable()) {
@@ -428,7 +378,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
                 log.debug("[{}][{}] Individual acks on {}", topicName, subName, positions);
             }
             cursor.asyncDelete(positions, deleteCallback, previousMarkDeletePosition);
-            if (config.isTransactionCoordinatorEnabled()) {
+            if (topic.getBrokerService().getPulsar().getConfig().isTransactionCoordinatorEnabled()) {
                 positions.forEach(position -> {
                     if (((ManagedCursorImpl) cursor).isMessageDeleted(position)) {
                         pendingAckHandle.clearIndividualPosition(position);
@@ -556,15 +506,9 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
         return "Null";
     }
 
+    @Override
     public CompletableFuture<AnalyzeBacklogResult> analyzeBacklog(Optional<Position> position) {
-        final ManagedLedger managedLedger = topic.getManagedLedger();
-        final String newNonDurableCursorName = "analyze-backlog-" + UUID.randomUUID();
-        ManagedCursor newNonDurableCursor;
-        try {
-            newNonDurableCursor = ((ManagedCursorImpl) cursor).duplicateNonDurableCursor(newNonDurableCursorName);
-        } catch (ManagedLedgerException e) {
-            return CompletableFuture.failedFuture(e);
-        }
+
         long start = System.currentTimeMillis();
         if (log.isDebugEnabled()) {
             log.debug("[{}][{}] Starting to analyze backlog", topicName, subName);
@@ -579,7 +523,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
         AtomicLong rejectedMessages = new AtomicLong();
         AtomicLong rescheduledMessages = new AtomicLong();
 
-        Position currentPosition = newNonDurableCursor.getMarkDeletedPosition();
+        Position currentPosition = cursor.getMarkDeletedPosition();
 
         if (log.isDebugEnabled()) {
             log.debug("[{}][{}] currentPosition {}",
@@ -588,9 +532,10 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
         final EntryFilterSupport entryFilterSupport = dispatcher != null
                 ? (EntryFilterSupport) dispatcher : new EntryFilterSupport(this);
         // we put some hard limits on the scan, in order to prevent denial of services
-        long maxEntries = config.getSubscriptionBacklogScanMaxEntries();
-        long timeOutMs = config.getSubscriptionBacklogScanMaxTimeMs();
-        int batchSize = config.getDispatcherMaxReadBatchSize();
+        ServiceConfiguration configuration = topic.getBrokerService().getPulsar().getConfiguration();
+        long maxEntries = configuration.getSubscriptionBacklogScanMaxEntries();
+        long timeOutMs = configuration.getSubscriptionBacklogScanMaxTimeMs();
+        int batchSize = configuration.getDispatcherMaxReadBatchSize();
         AtomicReference<Position> firstPosition = new AtomicReference<>();
         AtomicReference<Position> lastPosition = new AtomicReference<>();
         final Predicate<Entry> condition = entry -> {
@@ -638,7 +583,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
 
             return true;
         };
-        CompletableFuture<AnalyzeBacklogResult> res = newNonDurableCursor.scan(
+        return cursor.scan(
                 position,
                 condition,
                 batchSize,
@@ -665,22 +610,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
                     topicName, subName, end - start, result);
             return result;
         });
-        res.whenComplete((__, ex) -> {
-            managedLedger.asyncDeleteCursor(newNonDurableCursorName,
-                new AsyncCallbacks.DeleteCursorCallback(){
-                    @Override
-                    public void deleteCursorComplete(Object ctx) {
-                        // Nothing to do.
-                    }
 
-                    @Override
-                    public void deleteCursorFailed(ManagedLedgerException exception, Object ctx) {
-                        log.warn("[{}][{}] Delete non-durable cursor[{}] failed when analyze backlog.",
-                                topicName, subName, newNonDurableCursor.getName());
-                    }
-                }, null);
-        });
-        return res;
     }
 
     @Override
@@ -755,8 +685,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
     @Override
     public CompletableFuture<Void> resetCursor(long timestamp) {
         CompletableFuture<Void> future = new CompletableFuture<>();
-        PersistentMessageFinder persistentMessageFinder = new PersistentMessageFinder(topicName, cursor,
-                config.getManagedLedgerCursorResetLedgerCloseTimestampMaxClockSkewMillis());
+        PersistentMessageFinder persistentMessageFinder = new PersistentMessageFinder(topicName, cursor);
 
         if (log.isDebugEnabled()) {
             log.debug("[{}][{}] Resetting subscription to timestamp {}", topicName, subName, timestamp);
@@ -785,8 +714,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
                 } else {
                     finalPosition = position.getNext();
                 }
-                CompletableFuture<Void> resetCursorFuture = resetCursor(finalPosition);
-                FutureUtil.completeAfter(future, resetCursorFuture);
+                resetCursor(finalPosition, future);
             }
 
             @Override
@@ -805,13 +733,18 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
     }
 
     @Override
-    public CompletableFuture<Void> resetCursor(Position finalPosition) {
+    public CompletableFuture<Void> resetCursor(Position position) {
+        CompletableFuture<Void> future = new CompletableFuture<>();
+        resetCursor(position, future);
+        return future;
+    }
+
+    private void resetCursor(Position finalPosition, CompletableFuture<Void> future) {
         if (!IS_FENCED_UPDATER.compareAndSet(PersistentSubscription.this, FALSE, TRUE)) {
-            return CompletableFuture.failedFuture(new SubscriptionBusyException("Failed to fence subscription"));
+            future.completeExceptionally(new SubscriptionBusyException("Failed to fence subscription"));
+            return;
         }
 
-        final CompletableFuture<Void> future = new CompletableFuture<>();
-        inProgressResetCursorFuture = future;
         final CompletableFuture<Void> disconnectFuture;
 
         // Lock the Subscription object before locking the Dispatcher object to avoid deadlocks
@@ -831,7 +764,6 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
             if (throwable != null) {
                 log.error("[{}][{}] Failed to disconnect consumer from subscription", topicName, subName, throwable);
                 IS_FENCED_UPDATER.set(PersistentSubscription.this, FALSE);
-                inProgressResetCursorFuture = null;
                 future.completeExceptionally(
                         new SubscriptionBusyException("Failed to disconnect consumers from subscription"));
                 return;
@@ -840,16 +772,26 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
             log.info("[{}][{}] Successfully disconnected consumers from subscription, proceeding with cursor reset",
                     topicName, subName);
 
-            try {
-                boolean forceReset = false;
-                if (topic.getCompactedTopic() != null && topic.getCompactedTopic().getCompactionHorizon().isPresent()) {
-                    PositionImpl horizon = (PositionImpl) topic.getCompactedTopic().getCompactionHorizon().get();
+            CompletableFuture<Boolean> forceReset = new CompletableFuture<>();
+            if (topic.getTopicCompactionService() == null) {
+                forceReset.complete(false);
+            } else {
+                topic.getTopicCompactionService().getLastCompactedPosition().thenAccept(lastCompactedPosition -> {
                     PositionImpl resetTo = (PositionImpl) finalPosition;
-                    if (horizon.compareTo(resetTo) >= 0) {
-                        forceReset = true;
+                    if (lastCompactedPosition != null && resetTo.compareTo(lastCompactedPosition.getLedgerId(),
+                            lastCompactedPosition.getEntryId()) <= 0) {
+                        forceReset.complete(true);
+                    } else {
+                        forceReset.complete(false);
                     }
-                }
-                cursor.asyncResetCursor(finalPosition, forceReset, new AsyncCallbacks.ResetCursorCallback() {
+                }).exceptionally(ex -> {
+                    forceReset.completeExceptionally(ex);
+                    return null;
+                });
+            }
+
+            forceReset.thenAccept(forceResetValue -> {
+                cursor.asyncResetCursor(finalPosition, forceResetValue, new AsyncCallbacks.ResetCursorCallback() {
                     @Override
                     public void resetComplete(Object ctx) {
                         if (log.isDebugEnabled()) {
@@ -860,7 +802,6 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
                             dispatcher.cursorIsReset();
                         }
                         IS_FENCED_UPDATER.set(PersistentSubscription.this, FALSE);
-                        inProgressResetCursorFuture = null;
                         future.complete(null);
                     }
 
@@ -869,7 +810,6 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
                         log.error("[{}][{}] Failed to reset subscription to position {}", topicName, subName,
                                 finalPosition, exception);
                         IS_FENCED_UPDATER.set(PersistentSubscription.this, FALSE);
-                        inProgressResetCursorFuture = null;
                         // todo - retry on InvalidCursorPositionException
                         // or should we just ask user to retry one more time?
                         if (exception instanceof InvalidCursorPositionException) {
@@ -881,14 +821,13 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
                         }
                     }
                 });
-            } catch (Exception e) {
+            }).exceptionally((e) -> {
                 log.error("[{}][{}] Error while resetting cursor", topicName, subName, e);
                 IS_FENCED_UPDATER.set(PersistentSubscription.this, FALSE);
-                inProgressResetCursorFuture = null;
                 future.completeExceptionally(new BrokerServiceException(e));
-            }
+                return null;
+            });
         });
-        return future;
     }
 
     @Override
@@ -910,12 +849,6 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
             public void readEntryComplete(Entry entry, Object ctx) {
                 future.complete(entry);
             }
-
-            @Override
-            public String toString() {
-                return String.format("Subscription [%s-%s] async replay entries", PersistentSubscription.this.topicName,
-                        PersistentSubscription.this.subName);
-            }
         }, null);
 
         return future;
@@ -1155,29 +1088,8 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
         return cursor.getEstimatedSizeSinceMarkDeletePosition();
     }
 
-    /**
-     * @deprecated please call {@link #getStatsAsync(Boolean, boolean, boolean)}.
-     */
-    @Deprecated
     public SubscriptionStatsImpl getStats(Boolean getPreciseBacklog, boolean subscriptionBacklogSize,
                                           boolean getEarliestTimeInBacklog) {
-        // So far, there is no case hits this check.
-        if (getEarliestTimeInBacklog) {
-            throw new IllegalArgumentException("Calling the sync method subscription.getStats with"
-                    + " getEarliestTimeInBacklog, it may encountered a deadlock error.");
-        }
-        // The method "getStatsAsync" will be a sync method if the param "isGetEarliestTimeInBacklog" is false.
-        try {
-            return getStatsAsync(getPreciseBacklog, subscriptionBacklogSize, false).get(5, TimeUnit.SECONDS);
-        } catch (InterruptedException | ExecutionException | TimeoutException e) {
-            // This error will never occur.
-            throw new RuntimeException(e);
-        }
-    }
-
-    public CompletableFuture<SubscriptionStatsImpl> getStatsAsync(Boolean getPreciseBacklog,
-                                                                  boolean subscriptionBacklogSize,
-                                                                  boolean getEarliestTimeInBacklog) {
         SubscriptionStatsImpl subStats = new SubscriptionStatsImpl();
         subStats.lastExpireTimestamp = lastExpireTimestamp;
         subStats.lastConsumedFlowTimestamp = lastConsumedFlowTimestamp;
@@ -1239,7 +1151,6 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
                 subStats.unackedMessages = d.getTotalUnackedMessages();
                 subStats.blockedSubscriptionOnUnackedMsgs = d.isBlockedDispatcherOnUnackedMsgs();
                 subStats.msgDelayed = d.getNumberOfDelayedMessages();
-                subStats.msgInReplay = d.getNumberOfMessagesInReplay();
             }
         }
         subStats.msgBacklog = getNumberOfEntriesInBacklog(getPreciseBacklog);
@@ -1249,6 +1160,17 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
         } else {
             subStats.backlogSize = -1;
         }
+        if (getEarliestTimeInBacklog && subStats.msgBacklog > 0) {
+            ManagedLedgerImpl managedLedger = ((ManagedLedgerImpl) cursor.getManagedLedger());
+            PositionImpl markDeletedPosition = (PositionImpl) cursor.getMarkDeletedPosition();
+            long result = 0;
+            try {
+                result = managedLedger.getEarliestMessagePublishTimeOfPos(markDeletedPosition).get();
+            } catch (InterruptedException | ExecutionException e) {
+                result = -1;
+            }
+            subStats.earliestMsgPublishTimeInBacklog = result;
+        }
         subStats.msgBacklogNoDelayed = subStats.msgBacklog - subStats.msgDelayed;
         subStats.msgRateExpired = expiryMonitor.getMessageExpiryRate();
         subStats.totalMsgExpired = expiryMonitor.getTotalMessageExpired();
@@ -1273,20 +1195,7 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
         subStats.nonContiguousDeletedMessagesRanges = cursor.getTotalNonContiguousDeletedMessagesRange();
         subStats.nonContiguousDeletedMessagesRangesSerializedSize =
                 cursor.getNonContiguousDeletedMessagesRangeSerializedSize();
-        if (!getEarliestTimeInBacklog) {
-            return CompletableFuture.completedFuture(subStats);
-        }
-        if (subStats.msgBacklog > 0) {
-            ManagedLedgerImpl managedLedger = ((ManagedLedgerImpl) cursor.getManagedLedger());
-            PositionImpl markDeletedPosition = (PositionImpl) cursor.getMarkDeletedPosition();
-            return managedLedger.getEarliestMessagePublishTimeOfPos(markDeletedPosition).thenApply(v -> {
-                subStats.earliestMsgPublishTimeInBacklog = v;
-                return subStats;
-            });
-        } else {
-            subStats.earliestMsgPublishTimeInBacklog = -1;
-            return CompletableFuture.completedFuture(subStats);
-        }
+        return subStats;
     }
 
     @Override
@@ -1452,8 +1361,4 @@ public class PersistentSubscription extends AbstractSubscription implements Subs
 
     private static final Logger log = LoggerFactory.getLogger(PersistentSubscription.class);
 
-    @VisibleForTesting
-    public Boolean getReplicatedControlled() {
-        return replicatedControlled;
-    }
 }
