@@ -37,7 +37,6 @@ import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.Position;
-import org.apache.bookkeeper.mledger.impl.ManagedLedgerImpl;
 import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.commons.collections4.map.LinkedMap;
 import org.apache.pulsar.broker.service.BrokerServiceException;
@@ -76,8 +75,8 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
      */
     private final LinkedMap<TxnID, PositionImpl> ongoingTxns = new LinkedMap<>();
 
-    // when change max read position, the count will +1. Take snapshot will reset the count.
-    private final AtomicLong changeMaxReadPositionCount = new AtomicLong();
+    // when add abort or change max read position, the count will +1. Take snapshot will set 0 into it.
+    private final AtomicLong changeMaxReadPositionAndAddAbortTimes = new AtomicLong();
 
     private final LongAdder txnCommittedCounter = new LongAdder();
 
@@ -102,21 +101,9 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
 
     private final AbortedTxnProcessor snapshotAbortedTxnProcessor;
 
-    private final MaxReadPositionCallBack maxReadPositionCallBack;
-
-    private static AbortedTxnProcessor createSnapshotProcessor(PersistentTopic topic) {
-        return topic.getBrokerService().getPulsar().getConfiguration().isTransactionBufferSegmentedSnapshotEnabled()
-                ? new SnapshotSegmentAbortedTxnProcessorImpl(topic)
-                : new SingleSnapshotAbortedTxnProcessorImpl(topic);
-    }
-
+    private final AbortedTxnProcessor.SnapshotType snapshotType;
 
     public TopicTransactionBuffer(PersistentTopic topic) {
-        this(topic, createSnapshotProcessor(topic));
-    }
-
-    @VisibleForTesting
-    TopicTransactionBuffer(PersistentTopic topic, AbortedTxnProcessor snapshotAbortedTxnProcessor) {
         super(State.None);
         this.topic = topic;
         this.timer = topic.getBrokerService().getPulsar().getTransactionTimer();
@@ -125,8 +112,13 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
         this.takeSnapshotIntervalTime = topic.getBrokerService().getPulsar()
                 .getConfiguration().getTransactionBufferSnapshotMinTimeInMillis();
         this.maxReadPosition = (PositionImpl) topic.getManagedLedger().getLastConfirmedEntry();
-        this.snapshotAbortedTxnProcessor = snapshotAbortedTxnProcessor;
-        this.maxReadPositionCallBack = topic.getMaxReadPositionCallBack();
+        if (topic.getBrokerService().getPulsar().getConfiguration().isTransactionBufferSegmentedSnapshotEnabled()) {
+            snapshotAbortedTxnProcessor = new SnapshotSegmentAbortedTxnProcessorImpl(topic);
+            snapshotType = AbortedTxnProcessor.SnapshotType.Segment;
+        } else {
+            snapshotAbortedTxnProcessor = new SingleSnapshotAbortedTxnProcessorImpl(topic);
+            snapshotType = AbortedTxnProcessor.SnapshotType.Single;
+        }
         this.recover();
     }
 
@@ -177,15 +169,13 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
                         if (msgMetadata != null && msgMetadata.hasTxnidMostBits() && msgMetadata.hasTxnidLeastBits()) {
                             TxnID txnID = new TxnID(msgMetadata.getTxnidMostBits(), msgMetadata.getTxnidLeastBits());
                             PositionImpl position = PositionImpl.get(entry.getLedgerId(), entry.getEntryId());
-                            synchronized (TopicTransactionBuffer.this) {
-                                if (Markers.isTxnMarker(msgMetadata)) {
-                                    if (Markers.isTxnAbortMarker(msgMetadata)) {
-                                        snapshotAbortedTxnProcessor.putAbortedTxnAndPosition(txnID, position);
-                                    }
-                                    removeTxnAndUpdateMaxReadPosition(txnID);
-                                } else {
-                                    handleTransactionMessage(txnID, position);
+                            if (Markers.isTxnMarker(msgMetadata)) {
+                                if (Markers.isTxnAbortMarker(msgMetadata)) {
+                                    snapshotAbortedTxnProcessor.putAbortedTxnAndPosition(txnID, position);
                                 }
+                                updateMaxReadPosition(txnID);
+                            } else {
+                                handleTransactionMessage(txnID, position);
                             }
                         }
                     }
@@ -296,17 +286,11 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
                 .checkAbortedTransaction(txnId)) {
             ongoingTxns.put(txnId, (PositionImpl) position);
             PositionImpl firstPosition = ongoingTxns.get(ongoingTxns.firstKey());
-            // max read position is less than first ongoing transaction message position
-            updateMaxReadPosition(((ManagedLedgerImpl) topic.getManagedLedger()).getPreviousPosition(firstPosition),
-                    false);
+            //max read position is less than first ongoing transaction message position, so entryId -1
+            maxReadPosition = PositionImpl.get(firstPosition.getLedgerId(), firstPosition.getEntryId() - 1);
         }
     }
 
-    // ThreadSafe
-    private void updateLastDispatchablePosition(Position position) {
-        topic.updateLastDispatchablePosition(position);
-    }
-
     @Override
     public CompletableFuture<TransactionBufferReader> openTransactionBufferReader(TxnID txnID, long startSequenceId) {
         return null;
@@ -327,7 +311,7 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
                     @Override
                     public void addComplete(Position position, ByteBuf entryData, Object ctx) {
                         synchronized (TopicTransactionBuffer.this) {
-                            removeTxnAndUpdateMaxReadPosition(txnID);
+                            updateMaxReadPosition(txnID);
                             handleLowWaterMark(txnID, lowWaterMark);
                             snapshotAbortedTxnProcessor.trimExpiredAbortedTxns();
                             takeSnapshotByChangeTimes();
@@ -374,13 +358,13 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
                     public void addComplete(Position position, ByteBuf entryData, Object ctx) {
                         synchronized (TopicTransactionBuffer.this) {
                             snapshotAbortedTxnProcessor.putAbortedTxnAndPosition(txnID, (PositionImpl) position);
-                            removeTxnAndUpdateMaxReadPosition(txnID);
+                            updateMaxReadPosition(txnID);
                             snapshotAbortedTxnProcessor.trimExpiredAbortedTxns();
                             takeSnapshotByChangeTimes();
-                            txnAbortedCounter.increment();
-                            completableFuture.complete(null);
-                            handleLowWaterMark(txnID, lowWaterMark);
                         }
+                        txnAbortedCounter.increment();
+                        completableFuture.complete(null);
+                        handleLowWaterMark(txnID, lowWaterMark);
                     }
 
                     @Override
@@ -442,56 +426,33 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
     }
 
     private void takeSnapshotByChangeTimes() {
-        if (changeMaxReadPositionCount.get() >= takeSnapshotIntervalNumber) {
-            this.changeMaxReadPositionCount.set(0);
+        if (changeMaxReadPositionAndAddAbortTimes.get() >= takeSnapshotIntervalNumber) {
+            this.changeMaxReadPositionAndAddAbortTimes.set(0);
             this.snapshotAbortedTxnProcessor.takeAbortedTxnsSnapshot(this.maxReadPosition);
         }
     }
 
     private void takeSnapshotByTimeout() {
-        if (changeMaxReadPositionCount.get() > 0) {
-            this.changeMaxReadPositionCount.set(0);
+        if (changeMaxReadPositionAndAddAbortTimes.get() > 0) {
+            this.changeMaxReadPositionAndAddAbortTimes.set(0);
             this.snapshotAbortedTxnProcessor.takeAbortedTxnsSnapshot(this.maxReadPosition);
         }
         this.timer.newTimeout(TopicTransactionBuffer.this,
                 takeSnapshotIntervalTime, TimeUnit.MILLISECONDS);
     }
 
-    /**
-     * remove the specified transaction from ongoing transaction list and update the max read position.
-     * @param txnID
-     */
-    void removeTxnAndUpdateMaxReadPosition(TxnID txnID) {
+    void updateMaxReadPosition(TxnID txnID) {
+        PositionImpl preMaxReadPosition = this.maxReadPosition;
         ongoingTxns.remove(txnID);
         if (!ongoingTxns.isEmpty()) {
             PositionImpl position = ongoingTxns.get(ongoingTxns.firstKey());
-            updateMaxReadPosition(((ManagedLedgerImpl) topic.getManagedLedger()).getPreviousPosition(position), false);
+            //max read position is less than first ongoing transaction message position, so entryId -1
+            maxReadPosition = PositionImpl.get(position.getLedgerId(), position.getEntryId() - 1);
         } else {
-            updateMaxReadPosition((PositionImpl) topic.getManagedLedger().getLastConfirmedEntry(), false);
+            maxReadPosition = (PositionImpl) topic.getManagedLedger().getLastConfirmedEntry();
         }
-        // Update the last dispatchable position to null if there is a TXN finished.
-        updateLastDispatchablePosition(null);
-    }
-
-    /**
-     * update the max read position. if the new position is greater than the current max read position,
-     * we will trigger the callback, unless the disableCallback is true.
-     * Currently, we only use the callback to update the lastMaxReadPositionMovedForwardTimestamp.
-     * For non-transactional production, some marker messages will be sent to the topic, in which case we don't need
-     * to trigger the callback.
-     * @param newPosition new max read position to update.
-     * @param disableCallback whether disable the callback.
-     */
-    void updateMaxReadPosition(PositionImpl newPosition, boolean disableCallback) {
-        PositionImpl preMaxReadPosition = this.maxReadPosition;
-        this.maxReadPosition = newPosition;
-        if (preMaxReadPosition.compareTo(this.maxReadPosition) < 0) {
-            if (!checkIfNoSnapshot()) {
-                this.changeMaxReadPositionCount.getAndIncrement();
-            }
-            if (!disableCallback) {
-                maxReadPositionCallBack.maxReadPositionMovedForward(preMaxReadPosition, this.maxReadPosition);
-            }
+        if (preMaxReadPosition.compareTo(this.maxReadPosition) != 0) {
+            this.changeMaxReadPositionAndAddAbortTimes.getAndIncrement();
         }
     }
 
@@ -512,33 +473,29 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
     }
 
     @Override
-    public synchronized boolean isTxnAborted(TxnID txnID, PositionImpl readPosition) {
+    public boolean isTxnAborted(TxnID txnID, PositionImpl readPosition) {
         return snapshotAbortedTxnProcessor.checkAbortedTransaction(txnID);
     }
 
-    /**
-     * Sync max read position for normal publish.
-     * @param position {@link PositionImpl} the position to sync.
-     * @param isMarkerMessage whether the message is marker message, in such case, we
-     *                       don't need to trigger the callback to update lastMaxReadPositionMovedForwardTimestamp.
-     */
     @Override
-    public void syncMaxReadPositionForNormalPublish(PositionImpl position, boolean isMarkerMessage) {
+    public void syncMaxReadPositionForNormalPublish(PositionImpl position) {
         // when ongoing transaction is empty, proved that lastAddConfirm is can read max position, because callback
         // thread is the same tread, in this time the lastAddConfirm don't content transaction message.
         synchronized (TopicTransactionBuffer.this) {
             if (checkIfNoSnapshot()) {
-                updateMaxReadPosition(position, isMarkerMessage);
+                this.maxReadPosition = position;
             } else if (checkIfReady()) {
                 if (ongoingTxns.isEmpty()) {
-                    updateMaxReadPosition(position, isMarkerMessage);
+                    maxReadPosition = position;
+                    changeMaxReadPositionAndAddAbortTimes.incrementAndGet();
                 }
             }
         }
-        // If the message is a normal message, update the last dispatchable position.
-        if (!isMarkerMessage) {
-            updateLastDispatchablePosition(position);
-        }
+    }
+
+    @Override
+    public AbortedTxnProcessor.SnapshotType getSnapshotType() {
+        return snapshotType;
     }
 
     @Override
@@ -553,19 +510,18 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
     @Override
     public TransactionInBufferStats getTransactionInBufferStats(TxnID txnID) {
         TransactionInBufferStats transactionInBufferStats = new TransactionInBufferStats();
-        synchronized (this) {
-            transactionInBufferStats.aborted = isTxnAborted(txnID, null);
-            if (ongoingTxns.containsKey(txnID)) {
-                transactionInBufferStats.startPosition = ongoingTxns.get(txnID).toString();
-            }
+        transactionInBufferStats.aborted = isTxnAborted(txnID, null);
+        if (ongoingTxns.containsKey(txnID)) {
+            transactionInBufferStats.startPosition = ongoingTxns.get(txnID).toString();
         }
         return transactionInBufferStats;
     }
 
     @Override
-    public TransactionBufferStats getStats(boolean lowWaterMarks) {
-        TransactionBufferStats transactionBufferStats = new TransactionBufferStats();
-        transactionBufferStats.lastSnapshotTimestamps = this.snapshotAbortedTxnProcessor.getLastSnapshotTimestamps();
+    public TransactionBufferStats getStats(boolean lowWaterMarks, boolean segmentStats) {
+        TransactionBufferStats transactionBufferStats = this.snapshotAbortedTxnProcessor
+                .generateSnapshotStats(segmentStats);
+        transactionBufferStats.snapshotType = snapshotType.toString();
         transactionBufferStats.state = this.getState().name();
         transactionBufferStats.maxReadPosition = this.maxReadPosition.toString();
         if (lowWaterMarks) {
@@ -578,6 +534,11 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
         return transactionBufferStats;
     }
 
+    @Override
+    public TransactionBufferStats getStats(boolean lowWaterMarks) {
+        return getStats(lowWaterMarks, false);
+    }
+
     @Override
     public void run(Timeout timeout) {
         if (checkIfReady()) {
@@ -626,7 +587,7 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
                         this, topic.getName());
                 return;
             }
-            abortedTxnProcessor.recoverFromSnapshot().thenAccept(startReadCursorPosition -> {
+            abortedTxnProcessor.recoverFromSnapshot().thenAcceptAsync(startReadCursorPosition -> {
                 //Transaction is not use for this topic, so just make maxReadPosition as LAC.
                 if (startReadCursorPosition == null) {
                     callBack.noNeedToRecover();
@@ -672,7 +633,8 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
 
                 closeCursor(SUBSCRIPTION_NAME);
                 callBack.recoverComplete();
-            }).exceptionally(e -> {
+            }, topic.getBrokerService().getPulsar().getTransactionExecutorProvider()
+                    .getExecutor(this)).exceptionally(e -> {
                 callBack.recoverExceptionally(e.getCause());
                 log.error("[{}]Transaction buffer failed to recover snapshot!", topic.getName(), e);
                 return null;
@@ -708,18 +670,6 @@ public class TopicTransactionBuffer extends TopicTransactionBufferState implemen
         }
     }
 
-    /**
-     * A functional interface to handle the max read position move forward.
-     */
-    public interface MaxReadPositionCallBack {
-        /**
-         * callback method when max read position move forward.
-         * @param oldPosition the old max read position.
-         * @param newPosition the new max read position.
-         */
-        void maxReadPositionMovedForward(PositionImpl oldPosition, PositionImpl newPosition);
-    }
-
     static class FillEntryQueueCallback implements AsyncCallbacks.ReadEntriesCallback {
 
         private final AtomicLong outstandingReadsRequests = new AtomicLong(0);
