@@ -61,8 +61,6 @@ public class TransactionBufferHandlerImpl implements TransactionBufferHandler {
     private final PulsarService pulsarService;
     private final PulsarClientImpl pulsarClient;
 
-    private final int randomKeyForSelectConnection;
-
     private static final AtomicIntegerFieldUpdater<TransactionBufferHandlerImpl> REQUEST_CREDITS_UPDATER =
             AtomicIntegerFieldUpdater.newUpdater(TransactionBufferHandlerImpl.class, "requestCredits");
     private volatile int requestCredits;
@@ -76,7 +74,6 @@ public class TransactionBufferHandlerImpl implements TransactionBufferHandler {
         this.operationTimeoutInMills = operationTimeoutInMills;
         this.timer = timer;
         this.requestCredits = Math.max(100, maxConcurrentRequests);
-        this.randomKeyForSelectConnection = pulsarClient.getCnxPool().genRandomKeyToSelectCon();
     }
 
     @Override
@@ -127,9 +124,6 @@ public class TransactionBufferHandlerImpl implements TransactionBufferHandler {
             }
         } else {
             pendingRequests.add(op);
-            if (currentPermits != REQUEST_CREDITS_UPDATER.get(this)) {
-                checkPendingRequests();
-            }
             return false;
         }
     }
@@ -140,9 +134,8 @@ public class TransactionBufferHandlerImpl implements TransactionBufferHandler {
                 if (clientCnx.ctx().channel().isActive()) {
                     clientCnx.registerTransactionBufferHandler(TransactionBufferHandlerImpl.this);
                     outstandingRequests.put(op.requestId, op);
-                    final long requestId = op.requestId;
                     timer.newTimeout(timeout -> {
-                        OpRequestSend peek = outstandingRequests.remove(requestId);
+                        OpRequestSend peek = outstandingRequests.remove(op.requestId);
                         if (peek != null && !peek.cb.isDone() && !peek.cb.isCompletedExceptionally()) {
                             peek.cb.completeExceptionally(new TransactionBufferClientException
                                     .RequestTimeoutException());
@@ -253,7 +246,6 @@ public class TransactionBufferHandlerImpl implements TransactionBufferHandler {
                             invalid.recycle();
                         }
                         endTxn(polled);
-                        break;
                     } else {
                         REQUEST_CREDITS_UPDATER.incrementAndGet(this);
                     }
@@ -304,7 +296,7 @@ public class TransactionBufferHandlerImpl implements TransactionBufferHandler {
     }
 
     public CompletableFuture<ClientCnx> getClientCnxWithLookup(String topic) {
-        return pulsarClient.getConnection(topic, randomKeyForSelectConnection);
+        return pulsarClient.getConnection(topic);
     }
 
     public CompletableFuture<ClientCnx> getClientCnx(String topic) {
@@ -325,8 +317,7 @@ public class TransactionBufferHandlerImpl implements TransactionBufferHandler {
                                 }
                                 InetSocketAddress brokerAddress =
                                         InetSocketAddress.createUnresolved(uri.getHost(), uri.getPort());
-                                return pulsarClient.getConnection(brokerAddress, brokerAddress,
-                                        randomKeyForSelectConnection);
+                                return pulsarClient.getConnection(brokerAddress, brokerAddress);
                             } else {
                                 // Bundle is unloading, lookup topic
                                 return getClientCnxWithLookup(topic);
