@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,12 +18,16 @@
  */
 package org.apache.pulsar.broker.transaction.pendingack.impl;
 
+import io.netty.util.Timer;
+import io.prometheus.client.CollectorRegistry;
 import java.util.concurrent.CompletableFuture;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.mledger.AsyncCallbacks;
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedLedger;
 import org.apache.bookkeeper.mledger.ManagedLedgerException;
+import org.apache.pulsar.broker.PulsarService;
+import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.broker.service.persistent.PersistentSubscription;
 import org.apache.pulsar.broker.service.persistent.PersistentTopic;
 import org.apache.pulsar.broker.transaction.exception.pendingack.TransactionPendingAckException;
@@ -31,6 +35,10 @@ import org.apache.pulsar.broker.transaction.pendingack.PendingAckStore;
 import org.apache.pulsar.broker.transaction.pendingack.TransactionPendingAckStoreProvider;
 import org.apache.pulsar.common.api.proto.CommandSubscribe.InitialPosition;
 import org.apache.pulsar.common.naming.TopicName;
+import org.apache.pulsar.common.util.FutureUtil;
+import org.apache.pulsar.transaction.coordinator.impl.DisabledTxnLogBufferedWriterMetricsStats;
+import org.apache.pulsar.transaction.coordinator.impl.TxnLogBufferedWriterConfig;
+import org.apache.pulsar.transaction.coordinator.impl.TxnLogBufferedWriterMetricsStats;
 
 
 /**
@@ -39,6 +47,31 @@ import org.apache.pulsar.common.naming.TopicName;
 @Slf4j
 public class MLPendingAckStoreProvider implements TransactionPendingAckStoreProvider {
 
+    private static volatile TxnLogBufferedWriterMetricsStats bufferedWriterMetrics =
+            DisabledTxnLogBufferedWriterMetricsStats.DISABLED_BUFFERED_WRITER_METRICS;
+
+    public static void initBufferedWriterMetrics(String brokerAdvertisedAddress){
+        if (bufferedWriterMetrics != DisabledTxnLogBufferedWriterMetricsStats.DISABLED_BUFFERED_WRITER_METRICS){
+            return;
+        }
+        synchronized (MLPendingAckStoreProvider.class){
+            if (bufferedWriterMetrics != DisabledTxnLogBufferedWriterMetricsStats.DISABLED_BUFFERED_WRITER_METRICS){
+                return;
+            }
+            bufferedWriterMetrics = new MLTxnPendingAckLogBufferedWriterMetrics(brokerAdvertisedAddress);
+        }
+    }
+
+    public static void closeBufferedWriterMetrics() {
+        synchronized (MLPendingAckStoreProvider.class){
+            if (bufferedWriterMetrics == DisabledTxnLogBufferedWriterMetricsStats.DISABLED_BUFFERED_WRITER_METRICS){
+                return;
+            }
+            bufferedWriterMetrics.close();
+            bufferedWriterMetrics = DisabledTxnLogBufferedWriterMetricsStats.DISABLED_BUFFERED_WRITER_METRICS;
+        }
+    }
+
     @Override
     public CompletableFuture<PendingAckStore> newPendingAckStore(PersistentSubscription subscription) {
         CompletableFuture<PendingAckStore> pendingAckStoreFuture = new CompletableFuture<>();
@@ -49,7 +82,25 @@ public class MLPendingAckStoreProvider implements TransactionPendingAckStoreProv
                             .TransactionPendingAckStoreProviderException("The subscription is null."));
             return pendingAckStoreFuture;
         }
+
         PersistentTopic originPersistentTopic = (PersistentTopic) subscription.getTopic();
+        PulsarService pulsarService = originPersistentTopic.getBrokerService().getPulsar();
+
+        final Timer brokerClientSharedTimer =
+                pulsarService.getBrokerClientSharedTimer();
+        final ServiceConfiguration serviceConfiguration = pulsarService.getConfiguration();
+        final TxnLogBufferedWriterConfig txnLogBufferedWriterConfig = new TxnLogBufferedWriterConfig();
+        txnLogBufferedWriterConfig.setBatchEnabled(serviceConfiguration.isTransactionPendingAckBatchedWriteEnabled());
+        txnLogBufferedWriterConfig.setBatchedWriteMaxRecords(
+                serviceConfiguration.getTransactionPendingAckBatchedWriteMaxRecords()
+        );
+        txnLogBufferedWriterConfig.setBatchedWriteMaxSize(
+                serviceConfiguration.getTransactionPendingAckBatchedWriteMaxSize()
+        );
+        txnLogBufferedWriterConfig.setBatchedWriteMaxDelayInMillis(
+                serviceConfiguration.getTransactionPendingAckBatchedWriteMaxDelayInMillis()
+        );
+
         String pendingAckTopicName = MLPendingAckStore
                 .getTransactionPendingAckStoreSuffix(originPersistentTopic.getName(), subscription.getName());
         originPersistentTopic.getBrokerService().getManagedLedgerFactory()
@@ -81,7 +132,9 @@ public class MLPendingAckStoreProvider implements TransactionPendingAckStoreProv
                                                                         .getBrokerService()
                                                                         .getPulsar()
                                                                         .getConfiguration()
-                                                                        .getTransactionPendingAckLogIndexMinLag()));
+                                                                        .getTransactionPendingAckLogIndexMinLag(),
+                                                                txnLogBufferedWriterConfig,
+                                                                brokerClientSharedTimer, bufferedWriterMetrics));
                                                         if (log.isDebugEnabled()) {
                                                             log.debug("{},{} open MLPendingAckStore cursor success",
                                                                     originPersistentTopic.getName(),
@@ -107,15 +160,21 @@ public class MLPendingAckStoreProvider implements TransactionPendingAckStoreProv
                                         pendingAckStoreFuture.completeExceptionally(exception);
                                     }
                                 }, () -> true, null);
-            });
-        }).exceptionally(e -> {
-            log.error("Failed to obtain the existence of ManagerLedger with topic and subscription : "
-                    + originPersistentTopic.getSubscriptions() + "  "
-                    + subscription.getName());
-            pendingAckStoreFuture.completeExceptionally(
-                    e.getCause());
-            return null;
-        });
+                    }).exceptionally(e -> {
+                        Throwable t = FutureUtil.unwrapCompletionException(e);
+                        log.error("[{}] [{}] Failed to get managedLedger config when init pending ack store!",
+                                originPersistentTopic, subscription, t);
+                        pendingAckStoreFuture.completeExceptionally(t);
+                        return null;
+
+                    });
+                }).exceptionally(e -> {
+                    Throwable t = FutureUtil.unwrapCompletionException(e);
+                    log.error("[{}] [{}] Failed to check the pending ack topic exist when init pending ack store!",
+                            originPersistentTopic, subscription, t);
+                    pendingAckStoreFuture.completeExceptionally(t);
+                    return null;
+                });
         return pendingAckStoreFuture;
     }
 
@@ -127,4 +186,14 @@ public class MLPendingAckStoreProvider implements TransactionPendingAckStoreProv
         return originPersistentTopic.getBrokerService().getManagedLedgerFactory()
                 .asyncExists(TopicName.get(pendingAckTopicName).getPersistenceNamingEncoding());
     }
+
+    private static class MLTxnPendingAckLogBufferedWriterMetrics extends TxnLogBufferedWriterMetricsStats{
+
+        private MLTxnPendingAckLogBufferedWriterMetrics(String brokerAdvertisedAddress) {
+            super("pulsar_txn_pending_ack_store",
+                    new String[]{"broker"},
+                    new String[]{brokerAdvertisedAddress},
+                    CollectorRegistry.defaultRegistry);
+        }
+    }
 }
\ No newline at end of file
