@@ -28,7 +28,6 @@ import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentLinkedQueue;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.MessageId;
-import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.RawMessage;
 import org.apache.pulsar.client.api.RawReader;
 import org.apache.pulsar.client.api.Schema;
@@ -52,8 +51,7 @@ public class RawReaderImpl implements RawReader {
     private RawConsumerImpl consumer;
 
     public RawReaderImpl(PulsarClientImpl client, String topic, String subscription,
-                         CompletableFuture<Consumer<byte[]>> consumerFuture,
-                         boolean createTopicIfDoesNotExist, boolean retryOnRecoverableErrors) {
+                         CompletableFuture<Consumer<byte[]>> consumerFuture) {
         consumerConfiguration = new ConsumerConfigurationData<>();
         consumerConfiguration.getTopicNames().add(topic);
         consumerConfiguration.setSubscriptionName(subscription);
@@ -61,10 +59,9 @@ public class RawReaderImpl implements RawReader {
         consumerConfiguration.setReceiverQueueSize(DEFAULT_RECEIVER_QUEUE_SIZE);
         consumerConfiguration.setReadCompacted(true);
         consumerConfiguration.setSubscriptionInitialPosition(SubscriptionInitialPosition.Earliest);
-        consumerConfiguration.setAckReceiptEnabled(true);
 
-        consumer = new RawConsumerImpl(client, consumerConfiguration, consumerFuture, createTopicIfDoesNotExist,
-                retryOnRecoverableErrors);
+        consumer = new RawConsumerImpl(client, consumerConfiguration,
+                                       consumerFuture);
     }
 
     @Override
@@ -111,11 +108,9 @@ public class RawReaderImpl implements RawReader {
     static class RawConsumerImpl extends ConsumerImpl<byte[]> {
         final BlockingQueue<RawMessageAndCnx> incomingRawMessages;
         final Queue<CompletableFuture<RawMessage>> pendingRawReceives;
-        final boolean retryOnRecoverableErrors;
 
         RawConsumerImpl(PulsarClientImpl client, ConsumerConfigurationData<byte[]> conf,
-                CompletableFuture<Consumer<byte[]>> consumerFuture, boolean createTopicIfDoesNotExist,
-                boolean retryOnRecoverableErrors) {
+                CompletableFuture<Consumer<byte[]>> consumerFuture) {
             super(client,
                     conf.getSingleTopic(),
                     conf,
@@ -127,18 +122,10 @@ public class RawReaderImpl implements RawReader {
                     MessageId.earliest,
                     0 /* startMessageRollbackDurationInSec */,
                     Schema.BYTES, null,
-                    createTopicIfDoesNotExist
+                    true
             );
             incomingRawMessages = new GrowableArrayBlockingQueue<>();
             pendingRawReceives = new ConcurrentLinkedQueue<>();
-            this.retryOnRecoverableErrors = retryOnRecoverableErrors;
-        }
-
-        protected boolean isUnrecoverableError(Throwable t) {
-            if (!retryOnRecoverableErrors && (t instanceof PulsarClientException.ServiceNotReadyException)) {
-                return true;
-            }
-            return super.isUnrecoverableError(t);
         }
 
         void tryCompletePending() {
@@ -164,13 +151,14 @@ public class RawReaderImpl implements RawReader {
                     // TODO message validation
                     numMsg = 1;
                 }
-                MessageIdData messageId = messageAndCnx.msg.getMessageIdData();
-                lastDequeuedMessageId = new BatchMessageIdImpl(messageId.getLedgerId(), messageId.getEntryId(),
-                        messageId.getPartition(), numMsg - 1);
                 if (!future.complete(messageAndCnx.msg)) {
                     messageAndCnx.msg.close();
                     closeAsync();
                 }
+                MessageIdData messageId = messageAndCnx.msg.getMessageIdData();
+                lastDequeuedMessageId = new BatchMessageIdImpl(messageId.getLedgerId(), messageId.getEntryId(),
+                    messageId.getPartition(), numMsg - 1);
+
                 ClientCnx currentCnx = cnx();
                 if (currentCnx == messageAndCnx.cnx) {
                     increaseAvailablePermits(currentCnx, numMsg);
@@ -199,10 +187,7 @@ public class RawReaderImpl implements RawReader {
         private void failPendingRawReceives() {
             List<CompletableFuture<RawMessage>> toError = new ArrayList<>();
             while (!pendingRawReceives.isEmpty()) {
-                final CompletableFuture<RawMessage> ret = pendingRawReceives.poll();
-                if (ret != null) {
-                    toError.add(ret);
-                }
+                toError.add(pendingRawReceives.remove());
             }
             toError.forEach((f) -> f.cancel(false));
         }
