@@ -18,21 +18,30 @@
  */
 package org.apache.pulsar.compaction;
 
-import java.util.List;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
+import org.apache.bookkeeper.mledger.AsyncCallbacks.ReadEntriesCallback;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.Position;
-import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.pulsar.broker.service.Consumer;
 
 public interface CompactedTopic {
     CompletableFuture<CompactedTopicContext> newCompactedLedger(Position p, long compactedLedgerId);
     CompletableFuture<Void> deleteCompactedLedger(long compactedLedgerId);
-    CompletableFuture<List<Entry>> asyncReadEntriesOrWait(
-            ManagedCursor cursor, int maxEntries, long bytesToRead, PositionImpl maxReadPosition, boolean isFirstRead,
-            Consumer consumer);
+
+    /**
+     * Read entries from compacted topic.
+     *
+     * @deprecated Use {@link CompactedTopicUtils#asyncReadCompactedEntries(TopicCompactionService, ManagedCursor,
+     * int, long, boolean, ReadEntriesCallback, boolean, Consumer)} instead.
+     */
+    @Deprecated
+    void asyncReadEntriesOrWait(ManagedCursor cursor,
+                                int numberOfEntriesToRead,
+                                boolean isFirstRead,
+                                ReadEntriesCallback callback,
+                                Consumer consumer);
     CompletableFuture<Entry> readLastEntryOfCompactedLedger();
     Optional<Position> getCompactionHorizon();
 }
