@@ -18,7 +18,7 @@
  */
 package org.apache.pulsar.compaction;
 
-import static org.apache.bookkeeper.mledger.util.ManagedLedgerUtils.readEntriesWithSkipOrWait;
+import static org.apache.pulsar.common.protocol.Commands.DEFAULT_CONSUMER_EPOCH;
 import com.github.benmanes.caffeine.cache.AsyncLoadingCache;
 import com.github.benmanes.caffeine.cache.Caffeine;
 import com.google.common.annotations.VisibleForTesting;
@@ -32,21 +32,20 @@ import java.util.NoSuchElementException;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.TimeoutException;
 import javax.annotation.Nullable;
 import org.apache.bookkeeper.client.BKException;
 import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.client.LedgerEntry;
 import org.apache.bookkeeper.client.LedgerHandle;
+import org.apache.bookkeeper.mledger.AsyncCallbacks.ReadEntriesCallback;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.Position;
 import org.apache.bookkeeper.mledger.impl.EntryImpl;
-import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
 import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.pulsar.broker.service.Consumer;
+import org.apache.pulsar.broker.service.persistent.PersistentDispatcherSingleActiveConsumer.ReadEntriesCtx;
 import org.apache.pulsar.client.api.MessageId;
 import org.apache.pulsar.client.api.RawMessage;
 import org.apache.pulsar.client.impl.RawMessageImpl;
@@ -54,10 +53,6 @@ import org.apache.pulsar.common.api.proto.MessageIdData;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-/**
- * Note: If you want to guarantee that strong consistency between `compactionHorizon` and `compactedTopicContext`,
- * you need to call getting them method in "synchronized(CompactedTopicImpl){ ... }" lock block.
- */
 public class CompactedTopicImpl implements CompactedTopic {
     static final long NEWER_THAN_COMPACTED = -0xfeed0fbaL;
     static final long COMPACT_LEDGER_EMPTY = -0xfeed0fbbL;
@@ -66,7 +61,7 @@ public class CompactedTopicImpl implements CompactedTopic {
     private final BookKeeper bk;
 
     private volatile PositionImpl compactionHorizon = null;
-    private CompletableFuture<CompactedTopicContext> compactedTopicContext = null;
+    private volatile CompletableFuture<CompactedTopicContext> compactedTopicContext = null;
 
     public CompactedTopicImpl(BookKeeper bk) {
         this.bk = bk;
@@ -75,28 +70,14 @@ public class CompactedTopicImpl implements CompactedTopic {
     @Override
     public CompletableFuture<CompactedTopicContext> newCompactedLedger(Position p, long compactedLedgerId) {
         synchronized (this) {
+            compactionHorizon = (PositionImpl) p;
+
             CompletableFuture<CompactedTopicContext> previousContext = compactedTopicContext;
             compactedTopicContext = openCompactedLedger(bk, compactedLedgerId);
 
-            compactionHorizon = (PositionImpl) p;
-
             // delete the ledger from the old context once the new one is open
-            return compactedTopicContext.thenCompose(ctx -> {
-                if (previousContext != null) {
-                    previousContext.thenAccept(previousCtx -> {
-                        // Print an error log here, which is not expected.
-                        if (previousCtx != null && previousCtx.getLedger() != null
-                                && previousCtx.getLedger().getId() == compactedLedgerId) {
-                            log.error("[__compaction] Using the same compacted ledger to override the old one, which is"
-                                + " not expected and it may cause a ledger lost error. {} -> {}", compactedLedgerId,
-                                ctx.getLedger().getId());
-                        }
-                    });
-                    return previousContext;
-                } else {
-                    return CompletableFuture.completedFuture(null);
-                }
-            });
+            return compactedTopicContext.thenCompose(__ ->
+                    previousContext != null ? previousContext : CompletableFuture.completedFuture(null));
         }
     }
 
@@ -106,31 +87,27 @@ public class CompactedTopicImpl implements CompactedTopic {
     }
 
     @Override
-    public CompletableFuture<List<Entry>> asyncReadEntriesOrWait(
-            ManagedCursor cursor, int maxEntries, long bytesToRead, PositionImpl maxReadPosition, boolean isFirstRead,
-            Consumer consumer
-    ) {
+    @Deprecated
+    public void asyncReadEntriesOrWait(ManagedCursor cursor,
+                                       int numberOfEntriesToRead,
+                                       boolean isFirstRead,
+                                       ReadEntriesCallback callback, Consumer consumer) {
             PositionImpl cursorPosition;
-            boolean readFromEarliest = isFirstRead && MessageId.earliest.equals(consumer.getStartMessageId())
-                && (!cursor.isDurable() || cursor.getName().equals(Compactor.COMPACTION_SUBSCRIPTION)
-                || cursor.getMarkDeletedPosition() == null
-                || cursor.getMarkDeletedPosition().getEntryId() == -1L);
-            if (readFromEarliest){
+            if (isFirstRead && MessageId.earliest.equals(consumer.getStartMessageId())){
                 cursorPosition = PositionImpl.EARLIEST;
             } else {
                 cursorPosition = (PositionImpl) cursor.getReadPosition();
             }
 
+            // TODO: redeliver epoch link https://github.com/apache/pulsar/issues/13690
+            ReadEntriesCtx readEntriesCtx = ReadEntriesCtx.create(consumer, DEFAULT_CONSUMER_EPOCH);
+
             final PositionImpl currentCompactionHorizon = compactionHorizon;
 
             if (currentCompactionHorizon == null
                 || currentCompactionHorizon.compareTo(cursorPosition) < 0) {
-                return readEntriesWithSkipOrWait(cursor, maxEntries, bytesToRead, maxReadPosition, null);
+                cursor.asyncReadEntriesOrWait(numberOfEntriesToRead, callback, readEntriesCtx, PositionImpl.LATEST);
             } else {
-                ManagedCursorImpl managedCursor = (ManagedCursorImpl) cursor;
-                int numberOfEntriesToRead = managedCursor.applyMaxSizeCap(maxEntries, bytesToRead);
-
-                final var future = new CompletableFuture<List<Entry>>();
                 compactedTopicContext.thenCompose(
                     (context) -> findStartPoint(cursorPosition, context.ledger.getLastAddConfirmed(), context.cache)
                         .thenCompose((startPoint) -> {
@@ -138,19 +115,13 @@ public class CompactedTopicImpl implements CompactedTopic {
                             // the cursor just needs to be set to the compaction horizon
                             if (startPoint == COMPACT_LEDGER_EMPTY || startPoint == NEWER_THAN_COMPACTED) {
                                 cursor.seek(currentCompactionHorizon.getNext());
-                                future.complete(Collections.emptyList());
-                                return future;
+                                callback.readEntriesComplete(Collections.emptyList(), readEntriesCtx);
+                                return CompletableFuture.completedFuture(null);
                             } else {
                                 long endPoint = Math.min(context.ledger.getLastAddConfirmed(),
-                                                         startPoint + (numberOfEntriesToRead - 1));
+                                                         startPoint + numberOfEntriesToRead);
                                 return readEntries(context.ledger, startPoint, endPoint)
-                                    .thenApply((entries) -> {
-                                        long entriesSize = 0;
-                                        for (Entry entry : entries) {
-                                            entriesSize += entry.getLength();
-                                        }
-                                        managedCursor.updateReadStats(entries.size(), entriesSize);
-
+                                    .thenAccept((entries) -> {
                                         Entry lastEntry = entries.get(entries.size() - 1);
                                         // The compaction task depends on the last snapshot and the incremental
                                         // entries to build the new snapshot. So for the compaction cursor, we
@@ -158,21 +129,19 @@ public class CompactedTopicImpl implements CompactedTopic {
                                         // the complete last snapshot because of the compactor will read the data
                                         // before the compaction cursor mark delete position
                                         cursor.seek(lastEntry.getPosition().getNext(), true);
-                                        future.complete(entries);
-                                        return entries;
+                                        callback.readEntriesComplete(entries, readEntriesCtx);
                                     });
                             }
                         }))
-                    .exceptionally(exception -> {
+                    .exceptionally((exception) -> {
                         if (exception.getCause() instanceof NoSuchElementException) {
                             cursor.seek(currentCompactionHorizon.getNext());
-                            future.complete(Collections.emptyList());
+                            callback.readEntriesComplete(Collections.emptyList(), readEntriesCtx);
                         } else {
-                            future.completeExceptionally(new ManagedLedgerException(exception));
+                            callback.readEntriesFailed(new ManagedLedgerException(exception), readEntriesCtx);
                         }
                         return null;
                     });
-                return future;
             }
     }
 
@@ -282,7 +251,7 @@ public class CompactedTopicImpl implements CompactedTopic {
         return promise;
     }
 
-    private static CompletableFuture<List<Entry>> readEntries(LedgerHandle lh, long from, long to) {
+    static CompletableFuture<List<Entry>> readEntries(LedgerHandle lh, long from, long to) {
         CompletableFuture<Enumeration<LedgerEntry>> promise = new CompletableFuture<>();
 
         lh.asyncReadEntries(from, to,
@@ -314,10 +283,8 @@ public class CompactedTopicImpl implements CompactedTopic {
      * Getter for CompactedTopicContext.
      * @return CompactedTopicContext
      */
-    public Optional<CompactedTopicContext> getCompactedTopicContext() throws ExecutionException, InterruptedException,
-            TimeoutException {
-        return compactedTopicContext == null ? Optional.empty() :
-                Optional.of(compactedTopicContext.get(30, TimeUnit.SECONDS));
+    public Optional<CompactedTopicContext> getCompactedTopicContext() throws ExecutionException, InterruptedException {
+        return compactedTopicContext == null ? Optional.empty() : Optional.of(compactedTopicContext.get());
     }
 
     @Override
@@ -347,11 +314,6 @@ public class CompactedTopicImpl implements CompactedTopic {
         return Optional.ofNullable(this.compactionHorizon);
     }
 
-    public void reset() {
-        this.compactionHorizon = null;
-        this.compactedTopicContext = null;
-    }
-
     @Nullable
     public CompletableFuture<CompactedTopicContext> getCompactedTopicContextFuture() {
         return compactedTopicContext;
