@@ -18,10 +18,8 @@
  */
 package org.apache.pulsar.broker;
 
-import static org.assertj.core.api.Assertions.assertThat;
-import static org.testng.Assert.assertEquals;
-import static org.testng.Assert.assertTrue;
 import com.google.common.collect.Sets;
+
 import java.util.Queue;
 import java.util.UUID;
 import java.util.concurrent.ConcurrentLinkedQueue;
@@ -46,6 +44,9 @@ import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.assertEquals;
+import static org.testng.Assert.assertTrue;
+
 @Slf4j
 public class TopicEventsListenerTest extends BrokerTestBase {
 
@@ -125,7 +126,7 @@ public class TopicEventsListenerTest extends BrokerTestBase {
                            boolean forceDelete) throws Exception {
         String topicName = topicTypePersistence + "://" + namespace + "/" + "topic-" + UUID.randomUUID();
 
-        createTopicAndVerifyEvents(topicTypePersistence, topicTypePartitioned, topicName);
+        createTopicAndVerifyEvents(topicTypePartitioned, topicName);
 
         events.clear();
         if (topicTypePartitioned.equals("partitioned")) {
@@ -149,7 +150,7 @@ public class TopicEventsListenerTest extends BrokerTestBase {
                                      boolean forceDelete) throws Exception {
         String topicName = topicTypePersistence + "://" + namespace + "/" + "topic-" + UUID.randomUUID();
 
-        createTopicAndVerifyEvents(topicTypePersistence, topicTypePartitioned, topicName);
+        createTopicAndVerifyEvents(topicTypePartitioned, topicName);
 
         events.clear();
         admin.topics().unload(topicName);
@@ -181,7 +182,7 @@ public class TopicEventsListenerTest extends BrokerTestBase {
                                     boolean forceDelete) throws Exception {
         String topicName = topicTypePersistence + "://" + namespace + "/" + "topic-" + UUID.randomUUID();
 
-        createTopicAndVerifyEvents(topicTypePersistence, topicTypePartitioned, topicName);
+        createTopicAndVerifyEvents(topicTypePartitioned, topicName);
 
         Consumer<byte[]> consumer = pulsarClient.newConsumer().topic(topicName).subscriptionName("sub").subscribe();
         Producer<byte[]> producer = pulsarClient.newProducer().topic(topicName).create();
@@ -237,7 +238,7 @@ public class TopicEventsListenerTest extends BrokerTestBase {
     public void testTopicAutoGC(String topicTypePersistence, String topicTypePartitioned) throws Exception {
         String topicName = topicTypePersistence + "://" + namespace + "/" + "topic-" + UUID.randomUUID();
 
-        createTopicAndVerifyEvents(topicTypePersistence, topicTypePartitioned, topicName);
+        createTopicAndVerifyEvents(topicTypePartitioned, topicName);
 
         admin.namespaces().setInactiveTopicPolicies(namespace,
                 new InactiveTopicPolicies(InactiveTopicDeleteMode.delete_when_no_subscriptions, 1, true));
@@ -261,48 +262,25 @@ public class TopicEventsListenerTest extends BrokerTestBase {
         );
     }
 
-    private void createTopicAndVerifyEvents(String topicDomain, String topicTypePartitioned, String topicName) throws Exception {
+    private void createTopicAndVerifyEvents(String topicTypePartitioned, String topicName) throws Exception {
         final String[] expectedEvents;
-        if (topicDomain.equalsIgnoreCase("persistent") || topicTypePartitioned.equals("partitioned")) {
-            if (topicTypePartitioned.equals("partitioned")) {
-                if (topicDomain.equalsIgnoreCase("persistent")) {
-                    expectedEvents = new String[]{
-                            "CREATE__BEFORE",
-                            "CREATE__SUCCESS",
-                            "LOAD__BEFORE",
-                            "LOAD__SUCCESS"
-                    };
-                } else {
-                    // For non-persistent partitioned topic, only metadata is initially created;
-                    // partitions are created when the client connects.
-                    // PR #23680 currently records creation events at metadata creation,
-                    // and the broker records them again when partitions are loaded,
-                    // which can result in multiple events.
-                    // Ideally, #23680 should not record the event here,
-                    // because the topic is not fully created until the client connects.
-                    expectedEvents = new String[]{
-                            "CREATE__BEFORE",
-                            "CREATE__SUCCESS",
-                            "LOAD__BEFORE",
-                            "CREATE__BEFORE",
-                            "CREATE__SUCCESS",
-                            "LOAD__SUCCESS",
-                    };
-                }
-            } else {
-                expectedEvents = new String[]{
-                        "LOAD__BEFORE",
-                        "CREATE__BEFORE",
-                        "CREATE__SUCCESS",
-                        "LOAD__SUCCESS"
-                };
-            }
+        if (topicTypePartitioned.equals("partitioned")) {
+            topicNameToWatch = topicName + "-partition-1";
+            admin.topics().createPartitionedTopic(topicName, 2);
+            triggerPartitionsCreation(topicName);
+
+            expectedEvents = new String[]{
+                    "LOAD__BEFORE",
+                    "CREATE__BEFORE",
+                    "CREATE__SUCCESS",
+                    "LOAD__SUCCESS"
+            };
+
         } else {
+            topicNameToWatch = topicName;
+            admin.topics().createNonPartitionedTopic(topicName);
+
             expectedEvents = new String[]{
-                    // Before https://github.com/apache/pulsar/pull/21995, Pulsar will skip create topic if the topic
-                    //   was already exists, and the action "check topic exists" will try to load Managed ledger,
-                    //   the check triggers two exrtra events: [LOAD__BEFORE, LOAD__FAILURE].
-                    //   #21995 fixed this wrong behavior, so remove these two events.
                     "LOAD__BEFORE",
                     "LOAD__FAILURE",
                     "LOAD__BEFORE",
@@ -310,67 +288,13 @@ public class TopicEventsListenerTest extends BrokerTestBase {
                     "CREATE__SUCCESS",
                     "LOAD__SUCCESS"
             };
-        }
-        if (topicTypePartitioned.equals("partitioned")) {
-            topicNameToWatch = topicName + "-partition-1";
-            admin.topics().createPartitionedTopic(topicName, 2);
-            triggerPartitionsCreation(topicName);
-        } else {
-            topicNameToWatch = topicName;
-            admin.topics().createNonPartitionedTopic(topicName);
+
         }
 
         Awaitility.waitAtMost(10, TimeUnit.SECONDS).untilAsserted(() ->
                 Assert.assertEquals(events.toArray(), expectedEvents));
     }
 
-    @DataProvider(name = "createTopicEventType")
-    public static Object[][] createTopicEventType() {
-        return new Object[][] {
-                {"persistent", "partitioned"},
-                {"persistent", "non-partitioned"},
-                {"non-persistent", "partitioned"},
-                {"non-persistent", "non-partitioned"},
-        };
-    }
-
-    @Test(dataProvider = "createTopicEventType")
-    public void testCreateTopicEvent(String topicTypePersistence, String topicTypePartitioned) throws Exception {
-        String topicName = topicTypePersistence + "://" + namespace + "/" + "topic-" + UUID.randomUUID();
-
-        events.clear();
-        if (topicTypePartitioned.equals("partitioned")) {
-            topicNameToWatch = topicName + "-partition-0";
-            admin.topics().createPartitionedTopic(topicName, 1);
-        } else {
-            topicNameToWatch = topicName;
-            admin.topics().createNonPartitionedTopic(topicName);
-        }
-
-        triggerPartitionsCreation(topicName); // ensure partitions are really created
-        triggerPartitionsCreation(topicName); // trigger again to ensure no duplicate events
-
-        Awaitility.await().during(3, TimeUnit.SECONDS)
-                .untilAsserted(() -> {
-                    if (topicTypePartitioned.equals("partitioned") && topicTypePersistence.equals("non-persistent")) {
-                        // For non-persistent partitioned topic, only metadata is initially created;
-                        // partitions are created when the client connects.
-                        // PR #23680 currently records creation events at metadata creation,
-                        // and the broker records them again when partitions are loaded,
-                        // which can result in multiple events.
-                        // Ideally, #23680 should not record the event here,
-                        // because the topic is not fully created until the client connects.
-                        assertThat(events.toArray())
-                                .contains("CREATE__BEFORE")
-                                .contains("CREATE__SUCCESS");
-                    } else {
-                        assertThat(events.toArray())
-                                .containsOnlyOnce("CREATE__BEFORE")
-                                .containsOnlyOnce("CREATE__SUCCESS");
-                    }
-                });
-    }
-
     private PulsarAdmin createPulsarAdmin() throws PulsarClientException {
         return PulsarAdmin.builder()
                 .serviceHttpUrl(brokerUrl != null ? brokerUrl.toString() : brokerUrlTls.toString())
