@@ -19,18 +19,15 @@
 package org.apache.pulsar.broker.auth;
 
 import static org.apache.pulsar.broker.BrokerTestUtil.spyWithoutRecordingInvocations;
-import static org.testng.Assert.assertEquals;
 import com.google.common.collect.Sets;
 import java.lang.reflect.Field;
-import java.lang.reflect.Method;
 import java.net.InetSocketAddress;
 import java.net.URI;
 import java.net.URL;
-import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Date;
 import java.util.HashSet;
-import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
@@ -40,13 +37,10 @@ import java.util.function.Consumer;
 import java.util.function.Predicate;
 import javax.ws.rs.container.AsyncResponse;
 import javax.ws.rs.container.TimeoutHandler;
-import lombok.AllArgsConstructor;
-import lombok.Data;
-import org.apache.pulsar.broker.BrokerTestUtil;
 import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.broker.service.BrokerService;
-import org.apache.pulsar.broker.service.persistent.PersistentTopic;
+import org.apache.pulsar.broker.service.BrokerTestBase;
 import org.apache.pulsar.broker.testcontext.PulsarTestContext;
 import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.admin.PulsarAdminBuilder;
@@ -54,9 +48,6 @@ import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.api.ClientBuilder;
 import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
-import org.apache.pulsar.client.impl.ConnectionPool;
-import org.apache.pulsar.client.impl.ProducerImpl;
-import org.apache.pulsar.client.impl.PulsarClientImpl;
 import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.TenantInfo;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
@@ -65,7 +56,6 @@ import org.apache.pulsar.tests.TestRetrySupport;
 import org.apache.pulsar.utils.ResourceUtils;
 import org.apache.zookeeper.MockZooKeeper;
 import org.awaitility.Awaitility;
-import org.awaitility.reflect.WhiteboxImpl;
 import org.mockito.Mockito;
 import org.mockito.internal.util.MockUtil;
 import org.slf4j.Logger;
@@ -78,7 +68,7 @@ import org.testng.annotations.DataProvider;
 public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
     // All certificate-authority files are copied from the tests/certificate-authority directory and all share the same
     // root CA.
-    public static String getTlsFileForClient(String name) {
+    protected static String getTlsFileForClient(String name) {
         return ResourceUtils.getAbsolutePath(String.format("certificate-authority/client-keys/%s.pem", name));
     }
     public final static String CA_CERT_FILE_PATH =
@@ -141,11 +131,6 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
 
     protected boolean enableBrokerInterceptor = false;
 
-    private final List<AutoCloseable> closeables = new ArrayList<>();
-
-    // Set to true in test's constructor to use a real Zookeeper (TestZKServer)
-    protected boolean useTestZookeeper;
-
     public MockedPulsarServiceBaseTest() {
         resetConfig();
     }
@@ -236,8 +221,10 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
         this.conf.setBrokerShutdownTimeoutMs(0L);
         this.conf.setLoadBalancerOverrideBrokerNicSpeedGbps(Optional.of(1.0d));
         this.conf.setBrokerServicePort(Optional.of(0));
+        this.conf.setBrokerServicePortTls(Optional.of(0));
         this.conf.setAdvertisedAddress("localhost");
         this.conf.setWebServicePort(Optional.of(0));
+        this.conf.setWebServicePortTls(Optional.of(0));
         this.conf.setNumExecutorThreadPoolSize(5);
         this.conf.setExposeBundlesMetricsInPrometheus(true);
     }
@@ -264,43 +251,19 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
         }
         if (brokerGateway != null) {
             brokerGateway.close();
-            brokerGateway = null;
         }
         if (pulsarTestContext != null) {
             pulsarTestContext.close();
             pulsarTestContext = null;
         }
-
         resetConfig();
-        callCloseables(closeables);
-        closeables.clear();
         onCleanup();
-
-        // clear fields to avoid test runtime memory leak, pulsarTestContext already handles closing of these instances
-        pulsar = null;
-        mockZooKeeper = null;
-        mockZooKeeperGlobal = null;
     }
 
     protected void onCleanup() {
 
     }
 
-    protected <T extends AutoCloseable> T registerCloseable(T closeable) {
-        closeables.add(closeable);
-        return closeable;
-    }
-
-    private static void callCloseables(List<AutoCloseable> closeables) {
-        for (int i = closeables.size() - 1; i >= 0; i--) {
-            try {
-                closeables.get(i).close();
-            } catch (Exception e) {
-                log.error("Failure in calling close method", e);
-            }
-        }
-    }
-
     protected abstract void setup() throws Exception;
 
     protected abstract void cleanup() throws Exception;
@@ -331,14 +294,7 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
      * @throws Exception if an error occurs
      */
     protected void restartBroker() throws Exception {
-        restartBroker(null);
-    }
-
-    protected void restartBroker(Consumer<ServiceConfiguration> configurationChanger) throws Exception {
         stopBroker();
-        if (configurationChanger != null) {
-            configurationChanger.accept(conf);
-        }
         startBroker();
     }
 
@@ -429,6 +385,7 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
         PulsarTestContext.Builder builder = PulsarTestContext.builder()
                 .spyByDefault()
                 .config(conf)
+                .withMockZookeeper(true)
                 .pulsarServiceCustomizer(pulsarService -> {
                     try {
                         beforePulsarStart(pulsarService);
@@ -437,25 +394,9 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
                     }
                 })
                 .brokerServiceCustomizer(this::customizeNewBrokerService);
-        configureMetadataStores(builder);
         return builder;
     }
 
-    /**
-     * Configures the metadata stores for the PulsarTestContext.Builder instance.
-     * Set useTestZookeeper to true in the test's constructor to use TestZKServer which is a real ZooKeeper
-     * implementation.
-     *
-     * @param builder the PulsarTestContext.Builder instance to configure
-     */
-    protected void configureMetadataStores(PulsarTestContext.Builder builder) {
-        if (useTestZookeeper) {
-            builder.withTestZookeeper();
-        } else {
-            builder.withMockZookeeper(true);
-        }
-    }
-
     /**
      * This method can be used in test classes for creating additional PulsarTestContext instances
      * that share the same mock ZooKeeper and BookKeeper instances as the main PulsarTestContext instance.
@@ -523,7 +464,9 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
         configuration.setBrokerShutdownTimeoutMs(0L);
         configuration.setLoadBalancerOverrideBrokerNicSpeedGbps(Optional.of(1.0d));
         configuration.setBrokerServicePort(Optional.of(0));
+        configuration.setBrokerServicePortTls(Optional.of(0));
         configuration.setWebServicePort(Optional.of(0));
+        configuration.setWebServicePortTls(Optional.of(0));
         configuration.setBookkeeperClientExposeStatsToPrometheus(true);
         configuration.setNumExecutorThreadPoolSize(5);
         configuration.setBrokerMaxConnections(0);
@@ -643,24 +586,47 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
     }
 
     /**
-     * see {@link #deleteNamespaceWithRetry(String, boolean, PulsarAdmin)}
+     * see {@link BrokerTestBase#deleteNamespaceWithRetry(String, boolean, PulsarAdmin, Collection)}
      */
     protected void deleteNamespaceWithRetry(String ns, boolean force)
             throws Exception {
-        deleteNamespaceWithRetry(ns, force, admin);
+        BrokerTestBase.deleteNamespaceWithRetry(ns, force, admin, pulsar);
+    }
+
+    /**
+     * see {@link BrokerTestBase#deleteNamespaceWithRetry(String, boolean, PulsarAdmin, Collection)}
+     */
+    protected void deleteNamespaceWithRetry(String ns, boolean force, PulsarAdmin admin)
+            throws Exception {
+        BrokerTestBase.deleteNamespaceWithRetry(ns, force, admin, pulsar);
+    }
+
+    /**
+     * see {@link MockedPulsarServiceBaseTest#deleteNamespaceWithRetry(String, boolean, PulsarAdmin, Collection)}
+     */
+    public static void deleteNamespaceWithRetry(String ns, boolean force, PulsarAdmin admin, PulsarService...pulsars)
+            throws Exception {
+        deleteNamespaceWithRetry(ns, force, admin, Arrays.asList(pulsars));
     }
 
     /**
      * 1. Pause system "__change_event" topic creates.
      * 2. Do delete namespace with retry because maybe fail by race-condition with create topics.
      */
-    public static void deleteNamespaceWithRetry(String ns, boolean force, PulsarAdmin admin) throws Exception {
-        Awaitility.await().atMost(5, TimeUnit.SECONDS).until(() -> {
+    public static void deleteNamespaceWithRetry(String ns, boolean force, PulsarAdmin admin,
+                                                Collection<PulsarService> pulsars) throws Exception {
+        Awaitility.await()
+                .pollDelay(500, TimeUnit.MILLISECONDS)
+                .until(() -> {
             try {
                 // Maybe fail by race-condition with create topics, just retry.
                 admin.namespaces().deleteNamespace(ns, force);
                 return true;
-            } catch (Exception ex) {
+            } catch (PulsarAdminException.NotFoundException ex) {
+                // namespace was already deleted, ignore exception
+                return true;
+            } catch (Exception e) {
+                log.warn("Failed to delete namespace {} (force={})", ns, force, e);
                 return false;
             }
         });
@@ -678,52 +644,5 @@ public abstract class MockedPulsarServiceBaseTest extends TestRetrySupport {
         };
     }
 
-    protected ServiceProducer getServiceProducer(ProducerImpl clientProducer, String topicName) {
-        PersistentTopic persistentTopic =
-                (PersistentTopic) pulsar.getBrokerService().getTopic(topicName, false).join().get();
-        org.apache.pulsar.broker.service.Producer serviceProducer =
-                persistentTopic.getProducers().get(clientProducer.getProducerName());
-        long clientProducerId = WhiteboxImpl.getInternalState(clientProducer, "producerId");
-        assertEquals(serviceProducer.getProducerId(), clientProducerId);
-        assertEquals(serviceProducer.getEpoch(), clientProducer.getConnectionHandler().getEpoch());
-        return new ServiceProducer(serviceProducer, persistentTopic);
-    }
-
-    @Data
-    @AllArgsConstructor
-    public static class ServiceProducer {
-        private org.apache.pulsar.broker.service.Producer serviceProducer;
-        private PersistentTopic persistentTopic;
-    }
-
-    protected void sleepSeconds(int seconds){
-        try {
-            Thread.sleep(1000 * seconds);
-        } catch (InterruptedException e) {
-            log.warn("This thread has been interrupted", e);
-            Thread.currentThread().interrupt();
-        }
-    }
-
-    private static void reconnectAllConnections(PulsarClientImpl c) throws Exception {
-        ConnectionPool pool = c.getCnxPool();
-        Method closeAllConnections = ConnectionPool.class.getDeclaredMethod("closeAllConnections", new Class[]{});
-        closeAllConnections.setAccessible(true);
-        closeAllConnections.invoke(pool, new Object[]{});
-    }
-
-    protected void reconnectAllConnections() throws Exception {
-        reconnectAllConnections((PulsarClientImpl) pulsarClient);
-    }
-
-    protected void logTopicStats(String topic) {
-        BrokerTestUtil.logTopicStats(log, admin, topic);
-    }
-
-    @DataProvider(name = "trueFalse")
-    public static Object[][] trueFalse() {
-        return new Object[][] { { Boolean.TRUE }, { Boolean.FALSE } };
-    }
-
     private static final Logger log = LoggerFactory.getLogger(MockedPulsarServiceBaseTest.class);
 }
