@@ -38,19 +38,16 @@ import org.apache.pulsar.broker.PulsarServerException;
 import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.client.api.Producer;
+import org.apache.pulsar.client.api.ProducerBuilder;
 import org.apache.pulsar.client.api.Schema;
-import org.apache.pulsar.client.impl.ConnectionPool;
-import org.apache.pulsar.client.impl.ProducerBuilderImpl;
 import org.apache.pulsar.client.impl.PulsarClientImpl;
-import org.apache.pulsar.client.impl.conf.ProducerConfigurationData;
-import org.apache.pulsar.common.policies.data.stats.ReplicatorStatsImpl;
 import org.apache.pulsar.common.util.collections.ConcurrentOpenHashMap;
 import org.awaitility.Awaitility;
 import org.awaitility.reflect.WhiteboxImpl;
 import org.testng.Assert;
 import org.testng.annotations.Test;
 
-@Test(groups = "broker-replication")
+@Test(groups = "broker")
 public class AbstractReplicatorTest {
 
     @Test
@@ -65,13 +62,9 @@ public class AbstractReplicatorTest {
         final PulsarService pulsar = mock(PulsarService.class);
         final BrokerService broker = mock(BrokerService.class);
         final Topic localTopic = mock(Topic.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
         final PulsarClientImpl localClient = mock(PulsarClientImpl.class);
-        when(localClient.getCnxPool()).thenReturn(connectionPool);
         final PulsarClientImpl remoteClient = mock(PulsarClientImpl.class);
-        when(remoteClient.getCnxPool()).thenReturn(connectionPool);
-        final ProducerConfigurationData producerConf = new ProducerConfigurationData();
-        final ProducerBuilderImpl producerBuilder = mock(ProducerBuilderImpl.class);
+        final ProducerBuilder producerBuilder = mock(ProducerBuilder.class);
         final ConcurrentOpenHashMap<String, CompletableFuture<Optional<Topic>>> topics = new ConcurrentOpenHashMap<>();
         when(broker.executor()).thenReturn(eventLoopGroup);
         when(broker.getTopics()).thenReturn(topics);
@@ -87,7 +80,6 @@ public class AbstractReplicatorTest {
         when(producerBuilder.sendTimeout(anyInt(), any())).thenReturn(producerBuilder);
         when(producerBuilder.maxPendingMessages(anyInt())).thenReturn(producerBuilder);
         when(producerBuilder.producerName(anyString())).thenReturn(producerBuilder);
-        when(producerBuilder.getConf()).thenReturn(producerConf);
         // Mock create producer fail.
         when(producerBuilder.create()).thenThrow(new RuntimeException("mocked ex"));
         when(producerBuilder.createAsync())
@@ -96,7 +88,7 @@ public class AbstractReplicatorTest {
         final ReplicatorInTest replicator = new ReplicatorInTest(localCluster, localTopic, remoteCluster, topicName,
                 replicatorPrefix, broker, remoteClient);
         replicator.startProducer();
-        replicator.terminate();
+        replicator.disconnect();
 
         // Verify task will done.
         Awaitility.await().atMost(10, TimeUnit.SECONDS).untilAsserted(() -> {
@@ -131,7 +123,7 @@ public class AbstractReplicatorTest {
         }
 
         @Override
-        protected void setProducerAndTriggerReadEntries(Producer<byte[]> producer) {
+        protected void readEntries(Producer<byte[]> producer) {
 
         }
 
@@ -141,25 +133,7 @@ public class AbstractReplicatorTest {
         }
 
         @Override
-        public ReplicatorStatsImpl getStats() {
-            return null;
-        }
-
-        @Override
-        public void updateRates() {
-
-        }
-
-        @Override
-        public boolean isConnected() {
-            return false;
-        }
-
-        @Override
-        protected void beforeTerminate() {}
-
-        @Override
-        public long getNumberOfEntriesInBacklog() {
+        protected long getNumberOfEntriesInBacklog() {
             return 0;
         }
 
