@@ -20,12 +20,9 @@ package org.apache.pulsar.broker.service;
 
 import static org.apache.pulsar.common.naming.SystemTopicNames.TRANSACTION_COORDINATOR_ASSIGN;
 import static org.apache.pulsar.common.naming.SystemTopicNames.TRANSACTION_COORDINATOR_LOG;
-import static org.mockito.ArgumentMatchers.anyString;
 import static org.mockito.Mockito.any;
 import static org.mockito.Mockito.doReturn;
-import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.spy;
-import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertNotNull;
@@ -36,6 +33,7 @@ import com.google.common.collect.Sets;
 import com.google.gson.Gson;
 import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
+import com.google.gson.JsonPrimitive;
 import io.netty.buffer.ByteBuf;
 import io.netty.channel.EventLoopGroup;
 import io.netty.util.concurrent.DefaultThreadFactory;
@@ -43,7 +41,7 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
-import java.io.StringReader;
+import java.lang.reflect.Array;
 import java.lang.reflect.Field;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
@@ -67,16 +65,10 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
 import lombok.Cleanup;
 import lombok.extern.slf4j.Slf4j;
-import org.apache.bookkeeper.client.api.ReadHandle;
-import org.apache.bookkeeper.mledger.LedgerOffloader;
 import org.apache.bookkeeper.mledger.ManagedLedgerConfig;
 import org.apache.bookkeeper.mledger.ManagedLedgerException;
-import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
 import org.apache.bookkeeper.mledger.impl.ManagedLedgerFactoryImpl;
 import org.apache.bookkeeper.mledger.impl.ManagedLedgerImpl;
-import org.apache.bookkeeper.mledger.impl.NullLedgerOffloader;
-import org.apache.commons.lang3.StringUtils;
-import org.apache.bookkeeper.mledger.impl.NonAppendableLedgerOffloader;
 import org.apache.http.HttpResponse;
 import org.apache.http.client.HttpClient;
 import org.apache.http.client.methods.HttpGet;
@@ -85,10 +77,7 @@ import org.apache.pulsar.broker.PulsarService;
 import org.apache.pulsar.broker.loadbalance.extensions.channel.ServiceUnitStateChannelImpl;
 import org.apache.pulsar.broker.namespace.NamespaceService;
 import org.apache.pulsar.broker.service.BrokerServiceException.PersistenceException;
-import org.apache.pulsar.broker.service.PulsarMetadataEventSynchronizer.State;
-import org.apache.pulsar.broker.service.persistent.PersistentSubscription;
 import org.apache.pulsar.broker.service.persistent.PersistentTopic;
-import org.apache.pulsar.broker.stats.prometheus.PrometheusMetricsServlet;
 import org.apache.pulsar.broker.stats.prometheus.PrometheusRawMetricsProvider;
 import org.apache.pulsar.client.admin.BrokerStats;
 import org.apache.pulsar.client.admin.PulsarAdminException;
@@ -102,12 +91,9 @@ import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.api.SubscriptionInitialPosition;
-import org.apache.pulsar.client.api.SubscriptionMode;
 import org.apache.pulsar.client.api.SubscriptionType;
-import org.apache.pulsar.client.impl.ClientBuilderImpl;
 import org.apache.pulsar.client.impl.ClientCnx;
 import org.apache.pulsar.client.impl.ConnectionPool;
-import org.apache.pulsar.client.impl.PulsarClientImpl;
 import org.apache.pulsar.client.impl.PulsarServiceNameResolver;
 import org.apache.pulsar.client.impl.auth.AuthenticationTls;
 import org.apache.pulsar.client.impl.conf.ClientConfigurationData;
@@ -119,18 +105,11 @@ import org.apache.pulsar.common.naming.SystemTopicNames;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.policies.data.BundlesData;
 import org.apache.pulsar.common.policies.data.LocalPolicies;
-import org.apache.pulsar.common.policies.data.OffloadPoliciesImpl;
-import org.apache.pulsar.common.policies.data.OffloadedReadPriority;
 import org.apache.pulsar.common.policies.data.SubscriptionStats;
 import org.apache.pulsar.common.policies.data.TopicStats;
 import org.apache.pulsar.common.protocol.Commands;
 import org.apache.pulsar.common.util.netty.EventLoopUtil;
-import org.apache.pulsar.compaction.Compactor;
-import org.apache.zookeeper.KeeperException;
-import org.apache.zookeeper.MockZooKeeper;
 import org.awaitility.Awaitility;
-import org.glassfish.jersey.client.JerseyClient;
-import org.glassfish.jersey.client.JerseyClientBuilder;
 import org.mockito.Mockito;
 import org.testng.Assert;
 import org.testng.annotations.AfterClass;
@@ -176,11 +155,10 @@ public class BrokerServiceTest extends BrokerTestBase {
         assertEquals(bundlesData.getNumBundles(), bundleNum);
         List<String> list = admin.brokers().getActiveBrokers("test");
         assertEquals(list.size(), 1);
-        BrokerService brokerService = pulsar.getBrokerService();
         admin.brokers().shutDownBrokerGracefully(1, false);
         //We can only unload one bundle per second, so it takes at least 2 seconds.
         Awaitility.await().atLeast(bundleNum - 1, TimeUnit.SECONDS).untilAsserted(() -> {
-            assertEquals(brokerService.getTopics().size(), 0);
+            assertEquals(pulsar.getBrokerService().getTopics().size(), 0);
         });
         Awaitility.await().atMost(60, TimeUnit.SECONDS).untilAsserted(() -> {
             assertNull(pulsar.getBrokerService());
@@ -193,8 +171,6 @@ public class BrokerServiceTest extends BrokerTestBase {
             assertTrue(e instanceof PulsarClientException.TimeoutException);
         }
 
-        pulsar = null;
-
         producer.close();
         resetState();
     }
@@ -1029,12 +1005,12 @@ public class BrokerServiceTest extends BrokerTestBase {
             // for PMR
             // 2 lookup will succeed
             long reqId1 = reqId++;
-            ByteBuf request1 = Commands.newPartitionMetadataRequest(topicName, reqId1, true);
+            ByteBuf request1 = Commands.newPartitionMetadataRequest(topicName, reqId1);
             CompletableFuture<?> f1 = pool.getConnection(resolver.resolveHost())
                 .thenCompose(clientCnx -> clientCnx.newLookup(request1, reqId1));
 
             long reqId2 = reqId++;
-            ByteBuf request2 = Commands.newPartitionMetadataRequest(topicName, reqId2, true);
+            ByteBuf request2 = Commands.newPartitionMetadataRequest(topicName, reqId2);
             CompletableFuture<?> f2 = pool.getConnection(resolver.resolveHost())
                 .thenCompose(clientCnx -> {
                     CompletableFuture<?> future = clientCnx.newLookup(request2, reqId2);
@@ -1049,17 +1025,17 @@ public class BrokerServiceTest extends BrokerTestBase {
             // 3 lookup will fail
             latchRef.set(new CountDownLatch(1));
             long reqId3 = reqId++;
-            ByteBuf request3 = Commands.newPartitionMetadataRequest(topicName, reqId3, true);
+            ByteBuf request3 = Commands.newPartitionMetadataRequest(topicName, reqId3);
             f1 = pool.getConnection(resolver.resolveHost())
                 .thenCompose(clientCnx -> clientCnx.newLookup(request3, reqId3));
 
             long reqId4 = reqId++;
-            ByteBuf request4 = Commands.newPartitionMetadataRequest(topicName, reqId4, true);
+            ByteBuf request4 = Commands.newPartitionMetadataRequest(topicName, reqId4);
             f2 = pool.getConnection(resolver.resolveHost())
                 .thenCompose(clientCnx -> clientCnx.newLookup(request4, reqId4));
 
             long reqId5 = reqId++;
-            ByteBuf request5 = Commands.newPartitionMetadataRequest(topicName, reqId5, true);
+            ByteBuf request5 = Commands.newPartitionMetadataRequest(topicName, reqId5);
             CompletableFuture<?> f3 = pool.getConnection(resolver.resolveHost())
                 .thenCompose(clientCnx -> {
                     CompletableFuture<?> future = clientCnx.newLookup(request5, reqId5);
@@ -1181,66 +1157,65 @@ public class BrokerServiceTest extends BrokerTestBase {
     }
 
     @Test
-    public void testCheckInactiveSubscriptionsShouldNotDeleteCompactionCursor() throws Exception {
-        String namespace = "prop/test";
-
-        // set up broker set compaction threshold.
-        cleanup();
-        conf.setBrokerServiceCompactionThresholdInBytes(8);
-        setup();
-
+    public void testConcurrentLoadTopicExceedLimitShouldNotBeAutoCreated() throws Exception {
+        boolean needDeleteTopic = false;
+        final String namespace = "prop/concurrentLoad";
         try {
-            admin.namespaces().createNamespace(namespace);
-        } catch (PulsarAdminException.ConflictException e) {
-            // Ok.. (if test fails intermittently and namespace is already created)
+            // set up broker disable auto create and set concurrent load to 1 qps.
+            cleanup();
+            conf.setMaxConcurrentTopicLoadRequest(1);
+            conf.setAllowAutoTopicCreation(false);
+            setup();
+
+            try {
+                admin.namespaces().createNamespace(namespace);
+            } catch (PulsarAdminException.ConflictException e) {
+                // Ok.. (if test fails intermittently and namespace is already created)
+            }
+
+            // create 3 topic
+            String topicName = "persistent://" + namespace + "/my-topic";
+
+            for (int i = 0; i < 3; i++) {
+                admin.topics().createNonPartitionedTopic(topicName + "_" + i);
+            }
+
+            needDeleteTopic = true;
+
+            // try to load 10 topic
+            ArrayList<CompletableFuture<Optional<Topic>>> loadFutures = new ArrayList<>();
+            for (int i = 0; i < 10; i++) {
+                // try to create topic which should fail as bundle is disable
+                CompletableFuture<Optional<Topic>> futureResult = pulsar.getBrokerService()
+                        .loadOrCreatePersistentTopic(topicName + "_" + i, false, null);
+                loadFutures.add(futureResult);
+            }
+
+            CompletableFuture<?>[] o = (CompletableFuture<?>[]) Array.newInstance(CompletableFuture.class, 10);
+            CompletableFuture<?>[] completableFutures = loadFutures.toArray(o);
+            CompletableFuture.allOf(completableFutures).get();
+
+            // check topic load CompletableFuture. only first three topic should be success.
+            for (int i = 0; i < 10; i++) {
+                CompletableFuture<Optional<Topic>> load = loadFutures.get(i);
+                if (i < 3) {
+                    Assert.assertTrue(load.isDone());
+                    Assert.assertFalse(load.isCompletedExceptionally());
+                } else {
+                    // check topic should not be created if disable autoCreateTopic.
+                    Assert.assertTrue(load.isDone());
+                    Assert.assertTrue(load.get().isEmpty());
+                }
+            }
+        } finally {
+            if (needDeleteTopic) {
+                String topicName = "persistent://" + namespace + "/my-topic";
+
+                for (int i = 0; i < 3; i++) {
+                    admin.topics().delete(topicName + "_" + i);
+                }
+            }
         }
-
-        // set enable subscription expiration.
-        admin.namespaces().setSubscriptionExpirationTime(namespace, 1);
-
-        String compactionInactiveTestTopic = "persistent://prop/test/testCompactionCursorShouldNotDelete";
-
-        admin.topics().createNonPartitionedTopic(compactionInactiveTestTopic);
-
-        CompletableFuture<Optional<Topic>> topicCf =
-                pulsar.getBrokerService().getTopic(compactionInactiveTestTopic, true);
-
-        Optional<Topic> topicOptional = topicCf.get();
-        assertTrue(topicOptional.isPresent());
-
-        PersistentTopic topic = (PersistentTopic) topicOptional.get();
-
-        PersistentSubscription sub = (PersistentSubscription) topic.getSubscription(Compactor.COMPACTION_SUBSCRIPTION);
-        assertNotNull(sub);
-
-        topic.checkCompaction();
-
-        Field currentCompaction = PersistentTopic.class.getDeclaredField("currentCompaction");
-        currentCompaction.setAccessible(true);
-        CompletableFuture<Long> compactionFuture = (CompletableFuture<Long>)currentCompaction.get(topic);
-
-        compactionFuture.get();
-
-        ManagedCursorImpl cursor = (ManagedCursorImpl) sub.getCursor();
-
-        // make cursor last active time to very small to check if it will be deleted
-        Field cursorLastActiveField = ManagedCursorImpl.class.getDeclaredField("lastActive");
-        cursorLastActiveField.setAccessible(true);
-        cursorLastActiveField.set(cursor, 0);
-
-        // replace origin object. so we can check if subscription is deleted.
-        PersistentSubscription spySubscription = Mockito.spy(sub);
-        topic.getSubscriptions().put(Compactor.COMPACTION_SUBSCRIPTION, spySubscription);
-
-        // trigger inactive check.
-        topic.checkInactiveSubscriptions();
-
-        // Compaction subscription should not call delete method.
-        Mockito.verify(spySubscription, Mockito.never()).delete();
-
-        // check if the subscription exist.
-        assertNotNull(topic.getSubscription(Compactor.COMPACTION_SUBSCRIPTION));
-
     }
 
     /**
@@ -1267,7 +1242,7 @@ public class BrokerServiceTest extends BrokerTestBase {
         BrokerService service = spy(pulsar.getBrokerService());
         // create topic will fail to get managedLedgerConfig
         CompletableFuture<ManagedLedgerConfig> failedManagedLedgerConfig = new CompletableFuture<>();
-        failedManagedLedgerConfig.completeExceptionally(new NullPointerException("failed to peristent policy"));
+        failedManagedLedgerConfig.completeExceptionally(new NullPointerException("failed to persistent policy"));
         doReturn(failedManagedLedgerConfig).when(service).getManagedLedgerConfig(any());
 
         CompletableFuture<Void> topicCreation = new CompletableFuture<Void>();
@@ -1415,8 +1390,6 @@ public class BrokerServiceTest extends BrokerTestBase {
         HttpClient httpClient = HttpClientBuilder.create().build();
         final String metricsEndPoint = getPulsar().getWebServiceAddress() + "/metrics";
         HttpResponse response = httpClient.execute(new HttpGet(metricsEndPoint));
-        assertEquals(response.getEntity().getContentType().getValue(),
-                PrometheusMetricsServlet.PROMETHEUS_CONTENT_TYPE_004);
         InputStream inputStream = response.getEntity().getContent();
         InputStreamReader isReader = new InputStreamReader(inputStream);
         BufferedReader reader = new BufferedReader(isReader);
@@ -1555,10 +1528,8 @@ public class BrokerServiceTest extends BrokerTestBase {
 
         assertTrue(brokerService.isSystemTopic(TRANSACTION_COORDINATOR_ASSIGN));
         assertTrue(brokerService.isSystemTopic(TRANSACTION_COORDINATOR_LOG));
-        NamespaceName heartbeatNamespaceV1 = NamespaceService
-                .getHeartbeatNamespace(pulsar.getBrokerId(), pulsar.getConfig());
-        NamespaceName heartbeatNamespaceV2 = NamespaceService
-                .getHeartbeatNamespaceV2(pulsar.getBrokerId(), pulsar.getConfig());
+        NamespaceName heartbeatNamespaceV1 = NamespaceService.getHeartbeatNamespace(pulsar.getAdvertisedAddress(), pulsar.getConfig());
+        NamespaceName heartbeatNamespaceV2 = NamespaceService.getHeartbeatNamespaceV2(pulsar.getAdvertisedAddress(), pulsar.getConfig());
         assertTrue(brokerService.isSystemTopic("persistent://" + heartbeatNamespaceV1.toString() + "/healthcheck"));
         assertTrue(brokerService.isSystemTopic(heartbeatNamespaceV2.toString() + "/healthcheck"));
     }
@@ -1609,6 +1580,84 @@ public class BrokerServiceTest extends BrokerTestBase {
         });
     }
 
+    // this test is disabled since it is flaky
+    @Test(enabled = false)
+    public void testBrokerStatsTopicLoadFailed() throws Exception {
+        admin.namespaces().createNamespace("prop/ns-test");
+
+        String persistentTopic = "persistent://prop/ns-test/topic1_" + UUID.randomUUID();
+        String nonPersistentTopic = "non-persistent://prop/ns-test/topic2_" + UUID.randomUUID();
+
+        BrokerService brokerService = pulsar.getBrokerService();
+        brokerService = Mockito.spy(brokerService);
+        // mock create persistent topic failed
+        Mockito
+                .doAnswer(invocation -> {
+                    CompletableFuture<ManagedLedgerConfig> f = new CompletableFuture<>();
+                    f.completeExceptionally(new RuntimeException("This is an exception"));
+                    return f;
+                })
+                .when(brokerService).getManagedLedgerConfig(Mockito.eq(TopicName.get(persistentTopic)));
+
+        // mock create non-persistent topic failed
+        Mockito
+                .doAnswer(inv -> {
+                    CompletableFuture<Void> f = new CompletableFuture<>();
+                    f.completeExceptionally(new RuntimeException("This is an exception"));
+                    return f;
+                })
+                .when(brokerService).checkTopicNsOwnership(Mockito.eq(nonPersistentTopic));
+
+
+        PulsarService pulsarService = pulsar;
+        Field field = PulsarService.class.getDeclaredField("brokerService");
+        field.setAccessible(true);
+        field.set(pulsarService, brokerService);
+
+        CompletableFuture<Producer<String>> producer = pulsarClient.newProducer(Schema.STRING)
+                .topic(persistentTopic)
+                .createAsync();
+        CompletableFuture<Producer<String>> producer1 = pulsarClient.newProducer(Schema.STRING)
+                .topic(nonPersistentTopic)
+                .createAsync();
+
+        producer.whenComplete((v, t) -> {
+            if (t == null) {
+                try {
+                    v.close();
+                } catch (PulsarClientException e) {
+                    // ignore
+                }
+            }
+        });
+        producer1.whenComplete((v, t) -> {
+            if (t == null) {
+                try {
+                    v.close();
+                } catch (PulsarClientException e) {
+                    // ignore
+                }
+            }
+        });
+
+        Awaitility.waitAtMost(2, TimeUnit.MINUTES).until(() -> {
+            String json = admin.brokerStats().getMetrics();
+            JsonArray metrics = new Gson().fromJson(json, JsonArray.class);
+            AtomicBoolean flag = new AtomicBoolean(false);
+
+            metrics.forEach(ele -> {
+                JsonObject obj = ((JsonObject) ele);
+                JsonObject metrics0 = (JsonObject) obj.get("metrics");
+                JsonPrimitive v = (JsonPrimitive) metrics0.get("brk_topic_load_failed_count");
+                if (null != v && v.getAsDouble() >= 2D) {
+                    flag.set(true);
+                }
+            });
+
+            return flag.get();
+        });
+    }
+
     @Test
     public void testIsSystemTopicAllowAutoTopicCreationAsync() throws Exception {
         BrokerService brokerService = pulsar.getBrokerService();
@@ -1617,379 +1666,4 @@ public class BrokerServiceTest extends BrokerTestBase {
         assertTrue(brokerService.isAllowAutoTopicCreationAsync(
                 "persistent://pulsar/system/my-system-topic").get());
     }
-
-    @Test
-    public void testDuplicateAcknowledgement() throws Exception {
-        final String ns = "prop/ns-test";
-
-        admin.namespaces().createNamespace(ns, 2);
-        final String topicName = "persistent://prop/ns-test/duplicated-acknowledgement-test";
-        @Cleanup
-        Producer<byte[]> producer = pulsarClient.newProducer()
-                .topic(topicName)
-                .create();
-        @Cleanup
-        Consumer<byte[]> consumer1 = pulsarClient.newConsumer()
-                .topic(topicName)
-                .subscriptionName("sub-1")
-                .acknowledgmentGroupTime(0, TimeUnit.SECONDS)
-                .subscriptionType(SubscriptionType.Shared)
-                .isAckReceiptEnabled(true)
-                .subscribe();
-        producer.send("1".getBytes(StandardCharsets.UTF_8));
-        Message<byte[]> message = consumer1.receive();
-        consumer1.acknowledge(message);
-        consumer1.acknowledge(message);
-        assertEquals(admin.topics().getStats(topicName).getSubscriptions()
-                .get("sub-1").getUnackedMessages(), 0);
-    }
-
-    @Test
-    public void testBlockedConsumerOnUnackedMsgs() throws Exception {
-        final String ns = "prop/ns-test";
-        admin.namespaces().createNamespace(ns, 2);
-        admin.namespaces().setMaxUnackedMessagesPerConsumer(ns, 1);
-
-        final String topicName = "persistent://prop/ns-test/testBlockedConsumerOnUnackedMsgs";
-        @Cleanup
-        Producer<byte[]> producer = pulsarClient.newProducer()
-                .topic(topicName)
-                .create();
-        @Cleanup
-        Consumer<byte[]> consumer = pulsarClient.newConsumer()
-                .topic(topicName)
-                .subscriptionName("sub-test")
-                .acknowledgmentGroupTime(0, TimeUnit.SECONDS)
-                .subscriptionType(SubscriptionType.Shared)
-                .isAckReceiptEnabled(true)
-                .receiverQueueSize(0)
-                .subscribe();
-
-        producer.send("1".getBytes(StandardCharsets.UTF_8));
-        producer.send("2".getBytes(StandardCharsets.UTF_8));
-
-        // 1. receive message
-        Message<byte[]> message = consumer.receive();
-        Thread.sleep(ASYNC_EVENT_COMPLETION_WAIT);
-
-        SubscriptionStats subscriptionStats = admin.topics().getStats(topicName).getSubscriptions().get("sub-test");
-        assertEquals(subscriptionStats.getUnackedMessages(), 1);
-        assertTrue(subscriptionStats.getConsumers().get(0).isBlockedConsumerOnUnackedMsgs());
-
-        // 2、ack this message
-        consumer.acknowledge(message);
-        Thread.sleep(ASYNC_EVENT_COMPLETION_WAIT);
-
-        subscriptionStats = admin.topics().getStats(topicName).getSubscriptions().get("sub-test");
-        assertEquals(subscriptionStats.getUnackedMessages(), 0);
-        assertFalse(subscriptionStats.getConsumers().get(0).isBlockedConsumerOnUnackedMsgs());
-    }
-
-    @Test
-    public void testUnsubscribeNonDurableSub() throws Exception {
-        final String ns = "prop/ns-test";
-        final String topic = ns + "/testUnsubscribeNonDurableSub";
-
-        admin.namespaces().createNamespace(ns, 2);
-        admin.topics().createPartitionedTopic(String.format("persistent://%s", topic), 1);
-
-        pulsarClient.newProducer(Schema.STRING).topic(topic).create().close();
-        @Cleanup
-        Consumer<String> consumer = pulsarClient
-                .newConsumer(Schema.STRING)
-                .topic(topic)
-                .subscriptionMode(SubscriptionMode.NonDurable)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscriptionName("sub1")
-                .subscriptionType(SubscriptionType.Shared)
-                .subscribe();
-        try {
-            consumer.unsubscribe();
-        } catch (Exception ex) {
-            fail("Unsubscribe failed");
-        }
-    }
-
-    @Test
-    public void testMetricsPersistentTopicLoadFails() throws Exception {
-        final String namespace = "prop/" + UUID.randomUUID().toString().replaceAll("-", "");
-        String topic = "persistent://" + namespace + "/topic1_" + UUID.randomUUID();
-        admin.namespaces().createNamespace(namespace);
-        admin.topics().createNonPartitionedTopic(topic);
-        admin.topics().unload(topic);
-
-        // Inject an error that makes the topic load fails.
-        AtomicBoolean failMarker = new AtomicBoolean(true);
-        mockZooKeeper.failConditional(KeeperException.Code.NODEEXISTS, (op, path) -> {
-            if (failMarker.get() && op.equals(MockZooKeeper.Op.SET) &&
-                    path.endsWith(TopicName.get(topic).getPersistenceNamingEncoding())) {
-                return true;
-            }
-            return false;
-        });
-
-        // Do test
-        CompletableFuture<Producer<byte[]>> producer = pulsarClient.newProducer().topic(topic).createAsync();
-        JerseyClient httpClient = JerseyClientBuilder.createClient();
-        Awaitility.await().until(() -> {
-            String response = httpClient.target(pulsar.getWebServiceAddress()).path("/metrics/")
-                    .request().get(String.class);
-            BufferedReader reader = new BufferedReader(new StringReader(response));
-            String line;
-            String metricsLine = null;
-            while ((line = reader.readLine()) != null) {
-                if (StringUtils.isBlank(line)) {
-                    continue;
-                }
-                if (line.startsWith("#")) {
-                    continue;
-                }
-                if (line.contains("topic_load_failed")) {
-                    metricsLine = line;
-                    break;
-                }
-            }
-            log.info("topic_load_failed: {}", metricsLine);
-            if (metricsLine == null) {
-                return false;
-            }
-            reader.close();
-            String[] parts = metricsLine.split(" ");
-            Double value = Double.valueOf(parts[parts.length - 1]);
-            return value >= 1D;
-        });
-
-        // Remove the injection.
-        failMarker.set(false);
-        // cleanup.
-        httpClient.close();
-        producer.join().close();
-        admin.topics().delete(topic);
-        admin.namespaces().deleteNamespace(namespace);
-    }
-
-    @Test
-    public void testMetricsNonPersistentTopicLoadFails() throws Exception {
-        final String namespace = "prop/" + UUID.randomUUID().toString().replaceAll("-", "");
-        String topic = "non-persistent://" + namespace + "/topic1_" + UUID.randomUUID();
-        admin.namespaces().createNamespace(namespace);
-
-        // Inject an error that makes the topic load fails.
-        // Since we did not set a topic factory name, the "topicFactory" variable is null, inject a mocked
-        // "topicFactory".
-        Field fieldTopicFactory = BrokerService.class.getDeclaredField("topicFactory");
-        fieldTopicFactory.setAccessible(true);
-        TopicFactory originalTopicFactory = (TopicFactory) fieldTopicFactory.get(pulsar.getBrokerService());
-        assertNull(originalTopicFactory);
-        TopicFactory mockedTopicFactory = mock(TopicFactory.class);
-        when(mockedTopicFactory.create(anyString(), any(), any(), any()))
-                .thenThrow(new RuntimeException("mocked error"));
-        fieldTopicFactory.set(pulsar.getBrokerService(), mockedTopicFactory);
-
-        // Do test.
-        CompletableFuture<Producer<byte[]>> producer = pulsarClient.newProducer().topic(topic).createAsync();
-        JerseyClient httpClient = JerseyClientBuilder.createClient();
-        Awaitility.await().until(() -> {
-            String response = httpClient.target(pulsar.getWebServiceAddress()).path("/metrics/")
-                    .request().get(String.class);
-            BufferedReader reader = new BufferedReader(new StringReader(response));
-            String line;
-            String metricsLine = null;
-            while ((line = reader.readLine()) != null) {
-                if (StringUtils.isBlank(line)) {
-                    continue;
-                }
-                if (line.startsWith("#")) {
-                    continue;
-                }
-                if (line.contains("topic_load_failed")) {
-                    metricsLine = line;
-                    break;
-                }
-            }
-            log.info("topic_load_failed: {}", metricsLine);
-            if (metricsLine == null) {
-                return false;
-            }
-            reader.close();
-            String[] parts = metricsLine.split(" ");
-            Double value = Double.valueOf(parts[parts.length - 1]);
-            return value >= 1D;
-        });
-
-        // Remove the injection.
-        fieldTopicFactory.set(pulsar.getBrokerService(), null);
-
-        // cleanup.
-        httpClient.close();
-        producer.join().close();
-        admin.topics().delete(topic);
-        admin.namespaces().deleteNamespace(namespace);
-    }
-
-
-    @Test
-    public void testOffloadConfShouldNotAppliedForSystemTopic() throws PulsarAdminException {
-        final String driver = "aws-s3";
-        final String region = "test-region";
-        final String bucket = "test-bucket";
-        final String role = "test-role";
-        final String roleSessionName = "test-role-session-name";
-        final String credentialId = "test-credential-id";
-        final String credentialSecret = "test-credential-secret";
-        final String endPoint = "test-endpoint";
-        final Integer maxBlockSizeInBytes = 5;
-        final Integer readBufferSizeInBytes = 2;
-        final Long offloadThresholdInBytes = 10L;
-        final Long offloadThresholdInSeconds = 1000L;
-        final Long offloadDeletionLagInMillis = 5L;
-
-        final OffloadPoliciesImpl offloadPolicies = OffloadPoliciesImpl.create(
-                driver,
-                region,
-                bucket,
-                endPoint,
-                role,
-                roleSessionName,
-                credentialId,
-                credentialSecret,
-                maxBlockSizeInBytes,
-                readBufferSizeInBytes,
-                offloadThresholdInBytes,
-                offloadThresholdInSeconds,
-                offloadDeletionLagInMillis,
-                OffloadedReadPriority.TIERED_STORAGE_FIRST
-        );
-
-        var fakeOffloader = new LedgerOffloader() {
-            @Override
-            public String getOffloadDriverName() {
-                return driver;
-            }
-
-            @Override
-            public CompletableFuture<Void> offload(ReadHandle ledger, UUID uid, Map<String, String> extraMetadata) {
-                return CompletableFuture.completedFuture(null);
-            }
-
-            @Override
-            public CompletableFuture<ReadHandle> readOffloaded(long ledgerId, UUID uid, Map<String, String> offloadDriverMetadata) {
-                return CompletableFuture.completedFuture(null);
-            }
-
-            @Override
-            public CompletableFuture<Void> deleteOffloaded(long ledgerId, UUID uid, Map<String, String> offloadDriverMetadata) {
-                return CompletableFuture.completedFuture(null);
-            }
-
-            @Override
-            public OffloadPoliciesImpl getOffloadPolicies() {
-                return offloadPolicies;
-            }
-
-            @Override
-            public void close() {
-            }
-        };
-
-        final BrokerService brokerService = pulsar.getBrokerService();
-        final String namespace = "prop/" + UUID.randomUUID();
-        admin.namespaces().createNamespace(namespace);
-        admin.namespaces().setOffloadPolicies(namespace, offloadPolicies);
-        Awaitility.await().untilAsserted(() -> {
-            Object policiesGot = admin.namespaces().getOffloadPolicies(namespace);
-            assertNotNull(policiesGot);
-        });
-
-        // Inject the cache to avoid real load off-loader jar
-        final Map<NamespaceName, LedgerOffloader> ledgerOffloaderMap = pulsar.getLedgerOffloaderMap();
-        ledgerOffloaderMap.put(NamespaceName.get(namespace), fakeOffloader);
-
-        // (1) test normal topic
-        final String normalTopic = "persistent://" + namespace + "/" + UUID.randomUUID();
-        var managedLedgerConfig = brokerService.getManagedLedgerConfig(TopicName.get(normalTopic)).join();
-
-        Assert.assertEquals(managedLedgerConfig.getLedgerOffloader(), fakeOffloader);
-
-        // (2) test system topic
-        for (String eventTopicName : SystemTopicNames.EVENTS_TOPIC_NAMES) {
-            boolean offloadPoliciesExists = false;
-            try {
-                Object policiesGot =
-                        admin.namespaces().getOffloadPolicies(TopicName.get(eventTopicName).getNamespace());
-                offloadPoliciesExists = policiesGot != null;
-            } catch (PulsarAdminException.NotFoundException notFoundException) {
-                offloadPoliciesExists = false;
-            }
-            var managedLedgerConfig2 = brokerService.getManagedLedgerConfig(TopicName.get(eventTopicName)).join();
-            if (offloadPoliciesExists) {
-                Assert.assertTrue(managedLedgerConfig2.getLedgerOffloader() instanceof NonAppendableLedgerOffloader);
-            } else {
-                Assert.assertEquals(managedLedgerConfig2.getLedgerOffloader(), NullLedgerOffloader.INSTANCE);
-            }
-        }
-    }
-
-    @Test
-    public void testTlsWithAuthParams() throws Exception {
-        final String topicName = "persistent://prop/ns-abc/newTopic";
-        final String subName = "newSub";
-        Authentication auth;
-
-        Set<String> providers = new HashSet<>();
-        providers.add("org.apache.pulsar.broker.authentication.AuthenticationProviderTls");
-
-        conf.setAuthenticationEnabled(true);
-        conf.setAuthenticationProviders(providers);
-        conf.setBrokerServicePortTls(Optional.of(0));
-        conf.setWebServicePortTls(Optional.of(0));
-        conf.setTlsCertificateFilePath(BROKER_CERT_FILE_PATH);
-        conf.setTlsKeyFilePath(BROKER_KEY_FILE_PATH);
-        conf.setTlsAllowInsecureConnection(false);
-        conf.setTlsTrustCertsFilePath(CA_CERT_FILE_PATH);
-        conf.setNumExecutorThreadPoolSize(5);
-        restartBroker();
-
-        String authParam = String.format("tlsCertFile:%s,tlsKeyFile:%s", getTlsFileForClient("admin.cert"),
-                getTlsFileForClient("admin.key-pk8"));
-        String authClassName = "org.apache.pulsar.client.impl.auth.AuthenticationTls";
-        ClientConfigurationData conf = new ClientConfigurationData();
-        conf.setServiceUrl(brokerUrlTls.toString());
-        conf.setAuthParams(authParam);
-        conf.setAuthPluginClassName(authClassName);
-        conf.setTlsAllowInsecureConnection(true);
-
-        PulsarClient pulsarClient = null;
-        try {
-            pulsarClient = (new ClientBuilderImpl(conf)).build();
-
-            @Cleanup
-            Consumer<byte[]> consumer = pulsarClient.newConsumer().topic(topicName).subscriptionName(subName)
-                    .subscribe();
-        } catch (Exception e) {
-            fail("should not fail");
-        } finally {
-            pulsarClient.close();
-        }
-    }
-
-    @Test
-    public void testPulsarMetadataEventSyncProducerCreation() throws Exception {
-        final String topicName = "persistent://prop/usw/my-ns/syncTopic";
-        pulsar.getConfiguration().setMetadataSyncEventTopic(topicName);
-        PulsarMetadataEventSynchronizer sync = new PulsarMetadataEventSynchronizer(pulsar, topicName);
-        // set invalid client for retry
-        @Cleanup
-        PulsarClientImpl client = (PulsarClientImpl) PulsarClient.builder().serviceUrl("http://invalidhost:8080")
-                .operationTimeout(1000, TimeUnit.MILLISECONDS).build();
-        sync.client = client;
-        sync.STATE_UPDATER.set(sync, State.Starting_Producer);
-        sync.startProducer();
-        assertNull(sync.getProducer());
-        // update valid client which will set the producer
-        sync.client = (PulsarClientImpl) pulsarClient;
-        retryStrategically((test) -> sync.getProducer() != null, 1000, 10);
-        assertNotNull(sync.getProducer());
-    }
 }
-
