@@ -18,9 +18,6 @@
  */
 package org.apache.pulsar.broker.service.persistent;
 
-import com.google.common.collect.Multimap;
-import com.google.common.collect.Sets;
-import static org.apache.pulsar.broker.stats.prometheus.PrometheusMetricsClient.parseMetrics;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.Mockito.doAnswer;
@@ -36,6 +33,8 @@ import static org.testng.Assert.assertNotNull;
 import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
+import com.google.common.collect.Multimap;
+import com.google.common.collect.Sets;
 import java.io.ByteArrayOutputStream;
 import java.lang.reflect.Field;
 import java.nio.charset.StandardCharsets;
@@ -46,10 +45,7 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Optional;
 import java.util.Set;
-import java.util.Map;
-import java.util.Objects;
 import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CountDownLatch;
@@ -57,23 +53,15 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.function.Supplier;
-import java.util.stream.Collectors;
 import lombok.Cleanup;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.client.LedgerHandle;
-import org.apache.bookkeeper.mledger.AsyncCallbacks;
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedLedger;
-import org.apache.bookkeeper.mledger.impl.ManagedCursorContainer;
-import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
-import org.apache.bookkeeper.mledger.impl.ManagedLedgerImpl;
-import org.apache.bookkeeper.mledger.impl.PositionImpl;
-import org.apache.pulsar.PrometheusMetricsTestUtil;
 import org.apache.pulsar.broker.service.BrokerService;
 import org.apache.pulsar.broker.service.BrokerTestBase;
-import org.apache.pulsar.broker.service.Topic;
-import org.apache.pulsar.broker.service.TopicPoliciesService;
-import org.apache.pulsar.broker.stats.prometheus.PrometheusMetricsClient.Metric;
+import org.apache.pulsar.broker.stats.PrometheusMetricsTest;
+import org.apache.pulsar.broker.stats.prometheus.PrometheusMetricsGenerator;
 import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.Message;
@@ -84,25 +72,20 @@ import org.apache.pulsar.client.api.Producer;
 import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Schema;
-import org.apache.pulsar.client.api.SubscriptionInitialPosition;
-import org.apache.pulsar.client.api.SubscriptionMode;
 import org.apache.pulsar.client.api.SubscriptionType;
 import org.apache.pulsar.common.naming.NamespaceBundle;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.Policies;
-import org.apache.pulsar.common.policies.data.RetentionPolicies;
 import org.apache.pulsar.common.policies.data.TenantInfo;
-import org.apache.pulsar.common.policies.data.TopicPolicies;
 import org.apache.pulsar.common.policies.data.TopicStats;
 import org.awaitility.Awaitility;
-import org.junit.Assert;
+import org.testng.Assert;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
-
 @Slf4j
 @Test(groups = "broker")
 public class PersistentTopicTest extends BrokerTestBase {
@@ -119,11 +102,6 @@ public class PersistentTopicTest extends BrokerTestBase {
         super.internalCleanup();
     }
 
-    @Override protected void doInitConf() throws Exception {
-        super.doInitConf();
-        this.conf.setManagedLedgerCursorBackloggedThreshold(10);
-    }
-
     /**
      * Test validates that broker cleans up topic which failed to unload while bundle unloading.
      *
@@ -217,13 +195,6 @@ public class PersistentTopicTest extends BrokerTestBase {
         assertNotNull(msg);
         msg = consumer2.receive(5, TimeUnit.SECONDS);
         assertNotNull(msg);
-
-        org.apache.pulsar.broker.service.Consumer sharedConsumer = sharedDispatcher.getConsumers().get(0);
-        Field blockField = org.apache.pulsar.broker.service.Consumer.class.getDeclaredField("blockedConsumerOnUnackedMsgs");
-        blockField.setAccessible(true);
-        blockField.set(sharedConsumer, true);
-        producer.newMessage().value("test").eventTime(5).send();
-        assertFalse(sharedSub.checkAndUnblockIfStuck());
     }
 
     @Test
@@ -320,8 +291,7 @@ public class PersistentTopicTest extends BrokerTestBase {
 
         assertFalse(pulsar.getBrokerService().getTopics().containsKey(topicName));
         pulsar.getBrokerService().getTopicIfExists(topicName).get();
-        // The map topics should only contain partitions, does not contain partitioned topic.
-        assertFalse(pulsar.getBrokerService().getTopics().containsKey(topicName));
+        assertTrue(pulsar.getBrokerService().getTopics().containsKey(topicName));
 
         // ref of partitioned-topic name should be empty
         assertFalse(pulsar.getBrokerService().getTopicReference(topicName).isPresent());
@@ -334,83 +304,6 @@ public class PersistentTopicTest extends BrokerTestBase {
         }
     }
 
-    @DataProvider(name = "closeWithoutWaitingClientDisconnectInFirstBatch")
-    public Object[][] closeWithoutWaitingClientDisconnectInFirstBatch() {
-        return new Object[][]{
-                new Object[] {true},
-                new Object[] {false},
-        };
-    }
-
-    @Test(dataProvider = "closeWithoutWaitingClientDisconnectInFirstBatch")
-    public void testConcurrentClose(boolean closeWithoutWaitingClientDisconnectInFirstBatch) throws Exception {
-        final String topicName = "persistent://prop/ns/concurrentClose";
-        final String ns = "prop/ns";
-        admin.namespaces().createNamespace(ns, 1);
-        admin.topics().createNonPartitionedTopic(topicName);
-        final Topic topic = pulsar.getBrokerService().getTopicIfExists(topicName).get().get();
-        List<CompletableFuture<Void>> futureList =
-                make2ConcurrentBatchesOfClose(topic, 10, closeWithoutWaitingClientDisconnectInFirstBatch);
-        Map<Integer, List<CompletableFuture<Void>>> futureMap =
-                futureList.stream().collect(Collectors.groupingBy(Objects::hashCode));
-        /**
-         * The first call: get the return value of "topic.close".
-         * The other 19 calls: get the cached value which related {@link PersistentTopic#closeFutures}.
-         */
-        assertTrue(futureMap.size() <= 3);
-        for (List list : futureMap.values()){
-            if (list.size() == 1){
-                // This is the first call, the future is the return value of `topic.close`.
-            } else {
-                // Two types future list: wait client close or not.
-                assertTrue(list.size() >= 9 && list.size() <= 10);
-            }
-        }
-    }
-
-    private List<CompletableFuture<Void>> make2ConcurrentBatchesOfClose(Topic topic, int tryTimes,
-                                                              boolean closeWithoutWaitingClientDisconnectInFirstBatch){
-        final List<CompletableFuture<Void>> futureList = Collections.synchronizedList(new ArrayList<>());
-        final List<Thread> taskList = new ArrayList<>();
-        CountDownLatch allTaskBeginLatch = new CountDownLatch(1);
-        // Call a batch of close.
-        for (int i = 0; i < tryTimes; i++) {
-            Thread thread = new Thread(() -> {
-                try {
-                    allTaskBeginLatch.await(5, TimeUnit.SECONDS);
-                } catch (InterruptedException e) {
-                    throw new RuntimeException(e);
-                }
-                futureList.add(topic.close(closeWithoutWaitingClientDisconnectInFirstBatch));
-            });
-            thread.start();
-            taskList.add(thread);
-        }
-        // Call another batch of close.
-        for (int i = 0; i < tryTimes; i++) {
-            Thread thread = new Thread(() -> {
-                try {
-                    allTaskBeginLatch.await(5, TimeUnit.SECONDS);
-                } catch (InterruptedException e) {
-                    throw new RuntimeException(e);
-                }
-                futureList.add(topic.close(!closeWithoutWaitingClientDisconnectInFirstBatch));
-            });
-            thread.start();
-            taskList.add(thread);
-        }
-        // Wait close task executed.
-        allTaskBeginLatch.countDown();
-        Awaitility.await().atMost(5, TimeUnit.SECONDS).until(()->{
-            for (Thread thread : taskList){
-                if (thread.isAlive()){
-                    return false;
-                }
-            }
-            return true;
-        });
-        return futureList;
-    }
 
     @DataProvider(name = "topicAndMetricsLevel")
     public Object[][] indexPatternTestData() {
@@ -420,6 +313,7 @@ public class PersistentTopicTest extends BrokerTestBase {
         };
     }
 
+
     @Test(dataProvider = "topicAndMetricsLevel")
     public void testDelayedDeliveryTrackerMemoryUsageMetric(String topic, boolean exposeTopicLevelMetrics) throws Exception {
         PulsarClient client = pulsar.getClient();
@@ -431,11 +325,10 @@ public class PersistentTopicTest extends BrokerTestBase {
 
         @Cleanup
         Producer<String> producer = client.newProducer(Schema.STRING).topic(topic).enableBatching(false).create();
-        String subName = "test_sub";
         @Cleanup
         Consumer<String> consumer = client.newConsumer(Schema.STRING)
                 .topic(topic)
-                .subscriptionName(subName)
+                .subscriptionName("test_sub")
                 .subscriptionType(SubscriptionType.Shared)
                 .messageListener((MessageListener<String>) (consumer1, msg) -> {
                     try {
@@ -456,40 +349,28 @@ public class PersistentTopicTest extends BrokerTestBase {
 
         latch.await(10, TimeUnit.SECONDS);
         ByteArrayOutputStream output = new ByteArrayOutputStream();
-        PrometheusMetricsTestUtil.generate(pulsar, exposeTopicLevelMetrics, true, true, output);
+        PrometheusMetricsGenerator.generate(pulsar, exposeTopicLevelMetrics, true, true, output);
         String metricsStr = output.toString(StandardCharsets.UTF_8);
 
-        Multimap<String, Metric> metricsMap = parseMetrics(metricsStr);
-        Collection<Metric> metrics = metricsMap.get("pulsar_delayed_message_index_size_bytes");
-        Collection<Metric> subMetrics = metricsMap.get("pulsar_subscription_delayed_message_index_size_bytes");
-        assertFalse(metrics.isEmpty());
-        if (exposeTopicLevelMetrics) {
-            assertFalse(subMetrics.isEmpty());
-        } else {
-            assertTrue(subMetrics.isEmpty());
-        }
+        Multimap<String, PrometheusMetricsTest.Metric> metricsMap = PrometheusMetricsTest.parseMetrics(metricsStr);
+        Collection<PrometheusMetricsTest.Metric> metrics = metricsMap.get("pulsar_delayed_message_index_size_bytes");
+        Assert.assertTrue(metrics.size() > 0);
 
         int topicLevelNum = 0;
         int namespaceLevelNum = 0;
         int subscriptionLevelNum = 0;
-        for (Metric metric : metrics) {
+        for (PrometheusMetricsTest.Metric metric : metrics) {
             if (exposeTopicLevelMetrics && metric.tags.get("topic").equals(topic)) {
                 Assert.assertTrue(metric.value > 0);
                 topicLevelNum++;
+                if ("test_sub".equals(metric.tags.get("subscription"))) {
+                    subscriptionLevelNum++;
+                }
             } else if (!exposeTopicLevelMetrics && metric.tags.get("namespace").equals(namespace)) {
                 Assert.assertTrue(metric.value > 0);
                 namespaceLevelNum++;
             }
         }
-        if (exposeTopicLevelMetrics) {
-            for (Metric metric : subMetrics) {
-                if (metric.tags.get("topic").equals(topic) &&
-                        subName.equals(metric.tags.get("subscription"))) {
-                    Assert.assertTrue(metric.value > 0);
-                    subscriptionLevelNum++;
-                }
-            }
-        }
 
         if (exposeTopicLevelMetrics) {
             Assert.assertTrue(topicLevelNum > 0);
@@ -572,7 +453,8 @@ public class PersistentTopicTest extends BrokerTestBase {
                     .topic(partition.toString())
                     .create();
             fail("unexpected behaviour");
-        } catch (PulsarClientException.NotAllowedException ex) {
+        } catch (PulsarClientException.TopicDoesNotExistException ignored) {
+
         }
         Assert.assertEquals(admin.topics().getPartitionedTopicMetadata(topicName).partitions, 4);
     }
@@ -721,171 +603,4 @@ public class PersistentTopicTest extends BrokerTestBase {
             return !topic.getManagedLedger().getCursors().iterator().hasNext();
         });
     }
-
-    @Test
-    public void testCheckPersistencePolicies() throws Exception {
-        final String myNamespace = "prop/ns";
-        admin.namespaces().createNamespace(myNamespace, Sets.newHashSet("test"));
-        final String topic = "persistent://" + myNamespace + "/testConfig" + UUID.randomUUID();
-        conf.setForceDeleteNamespaceAllowed(true);
-        pulsarClient.newProducer().topic(topic).create().close();
-        RetentionPolicies retentionPolicies = new RetentionPolicies(1, 1);
-        PersistentTopic persistentTopic = spy((PersistentTopic) pulsar.getBrokerService().getTopicIfExists(topic).get().get());
-        TopicPoliciesService policiesService = spy(pulsar.getTopicPoliciesService());
-        doReturn(policiesService).when(pulsar).getTopicPoliciesService();
-        TopicPolicies policies = new TopicPolicies();
-        policies.setRetentionPolicies(retentionPolicies);
-        doReturn(CompletableFuture.completedFuture(Optional.of(policies))).when(policiesService)
-                .getTopicPoliciesAsync(TopicName.get(topic), false);
-        persistentTopic.onUpdate(policies);
-        verify(persistentTopic, times(1)).checkPersistencePolicies();
-        Awaitility.await().untilAsserted(() -> {
-            assertEquals(persistentTopic.getManagedLedger().getConfig().getRetentionSizeInMB(), 1L);
-            assertEquals(persistentTopic.getManagedLedger().getConfig().getRetentionTimeMillis(), TimeUnit.MINUTES.toMillis(1));
-        });
-        // throw exception
-        doReturn(CompletableFuture.failedFuture(new RuntimeException())).when(persistentTopic).checkPersistencePolicies();
-        policies.setRetentionPolicies(new RetentionPolicies(2, 2));
-        persistentTopic.onUpdate(policies);
-        assertEquals(persistentTopic.getManagedLedger().getConfig().getRetentionSizeInMB(), 1L);
-        assertEquals(persistentTopic.getManagedLedger().getConfig().getRetentionTimeMillis(), TimeUnit.MINUTES.toMillis(1));
-    }
-
-    @Test
-    public void testDynamicConfigurationAutoSkipNonRecoverableData() throws Exception {
-        pulsar.getConfiguration().setAutoSkipNonRecoverableData(false);
-        final String topicName = "persistent://prop/ns-abc/testAutoSkipNonRecoverableData";
-        final String subName = "test_sub";
-
-        Consumer<byte[]> subscribe = pulsarClient.newConsumer().topic(topicName).subscriptionName(subName).subscribe();
-        PersistentTopic persistentTopic =
-                (PersistentTopic) pulsar.getBrokerService().getTopic(topicName, false).join().get();
-        PersistentSubscription subscription = persistentTopic.getSubscription(subName);
-
-        assertFalse(persistentTopic.ledger.getConfig().isAutoSkipNonRecoverableData());
-        assertFalse(subscription.getExpiryMonitor().isAutoSkipNonRecoverableData());
-
-        String key = "autoSkipNonRecoverableData";
-        admin.brokers().updateDynamicConfiguration(key, "true");
-        Awaitility.await()
-                .untilAsserted(() -> assertEquals(admin.brokers().getAllDynamicConfigurations().get(key), "true"));
-
-        assertTrue(persistentTopic.ledger.getConfig().isAutoSkipNonRecoverableData());
-        assertTrue(subscription.getExpiryMonitor().isAutoSkipNonRecoverableData());
-
-        subscribe.close();
-        admin.topics().delete(topicName);
-    }
-
-    @Test
-    public void testCursorGetConfigAfterTopicPoliciesChanged() throws Exception {
-        final String topicName = "persistent://prop/ns-abc/" + UUID.randomUUID();
-        final String subName = "test_sub";
-
-        @Cleanup
-        Consumer<byte[]> subscribe = pulsarClient.newConsumer().topic(topicName).subscriptionName(subName).subscribe();
-        PersistentTopic persistentTopic =
-                (PersistentTopic) pulsar.getBrokerService().getTopic(topicName, false).join().get();
-        PersistentSubscription subscription = persistentTopic.getSubscription(subName);
-
-        int maxConsumers = 100;
-        admin.topicPolicies().setMaxConsumers(topicName, 100);
-        Awaitility.await().untilAsserted(() -> {
-            assertEquals(admin.topicPolicies().getMaxConsumers(topicName, false), maxConsumers);
-        });
-
-        ManagedCursorImpl cursor = (ManagedCursorImpl) subscription.getCursor();
-        assertEquals(cursor.getConfig(), persistentTopic.getManagedLedger().getConfig());
-
-        subscribe.close();
-        admin.topics().delete(topicName);
-    }
-
-    @Test
-    public void testAddWaitingCursorsForNonDurable() throws Exception {
-        final String ns = "prop/ns-test";
-        admin.namespaces().createNamespace(ns, 2);
-        final String topicName = "persistent://prop/ns-test/testAddWaitingCursors";
-        admin.topics().createNonPartitionedTopic(topicName);
-        final Optional<Topic> topic = pulsar.getBrokerService().getTopic(topicName, false).join();
-        assertNotNull(topic.get());
-        PersistentTopic persistentTopic = (PersistentTopic) topic.get();
-        ManagedLedgerImpl ledger = (ManagedLedgerImpl)persistentTopic.getManagedLedger();
-        final ManagedCursor spyCursor= spy(ledger.newNonDurableCursor(PositionImpl.LATEST, "sub-2"));
-        doAnswer((invocation) -> {
-            Thread.sleep(5_000);
-            invocation.callRealMethod();
-            return null;
-        }).when(spyCursor).asyncReadEntriesOrWait(any(int.class), any(long.class),
-                any(AsyncCallbacks.ReadEntriesCallback.class), any(Object.class), any(PositionImpl.class));
-        Field cursorField = ManagedLedgerImpl.class.getDeclaredField("cursors");
-        cursorField.setAccessible(true);
-        ManagedCursorContainer container = (ManagedCursorContainer) cursorField.get(ledger);
-        container.removeCursor("sub-2");
-        container.add(spyCursor, null);
-        final Consumer<String> consumer = pulsarClient.newConsumer(Schema.STRING).topic(topicName)
-                .subscriptionMode(SubscriptionMode.NonDurable)
-                .subscriptionType(SubscriptionType.Exclusive)
-                .subscriptionName("sub-2").subscribe();
-        final Producer<String> producer = pulsarClient.newProducer(Schema.STRING).topic(topicName).create();
-        producer.send("test");
-        producer.close();
-        final Message<String> receive = consumer.receive();
-        assertEquals("test", receive.getValue());
-        consumer.close();
-        Awaitility.await()
-                .pollDelay(5, TimeUnit.SECONDS)
-                .pollInterval(1, TimeUnit.SECONDS)
-                .untilAsserted(() -> {
-                    assertEquals(ledger.getWaitingCursorsCount(), 0);
-        });
-    }
-
-    @Test
-    public void testAddWaitingCursorsForNonDurable2() throws Exception {
-        final String ns = "prop/ns-test";
-        admin.namespaces().createNamespace(ns, 2);
-        final String topicName = "persistent://prop/ns-test/testAddWaitingCursors2";
-        admin.topics().createNonPartitionedTopic(topicName);
-        pulsarClient.newConsumer(Schema.STRING).topic(topicName)
-                .subscriptionMode(SubscriptionMode.Durable)
-                .subscriptionType(SubscriptionType.Shared)
-                .subscriptionName("sub-1").subscribe().close();
-        @Cleanup
-        final Producer<String> producer = pulsarClient.newProducer(Schema.STRING).enableBatching(false).topic(topicName).create();
-        for (int i = 0; i < 100; i ++) {
-            producer.sendAsync("test-" + i);
-        }
-        @Cleanup
-        final Consumer<String> consumer = pulsarClient.newConsumer(Schema.STRING).topic(topicName)
-                .subscriptionMode(SubscriptionMode.NonDurable)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscriptionType(SubscriptionType.Exclusive)
-                .subscriptionName("sub-2").subscribe();
-        int count = 0;
-        while(true) {
-            final Message<String> msg = consumer.receive(3, TimeUnit.SECONDS);
-            if (msg != null) {
-                consumer.acknowledge(msg);
-                count++;
-            } else {
-                break;
-            }
-        }
-        Assert.assertEquals(count, 100);
-        Thread.sleep(3_000);
-        for (int i = 0; i < 100; i ++) {
-            producer.sendAsync("test-" + i);
-        }
-        while(true) {
-            final Message<String> msg = consumer.receive(5, TimeUnit.SECONDS);
-            if (msg != null) {
-                consumer.acknowledge(msg);
-                count++;
-            } else {
-                break;
-            }
-        }
-        Assert.assertEquals(count, 200);
-    }
 }
