@@ -19,10 +19,12 @@
 
 package org.apache.pulsar.broker.testcontext;
 
+import com.google.common.util.concurrent.MoreExecutors;
 import io.netty.channel.EventLoopGroup;
 import java.io.IOException;
 import java.nio.charset.StandardCharsets;
 import java.time.Duration;
+import java.util.ArrayList;
 import java.util.List;
 import java.util.Optional;
 import java.util.function.Consumer;
@@ -31,7 +33,6 @@ import lombok.AccessLevel;
 import lombok.Builder;
 import lombok.Getter;
 import lombok.Singular;
-import lombok.SneakyThrows;
 import lombok.ToString;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.client.BookKeeper;
@@ -53,9 +54,9 @@ import org.apache.pulsar.broker.service.BrokerService;
 import org.apache.pulsar.broker.service.ServerCnx;
 import org.apache.pulsar.broker.storage.ManagedLedgerStorage;
 import org.apache.pulsar.common.util.GracefulExecutorServicesShutdown;
-import org.apache.pulsar.common.util.PortManager;
+import org.apache.pulsar.compaction.CompactionServiceFactory;
 import org.apache.pulsar.compaction.Compactor;
-import org.apache.pulsar.metadata.TestZKServer;
+import org.apache.pulsar.compaction.PulsarCompactionServiceFactory;
 import org.apache.pulsar.metadata.api.MetadataStore;
 import org.apache.pulsar.metadata.api.MetadataStoreConfig;
 import org.apache.pulsar.metadata.api.MetadataStoreException;
@@ -63,11 +64,9 @@ import org.apache.pulsar.metadata.api.extended.MetadataStoreExtended;
 import org.apache.pulsar.metadata.impl.MetadataStoreFactoryImpl;
 import org.apache.pulsar.metadata.impl.ZKMetadataStore;
 import org.apache.zookeeper.CreateMode;
-import org.apache.zookeeper.KeeperException;
 import org.apache.zookeeper.MockZooKeeper;
 import org.apache.zookeeper.MockZooKeeperSession;
-import org.apache.zookeeper.ZooDefs;
-import org.apache.zookeeper.ZooKeeper;
+import org.apache.zookeeper.data.ACL;
 import org.jetbrains.annotations.NotNull;
 import org.mockito.Mockito;
 import org.mockito.internal.util.MockUtil;
@@ -86,7 +85,7 @@ import org.mockito.internal.util.MockUtil;
  * There are few motivations for PulsarTestContext:
  * <ul>
  * <li>It reduces the reliance on Mockito for hooking into the PulsarService for injecting mocks or customizing the behavior of some
- * collaborators. Mockito is not thread-safe and some mocking operations get corrupted. Some examples of the issuess: https://github.com/apache/pulsar/issues/13620, https://github.com/apache/pulsar/issues/16444 and https://github.com/apache/pulsar/issues/16427.</li>
+ * collaborators. Mockito is not thread-safe and some mocking operations get corrupted. Some examples of the issues: https://github.com/apache/pulsar/issues/13620, https://github.com/apache/pulsar/issues/16444 and https://github.com/apache/pulsar/issues/16427.</li>
  * <li>Since the Mockito issue causes test flakiness, this change will improve reliability.</li>
  * <li>It makes it possible to use composition over inheritance in test classes. This can help reduce the dependency on
  * deep test base cases hierarchies.</li>
@@ -138,6 +137,8 @@ public class PulsarTestContext implements AutoCloseable {
 
     private final Compactor compactor;
 
+    private final CompactionServiceFactory compactionServiceFactory;
+
     private final BrokerService brokerService;
 
     @Getter(AccessLevel.NONE)
@@ -152,16 +153,10 @@ public class PulsarTestContext implements AutoCloseable {
 
     private final MockZooKeeper mockZooKeeperGlobal;
 
-    private final TestZKServer testZKServer;
-
-    private final TestZKServer testZKServerGlobal;
-
     private final SpyConfig spyConfig;
 
     private final boolean startable;
 
-    private final boolean preallocatePorts;
-
 
     public ManagedLedgerFactory getManagedLedgerFactory() {
         return managedLedgerClientFactory.getManagedLedgerFactory();
@@ -198,10 +193,6 @@ public class PulsarTestContext implements AutoCloseable {
      * @throws Exception if there is an error closing the resources
      */
     public void close() throws Exception {
-        callCloseables(closeables);
-    }
-
-    private static void callCloseables(List<AutoCloseable> closeables) {
         for (int i = closeables.size() - 1; i >= 0; i--) {
             try {
                 closeables.get(i).close();
@@ -222,18 +213,6 @@ public class PulsarTestContext implements AutoCloseable {
                 getPulsarService());
     }
 
-    private enum WithMockZooKeeperOrTestZKServer {
-        MOCKZOOKEEPER, MOCKZOOKEEPER_SEPARATE_GLOBAL, TEST_ZK_SERVER, TEST_ZK_SERVER_SEPARATE_GLOBAL;
-
-        boolean isMockZooKeeper() {
-            return this == MOCKZOOKEEPER || this == MOCKZOOKEEPER_SEPARATE_GLOBAL;
-        }
-
-        boolean isTestZKServer() {
-            return this == TEST_ZK_SERVER || this == TEST_ZK_SERVER_SEPARATE_GLOBAL;
-        }
-    }
-
     /**
      * A builder for a PulsarTestContext.
      *
@@ -245,12 +224,8 @@ public class PulsarTestContext implements AutoCloseable {
         protected SpyConfig.Builder spyConfigBuilder = SpyConfig.builder(SpyConfig.SpyType.NONE);
         protected Consumer<PulsarService> pulsarServiceCustomizer;
         protected ServiceConfiguration svcConfig = initializeConfig();
-        protected Consumer<ServiceConfiguration> configOverrideCustomizer;
-
-        protected boolean configOverrideCalled = false;
+        protected Consumer<ServiceConfiguration> configOverrideCustomizer = this::defaultOverrideServiceConfiguration;
         protected Function<BrokerService, BrokerService> brokerServiceCustomizer = Function.identity();
-        protected WithMockZooKeeperOrTestZKServer withMockZooKeeperOrTestZKServer;
-        protected PulsarTestContext otherContextToClose;
 
         /**
          * Initialize the ServiceConfiguration with default values.
@@ -374,7 +349,6 @@ public class PulsarTestContext implements AutoCloseable {
          */
         public Builder configOverride(Consumer<ServiceConfiguration> configOverrideCustomizer) {
             this.configOverrideCustomizer = configOverrideCustomizer;
-            this.configOverrideCalled = true;
             return this;
         }
 
@@ -397,17 +371,10 @@ public class PulsarTestContext implements AutoCloseable {
         public Builder reuseMockBookkeeperAndMetadataStores(PulsarTestContext otherContext) {
             bookKeeperClient(otherContext.getBookKeeperClient());
             if (otherContext.getMockZooKeeper() != null) {
-                withMockZooKeeperOrTestZKServer = null;
                 mockZooKeeper(otherContext.getMockZooKeeper());
                 if (otherContext.getMockZooKeeperGlobal() != null) {
                     mockZooKeeperGlobal(otherContext.getMockZooKeeperGlobal());
                 }
-            } else if (otherContext.getTestZKServer() != null) {
-                withMockZooKeeperOrTestZKServer = null;
-                testZKServer(otherContext.getTestZKServer());
-                if (otherContext.getTestZKServerGlobal() != null) {
-                    testZKServerGlobal(otherContext.getTestZKServerGlobal());
-                }
             } else {
                 localMetadataStore(NonClosingProxyHandler.createNonClosingProxy(otherContext.getLocalMetadataStore(),
                         MetadataStoreExtended.class
@@ -434,15 +401,7 @@ public class PulsarTestContext implements AutoCloseable {
          * The other PulsarTestContext will be closed when this one is closed.
          */
         public Builder chainClosing(PulsarTestContext otherContext) {
-            otherContextToClose = otherContext;
-            return this;
-        }
-
-        /**
-         * Registers a closeable to close as the last one by prepending it to the closeables list.
-         */
-        public Builder prependCloseable(AutoCloseable closeable) {
-            closeables.add(0, closeable);
+            registerCloseable(otherContext);
             return this;
         }
 
@@ -463,38 +422,29 @@ public class PulsarTestContext implements AutoCloseable {
          * @return the builder
          */
         public Builder withMockZookeeper(boolean useSeparateGlobalZk) {
-            if (useSeparateGlobalZk) {
-                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.MOCKZOOKEEPER_SEPARATE_GLOBAL;
-            } else {
-                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.MOCKZOOKEEPER;
+            try {
+                mockZooKeeper(createMockZooKeeper());
+                if (useSeparateGlobalZk) {
+                    mockZooKeeperGlobal(createMockZooKeeper());
+                }
+            } catch (Exception e) {
+                throw new RuntimeException(e);
             }
             return this;
         }
 
+        private MockZooKeeper createMockZooKeeper() throws Exception {
+            MockZooKeeper zk = MockZooKeeper.newInstance(MoreExecutors.newDirectExecutorService());
+            List<ACL> dummyAclList = new ArrayList<>(0);
 
-        /**
-         * Configure this PulsarTestContext to use a test ZooKeeper instance which is
-         * shared for both the local and configuration metadata stores.
-         *
-         * @return the builder
-         */
-        public Builder withTestZookeeper() {
-            return withTestZookeeper(false);
-        }
+            ZkUtils.createFullPathOptimistic(zk, "/ledgers/available/192.168.1.1:" + 5000,
+                    "".getBytes(StandardCharsets.UTF_8), dummyAclList, CreateMode.PERSISTENT);
 
-        /**
-         * Configure this PulsarTestContext to use a test ZooKeeper instance.
-         *
-         * @param useSeparateGlobalZk if true, the global (configuration) zookeeper will be a separate instance
-         * @return the builder
-         */
-        public Builder withTestZookeeper(boolean useSeparateGlobalZk) {
-            if (useSeparateGlobalZk) {
-                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.TEST_ZK_SERVER_SEPARATE_GLOBAL;
-            } else {
-                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.TEST_ZK_SERVER;
-            }
-            return this;
+            zk.create("/ledgers/LAYOUT", "1\nflat:1".getBytes(StandardCharsets.UTF_8), dummyAclList,
+                    CreateMode.PERSISTENT);
+
+            registerCloseable(zk::shutdown);
+            return zk;
         }
 
         /**
@@ -575,16 +525,9 @@ public class PulsarTestContext implements AutoCloseable {
             if (super.config == null) {
                 config(svcConfig);
             }
-            handlePreallocatePorts(super.config);
-            if (configOverrideCustomizer != null || !configOverrideCalled) {
-                // call defaultOverrideServiceConfiguration if configOverrideCustomizer
-                // isn't explicitly set to null with `.configOverride(null)` call
-                defaultOverrideServiceConfiguration(super.config);
-            }
             if (configOverrideCustomizer != null) {
                 configOverrideCustomizer.accept(super.config);
             }
-            createWithMockZooKeeperOrTestZKServerInstances();
             if (super.brokerInterceptor != null) {
                 super.config.setDisableBrokerInterceptors(false);
             }
@@ -597,116 +540,13 @@ public class PulsarTestContext implements AutoCloseable {
                 try {
                     super.pulsarService.start();
                 } catch (Exception e) {
-                    callCloseables(super.closeables);
-                    super.closeables.clear();
                     throw new RuntimeException(e);
                 }
             }
-            if (otherContextToClose != null) {
-                prependCloseable(otherContextToClose);
-            }
             brokerService(super.pulsarService.getBrokerService());
             return super.build();
         }
 
-        void createWithMockZooKeeperOrTestZKServerInstances() {
-            if (withMockZooKeeperOrTestZKServer == null) {
-                return;
-            }
-            int sessionTimeout = (int) super.config.getMetadataStoreSessionTimeoutMillis();
-            try {
-                if (withMockZooKeeperOrTestZKServer.isMockZooKeeper()) {
-                    if (super.mockZooKeeper == null) {
-                        mockZooKeeper(createMockZooKeeper(sessionTimeout));
-                    } else {
-                        log.warn("Skipping creating mockZooKeeper, already set");
-                    }
-                    if (withMockZooKeeperOrTestZKServer
-                            == WithMockZooKeeperOrTestZKServer.MOCKZOOKEEPER_SEPARATE_GLOBAL) {
-                        if (super.mockZooKeeperGlobal == null) {
-                            mockZooKeeperGlobal(createMockZooKeeper(sessionTimeout));
-                        } else {
-                            log.warn("Skipping creating mockZooKeeperGlobal, already set");
-                        }
-                    }
-                } else if (withMockZooKeeperOrTestZKServer.isTestZKServer()) {
-                    if (super.testZKServer == null) {
-                        testZKServer(createTestZookeeper(sessionTimeout));
-                    } else {
-                        log.warn("Skipping creating testZKServer, already set");
-                    }
-                    if (withMockZooKeeperOrTestZKServer
-                            == WithMockZooKeeperOrTestZKServer.TEST_ZK_SERVER_SEPARATE_GLOBAL) {
-                        if (super.testZKServerGlobal == null) {
-                            testZKServerGlobal(createTestZookeeper(sessionTimeout));
-                        } else {
-                            log.warn("Skipping creating testZKServerGlobal, already set");
-                        }
-                    }
-                }
-            } catch (Exception e) {
-                throw new RuntimeException(e);
-            }
-        }
-
-        private MockZooKeeper createMockZooKeeper(int sessionTimeout) throws Exception {
-            MockZooKeeper zk = MockZooKeeper.newInstance();
-            zk.setSessionTimeout(sessionTimeout);
-            initializeZookeeper(zk);
-            registerCloseable(zk::shutdown);
-            return zk;
-        }
-
-        // this might not be required at all, but it's kept here as an example
-        private static void initializeZookeeper(ZooKeeper zk) throws KeeperException, InterruptedException {
-            ZkUtils.createFullPathOptimistic(zk, "/ledgers/available/192.168.1.1:" + 5000,
-                    "".getBytes(StandardCharsets.UTF_8), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
-
-            zk.create("/ledgers/LAYOUT", "1\nflat:1".getBytes(StandardCharsets.UTF_8), ZooDefs.Ids.OPEN_ACL_UNSAFE,
-                    CreateMode.PERSISTENT);
-        }
-
-        private TestZKServer createTestZookeeper(int sessionTimeout) throws Exception {
-            TestZKServer testZKServer = new TestZKServer();
-            try (ZooKeeper zkc = new ZooKeeper(testZKServer.getConnectionString(), sessionTimeout, event -> {
-            })) {
-                initializeZookeeper(zkc);
-            }
-            registerCloseable(testZKServer);
-            return testZKServer;
-        }
-
-        protected void handlePreallocatePorts(ServiceConfiguration config) {
-            if (super.preallocatePorts) {
-                config.getBrokerServicePort().ifPresent(portNumber -> {
-                    if (portNumber == 0) {
-                        config.setBrokerServicePort(Optional.of(PortManager.nextLockedFreePort()));
-                    }
-                });
-                config.getBrokerServicePortTls().ifPresent(portNumber -> {
-                    if (portNumber == 0) {
-                        config.setBrokerServicePortTls(Optional.of(PortManager.nextLockedFreePort()));
-                    }
-                });
-                config.getWebServicePort().ifPresent(portNumber -> {
-                    if (portNumber == 0) {
-                        config.setWebServicePort(Optional.of(PortManager.nextLockedFreePort()));
-                    }
-                });
-                config.getWebServicePortTls().ifPresent(portNumber -> {
-                    if (portNumber == 0) {
-                        config.setWebServicePortTls(Optional.of(PortManager.nextLockedFreePort()));
-                    }
-                });
-                registerCloseable(() -> {
-                    config.getBrokerServicePort().ifPresent(PortManager::releaseLockedPort);
-                    config.getBrokerServicePortTls().ifPresent(PortManager::releaseLockedPort);
-                    config.getWebServicePort().ifPresent(PortManager::releaseLockedPort);
-                    config.getWebServicePortTls().ifPresent(PortManager::releaseLockedPort);
-                });
-            }
-        }
-
         private void initializeCommonPulsarServices(SpyConfig spyConfig) {
             if (super.bookKeeperClient == null && super.managedLedgerClientFactory == null) {
                 if (super.executor == null) {
@@ -737,39 +577,22 @@ public class PulsarTestContext implements AutoCloseable {
             if (super.localMetadataStore == null || super.configurationMetadataStore == null) {
                 if (super.mockZooKeeper != null) {
                     MetadataStoreExtended mockZookeeperMetadataStore =
-                            createMockZookeeperMetadataStore(super.mockZooKeeper, super.config,
-                                    MetadataStoreConfig.METADATA_STORE);
+                            createMockZookeeperMetadataStore(super.mockZooKeeper, MetadataStoreConfig.METADATA_STORE);
                     if (super.localMetadataStore == null) {
                         localMetadataStore(mockZookeeperMetadataStore);
                     }
                     if (super.configurationMetadataStore == null) {
                         if (super.mockZooKeeperGlobal != null) {
                             configurationMetadataStore(createMockZookeeperMetadataStore(super.mockZooKeeperGlobal,
-                                    super.config, MetadataStoreConfig.CONFIGURATION_METADATA_STORE));
+                                    MetadataStoreConfig.CONFIGURATION_METADATA_STORE));
                         } else {
                             configurationMetadataStore(mockZookeeperMetadataStore);
                         }
                     }
-                } else if (super.testZKServer != null) {
-                    MetadataStoreExtended testZookeeperMetadataStore =
-                            createTestZookeeperMetadataStore(super.testZKServer, super.config,
-                                    MetadataStoreConfig.METADATA_STORE);
-                    if (super.localMetadataStore == null) {
-                        localMetadataStore(testZookeeperMetadataStore);
-                    }
-                    if (super.configurationMetadataStore == null) {
-                        if (super.testZKServerGlobal != null) {
-                            configurationMetadataStore(createTestZookeeperMetadataStore(super.testZKServerGlobal,
-                                    super.config, MetadataStoreConfig.CONFIGURATION_METADATA_STORE));
-                        } else {
-                            configurationMetadataStore(testZookeeperMetadataStore);
-                        }
-                    }
                 } else {
                     try {
                         MetadataStoreExtended store = MetadataStoreFactoryImpl.createExtended("memory:local",
-                                MetadataStoreConfig.builder()
-                                        .metadataStoreName(MetadataStoreConfig.METADATA_STORE).build());
+                                MetadataStoreConfig.builder().build());
                         registerCloseable(() -> {
                             store.close();
                             resetSpyOrMock(store);
@@ -790,30 +613,16 @@ public class PulsarTestContext implements AutoCloseable {
             }
         }
 
-        private MetadataStoreConfig createMetadataStoreConfig(ServiceConfiguration config, String metadataStoreName) {
-            return MetadataStoreConfig.builder()
-                    .sessionTimeoutMillis((int) config.getMetadataStoreSessionTimeoutMillis())
-                    .allowReadOnlyOperations(config.isMetadataStoreAllowReadOnlyOperations())
-                    .batchingEnabled(config.isMetadataStoreBatchingEnabled())
-                    .batchingMaxDelayMillis(config.getMetadataStoreBatchingMaxDelayMillis())
-                    .batchingMaxOperations(config.getMetadataStoreBatchingMaxOperations())
-                    .batchingMaxSizeKb(config.getMetadataStoreBatchingMaxSizeKb())
-                    .metadataStoreName(metadataStoreName)
-                    .build();
-        }
-
         private MetadataStoreExtended createMockZookeeperMetadataStore(MockZooKeeper mockZooKeeper,
-                                                                       ServiceConfiguration config,
                                                                        String metadataStoreName) {
             // provide a unique session id for each instance
             MockZooKeeperSession mockZooKeeperSession = MockZooKeeperSession.newInstance(mockZooKeeper, false);
-            mockZooKeeperSession.setSessionTimeout((int) config.getMetadataStoreSessionTimeoutMillis());
             registerCloseable(() -> {
                 mockZooKeeperSession.close();
                 resetSpyOrMock(mockZooKeeperSession);
             });
-            ZKMetadataStore zkMetadataStore =
-                    new ZKMetadataStore(mockZooKeeperSession, createMetadataStoreConfig(config, metadataStoreName));
+            ZKMetadataStore zkMetadataStore = new ZKMetadataStore(mockZooKeeperSession,
+                    MetadataStoreConfig.builder().metadataStoreName(metadataStoreName).build());
             registerCloseable(() -> {
                 zkMetadataStore.close();
                 resetSpyOrMock(zkMetadataStore);
@@ -823,18 +632,6 @@ public class PulsarTestContext implements AutoCloseable {
             return nonClosingProxy;
         }
 
-        @SneakyThrows
-        private MetadataStoreExtended createTestZookeeperMetadataStore(TestZKServer zkServer,
-                                                                       ServiceConfiguration config,
-                                                                       String metadataStoreName) {
-            MetadataStoreExtended store = MetadataStoreExtended.create("zk:" + zkServer.getConnectionString(),
-                    createMetadataStoreConfig(config, metadataStoreName));
-            registerCloseable(store);
-            MetadataStoreExtended nonClosingProxy =
-                    NonClosingProxyHandler.createNonClosingProxy(store, MetadataStoreExtended.class);
-            return nonClosingProxy;
-        }
-
         protected abstract void initializePulsarServices(SpyConfig spyConfig, Builder builder);
     }
 
@@ -866,10 +663,19 @@ public class PulsarTestContext implements AutoCloseable {
         protected void initializePulsarServices(SpyConfig spyConfig, Builder builder) {
             BookKeeperClientFactory bookKeeperClientFactory =
                     new MockBookKeeperClientFactory(builder.bookKeeperClient);
+            CompactionServiceFactory compactionServiceFactory = builder.compactionServiceFactory;
+            if (builder.compactionServiceFactory == null && builder.config.getCompactionServiceFactoryClassName()
+                    .equals(PulsarCompactionServiceFactory.class.getName())) {
+                compactionServiceFactory = new MockPulsarCompactionServiceFactory(spyConfig, builder.compactor);
+            }
             PulsarService pulsarService = spyConfig.getPulsarService()
                     .spy(StartableTestPulsarService.class, spyConfig, builder.config, builder.localMetadataStore,
-                            builder.configurationMetadataStore, builder.compactor, builder.brokerInterceptor,
+                            builder.configurationMetadataStore, compactionServiceFactory,
+                            builder.brokerInterceptor,
                             bookKeeperClientFactory, builder.brokerServiceCustomizer);
+            if (compactionServiceFactory != null) {
+                compactionServiceFactory.initialize(pulsarService);
+            }
             registerCloseable(() -> {
                 pulsarService.close();
                 resetSpyOrMock(pulsarService);
@@ -909,8 +715,7 @@ public class PulsarTestContext implements AutoCloseable {
                     if (metadataStore == null) {
                         metadataStore = builder.configurationMetadataStore;
                     }
-                    NamespaceResources nsr = spyConfigPulsarResources.spy(NamespaceResources.class,
-                            builder.localMetadataStore, metadataStore, 30);
+                    NamespaceResources nsr = spyConfigPulsarResources.spy(NamespaceResources.class, metadataStore, 30);
                     TopicResources tsr = spyConfigPulsarResources.spy(TopicResources.class, metadataStore);
                     pulsarResources(
                             spyConfigPulsarResources.spy(
@@ -925,11 +730,20 @@ public class PulsarTestContext implements AutoCloseable {
             }
             BookKeeperClientFactory bookKeeperClientFactory =
                     new MockBookKeeperClientFactory(builder.bookKeeperClient);
+            CompactionServiceFactory compactionServiceFactory = builder.compactionServiceFactory;
+            if (builder.compactionServiceFactory == null && builder.config.getCompactionServiceFactoryClassName()
+                    .equals(PulsarCompactionServiceFactory.class.getName())) {
+                compactionServiceFactory = new MockPulsarCompactionServiceFactory(spyConfig, builder.compactor);
+            }
             PulsarService pulsarService = spyConfig.getPulsarService()
                     .spy(NonStartableTestPulsarService.class, spyConfig, builder.config, builder.localMetadataStore,
-                            builder.configurationMetadataStore, builder.compactor, builder.brokerInterceptor,
+                            builder.configurationMetadataStore, compactionServiceFactory,
+                            builder.brokerInterceptor,
                             bookKeeperClientFactory, builder.pulsarResources,
                             builder.managedLedgerClientFactory, builder.brokerServiceCustomizer);
+            if (compactionServiceFactory != null) {
+                compactionServiceFactory.initialize(pulsarService);
+            }
             registerCloseable(() -> {
                 pulsarService.close();
                 resetSpyOrMock(pulsarService);
