@@ -21,8 +21,8 @@ package org.apache.pulsar.broker.transaction;
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.pulsar.common.naming.SystemTopicNames.PENDING_ACK_STORE_CURSOR_NAME;
 import static org.apache.pulsar.common.naming.SystemTopicNames.PENDING_ACK_STORE_SUFFIX;
-import static org.apache.pulsar.transaction.coordinator.impl.MLTransactionLogImpl.TRANSACTION_LOG_PREFIX;
 import static org.apache.pulsar.transaction.coordinator.impl.DisabledTxnLogBufferedWriterMetricsStats.DISABLED_BUFFERED_WRITER_METRICS;
+import static org.apache.pulsar.transaction.coordinator.impl.MLTransactionLogImpl.TRANSACTION_LOG_PREFIX;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.Mockito.doAnswer;
@@ -42,12 +42,12 @@ import io.netty.util.Timeout;
 import io.netty.util.concurrent.DefaultThreadFactory;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
-import java.util.ArrayList;
 import java.nio.charset.StandardCharsets;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.Map;
 import java.util.List;
+import java.util.Map;
 import java.util.Optional;
 import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
@@ -69,7 +69,6 @@ import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.common.util.Bytes;
 import org.apache.bookkeeper.mledger.AsyncCallbacks;
 import org.apache.bookkeeper.mledger.ManagedCursor;
-import org.apache.bookkeeper.mledger.ManagedLedgerConfig;
 import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.ManagedLedgerFactory;
 import org.apache.bookkeeper.mledger.impl.ManagedCursorContainer;
@@ -100,14 +99,13 @@ import org.apache.pulsar.broker.transaction.buffer.impl.TopicTransactionBuffer;
 import org.apache.pulsar.broker.transaction.buffer.impl.TopicTransactionBufferProvider;
 import org.apache.pulsar.broker.transaction.buffer.impl.TopicTransactionBufferRecoverCallBack;
 import org.apache.pulsar.broker.transaction.buffer.impl.TopicTransactionBufferState;
-import org.apache.pulsar.broker.transaction.pendingack.PendingAckStore;
 import org.apache.pulsar.broker.transaction.buffer.metadata.TransactionBufferSnapshot;
+import org.apache.pulsar.broker.transaction.pendingack.PendingAckStore;
 import org.apache.pulsar.broker.transaction.pendingack.TransactionPendingAckStoreProvider;
 import org.apache.pulsar.broker.transaction.pendingack.impl.MLPendingAckReplyCallBack;
 import org.apache.pulsar.broker.transaction.pendingack.impl.MLPendingAckStore;
 import org.apache.pulsar.broker.transaction.pendingack.impl.MLPendingAckStoreProvider;
 import org.apache.pulsar.broker.transaction.pendingack.impl.PendingAckHandleImpl;
-import org.apache.pulsar.client.admin.LongRunningProcessStatus;
 import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.Message;
@@ -126,9 +124,9 @@ import org.apache.pulsar.client.impl.ClientCnx;
 import org.apache.pulsar.client.impl.ConsumerBase;
 import org.apache.pulsar.client.impl.MessageIdImpl;
 import org.apache.pulsar.client.impl.MessagesImpl;
+import org.apache.pulsar.client.impl.transaction.TransactionImpl;
 import org.apache.pulsar.client.util.ExecutorProvider;
 import org.apache.pulsar.common.api.proto.CommandSubscribe;
-import org.apache.pulsar.client.impl.transaction.TransactionImpl;
 import org.apache.pulsar.common.events.EventType;
 import org.apache.pulsar.common.naming.NamespaceName;
 import org.apache.pulsar.common.naming.SystemTopicNames;
@@ -140,14 +138,16 @@ import org.apache.pulsar.common.policies.data.TopicPolicies;
 import org.apache.pulsar.common.policies.data.stats.TopicStatsImpl;
 import org.apache.pulsar.common.schema.SchemaInfo;
 import org.apache.pulsar.common.util.collections.ConcurrentOpenHashMap;
+import org.apache.pulsar.compaction.CompactionServiceFactory;
+import org.apache.pulsar.compaction.PulsarCompactionServiceFactory;
 import org.apache.pulsar.transaction.coordinator.TransactionCoordinatorID;
 import org.apache.pulsar.transaction.coordinator.TransactionMetadataStore;
 import org.apache.pulsar.transaction.coordinator.TransactionMetadataStoreState;
 import org.apache.pulsar.transaction.coordinator.TransactionRecoverTracker;
 import org.apache.pulsar.transaction.coordinator.TransactionTimeoutTracker;
 import org.apache.pulsar.transaction.coordinator.impl.MLTransactionLogImpl;
-import org.apache.pulsar.transaction.coordinator.impl.MLTransactionSequenceIdGenerator;
 import org.apache.pulsar.transaction.coordinator.impl.MLTransactionMetadataStore;
+import org.apache.pulsar.transaction.coordinator.impl.MLTransactionSequenceIdGenerator;
 import org.apache.pulsar.transaction.coordinator.impl.TxnLogBufferedWriterConfig;
 import org.awaitility.Awaitility;
 import org.mockito.invocation.InvocationOnMock;
@@ -169,10 +169,7 @@ public class TransactionTest extends TransactionTestBase {
 
     @BeforeClass
     protected void setup() throws Exception {
-        // Use a single transaction thread to reproduce possible deadlock easily
-        conf.setNumTransactionReplayThreadPoolSize(1);
-        conf.setManagedLedgerNumSchedulerThreads(1);
-        setUpBase(NUM_BROKERS, NUM_PARTITIONS, NAMESPACE1 + "/test", 0);
+       setUpBase(NUM_BROKERS, NUM_PARTITIONS, NAMESPACE1 + "/test", 0);
     }
 
     @AfterClass(alwaysRun = true)
@@ -746,7 +743,7 @@ public class TransactionTest extends TransactionTestBase {
 
         TransactionBuffer buffer2 = new TopicTransactionBuffer(persistentTopic);
         Awaitility.await().atMost(30, TimeUnit.SECONDS).untilAsserted(() ->
-                assertEquals(buffer2.getStats(false).state, "Ready"));
+                assertEquals(buffer2.getStats(false, false).state, "Ready"));
         managedCursors.removeCursor("transaction-buffer-sub");
 
         doAnswer(invocation -> {
@@ -758,7 +755,7 @@ public class TransactionTest extends TransactionTestBase {
         managedCursors.add(managedCursor, managedCursor.getMarkDeletedPosition());
         TransactionBuffer buffer3 = new TopicTransactionBuffer(persistentTopic);
         Awaitility.await().atMost(30, TimeUnit.SECONDS).untilAsserted(() ->
-                assertEquals(buffer3.getStats(false).state, "Ready"));
+                assertEquals(buffer3.getStats(false, false).state, "Ready"));
         persistentTopic.getInternalStats(false).thenAccept(internalStats -> {
             assertTrue(internalStats.cursors.isEmpty());
         });
@@ -1054,10 +1051,10 @@ public class TransactionTest extends TransactionTestBase {
     }
 
     @Test
-    public void testNotChangeMaxReadPositionCountWhenCheckIfNoSnapshot() throws Exception {
+    public void testNotChangeMaxReadPositionAndAddAbortTimesWhenCheckIfNoSnapshot() throws Exception {
         PersistentTopic persistentTopic = (PersistentTopic) getPulsarServiceList().get(0)
                 .getBrokerService()
-                .getTopic(NAMESPACE1 + "/changeMaxReadPositionCount" + UUID.randomUUID(), true)
+                .getTopic(NAMESPACE1 + "/changeMaxReadPositionAndAddAbortTimes" + UUID.randomUUID(), true)
                 .get().get();
         TransactionBuffer buffer = persistentTopic.getTransactionBuffer();
         Field processorField = TopicTransactionBuffer.class.getDeclaredField("snapshotAbortedTxnProcessor");
@@ -1065,9 +1062,9 @@ public class TransactionTest extends TransactionTestBase {
 
         AbortedTxnProcessor abortedTxnProcessor = (AbortedTxnProcessor) processorField.get(buffer);
         Field changeTimeField = TopicTransactionBuffer
-                .class.getDeclaredField("changeMaxReadPositionCount");
+                .class.getDeclaredField("changeMaxReadPositionAndAddAbortTimes");
         changeTimeField.setAccessible(true);
-        AtomicLong changeMaxReadPositionCount = (AtomicLong) changeTimeField.get(buffer);
+        AtomicLong changeMaxReadPositionAndAddAbortTimes = (AtomicLong) changeTimeField.get(buffer);
 
         Field field1 = TopicTransactionBufferState.class.getDeclaredField("state");
         field1.setAccessible(true);
@@ -1076,10 +1073,10 @@ public class TransactionTest extends TransactionTestBase {
                     TopicTransactionBufferState.State state = (TopicTransactionBufferState.State) field1.get(buffer);
                     Assert.assertEquals(state, TopicTransactionBufferState.State.NoSnapshot);
         });
-        Assert.assertEquals(changeMaxReadPositionCount.get(), 0L);
+        Assert.assertEquals(changeMaxReadPositionAndAddAbortTimes.get(), 0L);
 
-        buffer.syncMaxReadPositionForNormalPublish(new PositionImpl(1, 1), false);
-        Assert.assertEquals(changeMaxReadPositionCount.get(), 0L);
+        buffer.syncMaxReadPositionForNormalPublish(new PositionImpl(1, 1));
+        Assert.assertEquals(changeMaxReadPositionAndAddAbortTimes.get(), 0L);
 
     }
 
@@ -1507,7 +1504,7 @@ public class TransactionTest extends TransactionTestBase {
             fail("Expect failure by PendingAckHandle closed, but success");
         } catch (ExecutionException executionException){
             Throwable t = executionException.getCause();
-            Assert.assertTrue(t instanceof BrokerServiceException);
+            Assert.assertTrue(t instanceof BrokerServiceException.ServiceUnitNotReadyException);
         }
     }
 
@@ -1576,6 +1573,9 @@ public class TransactionTest extends TransactionTestBase {
         when(pulsar.getTransactionBufferSnapshotServiceFactory()).thenReturn(transactionBufferSnapshotServiceFactory);
         TopicTransactionBufferProvider topicTransactionBufferProvider = new TopicTransactionBufferProvider();
         when(pulsar.getTransactionBufferProvider()).thenReturn(topicTransactionBufferProvider);
+        CompactionServiceFactory compactionServiceFactory = new PulsarCompactionServiceFactory();
+        compactionServiceFactory.initialize(pulsar);
+        when(pulsar.getCompactionServiceFactory()).thenReturn(compactionServiceFactory);
         // Mock BacklogQuotaManager
         BacklogQuotaManager backlogQuotaManager = mock(BacklogQuotaManager.class);
         // Mock brokerService.
@@ -1586,7 +1586,6 @@ public class TransactionTest extends TransactionTestBase {
         // Mock managedLedger.
         ManagedLedgerImpl managedLedger = mock(ManagedLedgerImpl.class);
         ManagedCursorContainer managedCursors = new ManagedCursorContainer();
-        when(managedLedger.getConfig()).thenReturn(new ManagedLedgerConfig());
         when(managedLedger.getCursors()).thenReturn(managedCursors);
         PositionImpl position = PositionImpl.EARLIEST;
         when(managedLedger.getLastConfirmedEntry()).thenReturn(position);
@@ -1653,7 +1652,7 @@ public class TransactionTest extends TransactionTestBase {
 
     @Test
     public void testEncryptionRequired() throws Exception {
-        final String namespace = "tnx/testEncryptionRequired";
+        final String namespace = "tnx/ns-prechecks";
         final String topic = "persistent://" + namespace + "/test_transaction_topic";
         admin.namespaces().createNamespace(namespace);
         admin.namespaces().setEncryptionRequiredStatus(namespace, true);
@@ -1784,187 +1783,4 @@ public class TransactionTest extends TransactionTestBase {
                 });
     }
 
-    @Test
-    public void testReadCommittedWithReadCompacted() throws Exception{
-        final String namespace = "tnx/testReadCommittedWithReadCompacted";
-        final String topic = "persistent://" + namespace + "/test_transaction_topic";
-        admin.namespaces().createNamespace(namespace);
-        admin.topics().createNonPartitionedTopic(topic);
-
-        admin.topicPolicies().setCompactionThreshold(topic, 100 * 1024 * 1024);
-
-        @Cleanup
-        Consumer<String> consumer = this.pulsarClient.newConsumer(Schema.STRING)
-                .topic(topic)
-                .subscriptionName("sub")
-                .subscriptionType(SubscriptionType.Exclusive)
-                .readCompacted(true)
-                .subscribe();
-
-        @Cleanup
-        Producer<String> producer = this.pulsarClient.newProducer(Schema.STRING)
-                .topic(topic)
-                .create();
-
-        producer.newMessage().key("K1").value("V1").send();
-
-        Transaction txn = pulsarClient.newTransaction()
-                .withTransactionTimeout(1, TimeUnit.MINUTES).build().get();
-        producer.newMessage(txn).key("K2").value("V2").send();
-        producer.newMessage(txn).key("K3").value("V3").send();
-
-        List<String> messages = new ArrayList<>();
-        while (true) {
-            Message<String> message = consumer.receive(5, TimeUnit.SECONDS);
-            if (message == null) {
-                break;
-            }
-            messages.add(message.getValue());
-        }
-
-        Assert.assertEquals(messages, List.of("V1"));
-
-        txn.commit();
-
-        messages.clear();
-
-        while (true) {
-            Message<String> message = consumer.receive(5, TimeUnit.SECONDS);
-            if (message == null) {
-                break;
-            }
-            messages.add(message.getValue());
-        }
-
-        Assert.assertEquals(messages, List.of("V2", "V3"));
-    }
-
-
-    @Test
-    public void testReadCommittedWithCompaction() throws Exception{
-        final String namespace = "tnx/ns-prechecks";
-        final String topic = "persistent://" + namespace + "/test_transaction_topic" + UUID.randomUUID();
-        admin.namespaces().createNamespace(namespace);
-        admin.topics().createNonPartitionedTopic(topic);
-
-        admin.topicPolicies().setCompactionThreshold(topic, 100 * 1024 * 1024);
-
-        @Cleanup
-        Producer<String> producer = this.pulsarClient.newProducer(Schema.STRING)
-                .topic(topic)
-                .create();
-
-        producer.newMessage().key("K1").value("V1").send();
-
-        Transaction txn = pulsarClient.newTransaction()
-                .withTransactionTimeout(1, TimeUnit.MINUTES).build().get();
-        producer.newMessage(txn).key("K2").value("V2").send();
-        producer.newMessage(txn).key("K3").value("V3").send();
-        txn.commit().get();
-
-        producer.newMessage().key("K1").value("V4").send();
-
-        Transaction txn2 = pulsarClient.newTransaction()
-                .withTransactionTimeout(1, TimeUnit.MINUTES).build().get();
-        producer.newMessage(txn2).key("K2").value("V5").send();
-        producer.newMessage(txn2).key("K3").value("V6").send();
-        txn2.commit().get();
-
-        admin.topics().triggerCompaction(topic);
-
-        Awaitility.await().untilAsserted(() -> {
-            assertEquals(admin.topics().compactionStatus(topic).status,
-                    LongRunningProcessStatus.Status.SUCCESS);
-        });
-
-        @Cleanup
-        Consumer<String> consumer = this.pulsarClient.newConsumer(Schema.STRING)
-                .topic(topic)
-                .subscriptionName("sub")
-                .subscriptionType(SubscriptionType.Exclusive)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .readCompacted(true)
-                .subscribe();
-        List<String> result = new ArrayList<>();
-        while (true) {
-            Message<String> receive = consumer.receive(2, TimeUnit.SECONDS);
-            if (receive == null) {
-                break;
-            }
-
-            result.add(receive.getValue());
-        }
-
-        Assert.assertEquals(result, List.of("V4", "V5", "V6"));
-    }
-
-
-    @Test
-    public void testPersistentTopicGetLastDispatchablePositionWithTxn() throws Exception {
-        String topic = "persistent://" + NAMESPACE1 + "/testPersistentTopicGetLastDispatchablePositionWithTxn";
-
-        @Cleanup
-        Producer<String> producer = pulsarClient.newProducer(Schema.STRING)
-                .topic(topic)
-                .enableBatching(false)
-                .create();
-
-        BrokerService brokerService = pulsarTestContexts.get(0).getBrokerService();
-        PersistentTopic persistentTopic = (PersistentTopic) brokerService.getTopicReference(topic).get();
-
-
-        // send a normal message
-        String body = UUID.randomUUID().toString();
-        MessageIdImpl msgId = (MessageIdImpl) producer.send(body);
-
-        // send 3 txn messages
-        Transaction txn = pulsarClient.newTransaction().build().get();
-        producer.newMessage(txn).value(UUID.randomUUID().toString()).send();
-        producer.newMessage(txn).value(UUID.randomUUID().toString()).send();
-        producer.newMessage(txn).value(UUID.randomUUID().toString()).send();
-
-        // get last dispatchable position
-        PositionImpl lastDispatchablePosition = (PositionImpl) persistentTopic.getLastDispatchablePosition().get();
-        // the last dispatchable position should be the message id of the normal message
-        assertEquals(lastDispatchablePosition, PositionImpl.get(msgId.getLedgerId(), msgId.getEntryId()));
-
-        // abort the txn
-        txn.abort().get(5, TimeUnit.SECONDS);
-
-        // get last dispatchable position
-        lastDispatchablePosition = (PositionImpl) persistentTopic.getLastDispatchablePosition().get();
-        // the last dispatchable position should be the message id of the normal message
-        assertEquals(lastDispatchablePosition, PositionImpl.get(msgId.getLedgerId(), msgId.getEntryId()));
-
-
-        @Cleanup
-        Reader<String> reader = pulsarClient.newReader(Schema.STRING)
-                .topic(topic)
-                .startMessageId(MessageId.earliest)
-                .create();
-        Transaction txn1 = pulsarClient.newTransaction().build().get();
-        producer.newMessage(txn1).value(UUID.randomUUID().toString()).send();
-        producer.newMessage(txn1).value(UUID.randomUUID().toString()).send();
-        producer.newMessage(txn1).value(UUID.randomUUID().toString()).send();
-        List<Message<String>> messages = new ArrayList<>();
-        while (reader.hasMessageAvailable()) {
-            messages.add(reader.readNext());
-        }
-        assertEquals(messages.size(), 1);
-        assertEquals(messages.get(0).getValue(), body);
-
-        txn1.abort().get(5, TimeUnit.SECONDS);
-
-        @Cleanup
-        Reader<String> reader1 = pulsarClient.newReader(Schema.STRING)
-                .topic(topic)
-                .startMessageId(MessageId.earliest)
-                .create();
-        List<Message<String>> messages1 = new ArrayList<>();
-        while (reader1.hasMessageAvailable()) {
-            messages1.add(reader1.readNext());
-        }
-        assertEquals(messages1.size(), 1);
-        assertEquals(messages1.get(0).getValue(), body);
-    }
 }
