@@ -27,12 +27,10 @@ import static org.testng.Assert.assertNotEquals;
 import static org.testng.Assert.assertNotNull;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
-import com.github.benmanes.caffeine.cache.AsyncLoadingCache;
 import com.google.common.collect.Sets;
 import com.google.common.util.concurrent.MoreExecutors;
 import io.netty.handler.codec.http.HttpRequest;
 import io.netty.handler.codec.http.HttpResponse;
-import io.prometheus.client.CollectorRegistry;
 import java.io.IOException;
 import java.io.InputStream;
 import java.lang.reflect.Field;
@@ -71,30 +69,20 @@ import org.apache.pulsar.broker.loadbalance.ResourceUnit;
 import org.apache.pulsar.broker.loadbalance.impl.ModularLoadManagerImpl;
 import org.apache.pulsar.broker.loadbalance.impl.ModularLoadManagerWrapper;
 import org.apache.pulsar.broker.loadbalance.impl.SimpleResourceUnit;
-import org.apache.pulsar.broker.namespace.NamespaceEphemeralData;
 import org.apache.pulsar.broker.namespace.NamespaceService;
-import org.apache.pulsar.broker.namespace.OwnedBundle;
-import org.apache.pulsar.broker.namespace.OwnershipCache;
-import org.apache.pulsar.broker.namespace.ServiceUnitUtils;
 import org.apache.pulsar.broker.service.BrokerService;
 import org.apache.pulsar.broker.testcontext.PulsarTestContext;
-import org.apache.pulsar.client.impl.BinaryProtoLookupService;
-import org.apache.pulsar.client.impl.ClientCnx;
-import org.apache.pulsar.client.impl.LookupService;
-import org.apache.pulsar.client.impl.PulsarClientImpl;
 import org.apache.pulsar.common.naming.NamespaceBundle;
 import org.apache.pulsar.common.naming.NamespaceName;
 import org.apache.pulsar.common.naming.ServiceUnitId;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.partition.PartitionedTopicMetadata;
 import org.apache.pulsar.common.policies.data.ClusterData;
-import org.apache.pulsar.common.policies.data.PoliciesUtil;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
 import org.apache.pulsar.common.util.ObjectMapperFactory;
 import org.apache.pulsar.common.util.SecurityUtility;
 import org.apache.pulsar.policies.data.loadbalancer.LocalBrokerData;
 import org.apache.pulsar.policies.data.loadbalancer.NamespaceBundleStats;
-import org.apache.zookeeper.KeeperException;
 import org.asynchttpclient.AsyncCompletionHandler;
 import org.asynchttpclient.AsyncHttpClient;
 import org.asynchttpclient.AsyncHttpClientConfig;
@@ -106,44 +94,17 @@ import org.asynchttpclient.Request;
 import org.asynchttpclient.Response;
 import org.asynchttpclient.channel.DefaultKeepAliveStrategy;
 import org.awaitility.Awaitility;
-import org.awaitility.reflect.WhiteboxImpl;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import org.testng.ITest;
-import org.testng.SkipException;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
-import org.testng.annotations.DataProvider;
-import org.testng.annotations.Factory;
 import org.testng.annotations.Test;
 
 @Test(groups = "broker-api")
-public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITest {
+public class BrokerServiceLookupTest extends ProducerConsumerBase {
     private static final Logger log = LoggerFactory.getLogger(BrokerServiceLookupTest.class);
-    private String testName;
-
-    @DataProvider
-    private static Object[] booleanValues() {
-        return new Object[]{ true, false };
-    }
-
-    @Factory(dataProvider = "booleanValues")
-    public BrokerServiceLookupTest(boolean useTestZookeeper) {
-        // when set to true, TestZKServer is used which is a real ZooKeeper implementation
-        this.useTestZookeeper = useTestZookeeper;
-    }
-
-    @Override
-    public String getTestName() {
-        return testName;
-    }
 
     @BeforeMethod
-    public void applyTestName(Method method) {
-        testName = method.getName() + " with " + (useTestZookeeper ? "TestZKServer" : "MockZooKeeper");
-    }
-
-    @BeforeMethod(dependsOnMethods = "setTestMethodName")
     @Override
     protected void setup() throws Exception {
         conf.setDefaultNumberOfNamespaceBundles(1);
@@ -152,43 +113,10 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         producerBaseSetup();
     }
 
-    @Override
-    protected void doInitConf() throws Exception {
-        super.doInitConf();
-        switch (methodName) {
-            case "testMultipleBrokerDifferentClusterLookup" -> {
-                conf.setAuthenticationEnabled(true);
-            }
-            case "testWebserviceServiceTls" -> {
-                // broker1 with tls enabled
-                conf.setBrokerServicePortTls(Optional.of(0));
-                conf.setWebServicePortTls(Optional.of(0));
-                conf.setTlsTrustCertsFilePath(CA_CERT_FILE_PATH);
-                conf.setTlsRequireTrustedClientCertOnConnect(true);
-                conf.setTlsCertificateFilePath(BROKER_CERT_FILE_PATH);
-                conf.setTlsKeyFilePath(BROKER_KEY_FILE_PATH);
-                conf.setNumExecutorThreadPoolSize(5);
-                // Not in use, and because TLS is not configured, it will fail to start
-                conf.setSystemTopicEnabled(false);
-            }
-            case "testSkipSplitBundleIfOnlyOneBroker" -> {
-                conf.setDefaultNumberOfNamespaceBundles(1);
-                conf.setLoadBalancerNamespaceBundleMaxTopics(1);
-                conf.setLoadManagerClassName(ModularLoadManagerImpl.class.getName());
-            }
-            case "testPartitionedMetadataWithDeprecatedVersion" -> {
-                conf.setBrokerServicePortTls(Optional.empty());
-                conf.setWebServicePortTls(Optional.empty());
-                conf.setClientLibraryVersionCheckEnabled(true);
-            }
-        }
-    }
-
     @AfterMethod(alwaysRun = true)
     @Override
     protected void cleanup() throws Exception {
         internalCleanup();
-        testName = null;
     }
 
     /**
@@ -233,7 +161,7 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
 
         // mock: return Broker2 as a Least-loaded broker when leader receives request [3]
         doReturn(true).when(loadManager1).isCentralized();
-        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar2.getBrokerId(), null);
+        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar2.getSafeWebServiceAddress(), null);
         doReturn(Optional.of(resourceUnit)).when(loadManager1).getLeastLoaded(any(ServiceUnitId.class));
         doReturn(Optional.of(resourceUnit)).when(loadManager2).getLeastLoaded(any(ServiceUnitId.class));
         loadManagerField.set(pulsar.getNamespaceService(), new AtomicReference<>(loadManager1));
@@ -274,11 +202,9 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
     @Test
     public void testConcurrentWriteBrokerData() throws Exception {
         Map<String, NamespaceBundleStats> map = new ConcurrentHashMap<>();
-        List<String> boundaries = PoliciesUtil.getBundles(100).getBoundaries();
         for (int i = 0; i < 100; i++) {
-            map.put("my-property/my-ns/" + boundaries.get(i), new NamespaceBundleStats());
+            map.put("key"+ i, new NamespaceBundleStats());
         }
-        BrokerService originalBrokerService = pulsar.getBrokerService();
         BrokerService brokerService = mock(BrokerService.class);
         doReturn(brokerService).when(pulsar).getBrokerService();
         doReturn(map).when(brokerService).getBundleStats();
@@ -309,8 +235,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         for (Future<?> future : list) {
             future.get();
         }
-        // allow proper shutdown so that resources aren't leaked
-        doReturn(originalBrokerService).when(pulsar).getBrokerService();
     }
 
     /**
@@ -358,13 +282,19 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         @Cleanup
         PulsarClient pulsarClient2 = PulsarClient.builder().serviceUrl(brokerServiceUrl.toString()).build();
 
+        // enable authorization: so, broker can validate cluster and redirect if finds different cluster
+        pulsar.getConfiguration().setAuthorizationEnabled(true);
+        // restart broker with authorization enabled: it initialize AuthorizationService
+        stopBroker();
+        startBroker();
+
         LoadManager loadManager2 = spy(pulsar2.getLoadManager().get());
         Field loadManagerField = NamespaceService.class.getDeclaredField("loadManager");
         loadManagerField.setAccessible(true);
 
         // mock: return Broker2 as a Least-loaded broker when leader receives request
         doReturn(true).when(loadManager2).isCentralized();
-        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar2.getBrokerId(), null);
+        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar2.getSafeWebServiceAddress(), null);
         doReturn(Optional.of(resourceUnit)).when(loadManager2).getLeastLoaded(any(ServiceUnitId.class));
         loadManagerField.set(pulsar.getNamespaceService(), new AtomicReference<>(loadManager2));
         /**** started broker-2 ****/
@@ -394,6 +324,10 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         consumer.acknowledgeCumulative(msg);
         consumer.close();
         producer.close();
+
+        // disable authorization
+        pulsar.getConfiguration().setAuthorizationEnabled(false);
+        loadManager2 = null;
     }
 
     /**
@@ -511,6 +445,18 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         PulsarTestContext pulsarTestContext2 = createAdditionalPulsarTestContext(conf2);
         PulsarService pulsar2 = pulsarTestContext2.getPulsarService();
 
+        // restart broker1 with tls enabled
+        conf.setBrokerServicePortTls(Optional.of(0));
+        conf.setWebServicePortTls(Optional.of(0));
+        conf.setTlsTrustCertsFilePath(CA_CERT_FILE_PATH);
+        conf.setTlsRequireTrustedClientCertOnConnect(true);
+        conf.setTlsCertificateFilePath(BROKER_CERT_FILE_PATH);
+        conf.setTlsKeyFilePath(BROKER_KEY_FILE_PATH);
+        conf.setNumExecutorThreadPoolSize(5);
+        // Not in use, and because TLS is not configured, it will fail to start
+        conf.setSystemTopicEnabled(false);
+        stopBroker();
+        startBroker();
         pulsar.getLoadManager().get().writeLoadReportOnZookeeper();
         pulsar2.getLoadManager().get().writeLoadReportOnZookeeper();
 
@@ -528,7 +474,7 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         // request [3]
         doReturn(true).when(loadManager1).isCentralized();
         doReturn(true).when(loadManager2).isCentralized();
-        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar.getBrokerId(), null);
+        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar.getWebServiceAddress(), null);
         doReturn(Optional.of(resourceUnit)).when(loadManager2).getLeastLoaded(any(ServiceUnitId.class));
         doReturn(Optional.of(resourceUnit)).when(loadManager1).getLeastLoaded(any(ServiceUnitId.class));
 
@@ -561,9 +507,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
 
         loadManager1 = null;
         loadManager2 = null;
-
-        conf.setBrokerServicePortTls(Optional.empty());
-        conf.setWebServicePortTls(Optional.empty());
     }
 
     /**
@@ -622,7 +565,7 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         loadManagerField.set(pulsar2.getNamespaceService(), new AtomicReference<>(loadManager2));
         // mock: return Broker1 as a Least-loaded broker when leader receives request [3]
         doReturn(true).when(loadManager1).isCentralized();
-        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar.getBrokerId(), null);
+        SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar.getSafeWebServiceAddress(), null);
         doReturn(Optional.of(resourceUnit)).when(loadManager1).getLeastLoaded(any(ServiceUnitId.class));
         doReturn(Optional.of(resourceUnit)).when(loadManager2).getLeastLoaded(any(ServiceUnitId.class));
         loadManagerField.set(pulsar.getNamespaceService(), new AtomicReference<>(loadManager1));
@@ -714,6 +657,11 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
             conf2.setLoadBalancerAutoUnloadSplitBundlesEnabled(true);
             conf2.setLoadBalancerNamespaceBundleMaxTopics(1);
 
+            // configure broker-1 with ModularLoadManager
+            stopBroker();
+            conf.setLoadManagerClassName(ModularLoadManagerImpl.class.getName());
+            startBroker();
+
             @Cleanup
             PulsarTestContext pulsarTestContext2 = createAdditionalPulsarTestContext(conf2);
             PulsarService pulsar2 = pulsarTestContext2.getPulsarService();
@@ -732,7 +680,7 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
             loadManagerField.set(pulsar2.getNamespaceService(), new AtomicReference<>(loadManager2));
             // mock: return Broker1 as a Least-loaded broker when leader receives request [3]
             doReturn(true).when(loadManager1).isCentralized();
-            SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar.getBrokerId(), null);
+            SimpleResourceUnit resourceUnit = new SimpleResourceUnit(pulsar.getSafeWebServiceAddress(), null);
             Optional<ResourceUnit> res = Optional.of(resourceUnit);
             doReturn(res).when(loadManager1).getLeastLoaded(any(ServiceUnitId.class));
             doReturn(res).when(loadManager2).getLeastLoaded(any(ServiceUnitId.class));
@@ -798,7 +746,7 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
             });
 
             // Unload the NamespacePolicies and AntiAffinity check.
-            String currentBroker = pulsar.getBrokerId();
+            String currentBroker = String.format("%s:%d", "localhost", pulsar.getListenPortHTTP().get());
             assertTrue(loadManager.shouldNamespacePoliciesUnload(namespace,"0x00000000_0xffffffff", currentBroker));
             assertTrue(loadManager.shouldAntiAffinityNamespaceUnload(namespace,"0x00000000_0xffffffff", currentBroker));
 
@@ -831,6 +779,12 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         final String topicName1 = BrokerTestUtil.newUniqueName("persistent://" + namespace + "/tp_");
         final String topicName2 = BrokerTestUtil.newUniqueName("persistent://" + namespace + "/tp_");
         try {
+            // configure broker with ModularLoadManager.
+            stopBroker();
+            conf.setDefaultNumberOfNamespaceBundles(1);
+            conf.setLoadBalancerNamespaceBundleMaxTopics(1);
+            conf.setLoadManagerClassName(ModularLoadManagerImpl.class.getName());
+            startBroker();
             final ModularLoadManagerWrapper modularLoadManagerWrapper =
                     (ModularLoadManagerWrapper) pulsar.getLoadManager().get();
             final ModularLoadManagerImpl modularLoadManager =
@@ -869,104 +823,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         }
     }
 
-    @Test
-    public void testMergeGetPartitionedMetadataRequests() throws Exception {
-        // Assert the lookup service is a "BinaryProtoLookupService".
-        final PulsarClientImpl pulsarClientImpl = (PulsarClientImpl) pulsarClient;
-        final LookupService lookupService = pulsarClientImpl.getLookup();
-        assertTrue(lookupService instanceof BinaryProtoLookupService);
-
-        final String tpName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
-        final int topicPartitions = 10;
-        admin.topics().createPartitionedTopic(tpName, topicPartitions);
-
-        // Verify the request is works after merge the requests.
-        List<CompletableFuture<PartitionedTopicMetadata>> futures = new ArrayList<>();
-        for (int i = 0; i < 100; i++) {
-            futures.add(lookupService.getPartitionedTopicMetadata(TopicName.get(tpName), false));
-        }
-        for (CompletableFuture<PartitionedTopicMetadata> future : futures) {
-            assertEquals(future.join().partitions, topicPartitions);
-        }
-
-        // cleanup.
-        admin.topics().deletePartitionedTopic(tpName);
-    }
-
-    @Test
-    public void testMergeLookupRequests() throws Exception {
-        // Assert the lookup service is a "BinaryProtoLookupService".
-        final PulsarClientImpl pulsarClientImpl = (PulsarClientImpl) pulsarClient;
-        final LookupService lookupService = pulsarClientImpl.getLookup();
-        assertTrue(lookupService instanceof BinaryProtoLookupService);
-
-        final String tpName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
-        admin.topics().createNonPartitionedTopic(tpName);
-
-        // Create 1 producer and 100 consumers.
-        List<Producer<String>> producers = new ArrayList<>();
-        List<Consumer<String>> consumers = new ArrayList<>();
-        for (int i = 0; i < 20; i++) {
-            producers.add(pulsarClient.newProducer(Schema.STRING).topic(tpName).create());
-        }
-        for (int i = 0; i < 20; i++) {
-            consumers.add(pulsarClient.newConsumer(Schema.STRING).topic(tpName).subscriptionName("s" + i).subscribe());
-        }
-
-        // Verify the lookup count will be smaller than before improve.
-        int lookupCountBeforeUnload = calculateLookupRequestCount();
-        admin.namespaces().unload(TopicName.get(tpName).getNamespace());
-        Awaitility.await().untilAsserted(() -> {
-            for (Producer p : producers) {
-                assertEquals(WhiteboxImpl.getInternalState(p, "state").toString(), "Ready");
-            }
-            for (Consumer c : consumers) {
-                assertEquals(WhiteboxImpl.getInternalState(c, "state").toString(), "Ready");
-            }
-        });
-        int lookupCountAfterUnload = calculateLookupRequestCount();
-        log.info("lookup count before unload: {}, after unload: {}", lookupCountBeforeUnload, lookupCountAfterUnload);
-        assertTrue(lookupCountAfterUnload < lookupCountBeforeUnload * 2,
-                "the lookup count should be smaller than before improve");
-
-        // Verify the producers and consumers is still works.
-        List<String> messagesSent = new ArrayList<>();
-        int index = 0;
-        for (Producer producer: producers) {
-            String message = Integer.valueOf(index++).toString();
-            producer.send(message);
-            messagesSent.add(message);
-        }
-        HashSet<String> messagesReceived = new HashSet<>();
-        for (Consumer<String> consumer : consumers) {
-            while (true) {
-                Message<String> msg = consumer.receive(2, TimeUnit.SECONDS);
-                if (msg == null) {
-                    break;
-                }
-                messagesReceived.add(msg.getValue());
-            }
-        }
-        assertEquals(messagesReceived.size(), producers.size());
-
-        // cleanup.
-        for (Producer producer: producers) {
-            producer.close();
-        }
-        for (Consumer consumer : consumers) {
-            consumer.close();
-        }
-        admin.topics().delete(tpName);
-    }
-
-    private int calculateLookupRequestCount() throws Exception {
-        int failures = CollectorRegistry.defaultRegistry.getSampleValue("pulsar_broker_lookup_failures_total")
-                .intValue();
-        int answers = CollectorRegistry.defaultRegistry.getSampleValue("pulsar_broker_lookup_answers_total")
-                .intValue();
-        return failures + answers;
-    }
-
     @Test(timeOut = 10000)
     public void testPartitionedMetadataWithDeprecatedVersion() throws Exception {
 
@@ -983,6 +839,10 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
         admin.namespaces().createNamespace(property + "/" + cluster + "/" + namespace);
         admin.topics().createPartitionedTopic(dest.toString(), totalPartitions);
 
+        stopBroker();
+        conf.setClientLibraryVersionCheckEnabled(true);
+        startBroker();
+
         URI brokerServiceUrl = new URI(pulsar.getSafeWebServiceAddress());
 
         URL url = brokerServiceUrl.toURL();
@@ -1086,7 +946,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
 
     private AsyncHttpClient getHttpClient(String version) {
         DefaultAsyncHttpClientConfig.Builder confBuilder = new DefaultAsyncHttpClientConfig.Builder();
-        confBuilder.setCookieStore(null);
         confBuilder.setUseProxyProperties(true);
         confBuilder.setFollowRedirect(true);
         confBuilder.setUserAgent(version);
@@ -1138,143 +997,4 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITe
             return "invalid";
         }
     }
-
-    @Test
-    public void testLookupConnectionNotCloseIfGetUnloadingExOrMetadataEx() throws Exception {
-        if (useTestZookeeper) {
-            throw new SkipException("This test case depends on MockZooKeeper");
-        }
-        String tpName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
-        admin.topics().createNonPartitionedTopic(tpName);
-        PulsarClientImpl pulsarClientImpl = (PulsarClientImpl) pulsarClient;
-        Producer<String> producer = pulsarClientImpl.newProducer(Schema.STRING).topic(tpName).create();
-        Consumer<String> consumer = pulsarClientImpl.newConsumer(Schema.STRING).topic(tpName)
-                .subscriptionName("s1").isAckReceiptEnabled(true).subscribe();
-        LookupService lookupService = pulsarClientImpl.getLookup();
-        assertTrue(lookupService instanceof BinaryProtoLookupService);
-        ClientCnx lookupConnection = pulsarClientImpl.getCnxPool().getConnection(lookupService.resolveHost()).join();
-
-        // Verify the socket will not be closed if the bundle is unloading.
-        BundleOfTopic bundleOfTopic = new BundleOfTopic(tpName);
-        bundleOfTopic.setBundleIsUnloading();
-        try {
-            lookupService.getBroker(TopicName.get(tpName)).get();
-            fail("It should failed due to the namespace bundle is unloading.");
-        } catch (Exception ex) {
-            assertTrue(ex.getMessage().contains("is being unloaded"));
-        }
-        // Do unload topic, trigger producer & consumer reconnection.
-        pulsar.getBrokerService().getTopic(tpName, false).join().get().close(true);
-        assertTrue(lookupConnection.ctx().channel().isActive());
-        bundleOfTopic.setBundleIsNotUnloading();
-        //  Assert producer & consumer could reconnect successful.
-        producer.send("1");
-        HashSet<String> messagesReceived = new HashSet<>();
-        while (true) {
-            Message<String> msg = consumer.receive(2, TimeUnit.SECONDS);
-            if (msg == null) {
-                break;
-            }
-            messagesReceived.add(msg.getValue());
-        }
-        assertTrue(messagesReceived.contains("1"));
-
-        // Verify the socket will not be closed if get a metadata ex.
-        bundleOfTopic.releaseBundleLockAndMakeAcquireFail();
-        try {
-            lookupService.getBroker(TopicName.get(tpName)).get();
-            fail("It should failed due to the acquire bundle lock fail.");
-        } catch (Exception ex) {
-            assertTrue(ex.getMessage().contains("OperationTimeout"));
-        }
-        // Do unload topic, trigger producer & consumer reconnection.
-        pulsar.getBrokerService().getTopic(tpName, false).join().get().close(true);
-        assertTrue(lookupConnection.ctx().channel().isActive());
-        bundleOfTopic.makeAcquireBundleLockSuccess();
-        // Assert producer could reconnect successful.
-        producer.send("2");
-        while (true) {
-            Message<String> msg = consumer.receive(2, TimeUnit.SECONDS);
-            if (msg == null) {
-                break;
-            }
-            messagesReceived.add(msg.getValue());
-        }
-        assertTrue(messagesReceived.contains("2"));
-
-        // cleanup.
-        producer.close();
-        consumer.close();
-        admin.topics().delete(tpName);
-    }
-
-    private class BundleOfTopic {
-
-        private NamespaceBundle namespaceBundle;
-        private OwnershipCache ownershipCache;
-        private AsyncLoadingCache<NamespaceBundle, OwnedBundle> ownedBundlesCache;
-
-        public BundleOfTopic(String tpName) {
-            namespaceBundle = pulsar.getNamespaceService().getBundle(TopicName.get(tpName));
-            ownershipCache = pulsar.getNamespaceService().getOwnershipCache();
-            ownedBundlesCache = WhiteboxImpl.getInternalState(ownershipCache, "ownedBundlesCache");
-        }
-
-        private void setBundleIsUnloading() {
-            ownedBundlesCache.get(namespaceBundle).join().setActive(false);
-        }
-
-        private void setBundleIsNotUnloading() {
-            ownedBundlesCache.get(namespaceBundle).join().setActive(true);
-        }
-
-        private void releaseBundleLockAndMakeAcquireFail() throws Exception {
-            ownedBundlesCache.synchronous().invalidateAll();
-            mockZooKeeper.delete(ServiceUnitUtils.path(namespaceBundle), -1);
-            mockZooKeeper.setAlwaysFail(KeeperException.Code.OPERATIONTIMEOUT);
-        }
-
-        private void makeAcquireBundleLockSuccess() throws Exception {
-            mockZooKeeper.unsetAlwaysFail();
-        }
-    }
-
-    // TODO: This test is disabled since it's invalid. The test fails for both TestZKServer and MockZooKeeper.
-    @Test(timeOut = 30000, enabled = false)
-    public void testLookupConnectionNotCloseIfFailedToAcquireOwnershipOfBundle() throws Exception {
-        String tpName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
-        admin.topics().createNonPartitionedTopic(tpName);
-        final var pulsarClientImpl = (PulsarClientImpl) pulsarClient;
-        final var cache = pulsar.getNamespaceService().getOwnershipCache();
-        final var bundle = pulsar.getNamespaceService().getBundle(TopicName.get(tpName));
-        final var value = cache.getOwnerAsync(bundle).get().orElse(null);
-        assertNotNull(value);
-
-        cache.invalidateLocalOwnerCache();
-        final var lock = pulsar.getCoordinationService().getLockManager(NamespaceEphemeralData.class)
-                .acquireLock(ServiceUnitUtils.path(bundle), new NamespaceEphemeralData()).join();
-        lock.updateValue(null);
-        log.info("Updated bundle {} with null", bundle.getBundleRange());
-
-        // wait for the system topic reader to __change_events is closed, otherwise the test will be affected
-        Thread.sleep(500);
-
-        final var future = pulsarClientImpl.getLookup().getBroker(TopicName.get(tpName));
-        final var cnx = pulsarClientImpl.getCnxPool().getConnections().stream().findAny()
-                .map(CompletableFuture::join).orElse(null);
-        assertNotNull(cnx);
-
-        try {
-            future.get();
-            fail();
-        } catch (ExecutionException e) {
-            log.info("getBroker failed with {}: {}", e.getCause().getClass().getName(), e.getMessage());
-            assertTrue(e.getCause() instanceof PulsarClientException.BrokerMetadataException);
-            assertTrue(cnx.ctx().channel().isActive());
-            lock.updateValue(value);
-            lock.release();
-            assertTrue(e.getMessage().contains("Failed to acquire ownership"));
-            pulsarClientImpl.getLookup().getBroker(TopicName.get(tpName)).get();
-        }
-    }
 }
