@@ -18,20 +18,17 @@
  */
 package org.apache.pulsar.client.api;
 
-import static org.assertj.core.api.Assertions.assertThat;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNotNull;
 import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
 import java.time.Duration;
-import java.time.Instant;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Random;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutorService;
@@ -41,20 +38,17 @@ import java.util.concurrent.atomic.AtomicInteger;
 import lombok.Cleanup;
 import lombok.Data;
 import org.apache.avro.reflect.Nullable;
-import org.apache.pulsar.broker.BrokerTestUtil;
 import org.apache.pulsar.client.api.schema.GenericRecord;
 import org.apache.pulsar.client.impl.ConsumerBuilderImpl;
 import org.apache.pulsar.client.util.RetryMessageUtil;
-import org.apache.pulsar.common.policies.data.SchemaCompatibilityStrategy;
 import org.awaitility.Awaitility;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
-import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
-@Test(groups = "broker-impl")
+@Test(groups = "flaky")
 public class DeadLetterTopicTest extends ProducerConsumerBase {
 
     private static final Logger log = LoggerFactory.getLogger(DeadLetterTopicTest.class);
@@ -62,7 +56,6 @@ public class DeadLetterTopicTest extends ProducerConsumerBase {
     @BeforeMethod(alwaysRun = true)
     @Override
     protected void setup() throws Exception {
-        this.conf.setMaxMessageSize(5 * 1024);
         super.internalSetup();
         super.producerBaseSetup();
     }
@@ -73,15 +66,6 @@ public class DeadLetterTopicTest extends ProducerConsumerBase {
         super.internalCleanup();
     }
 
-    private String createMessagePayload(int size) {
-        StringBuilder str = new StringBuilder();
-        Random rand = new Random();
-        for (int i = 0; i < size; i++) {
-            str.append(rand.nextInt(10));
-        }
-        return str.toString();
-    }
-
     @Test
     public void testDeadLetterTopicWithMessageKey() throws Exception {
         final String topic = "persistent://my-property/my-ns/dead-letter-topic";
@@ -141,194 +125,9 @@ public class DeadLetterTopicTest extends ProducerConsumerBase {
         consumer.close();
     }
 
-    @Test
-    public void testDeadLetterTopicWithBinaryMessageKey() throws Exception {
-        final String topic = "persistent://my-property/my-ns/dead-letter-topic";
 
-        final int maxRedeliveryCount = 1;
-
-        final int sendMessages = 100;
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer(Schema.BYTES)
-                .topic(topic)
-                .subscriptionName("my-subscription")
-                .subscriptionType(SubscriptionType.Shared)
-                .ackTimeout(1, TimeUnit.SECONDS)
-                .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(maxRedeliveryCount).build())
-                .receiverQueueSize(100)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        @Cleanup
-        PulsarClient newPulsarClient = newPulsarClient(lookupUrl.toString(), 0);// Creates new client connection
-        Consumer<byte[]> deadLetterConsumer = newPulsarClient.newConsumer(Schema.BYTES)
-                .topic("persistent://my-property/my-ns/dead-letter-topic-my-subscription-DLQ")
-                .subscriptionName("my-subscription")
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        Producer<byte[]> producer = pulsarClient.newProducer(Schema.BYTES)
-                .topic(topic)
-                .create();
-
-        byte[] key = new byte[]{1, 2, 3, 4};
-        for (int i = 0; i < sendMessages; i++) {
-            producer.newMessage()
-                    .keyBytes(key)
-                    .value(String.format("Hello Pulsar [%d]", i).getBytes())
-                    .send();
-        }
-
-        producer.close();
-
-        int totalReceived = 0;
-        do {
-            Message<byte[]> message = consumer.receive();
-            log.info("consumer received message : {} {}", message.getMessageId(), new String(message.getData()));
-            totalReceived++;
-        } while (totalReceived < sendMessages * (maxRedeliveryCount + 1));
-
-        int totalInDeadLetter = 0;
-        do {
-            Message message = deadLetterConsumer.receive();
-            assertEquals(message.getKeyBytes(), key);
-            log.info("dead letter consumer received message : {} {}", message.getMessageId(), new String(message.getData()));
-            deadLetterConsumer.acknowledge(message);
-            totalInDeadLetter++;
-        } while (totalInDeadLetter < sendMessages);
-
-        deadLetterConsumer.close();
-        consumer.close();
-    }
-
-    @Test
-    public void testDeadLetterTopicMessagesWithOrderingKey() throws Exception {
-        final String topic = "persistent://my-property/my-ns/dead-letter-topic";
-
-        final int maxRedeliveryCount = 1;
-
-        final int sendMessages = 100;
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer(Schema.BYTES)
-                .topic(topic)
-                .subscriptionName("my-subscription")
-                .subscriptionType(SubscriptionType.Shared)
-                .ackTimeout(1, TimeUnit.SECONDS)
-                .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(maxRedeliveryCount).build())
-                .receiverQueueSize(100)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        @Cleanup
-        PulsarClient newPulsarClient = newPulsarClient(lookupUrl.toString(), 0);// Creates new client connection
-        Consumer<byte[]> deadLetterConsumer = newPulsarClient.newConsumer(Schema.BYTES)
-                .topic("persistent://my-property/my-ns/dead-letter-topic-my-subscription-DLQ")
-                .subscriptionName("my-subscription")
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        Producer<byte[]> producer = pulsarClient.newProducer(Schema.BYTES)
-                .topic(topic)
-                .create();
-
-        byte[] key = new byte[]{1, 2, 3, 4};
-        for (int i = 0; i < sendMessages; i++) {
-            producer.newMessage()
-                    .orderingKey(key)
-                    .value(String.format("Hello Pulsar [%d]", i).getBytes())
-                    .send();
-        }
-
-        producer.close();
-
-        int totalReceived = 0;
-        do {
-            Message<byte[]> message = consumer.receive();
-            log.info("consumer received message : {} {}", message.getMessageId(), new String(message.getData()));
-            totalReceived++;
-        } while (totalReceived < sendMessages * (maxRedeliveryCount + 1));
-
-        int totalInDeadLetter = 0;
-        do {
-            Message message = deadLetterConsumer.receive();
-            assertEquals(message.getOrderingKey(), key);
-            log.info("dead letter consumer received message : {} {}", message.getMessageId(), new String(message.getData()));
-            deadLetterConsumer.acknowledge(message);
-            totalInDeadLetter++;
-        } while (totalInDeadLetter < sendMessages);
-
-        deadLetterConsumer.close();
-        consumer.close();
-    }
-
-    @Test
-    public void testDeadLetterTopicMessagesWithEventTime() throws Exception {
-        final String topic = "persistent://my-property/my-ns/dead-letter-topic";
-
-        final int maxRedeliveryCount = 1;
-
-        final int sendMessages = 100;
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer(Schema.BYTES)
-                .topic(topic)
-                .subscriptionName("my-subscription")
-                .subscriptionType(SubscriptionType.Shared)
-                .ackTimeout(1, TimeUnit.SECONDS)
-                .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(maxRedeliveryCount).build())
-                .receiverQueueSize(100)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        @Cleanup
-        PulsarClient newPulsarClient = newPulsarClient(lookupUrl.toString(), 0);// Creates new client connection
-        Consumer<byte[]> deadLetterConsumer = newPulsarClient.newConsumer(Schema.BYTES)
-                .topic("persistent://my-property/my-ns/dead-letter-topic-my-subscription-DLQ")
-                .subscriptionName("my-subscription")
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        Producer<byte[]> producer = pulsarClient.newProducer(Schema.BYTES)
-                .topic(topic)
-                .create();
-
-        long testEventTime = Instant.now().toEpochMilli();
-        for (int i = 0; i < sendMessages; i++) {
-            producer.newMessage()
-                    .eventTime(testEventTime)
-                    .value(String.format("Hello Pulsar, eventTime: [%d]", testEventTime).getBytes())
-                    .send();
-        }
-
-        producer.close();
-
-        int totalReceived = 0;
-        do {
-            Message<byte[]> message = consumer.receive();
-            log.info("consumer received message : {} {}", message.getMessageId(),
-                    new String(message.getData()));
-            totalReceived++;
-        } while (totalReceived < sendMessages * (maxRedeliveryCount + 1));
-
-        int totalInDeadLetter = 0;
-        do {
-            Message<byte[]> message = deadLetterConsumer.receive();
-            assertEquals(message.getEventTime(), testEventTime);
-            log.info("dead letter consumer received message : {} {}", message.getMessageId(), new String(message.getData()));
-            deadLetterConsumer.acknowledge(message);
-            totalInDeadLetter++;
-        } while (totalInDeadLetter < sendMessages);
-
-        deadLetterConsumer.close();
-        consumer.close();
-    }
-
-    @DataProvider(name = "produceLargeMessages")
-    public Object[][] produceLargeMessages() {
-        return new Object[][] { { false }, { true } };
-    }
-
-    @Test(dataProvider = "produceLargeMessages")
-    public void testDeadLetterTopic(boolean produceLargeMessages) throws Exception {
+    @Test(groups = "quarantine")
+    public void testDeadLetterTopic() throws Exception {
         final String topic = "persistent://my-property/my-ns/dead-letter-topic";
 
         final int maxRedeliveryCount = 2;
@@ -355,44 +154,28 @@ public class DeadLetterTopicTest extends ProducerConsumerBase {
 
         Producer<byte[]> producer = pulsarClient.newProducer(Schema.BYTES)
                 .topic(topic)
-                .enableChunking(produceLargeMessages)
-                .enableBatching(!produceLargeMessages)
                 .create();
 
-        Map<Integer, String> messageContent = new HashMap<>();
-
         for (int i = 0; i < sendMessages; i++) {
-            String data;
-            if (!produceLargeMessages) {
-                data = String.format("Hello Pulsar [%d]", i);
-            } else {
-                data = createMessagePayload(1024 * 10);
-            }
-            producer.newMessage().key(String.valueOf(i)).value(data.getBytes()).send();
-            messageContent.put(i, data);
+            producer.send(String.format("Hello Pulsar [%d]", i).getBytes());
         }
 
         producer.close();
 
         int totalReceived = 0;
         do {
-            Message<byte[]> message = consumer.receive(5, TimeUnit.SECONDS);
-            assertNotNull(message, "The consumer should be able to receive messages.");
-            log.info("consumer received message : {}", message.getMessageId());
+            Message<byte[]> message = consumer.receive();
+            log.info("consumer received message : {} {}", message.getMessageId(), new String(message.getData()));
             totalReceived++;
         } while (totalReceived < sendMessages * (maxRedeliveryCount + 1));
 
         int totalInDeadLetter = 0;
         do {
-            Message message = deadLetterConsumer.receive(5, TimeUnit.SECONDS);
-            assertNotNull(message, "the deadLetterConsumer should receive messages.");
-            assertEquals(new String(message.getData()), messageContent.get(Integer.parseInt(message.getKey())));
-            messageContent.remove(Integer.parseInt(message.getKey()));
-            log.info("dead letter consumer received message : {}", message.getMessageId());
+            Message message = deadLetterConsumer.receive();
+            log.info("dead letter consumer received message : {} {}", message.getMessageId(), new String(message.getData()));
             deadLetterConsumer.acknowledge(message);
             totalInDeadLetter++;
         } while (totalInDeadLetter < sendMessages);
-        assertTrue(messageContent.isEmpty());
 
         deadLetterConsumer.close();
         consumer.close();
@@ -1012,7 +795,7 @@ public class DeadLetterTopicTest extends ProducerConsumerBase {
                         .maxRedeliverCount(maxRedeliveryCount)
                         .initialSubscriptionName(dlqInitialSub)
                         .build())
-                .receiverQueueSize(20)
+                .receiverQueueSize(100)
                 .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
                 .subscribe();
 
@@ -1025,7 +808,7 @@ public class DeadLetterTopicTest extends ProducerConsumerBase {
                         .maxRedeliverCount(maxRedeliveryCount)
                         .initialSubscriptionName(dlqInitialSub)
                         .build())
-                .receiverQueueSize(20)
+                .receiverQueueSize(100)
                 .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
                 .subscribe();
 
@@ -1084,254 +867,4 @@ public class DeadLetterTopicTest extends ProducerConsumerBase {
         consumerBuilder.loadConf(config);
         assertEquals(((ConsumerBuilderImpl)consumerBuilder).getConf().getDeadLetterPolicy(), policy);
     }
-
-    @Data
-    static class Payload {
-        String number;
-
-        public Payload() {
-
-        }
-
-        public Payload(String number) {
-            this.number = number;
-        }
-    }
-
-    @Data
-    static class PayloadIncompatible {
-        long number;
-
-        public PayloadIncompatible() {
-
-        }
-
-        public PayloadIncompatible(long number) {
-            this.number = number;
-        }
-    }
-
-    // reproduce issue reported in https://github.com/apache/pulsar/issues/20635#issuecomment-1709616321
-    @Test
-    public void testCloseDeadLetterTopicProducerOnExceptionToPreventProducerLeak() throws Exception {
-        String namespace = BrokerTestUtil.newUniqueName("my-property/my-ns");
-        admin.namespaces().createNamespace(namespace);
-        // don't enforce schema validation
-        admin.namespaces().setSchemaValidationEnforced(namespace, false);
-        // set schema compatibility strategy to always compatible
-        admin.namespaces().setSchemaCompatibilityStrategy(namespace, SchemaCompatibilityStrategy.ALWAYS_COMPATIBLE);
-
-        Schema<Payload> schema = Schema.AVRO(Payload.class);
-        Schema<PayloadIncompatible> schemaIncompatible = Schema.AVRO(PayloadIncompatible.class);
-        String topic = BrokerTestUtil.newUniqueName("persistent://" + namespace
-                        + "/testCloseDeadLetterTopicProducerOnExceptionToPreventProducerLeak");
-        String dlqTopic = topic + "-DLQ";
-
-        // create topics
-        admin.topics().createNonPartitionedTopic(topic);
-        admin.topics().createNonPartitionedTopic(dlqTopic);
-
-        AtomicInteger nackCounter = new AtomicInteger(0);
-        Consumer<Payload> payloadConsumer = null;
-        try {
-            payloadConsumer = pulsarClient.newConsumer(schema).topic(topic)
-                    .subscriptionType(SubscriptionType.Shared).subscriptionName("sub")
-                    .ackTimeout(1, TimeUnit.SECONDS)
-                    .negativeAckRedeliveryDelay(1, TimeUnit.MILLISECONDS)
-                    .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(3).deadLetterTopic(dlqTopic).build())
-                    .messageListener((c, msg) -> {
-                        if (nackCounter.incrementAndGet() < 10) {
-                            c.negativeAcknowledge(msg);
-                        }
-                    }).subscribe();
-
-            // send a message to the topic with the incompatible schema
-            PayloadIncompatible payloadIncompatible = new PayloadIncompatible(123);
-            try (Producer<PayloadIncompatible> producer = pulsarClient.newProducer(schemaIncompatible).topic(topic)
-                    .create()) {
-                producer.send(payloadIncompatible);
-            }
-
-            Thread.sleep(2000L);
-
-            assertThat(pulsar.getBrokerService().getTopicReference(dlqTopic).get().getProducers().size())
-                    .describedAs("producer count of dlq topic %s should be <= 1 so that it doesn't leak producers",
-                            dlqTopic)
-                    .isLessThanOrEqualTo(1);
-
-        } finally {
-            if (payloadConsumer != null) {
-                try {
-                    payloadConsumer.close();
-                } catch (PulsarClientException e) {
-                    // ignore
-                }
-            }
-        }
-
-        assertThat(pulsar.getBrokerService().getTopicReference(dlqTopic).get().getProducers().size())
-                .describedAs("producer count of dlq topic %s should be 0 here",
-                        dlqTopic)
-                .isEqualTo(0);
-    }
-
-    @Test
-    public void testDeadLetterTopicWithMaxUnackedMessagesBlocking() throws Exception {
-        final String topic = "persistent://my-property/my-ns/dead-letter-topic-unacked-blocking";
-        final String dlq = "persistent://my-property/my-ns/dead-letter-topic-unacked-blocking-my-subscription-DLQ";
-        final int maxRedeliveryCount = 3;
-        final int maxUnackedMessages = 100;
-        final int sendMessages = 1000;
-
-        admin.topics().createNonPartitionedTopic(topic);
-        admin.topicPolicies().setMaxUnackedMessagesOnConsumer(topic, maxUnackedMessages);
-        admin.topicPolicies().setMaxUnackedMessagesOnSubscription(topic, maxUnackedMessages);
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer(Schema.BYTES)
-                .topic(topic)
-                .subscriptionName("my-subscription")
-                .subscriptionType(SubscriptionType.Shared)
-                .deadLetterPolicy(DeadLetterPolicy.builder()
-                        .maxRedeliverCount(maxRedeliveryCount)
-                        .deadLetterTopic(dlq)
-                        .build())
-                .receiverQueueSize(200)
-                .negativeAckRedeliveryDelay(1, TimeUnit.MILLISECONDS)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        Consumer<byte[]> deadLetterConsumer = pulsarClient.newConsumer(Schema.BYTES)
-                .topic(dlq)
-                .subscriptionName("my-subscription")
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        Producer<byte[]> producer = pulsarClient.newProducer(Schema.BYTES)
-                .topic(topic)
-                .create();
-
-        for (int i = 0; i < sendMessages; i++) {
-            producer.newMessage()
-                    .value(String.format("Hello Pulsar [%d]", i).getBytes())
-                    .send();
-        }
-        producer.close();
-
-        Set<String> receivedMessages = new HashSet<>();
-        int totalReceived = 0;
-
-        while (totalReceived < sendMessages * (maxRedeliveryCount + 1)) {
-            try {
-                Message<byte[]> message = consumer.receive();
-                if (message != null) {
-                    String messageContent = new String(message.getData());
-                    receivedMessages.add(messageContent);
-                    totalReceived++;
-                    log.info("Received message: {} (total: {}), redelivery count: {}", messageContent,
-                            totalReceived, message.getRedeliveryCount());
-                    consumer.negativeAcknowledge(message);
-                }
-            } catch (Exception e) {
-                log.warn("Exception while receiving message", e);
-                break;
-            }
-        }
-
-        log.info("Total messages received: {}, Expected: {}", totalReceived, sendMessages);
-        log.info("Unique messages received: {}", receivedMessages.size());
-
-        int totalInDeadLetter = 0;
-
-        while (totalInDeadLetter < sendMessages) {
-            try {
-                Message<byte[]> message = deadLetterConsumer.receive();
-                if (message != null) {
-                    String messageContent = new String(message.getData());
-                    log.info("Dead letter message received: {}", messageContent);
-                    deadLetterConsumer.acknowledge(message);
-                    totalInDeadLetter++;
-                } else {
-                    log.warn("No more messages in DLQ");
-                    break;
-                }
-            } catch (Exception e) {
-                log.warn("Exception while receiving from DLQ", e);
-                break;
-            }
-        }
-
-        log.info("Total messages in dead letter queue: {}, Expected: {}", totalInDeadLetter, sendMessages);
-        assertEquals(totalInDeadLetter, sendMessages,
-                "All messages should eventually reach DLQ, but flow control may prevent this");
-
-        deadLetterConsumer.close();
-        consumer.close();
-
-    }
-
-    // reproduce issue reported in https://github.com/apache/pulsar/issues/24541
-    @Test
-    public void sendDeadLetterTopicWithMismatchSchemaProducer() throws Exception {
-        String namespace = BrokerTestUtil.newUniqueName("my-property/my-ns");
-        admin.namespaces().createNamespace(namespace);
-        // don't enforce schema validation
-        admin.namespaces().setSchemaValidationEnforced(namespace, false);
-        // set schema compatibility strategy to always compatible
-        admin.namespaces().setSchemaCompatibilityStrategy(namespace, SchemaCompatibilityStrategy.ALWAYS_COMPATIBLE);
-        final String topic = BrokerTestUtil.newUniqueName("persistent://" + namespace
-                + "/sendDeadLetterTopicWithMismatchSchemaProducer");
-        final String retryTopic = topic + "-RETRY";
-        final String deadLetterTopic = topic + "-DLQ";
-        final Long deadLetterMessageValue = 1234567890L;
-        // create topics
-        admin.topics().createNonPartitionedTopic(topic);
-        admin.topics().createNonPartitionedTopic(deadLetterTopic);
-        admin.topics().createNonPartitionedTopic(retryTopic);
-
-        final int maxRedeliverCount = 1;
-        final String subscriptionName = "my-subscription";
-        Consumer<String> consumer = pulsarClient.newConsumer(Schema.AVRO(String.class))
-                .topic(topic)
-                .subscriptionType(SubscriptionType.Shared)
-                .subscriptionName(subscriptionName)
-                .deadLetterPolicy(DeadLetterPolicy.builder()
-                        .deadLetterTopic(deadLetterTopic)
-                        .retryLetterTopic(retryTopic)
-                        .maxRedeliverCount(maxRedeliverCount)
-                        .build())
-                .negativeAckRedeliveryDelay(1, TimeUnit.SECONDS)
-                .messageListener((Consumer::negativeAcknowledge))
-                .subscribe();
-
-        var brokerConsumer = pulsar.getBrokerService().getTopicReference(topic).get()
-                .getSubscription(subscriptionName).getConsumers().get(0);
-        brokerConsumer.updateRates();
-
-        Consumer<GenericRecord> deadLetterConsumer = pulsarClient.newConsumer(Schema.AUTO_CONSUME())
-                .topic(deadLetterTopic)
-                .subscriptionType(SubscriptionType.Shared)
-                .subscriptionName(subscriptionName)
-                .subscribe();
-        Producer<Long> producer = pulsarClient.newProducer(Schema.AVRO(Long.class)).topic(topic).create();
-        producer.send(deadLetterMessageValue);
-
-        Thread.sleep(3000L);
-
-        brokerConsumer.updateRates();
-        var consumerStats = brokerConsumer.getStats();
-
-        assertThat(consumerStats.getMsgRateRedeliver())
-                .describedAs("redeliver count of topic %s should be less than or equal to 2 because of mismatch schema",
-                        topic)
-                .isLessThanOrEqualTo((maxRedeliverCount + 1) / 3.0);
-
-        Message<GenericRecord> deadLetterMessage = deadLetterConsumer.receive(3, TimeUnit.SECONDS);
-        assertNotNull(deadLetterMessage);
-        assertTrue(deadLetterMessage.getValue().getNativeObject() instanceof Long);
-        assertEquals(deadLetterMessage.getValue().getNativeObject(), deadLetterMessageValue);
-
-        producer.close();
-        consumer.close();
-        deadLetterConsumer.close();
-    }
 }
