@@ -18,20 +18,19 @@
  */
 package org.apache.pulsar.client.api;
 
-import com.google.common.collect.Sets;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+
+import com.google.common.collect.Sets;
 import lombok.Cleanup;
 import org.apache.commons.lang3.RandomStringUtils;
 import org.apache.commons.lang3.RandomUtils;
@@ -80,12 +79,6 @@ public class InterceptorsTest extends ProducerConsumerBase {
         return new Object[][] {{ 0 }, { 3 }};
     }
 
-    @DataProvider(name = "topics")
-    public Object[][] getTopics() {
-        return new Object[][] {{ List.of("persistent://my-property/my-ns/my-topic") },
-                { List.of("persistent://my-property/my-ns/my-topic", "persistent://my-property/my-ns/my-topic1") }};
-    }
-
     @Test
     public void testProducerInterceptor() throws Exception {
         Map<MessageId, List<String>> ackCallback = new HashMap<>();
@@ -217,48 +210,6 @@ public class InterceptorsTest extends ProducerConsumerBase {
         producer.close();
     }
 
-    @Test
-    public void testProducerInterceptorAccessMessageData() throws PulsarClientException {
-        List<String> messageDataInBeforeSend = Collections.synchronizedList(new ArrayList<>());
-        List<String> messageDataOnSendAcknowledgement = Collections.synchronizedList(new ArrayList<>());
-        ProducerInterceptor<String> interceptor = new ProducerInterceptor<>() {
-            @Override
-            public void close() {
-            }
-
-            @Override
-            public Message<String> beforeSend(Producer<String> producer, Message<String> message) {
-                messageDataInBeforeSend.add(new String(message.getData()));
-                return message;
-            }
-
-            @Override
-            public void onSendAcknowledgement(Producer<String> producer, Message<String> message, MessageId msgId,
-                                              Throwable exception) {
-                messageDataOnSendAcknowledgement.add(new String(message.getData()));
-            }
-        };
-        @Cleanup
-        Producer<String> producer = pulsarClient.newProducer(Schema.STRING)
-                .topic("persistent://my-property/my-ns/my-topic")
-                .intercept(interceptor)
-                .create();
-
-        final String messageValue = UUID.randomUUID().toString();
-        try {
-            producer.newMessage().value(messageValue).send();
-        } catch (Exception ignore) {
-        }
-        Assert.assertEquals(messageDataInBeforeSend.size(), 1,
-                "Message data should be available in beforeSend");
-        Assert.assertEquals(messageDataInBeforeSend.get(0), messageValue,
-                "Message data should be available in beforeSend");
-        Assert.assertEquals(messageDataOnSendAcknowledgement.size(), 1,
-                "Message data should be available in onSendAcknowledgement");
-        Assert.assertEquals(messageDataOnSendAcknowledgement.get(0), messageValue,
-                "Message data should be available in onSendAcknowledgement");
-    }
-
     @Test
     public void testConsumerInterceptorWithErrors() throws PulsarClientException {
         ConsumerInterceptor<String> interceptor = new ConsumerInterceptor<String>() {
@@ -452,9 +403,9 @@ public class InterceptorsTest extends ProducerConsumerBase {
 
             @Override
             public Message<String> beforeConsume(Consumer<String> consumer, Message<String> message) {
-                MessageImpl<String> msg = ((MessageImpl<String>) ((TopicMessageImpl<String>) message).getMessage());
+                MessageImpl<String> msg = (MessageImpl<String>) message;
                 msg.getMessageBuilder().addProperty().setKey("beforeConsumer").setValue("1");
-                return message;
+                return msg;
             }
 
             @Override
@@ -498,19 +449,13 @@ public class InterceptorsTest extends ProducerConsumerBase {
 
         int keyCount = 0;
         for (int i = 0; i < 2; i++) {
-            Message<String> received;
-            if (i % 2 == 0) {
-                received = consumer.receive();
-            } else {
-                received = consumer.receiveAsync().join();
-            }
+            Message<String> received = consumer.receive();
             MessageImpl<String> msg = (MessageImpl<String>) ((TopicMessageImpl<String>) received).getMessage();
             for (KeyValue keyValue : msg.getMessageBuilder().getPropertiesList()) {
                 if ("beforeConsumer".equals(keyValue.getKey())) {
                     keyCount++;
                 }
             }
-            Assert.assertEquals(keyCount, i + 1);
             consumer.acknowledge(received);
         }
         Assert.assertEquals(2, keyCount);
@@ -519,84 +464,6 @@ public class InterceptorsTest extends ProducerConsumerBase {
         consumer.close();
     }
 
-    @Test(dataProvider = "topicPartition")
-    public void testDoNotEarlierHitBeforeConsumerWithMessageListener(int partitions) throws Exception {
-
-        AtomicInteger beforeConsumeCount = new AtomicInteger(0);
-        PulsarClient client = PulsarClient.builder()
-                .serviceUrl(lookupUrl.toString())
-                .listenerThreads(1)
-                .build();
-
-        ConsumerInterceptor<String> interceptor = new ConsumerInterceptor<>() {
-            @Override
-            public void close() {
-            }
-
-            @Override
-            public Message<String> beforeConsume(Consumer<String> consumer, Message<String> message) {
-                beforeConsumeCount.incrementAndGet();
-                log.info("beforeConsume messageId: {}", message.getMessageId());
-                return message;
-            }
-
-            @Override
-            public void onAcknowledge(Consumer<String> consumer, MessageId messageId, Throwable cause) {
-            }
-
-            @Override
-            public void onAcknowledgeCumulative(Consumer<String> consumer, MessageId messageId, Throwable cause) {
-            }
-
-            @Override
-            public void onNegativeAcksSend(Consumer<String> consumer, Set<MessageId> messageIds) {
-            }
-
-            @Override
-            public void onAckTimeoutSend(Consumer<String> consumer, Set<MessageId> messageIds) {
-            }
-        };
-
-        final String topicName = "persistent://my-property/my-ns/my-topic";
-
-        if (partitions > 0) {
-            admin.topics().createPartitionedTopic(topicName, partitions);
-        } else {
-            admin.topics().createNonPartitionedTopic(topicName);
-        }
-
-        Consumer<String> consumer = client.newConsumer(Schema.STRING)
-                .topic(topicName)
-                .subscriptionType(SubscriptionType.Shared)
-                .intercept(interceptor)
-                .subscriptionName("my-subscription")
-                .messageListener((c, m) -> {
-                    // Simulate a long processing time
-                    try {
-                        Thread.sleep(60000);
-                    } catch (InterruptedException e) {
-                        throw new RuntimeException(e);
-                    }
-                })
-                .subscribe();
-
-        Producer<String> producer = client.newProducer(Schema.STRING)
-                .topic("persistent://my-property/my-ns/my-topic")
-                .create();
-
-        final int messages = 10;
-        for (int i = 0; i < messages; i++) {
-            producer.newMessage().value("Hello Pulsar!").send();
-        }
-        Awaitility.await().untilAsserted(() -> {
-            // Ensure that the interceptor is not hit before the message listener
-            Assert.assertEquals(beforeConsumeCount.get(), 1);
-        });
-        producer.close();
-        consumer.close();
-        client.close();
-    }
-
     @Test
     public void testConsumerInterceptorWithPatternTopicSubscribe() throws PulsarClientException {
 
@@ -608,9 +475,9 @@ public class InterceptorsTest extends ProducerConsumerBase {
 
             @Override
             public Message<String> beforeConsume(Consumer<String> consumer, Message<String> message) {
-                MessageImpl<String> msg = ((MessageImpl<String>) ((TopicMessageImpl<String>) message).getMessage());
+                MessageImpl<String> msg = (MessageImpl<String>) message;
                 msg.getMessageBuilder().addProperty().setKey("beforeConsumer").setValue("1");
-                return message;
+                return msg;
             }
 
             @Override
@@ -745,8 +612,8 @@ public class InterceptorsTest extends ProducerConsumerBase {
         consumer.close();
     }
 
-    @Test(dataProvider = "topics")
-    public void testConsumerInterceptorForNegativeAcksSend(List<String> topics) throws PulsarClientException, InterruptedException {
+    @Test
+    public void testConsumerInterceptorForNegativeAcksSend() throws PulsarClientException, InterruptedException {
         final int totalNumOfMessages = 100;
         CountDownLatch latch = new CountDownLatch(totalNumOfMessages / 2);
 
@@ -773,7 +640,6 @@ public class InterceptorsTest extends ProducerConsumerBase {
 
             @Override
             public void onNegativeAcksSend(Consumer<String> consumer, Set<MessageId> messageIds) {
-                Assert.assertTrue(latch.getCount() > 0);
                 messageIds.forEach(messageId -> latch.countDown());
             }
 
@@ -784,7 +650,7 @@ public class InterceptorsTest extends ProducerConsumerBase {
         };
 
         Consumer<String> consumer = pulsarClient.newConsumer(Schema.STRING)
-                .topics(topics)
+                .topic("persistent://my-property/my-ns/my-topic")
                 .subscriptionType(SubscriptionType.Failover)
                 .intercept(interceptor)
                 .negativeAckRedeliveryDelay(100, TimeUnit.MILLISECONDS)
@@ -792,7 +658,7 @@ public class InterceptorsTest extends ProducerConsumerBase {
                 .subscribe();
 
         Producer<String> producer = pulsarClient.newProducer(Schema.STRING)
-                .topic(topics.get(0))
+                .topic("persistent://my-property/my-ns/my-topic")
                 .create();
 
         for (int i = 0; i < totalNumOfMessages; i++) {
@@ -816,9 +682,8 @@ public class InterceptorsTest extends ProducerConsumerBase {
         consumer.close();
     }
 
-    @Test(dataProvider = "topics")
-    public void testConsumerInterceptorForAckTimeoutSend(List<String> topics) throws PulsarClientException,
-            InterruptedException {
+    @Test
+    public void testConsumerInterceptorForAckTimeoutSend() throws PulsarClientException, InterruptedException {
         final int totalNumOfMessages = 100;
         CountDownLatch latch = new CountDownLatch(totalNumOfMessages / 2);
 
@@ -849,17 +714,16 @@ public class InterceptorsTest extends ProducerConsumerBase {
 
             @Override
             public void onAckTimeoutSend(Consumer<String> consumer, Set<MessageId> messageIds) {
-                Assert.assertTrue(latch.getCount() > 0);
                 messageIds.forEach(messageId -> latch.countDown());
             }
         };
 
         Producer<String> producer = pulsarClient.newProducer(Schema.STRING)
-                .topic(topics.get(0))
+                .topic("persistent://my-property/my-ns/my-topic")
                 .create();
 
         Consumer<String> consumer = pulsarClient.newConsumer(Schema.STRING)
-                .topics(topics)
+                .topic("persistent://my-property/my-ns/my-topic")
                 .subscriptionName("foo")
                 .intercept(interceptor)
                 .ackTimeout(2, TimeUnit.SECONDS)
