@@ -35,17 +35,14 @@ import java.util.HashSet;
 import java.util.Properties;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
-import lombok.Cleanup;
 import org.apache.pulsar.broker.authentication.AuthenticationProviderToken;
 import org.apache.pulsar.client.admin.PulsarAdmin;
-import org.apache.pulsar.client.impl.auth.AuthenticationToken;
 import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import org.testng.Assert;
-import org.testng.annotations.AfterClass;
-import org.testng.annotations.BeforeClass;
+import org.testng.annotations.AfterMethod;
+import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
@@ -58,35 +55,31 @@ import org.testng.annotations.Test;
 public class TokenAuthenticatedProducerConsumerTest extends ProducerConsumerBase {
     private static final Logger log = LoggerFactory.getLogger(TokenAuthenticatedProducerConsumerTest.class);
 
-    private final static String ADMIN_ROLE = "admin";
     private final String ADMIN_TOKEN;
-    private final String USER_TOKEN;
     private final String TOKEN_PUBLIC_KEY;
-    private final KeyPair kp;
 
     TokenAuthenticatedProducerConsumerTest() throws NoSuchAlgorithmException {
         KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA");
-        kp = kpg.generateKeyPair();
+        KeyPair kp = kpg.generateKeyPair();
 
         byte[] encodedPublicKey = kp.getPublic().getEncoded();
         TOKEN_PUBLIC_KEY = "data:;base64," + Base64.getEncoder().encodeToString(encodedPublicKey);
-        ADMIN_TOKEN = generateToken(ADMIN_ROLE);
-        USER_TOKEN = generateToken("user");
+        ADMIN_TOKEN = generateToken(kp);
     }
 
-    private String generateToken(String subject) {
+    private String generateToken(KeyPair kp) {
         PrivateKey pkey = kp.getPrivate();
         long expMillis = System.currentTimeMillis() + Duration.ofHours(1).toMillis();
         Date exp = new Date(expMillis);
 
         return Jwts.builder()
-            .setSubject(subject)
+            .setSubject("admin")
             .setExpiration(exp)
             .signWith(pkey, SignatureAlgorithm.forSigningKey(pkey))
             .compact();
     }
 
-    @BeforeClass
+    @BeforeMethod
     @Override
     protected void setup() throws Exception {
         conf.setAuthenticationEnabled(true);
@@ -99,8 +92,6 @@ public class TokenAuthenticatedProducerConsumerTest extends ProducerConsumerBase
         Set<String> providers = new HashSet<>();
         providers.add(AuthenticationProviderToken.class.getName());
         conf.setAuthenticationProviders(providers);
-        conf.setBrokerClientAuthenticationPlugin(AuthenticationToken.class.getName());
-        conf.setBrokerClientAuthenticationParameters("token:" + ADMIN_TOKEN);
 
         conf.setClusterName("test");
 
@@ -123,7 +114,7 @@ public class TokenAuthenticatedProducerConsumerTest extends ProducerConsumerBase
                 .authentication(AuthenticationFactory.token(ADMIN_TOKEN)));
     }
 
-    @AfterClass(alwaysRun = true)
+    @AfterMethod(alwaysRun = true)
     @Override
     protected void cleanup() throws Exception {
         super.internalCleanup();
@@ -177,53 +168,4 @@ public class TokenAuthenticatedProducerConsumerTest extends ProducerConsumerBase
         log.info("-- Exiting {} test --", methodName);
     }
 
-    @DataProvider
-    public static Object[][] provider() {
-        // The 1st element specifies whether to use TCP service URL
-        // The 2nd element specifies whether to use a token with correct permission
-        return new Object[][] {
-                { true, true },
-                { true, false },
-                { false, true },
-                { false, false },
-        };
-    }
-
-    @Test(dataProvider = "provider")
-    public void testTenantNotExist(boolean useTcpServiceUrl, boolean useCorrectToken) throws Exception {
-        final var operationTimeoutMs = 10000;
-        final var url = useTcpServiceUrl ? pulsar.getBrokerServiceUrl() : pulsar.getWebServiceAddress();
-        final var token = useCorrectToken ? ADMIN_TOKEN : USER_TOKEN;
-        @Cleanup final var client = PulsarClient.builder().serviceUrl(url)
-                .operationTimeout(operationTimeoutMs, TimeUnit.MILLISECONDS)
-                .authentication(AuthenticationFactory.token(token)).build();
-        final var topic = "non-exist/not-exist/tp"; // the namespace does not exist
-        var start = System.currentTimeMillis();
-        try {
-            client.newProducer().topic(topic).create();
-            Assert.fail();
-        } catch (PulsarClientException e) {
-            final var elapsedMs = System.currentTimeMillis() - start;
-            log.info("Failed to create producer after {} ms: {} {}", elapsedMs, e.getClass().getName(), e.getMessage());
-            Assert.assertTrue(elapsedMs < operationTimeoutMs);
-            if (useTcpServiceUrl) {
-                Assert.assertTrue(e instanceof PulsarClientException.TopicDoesNotExistException);
-            } else {
-                Assert.assertTrue(e instanceof PulsarClientException.NotFoundException);
-            }
-        }
-        start = System.currentTimeMillis();
-        try {
-            client.newConsumer().topic(topic).subscriptionName("sub").subscribe();
-        } catch (PulsarClientException e) {
-            final var elapsedMs = System.currentTimeMillis() - start;
-            log.info("Failed to subscribe after {} ms: {} {}", elapsedMs, e.getClass().getName(), e.getMessage());
-            Assert.assertTrue(elapsedMs < operationTimeoutMs);
-            if (useTcpServiceUrl) {
-                Assert.assertTrue(e instanceof PulsarClientException.TopicDoesNotExistException);
-            } else {
-                Assert.assertTrue(e instanceof PulsarClientException.NotFoundException);
-            }
-        }
-    }
 }
