@@ -356,80 +356,6 @@ public class MessageChunkingTest extends ProducerConsumerBase {
         assertNull(consumer.receive(5, TimeUnit.SECONDS));
     }
 
-    @Test
-    public void testResendChunkMessagesWithoutAckHole() throws Exception {
-        log.info("-- Starting {} test --", methodName);
-        final String topicName = "persistent://my-property/my-ns/testResendChunkMessagesWithoutAckHole";
-        final String subName = "my-subscriber-name";
-        @Cleanup
-        Consumer<String> consumer = pulsarClient.newConsumer(Schema.STRING)
-                .topic(topicName)
-                .subscriptionName(subName)
-                .maxPendingChunkedMessage(10)
-                .autoAckOldestChunkedMessageOnQueueFull(true)
-                .subscribe();
-        @Cleanup
-        Producer<String> producer = pulsarClient.newProducer(Schema.STRING)
-                .topic(topicName)
-                .chunkMaxMessageSize(100)
-                .enableChunking(true)
-                .enableBatching(false)
-                .create();
-
-        sendSingleChunk(producer, "0", 0, 2);
-
-        sendSingleChunk(producer, "0", 0, 2); // Resending the first chunk
-        sendSingleChunk(producer, "0", 1, 2);
-
-        Message<String> receivedMsg = consumer.receive(5, TimeUnit.SECONDS);
-        assertEquals(receivedMsg.getValue(), "chunk-0-0|chunk-0-1|");
-        consumer.acknowledge(receivedMsg);
-        assertEquals(admin.topics().getStats(topicName).getSubscriptions().get(subName)
-                .getNonContiguousDeletedMessagesRanges(), 0);
-    }
-
-    @Test
-    public void testResendChunkMessages() throws Exception {
-        log.info("-- Starting {} test --", methodName);
-        final String topicName = "persistent://my-property/my-ns/testResendChunkMessages";
-
-        @Cleanup
-        Consumer<String> consumer = pulsarClient.newConsumer(Schema.STRING)
-                .topic(topicName)
-                .subscriptionName("my-subscriber-name")
-                .maxPendingChunkedMessage(10)
-                .autoAckOldestChunkedMessageOnQueueFull(true)
-                .subscribe();
-        @Cleanup
-        Producer<String> producer = pulsarClient.newProducer(Schema.STRING)
-                .topic(topicName)
-                .chunkMaxMessageSize(100)
-                .enableChunking(true)
-                .enableBatching(false)
-                .create();
-
-        sendSingleChunk(producer, "0", 0, 2);
-
-        sendSingleChunk(producer, "0", 0, 2); // Resending the first chunk
-        sendSingleChunk(producer, "1", 0, 3); // This is for testing the interwoven chunked message
-        sendSingleChunk(producer, "1", 1, 3);
-        sendSingleChunk(producer, "1", 0, 3); // Resending the UUID-1 chunked message
-
-        sendSingleChunk(producer, "0", 1, 2);
-
-        Message<String> receivedMsg = consumer.receive(5, TimeUnit.SECONDS);
-        assertEquals(receivedMsg.getValue(), "chunk-0-0|chunk-0-1|");
-        consumer.acknowledge(receivedMsg);
-
-        sendSingleChunk(producer, "1", 1, 3);
-        sendSingleChunk(producer, "1", 2, 3);
-
-        receivedMsg = consumer.receive(5, TimeUnit.SECONDS);
-        assertEquals(receivedMsg.getValue(), "chunk-1-0|chunk-1-1|chunk-1-2|");
-        consumer.acknowledge(receivedMsg);
-        Assert.assertEquals(((ConsumerImpl<String>) consumer).getAvailablePermits(), 8);
-    }
-
     /**
      * Validate that chunking is not supported with batching and non-persistent topic
      *
@@ -543,12 +469,8 @@ public class MessageChunkingTest extends ProducerConsumerBase {
         clientBuilder.memoryLimit(10000L, SizeUnit.BYTES);
     }
 
-    interface ThrowingBiConsumer<T, U> {
-        void accept(T t, U u) throws Exception;
-    }
-
     @Test
-    public void testSeekChunkMessages() throws Exception {
+    public void testSeekChunkMessages() throws PulsarClientException {
         log.info("-- Starting {} test --", methodName);
         this.conf.setMaxMessageSize(50);
         final int totalMessages = 5;
@@ -598,17 +520,14 @@ public class MessageChunkingTest extends ProducerConsumerBase {
             assertEquals(msgIds.get(i), msgAfterSeek.getMessageId());
         }
 
-        ThrowingBiConsumer<Boolean, MessageId> assertStartMessageId = (inclusive, expectedFirstMsgId) -> {
-            final var builder = pulsarClient.newReader().topic(topicName).startMessageId(msgIds.get(1));
-            if (inclusive) {
-                builder.startMessageIdInclusive();
-            }
-            @Cleanup final var reader = builder.create();
-            final var readMsg = reader.readNext(5, TimeUnit.SECONDS);
-            assertEquals(expectedFirstMsgId, readMsg.getMessageId());
-        };
-        assertStartMessageId.accept(true, msgIds.get(1));
-        assertStartMessageId.accept(false, msgIds.get(2));
+        Reader<byte[]> reader = pulsarClient.newReader()
+                .topic(topicName)
+                .startMessageIdInclusive()
+                .startMessageId(msgIds.get(1))
+                .create();
+
+        Message<byte[]> readMsg = reader.readNext(5, TimeUnit.SECONDS);
+        assertEquals(msgIds.get(1), readMsg.getMessageId());
 
         consumer1.close();
         consumer2.close();
