@@ -19,17 +19,14 @@
 package org.apache.pulsar.client.impl;
 
 import static org.mockito.Mockito.doReturn;
+import static org.mockito.Mockito.spy;
 import static org.testng.Assert.assertEquals;
-import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertSame;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
 
 import com.google.common.collect.Lists;
 
-import java.time.Duration;
-import java.util.ArrayList;
-import java.util.Collections;
 import java.util.List;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
@@ -37,32 +34,24 @@ import java.util.concurrent.TimeUnit;
 import java.util.regex.Pattern;
 import java.util.stream.IntStream;
 
-import org.apache.pulsar.broker.BrokerTestUtil;
+import io.netty.util.Timeout;
 import org.apache.pulsar.broker.namespace.NamespaceService;
 import org.apache.pulsar.client.api.Consumer;
-import org.apache.pulsar.client.api.InjectedClientCnxClientBuilder;
 import org.apache.pulsar.client.api.Message;
 import org.apache.pulsar.client.api.MessageRoutingMode;
 import org.apache.pulsar.client.api.Producer;
 import org.apache.pulsar.client.api.ProducerConsumerBase;
-import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.RegexSubscriptionMode;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.api.SubscriptionType;
-import org.apache.pulsar.common.api.proto.BaseCommand;
-import org.apache.pulsar.common.api.proto.CommandGetTopicsOfNamespace;
-import org.apache.pulsar.common.api.proto.CommandWatchTopicListSuccess;
 import org.apache.pulsar.common.naming.NamespaceName;
-import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
 import org.awaitility.Awaitility;
-import org.awaitility.reflect.WhiteboxImpl;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
-import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
 @Test(groups = "broker-impl")
@@ -78,7 +67,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
         isTcpLookup = true;
         // enabled transaction, to test pattern consumers not subscribe to transaction system topic.
         conf.setTransactionCoordinatorEnabled(true);
-        conf.setSubscriptionPatternMaxLength(10000);
         super.internalSetup();
         super.producerBaseSetup();
     }
@@ -222,12 +210,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .subscribe();
         assertTrue(consumer.getTopic().startsWith(PatternMultiTopicsConsumerImpl.DUMMY_TOPIC_NAME_PREFIX));
 
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-
         // 4. verify consumer get methods, to get right number of partitions and topics.
         assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
         List<String> topics = ((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions();
@@ -305,12 +287,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
                 .subscribe();
         assertTrue(consumer.getTopic().startsWith(PatternMultiTopicsConsumerImpl.DUMMY_TOPIC_NAME_PREFIX));
 
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-
         // 4. verify consumer get methods, to get right number of partitions and topics.
         assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
         List<String> topics = ((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions();
@@ -388,12 +364,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .subscriptionTopicsMode(RegexSubscriptionMode.NonPersistentOnly)
             .subscribe();
 
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-
         // 4. verify consumer get methods, to get right number of partitions and topics.
         assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
         List<String> topics = ((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions();
@@ -485,12 +455,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .ackTimeout(ackTimeOutMillis, TimeUnit.MILLISECONDS)
             .subscribe();
 
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-
         // 4. verify consumer get methods, to get right number of partitions and topics.
         assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
         List<String> topics = ((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions();
@@ -552,7 +516,7 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
         admin.topics().createPartitionedTopic(topicName2, 2);
         admin.topics().createPartitionedTopic(topicName3, 3);
 
-        // 2. Create consumer, this should success, but with empty sub-consumser internal
+        // 2. Create consumer, this should success, but with empty sub-consumer internal
         Consumer<byte[]> consumer = pulsarClient.newConsumer()
             .topicsPattern(pattern)
             .patternAutoDiscoveryPeriod(2)
@@ -561,11 +525,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .ackTimeout(ackTimeOutMillis, TimeUnit.MILLISECONDS)
             .receiverQueueSize(4)
             .subscribe();
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
 
         // 3. verify consumer get methods, to get 5 number of partitions and topics.
         assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
@@ -590,7 +549,12 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .messageRoutingMode(org.apache.pulsar.client.api.MessageRoutingMode.RoundRobinPartition)
             .create();
 
-        // 5. verify consumer get methods, to get number of partitions and topics, value 6=1+2+3.
+        // 5. call recheckTopics to subscribe each added topics above
+        log.debug("recheck topics change");
+        PatternMultiTopicsConsumerImpl<byte[]> consumer1 = ((PatternMultiTopicsConsumerImpl<byte[]>) consumer);
+        consumer1.run(consumer1.getRecheckPatternTimeout());
+
+        // 6. verify consumer get methods, to get number of partitions and topics, value 6=1+2+3.
         Awaitility.await().untilAsserted(() -> {
             assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
             assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions().size(), 6);
@@ -625,28 +589,13 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
         producer3.close();
     }
 
-    @DataProvider(name= "delayTypesOfWatchingTopics")
-    public Object[][] delayTypesOfWatchingTopics(){
-        return new Object[][]{
-            {true},
-            {false}
-        };
-    }
+    @Test(timeOut = testTimeout)
+    public void testAutoSubscribePatterConsumerFromBrokerWatcher() throws Exception {
+        String key = "AutoSubscribePatternConsumer";
+        String subscriptionName = "my-ex-subscription-" + key;
 
-    @Test(timeOut = testTimeout, dataProvider = "delayTypesOfWatchingTopics")
-    public void testAutoSubscribePatterConsumerFromBrokerWatcher(boolean delayWatchingTopics) throws Exception {
-        final String key = "AutoSubscribePatternConsumer";
-        final String subscriptionName = "my-ex-subscription-" + key;
-        final Pattern pattern = Pattern.compile("persistent://my-property/my-ns/pattern-topic.*");
-
-        PulsarClient client = null;
-        if (delayWatchingTopics) {
-            client = createDelayWatchTopicsClient();
-        } else {
-            client = pulsarClient;
-        }
-
-        Consumer<byte[]> consumer = client.newConsumer()
+        Pattern pattern = Pattern.compile("persistent://my-property/my-ns/pattern-topic.*");
+        Consumer<byte[]> consumer = pulsarClient.newConsumer()
                 .topicsPattern(pattern)
                 // Disable automatic discovery.
                 .patternAutoDiscoveryPeriod(1000)
@@ -671,260 +620,7 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitionedTopics().size(), 1);
         });
 
-        // cleanup.
         consumer.close();
-        admin.topics().deletePartitionedTopic(topicName);
-        if (delayWatchingTopics) {
-            client.close();
-        }
-    }
-
-    @Test(timeOut = testTimeout)
-    public void testSubscribePatterWithOutTopicDomain() throws Exception {
-        final String key = "testSubscribePatterWithOutTopicDomain";
-        final String subscriptionName = "my-ex-subscription-" + key;
-        final Pattern pattern = Pattern.compile("my-property/my-ns/test-pattern.*");
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer()
-                .topicsPattern(pattern)
-                .subscriptionName(subscriptionName)
-                .subscriptionType(SubscriptionType.Shared)
-                .receiverQueueSize(4)
-                .subscribe();
-
-        // 0. Need make sure topic watcher started
-        waitForTopicListWatcherStarted(consumer);
-
-        // if broker enable watch topic, then recheckPatternTimeout will be null.
-        assertNull(((PatternMultiTopicsConsumerImpl<?>) consumer).getRecheckPatternTimeout());
-
-        // 1. create partition topic
-        String topicName = "persistent://my-property/my-ns/test-pattern" + key;
-        admin.topics().createPartitionedTopic(topicName, 4);
-
-        // 2. verify broker will push the changes to update(CommandWatchTopicUpdate).
-        assertSame(pattern.pattern(), ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern().pattern());
-        Awaitility.await().atMost(Duration.ofSeconds(5)).untilAsserted(() -> {
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions().size(), 4);
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getConsumers().size(), 4);
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitionedTopics().size(), 1);
-        });
-
-        // cleanup.
-        consumer.close();
-        admin.topics().deletePartitionedTopic(topicName);
-        pulsarClient.close();
-    }
-
-    @DataProvider(name= "topicDomain")
-    public Object[][] topicDomain(){
-        return new Object[][]{
-                {"persistent"},
-                {"non-persistent"},
-        };
-    }
-
-    @Test(timeOut = testTimeout, dataProvider = "topicDomain")
-    public void testSubscribePatterBrokerDisable(String topicDomain) throws Exception {
-        if(topicDomain.equals("persistent")) {
-            conf.setEnableBrokerSideSubscriptionPatternEvaluation(false);
-        }
-        final String key = "testSubscribePatterWithOutTopicDomain";
-        final String subscriptionName = "my-ex-subscription-" + key;
-        final Pattern pattern = Pattern.compile("my-property/my-ns/test-pattern.*");
-
-        // 0. Create topic1 with 4 partition
-        String topicName1 = topicDomain + "://my-property/my-ns/test-pattern-0" + key;
-        admin.topics().createPartitionedTopic(topicName1, 4);
-        Producer<byte[]> producer1 = pulsarClient.newProducer().topic(topicName1).create();
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer()
-                .topicsPattern(pattern)
-                .subscriptionName(subscriptionName)
-                .subscriptionType(SubscriptionType.Shared)
-                .subscriptionTopicsMode(topicDomain.equals("persistent") ? RegexSubscriptionMode.PersistentOnly 
-                        : RegexSubscriptionMode.NonPersistentOnly)
-                .patternAutoDiscoveryPeriod(1, TimeUnit.SECONDS)
-                .receiverQueueSize(4)
-                .subscribe();
-
-        // 1. Need make sure first check complete
-        Awaitility.await().untilAsserted(() -> {
-            int recheckEpoch = ((PatternMultiTopicsConsumerImpl<?>) consumer).getRecheckPatternEpoch();
-            assertTrue(recheckEpoch > 0);
-        });
-
-        // 2. Create topic2 with 4 partition
-        String topicName2 = topicDomain + "://my-property/my-ns/test-pattern" + key;
-        admin.topics().createPartitionedTopic(topicName2, 4);
-        Producer<byte[]> producer2 = pulsarClient.newProducer().topic(topicName2).create();
-
-        // 3. verify will update the partitions and consumers
-        assertSame(pattern.pattern(), ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern().pattern());
-        Awaitility.await().atMost(Duration.ofSeconds(10)).untilAsserted(() -> {
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions().size(), 8);
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getConsumers().size(), 8);
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitionedTopics().size(), 2);
-        });
-
-        // cleanup.
-        consumer.close();
-        producer1.close();
-        producer2.close();
-        admin.topics().deletePartitionedTopic(topicName1);
-        admin.topics().deletePartitionedTopic(topicName2);
-        pulsarClient.close();
-        conf.setEnableBrokerSideSubscriptionPatternEvaluation(true);
-    }
-
-    @DataProvider(name= "regexpConsumerArgs")
-    public Object[][] regexpConsumerArgs(){
-        return new Object[][]{
-                {true, true},
-                {true, false},
-                {false, true},
-                {false, false}
-        };
-    }
-
-    private void waitForTopicListWatcherStarted(Consumer consumer) {
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            log.info("isDone: {}, isCompletedExceptionally: {}", completableFuture.isDone(),
-                    completableFuture.isCompletedExceptionally());
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-    }
-
-    @Test(timeOut = testTimeout, dataProvider = "regexpConsumerArgs")
-    public void testPreciseRegexpSubscribe(boolean partitioned, boolean createTopicAfterWatcherStarted) throws Exception {
-        final String topicName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
-        final String subscriptionName = "s1";
-        final Pattern pattern = Pattern.compile(String.format("%s$", topicName));
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer()
-                .topicsPattern(pattern)
-                // Disable automatic discovery.
-                .patternAutoDiscoveryPeriod(1000)
-                .subscriptionName(subscriptionName)
-                .subscriptionType(SubscriptionType.Shared)
-                .ackTimeout(ackTimeOutMillis, TimeUnit.MILLISECONDS)
-                .receiverQueueSize(4)
-                .subscribe();
-        if (createTopicAfterWatcherStarted) {
-            waitForTopicListWatcherStarted(consumer);
-        }
-
-        // 1. create topic.
-        if (partitioned) {
-            admin.topics().createPartitionedTopic(topicName, 1);
-        } else {
-            admin.topics().createNonPartitionedTopic(topicName);
-        }
-
-        // 2. verify consumer can subscribe the topic.
-        assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
-        Awaitility.await().untilAsserted(() -> {
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions().size(), 1);
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getConsumers().size(), 1);
-            if (partitioned) {
-                assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitionedTopics().size(), 1);
-            } else {
-                assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitionedTopics().size(), 0);
-            }
-        });
-
-        // cleanup.
-        consumer.close();
-        if (partitioned) {
-            admin.topics().deletePartitionedTopic(topicName);
-        } else {
-            admin.topics().delete(topicName);
-        }
-    }
-
-    @DataProvider(name= "partitioned")
-    public Object[][] partitioned(){
-        return new Object[][]{
-                {true},
-                {true}
-        };
-    }
-
-    @Test(timeOut = 240 * 1000, dataProvider = "partitioned")
-    public void testPreciseRegexpSubscribeDisabledTopicWatcher(boolean partitioned) throws Exception {
-        final String topicName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
-        final String subscriptionName = "s1";
-        final Pattern pattern = Pattern.compile(String.format("%s$", topicName));
-
-        // Close all ServerCnx by close client-side sockets to make the config changes effect.
-        pulsar.getConfig().setEnableBrokerSideSubscriptionPatternEvaluation(false);
-        reconnectAllConnections();
-
-        Consumer<byte[]> consumer = pulsarClient.newConsumer()
-                .topicsPattern(pattern)
-                // Disable brokerSideSubscriptionPatternEvaluation will leading disable topic list watcher.
-                // So set patternAutoDiscoveryPeriod to a little value.
-                .patternAutoDiscoveryPeriod(1)
-                .subscriptionName(subscriptionName)
-                .subscriptionType(SubscriptionType.Shared)
-                .ackTimeout(ackTimeOutMillis, TimeUnit.MILLISECONDS)
-                .receiverQueueSize(4)
-                .subscribe();
-
-        // 1. create topic.
-        if (partitioned) {
-            admin.topics().createPartitionedTopic(topicName, 1);
-        } else {
-            admin.topics().createNonPartitionedTopic(topicName);
-        }
-
-        // 2. verify consumer can subscribe the topic.
-        // Since the minimum value of `patternAutoDiscoveryPeriod` is 60s, we set the test timeout to a triple value.
-        assertSame(pattern, ((PatternMultiTopicsConsumerImpl<?>) consumer).getPattern());
-        Awaitility.await().atMost(Duration.ofMinutes(3)).untilAsserted(() -> {
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions().size(), 1);
-            assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getConsumers().size(), 1);
-            if (partitioned) {
-                assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitionedTopics().size(), 1);
-            } else {
-                assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitionedTopics().size(), 0);
-            }
-        });
-
-        // cleanup.
-        consumer.close();
-        if (partitioned) {
-            admin.topics().deletePartitionedTopic(topicName);
-        } else {
-            admin.topics().delete(topicName);
-        }
-        // Close all ServerCnx by close client-side sockets to make the config changes effect.
-        pulsar.getConfig().setEnableBrokerSideSubscriptionPatternEvaluation(true);
-        reconnectAllConnections();
-    }
-
-    private PulsarClient createDelayWatchTopicsClient() throws Exception {
-        ClientBuilderImpl clientBuilder = (ClientBuilderImpl) PulsarClient.builder().serviceUrl(lookupUrl.toString());
-        return InjectedClientCnxClientBuilder.create(clientBuilder,
-            (conf, eventLoopGroup) -> new ClientCnx(conf, eventLoopGroup) {
-                public CompletableFuture<CommandWatchTopicListSuccess> newWatchTopicList(
-                        BaseCommand command, long requestId) {
-                    // Inject 2 seconds delay when sending command New Watch Topics.
-                    CompletableFuture<CommandWatchTopicListSuccess> res = new CompletableFuture<>();
-                    new Thread(() -> {
-                        sleepSeconds(2);
-                        super.newWatchTopicList(command, requestId).whenComplete((v, ex) -> {
-                            if (ex != null) {
-                                res.completeExceptionally(ex);
-                            } else {
-                                res.complete(v);
-                            }
-                        });
-                    }).start();
-                    return res;
-                }
-            });
     }
 
     // simulate subscribe a pattern which has 3 topics, but then matched topic added in.
@@ -969,12 +665,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .receiverQueueSize(4)
             .subscribe();
 
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-
         assertTrue(consumer instanceof PatternMultiTopicsConsumerImpl);
 
         // 4. verify consumer get methods, to get 6 number of partitions and topics: 6=1+2+3
@@ -1012,6 +702,7 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
         // 7. call recheckTopics to subscribe each added topics above, verify topics number: 10=1+2+3+4
         log.debug("recheck topics change");
         PatternMultiTopicsConsumerImpl<byte[]> consumer1 = ((PatternMultiTopicsConsumerImpl<byte[]>) consumer);
+        consumer1.run(consumer1.getRecheckPatternTimeout());
         Awaitility.await().untilAsserted(() -> {
             assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getPartitions().size(), 10);
             assertEquals(((PatternMultiTopicsConsumerImpl<?>) consumer).getConsumers().size(), 10);
@@ -1084,12 +775,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .receiverQueueSize(4)
             .subscribe();
 
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-
         assertTrue(consumer instanceof PatternMultiTopicsConsumerImpl);
 
         // 4. verify consumer get methods, to get 0 number of partitions and topics: 6=1+2+3
@@ -1118,17 +803,17 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
 
         // 6. remove producer 1,3; verify only consumer 2 left
         // seems no direct way to verify auto-unsubscribe, because this patternConsumer also referenced the topic.
-        String tp2p0 = TopicName.get(topicName2).getPartition(0).toString();
-        String tp2p1 = TopicName.get(topicName2).getPartition(1).toString();
-        List<String> topicNames = Lists.newArrayList(tp2p0, tp2p1);
+        List<String> topicNames = Lists.newArrayList(topicName2);
         NamespaceService nss = pulsar.getNamespaceService();
         doReturn(CompletableFuture.completedFuture(topicNames)).when(nss)
                 .getListOfPersistentTopics(NamespaceName.get("my-property/my-ns"));
 
         // 7. call recheckTopics to unsubscribe topic 1,3, verify topics number: 2=6-1-3
         log.debug("recheck topics change");
-        PatternConsumerUpdateQueue taskQueue = WhiteboxImpl.getInternalState(consumer, "updateTaskQueue");
-        taskQueue.appendRecheckOp();
+        PatternMultiTopicsConsumerImpl<byte[]> consumer1 = ((PatternMultiTopicsConsumerImpl<byte[]>) consumer);
+        Timeout recheckPatternTimeout = spy(consumer1.getRecheckPatternTimeout());
+        doReturn(false).when(recheckPatternTimeout).isCancelled();
+        consumer1.run(recheckPatternTimeout);
         Thread.sleep(100);
         assertEquals(((PatternMultiTopicsConsumerImpl<byte[]>) consumer).getPartitions().size(), 2);
         assertEquals(((PatternMultiTopicsConsumerImpl<byte[]>) consumer).getConsumers().size(), 2);
@@ -1176,12 +861,6 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
             .subscriptionName("sub")
             .subscribe();
 
-        // Wait topic list watcher creation.
-        Awaitility.await().untilAsserted(() -> {
-            CompletableFuture completableFuture = WhiteboxImpl.getInternalState(consumer, "watcherFuture");
-            assertTrue(completableFuture.isDone() && !completableFuture.isCompletedExceptionally());
-        });
-
         assertTrue(consumer instanceof PatternMultiTopicsConsumerImpl);
         PatternMultiTopicsConsumerImpl<String> consumerImpl = (PatternMultiTopicsConsumerImpl<String>) consumer;
 
@@ -1209,57 +888,4 @@ public class PatternTopicsConsumerImplTest extends ProducerConsumerBase {
         assertEquals(pulsar.getBrokerService().getTopicIfExists(baseTopicName + "-1").join(), Optional.empty());
         assertTrue(pulsar.getBrokerService().getTopicIfExists(baseTopicName + "-2").join().isPresent());
     }
-
-    @Test(dataProvider = "partitioned")
-    public void testPatternQuote(boolean partitioned) throws Exception {
-        final NamespaceName namespace = NamespaceName.get("public/default");
-        final String topicName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
-        final PulsarClientImpl client = (PulsarClientImpl) pulsarClient;
-        final LookupService lookup = client.getLookup();
-        List<String> expectedRes = new ArrayList<>();
-        if (partitioned) {
-            admin.topics().createPartitionedTopic(topicName, 2);
-            expectedRes.add(TopicName.get(topicName).getPartition(0).toString());
-            expectedRes.add(TopicName.get(topicName).getPartition(1).toString());
-            Collections.sort(expectedRes);
-        } else {
-            admin.topics().createNonPartitionedTopic(topicName);
-            expectedRes.add(topicName);
-        }
-
-        // Verify 1: "java.util.regex.Pattern.quote".
-        String pattern1 = java.util.regex.Pattern.quote(topicName);
-        List<String> res1 = lookup.getTopicsUnderNamespace(namespace, CommandGetTopicsOfNamespace.Mode.PERSISTENT,
-                        pattern1, null).join().getNonPartitionedOrPartitionTopics();
-        Collections.sort(res1);
-        assertEquals(res1, expectedRes);
-
-        // Verify 2: "com.google.re2j.Pattern.quote"
-        String pattern2 = com.google.re2j.Pattern.quote(topicName);
-        List<String> res2 = lookup.getTopicsUnderNamespace(namespace, CommandGetTopicsOfNamespace.Mode.PERSISTENT,
-                pattern2, null).join().getNonPartitionedOrPartitionTopics();
-        Collections.sort(res2);
-        assertEquals(res2, expectedRes);
-
-        // Verify 3: "java.util.regex.Pattern.quote" & "^$"
-        String pattern3 = "^" + java.util.regex.Pattern.quote(topicName) + "$";
-        List<String> res3 = lookup.getTopicsUnderNamespace(namespace, CommandGetTopicsOfNamespace.Mode.PERSISTENT,
-                pattern3, null).join().getNonPartitionedOrPartitionTopics();
-        Collections.sort(res3);
-        assertEquals(res3, expectedRes);
-
-        // Verify 4: "com.google.re2j.Pattern.quote" & "^$"
-        String pattern4 = "^" + com.google.re2j.Pattern.quote(topicName) + "$";
-        List<String> res4 = lookup.getTopicsUnderNamespace(namespace, CommandGetTopicsOfNamespace.Mode.PERSISTENT,
-                pattern4, null).join().getNonPartitionedOrPartitionTopics();
-        Collections.sort(res4);
-        assertEquals(res4, expectedRes);
-
-        // cleanup.
-        if (partitioned) {
-            admin.topics().deletePartitionedTopic(topicName, false);
-        } else {
-            admin.topics().delete(topicName, false);
-        }
-    }
 }
