@@ -18,9 +18,6 @@
  */
 package org.apache.pulsar.client.impl;
 
-import static org.mockito.ArgumentMatchers.any;
-import static org.mockito.Mockito.doAnswer;
-import static org.mockito.Mockito.spy;
 import com.google.common.collect.Sets;
 import io.netty.buffer.ByteBuf;
 import java.util.ArrayList;
@@ -30,44 +27,30 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.CancellationException;
-import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
-import lombok.extern.slf4j.Slf4j;
-import org.apache.bookkeeper.mledger.AsyncCallbacks;
-import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedLedger;
-import org.apache.bookkeeper.mledger.impl.ManagedLedgerImpl;
 import org.apache.commons.lang3.tuple.ImmutableTriple;
-import org.apache.pulsar.broker.BrokerTestUtil;
 import org.apache.pulsar.broker.auth.MockedPulsarServiceBaseTest;
 import org.apache.pulsar.broker.service.persistent.PersistentTopic;
-import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.api.MessageId;
 import org.apache.pulsar.client.api.MessageRoutingMode;
 import org.apache.pulsar.client.api.Producer;
-import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.RawMessage;
 import org.apache.pulsar.client.api.RawReader;
-import org.apache.pulsar.client.api.Schema;
+import org.apache.pulsar.common.api.proto.BrokerEntryMetadata;
 import org.apache.pulsar.common.api.proto.MessageMetadata;
 import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
-import org.apache.pulsar.common.policies.data.TopicStats;
 import org.apache.pulsar.common.protocol.Commands;
 import org.awaitility.Awaitility;
-import org.mockito.invocation.InvocationOnMock;
-import org.mockito.stubbing.Answer;
 import org.testng.Assert;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
 
 @Test(groups = "broker-impl")
-@Slf4j
 public class RawReaderTest extends MockedPulsarServiceBaseTest {
 
     private static final String subscription = "foobar-sub";
@@ -79,7 +62,6 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
                 "org.apache.pulsar.common.intercept.AppendBrokerTimestampMetadataInterceptor",
                 "org.apache.pulsar.common.intercept.AppendIndexMetadataInterceptor"
         ));
-        conf.setSystemTopicEnabled(false);
         conf.setExposingBrokerEntryMetadataToClientEnabled(true);
         super.internalSetup();
 
@@ -134,7 +116,7 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
     @Test
     public void testHasMessageAvailableWithoutBatch() throws Exception {
         int numKeys = 10;
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
         Set<String> keys = publishMessages(topic, numKeys);
         RawReader reader = RawReader.create(pulsarClient, topic, subscription).get();
         while (true) {
@@ -151,18 +133,20 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
             }
         }
         Assert.assertTrue(keys.isEmpty());
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
     }
 
     @Test
     public void testHasMessageAvailableWithBatch() throws Exception {
         int numKeys = 20;
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
         Set<String> keys = publishMessages(topic, numKeys, true);
         RawReader reader = RawReader.create(pulsarClient, topic, subscription).get();
         int messageCount = 0;
         while (true) {
             boolean hasMsg = reader.hasMessageAvailableAsync().get();
+            if (hasMsg && (messageCount == numKeys)) {
+                Assert.fail("HasMessageAvailable shows still has message when there is no message");
+            }
             if (hasMsg) {
                 try (RawMessage m = reader.readNextAsync().get()) {
                     MessageMetadata meta = Commands.parseMessageMetadata(m.getHeadersAndPayload());
@@ -179,14 +163,13 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
         }
         Assert.assertEquals(messageCount, numKeys);
         Assert.assertTrue(keys.isEmpty());
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
     }
 
     @Test
     public void testRawReader() throws Exception {
         int numKeys = 10;
 
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         Set<String> keys = publishMessages(topic, numKeys);
 
@@ -202,13 +185,12 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
             }
         }
         Assert.assertTrue(keys.isEmpty());
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
     }
 
     @Test
     public void testSeekToStart() throws Exception {
         int numKeys = 10;
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         publishMessages(topic, numKeys);
 
@@ -237,13 +219,12 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
             }
         }
         Assert.assertTrue(readKeys.isEmpty());
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
     }
 
     @Test
     public void testSeekToMiddle() throws Exception {
         int numKeys = 10;
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         publishMessages(topic, numKeys);
 
@@ -281,7 +262,6 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
             }
         }
         Assert.assertTrue(readKeys.isEmpty());
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
     }
 
     /**
@@ -290,7 +270,7 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
     @Test
     public void testFlowControl() throws Exception {
         int numMessages = RawReaderImpl.DEFAULT_RECEIVER_QUEUE_SIZE * 5;
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         publishMessages(topic, numMessages);
 
@@ -316,13 +296,12 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
         }
         Assert.assertEquals(timeouts, 1);
         Assert.assertEquals(keys.size(), numMessages);
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
     }
 
     @Test
     public void testFlowControlBatch() throws Exception {
         int numMessages = RawReaderImpl.DEFAULT_RECEIVER_QUEUE_SIZE * 5;
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         publishMessages(topic, numMessages, true);
 
@@ -345,12 +324,11 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
             }
         }
         Assert.assertEquals(keys.size(), numMessages);
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
     }
 
     @Test
     public void testBatchingExtractKeysAndIds() throws Exception {
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         try (Producer<byte[]> producer = pulsarClient.newProducer().topic(topic)
             .maxPendingMessages(3)
@@ -385,7 +363,7 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
 
     @Test
     public void testBatchingRebatch() throws Exception {
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         try (Producer<byte[]> producer = pulsarClient.newProducer().topic(topic)
             .maxPendingMessages(3)
@@ -412,11 +390,45 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
         }
     }
 
+    @Test
+    public void testBatchingRebatchWithBrokerEntryMetadata() throws Exception {
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
+
+        try (Producer<byte[]> producer = pulsarClient.newProducer().topic(topic)
+                .maxPendingMessages(3)
+                .enableBatching(true)
+                .batchingMaxMessages(3)
+                .batchingMaxPublishDelay(1, TimeUnit.HOURS)
+                .messageRoutingMode(MessageRoutingMode.SinglePartition)
+                .create()) {
+            producer.newMessage().key("key1").value("my-content-1".getBytes()).sendAsync();
+            producer.newMessage().key("key2").value("my-content-2".getBytes()).sendAsync();
+            producer.newMessage().key("key3").value("my-content-3".getBytes()).send();
+        }
+
+        RawReader reader = RawReader.create(pulsarClient, topic, subscription).get();
+        try (RawMessage m1 = reader.readNextAsync().get()) {
+            RawMessage m2 = RawBatchConverter.rebatchMessage(m1, (key, id) -> key.equals("key2")).get();
+            BrokerEntryMetadata brokerEntryMetadata =
+                    Commands.parseBrokerEntryMetadataIfExist(m2.getHeadersAndPayload());
+            Assert.assertNotNull(brokerEntryMetadata);
+            Assert.assertEquals(brokerEntryMetadata.getIndex(), 2);
+            Assert.assertTrue(brokerEntryMetadata.getBrokerTimestamp() < System.currentTimeMillis());
+            List<ImmutableTriple<MessageId, String, Integer>> idsAndKeys = RawBatchConverter.extractIdsAndKeysAndSize(m2);
+            Assert.assertEquals(idsAndKeys.size(), 1);
+            Assert.assertEquals(idsAndKeys.get(0).getMiddle(), "key2");
+            m2.close();
+            Assert.assertEquals(m1.getHeadersAndPayload().refCnt(), 1);
+        } finally {
+            reader.closeAsync().get();
+        }
+    }
+
     @Test
     public void testAcknowledgeWithProperties() throws Exception {
         int numKeys = 10;
 
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
 
         Set<String> keys = publishMessages(topic, numKeys);
 
@@ -442,19 +454,19 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
         ManagedLedger ledger = topicRef.getManagedLedger();
 
         Awaitility.await()
-
+                
                 .untilAsserted(() ->
                         Assert.assertEquals(
                                 ledger.openCursor(subscription).getProperties().get("foobar"),
                                 Long.valueOf(0xdeadbeefdecaL)));
-        reader.closeAsync().get(3, TimeUnit.SECONDS);
+
     }
 
     @Test
     public void testReadCancellationOnClose() throws Exception {
         int numKeys = 10;
 
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
+        String topic = "persistent://my-property/my-ns/my-raw-topic";
         publishMessages(topic, numKeys/2);
 
         RawReader reader = RawReader.create(pulsarClient, topic, subscription).get();
@@ -476,74 +488,4 @@ public class RawReaderTest extends MockedPulsarServiceBaseTest {
             }
         }
     }
-
-    @Test
-    public void testAutoCreateTopic() throws ExecutionException, InterruptedException, PulsarAdminException {
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
-
-        RawReader reader = RawReader.create(pulsarClient, topic, subscription).get();
-        TopicStats stats = admin.topics().getStats(topic);
-        Assert.assertNotNull(stats);
-        reader.closeAsync().join();
-
-        String topic2 = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
-        try {
-            reader = RawReader.create(pulsarClient, topic2, subscription, false, true).get();
-            Assert.fail();
-        } catch (Exception e) {
-            Assert.assertTrue(e.getCause() instanceof PulsarClientException.TopicDoesNotExistException);
-        }
-        reader.closeAsync().join();
-    }
-
-    @Test(timeOut = 60000)
-    public void testReconnectsWhenServiceNotReady() throws Exception {
-        String topic = "persistent://my-property/my-ns/" + BrokerTestUtil.newUniqueName("reader");
-        String subscriptionName = "s1";
-        admin.topics().createNonPartitionedTopic(topic);
-        Producer<String> producer = pulsarClient.newProducer(Schema.STRING).topic(topic).create();
-        RawReader reader = RawReader.create(pulsarClient, topic, subscription).get();
-
-        // Inject a delay event for topic close, which leads to that the raw-reader will get a ServiceNotReady error,
-        PersistentTopic persistentTopic =
-                (PersistentTopic) pulsar.getBrokerService().getTopic(topic, false).get().get();
-        ManagedLedgerImpl ml = (ManagedLedgerImpl) persistentTopic.getManagedLedger();
-        ManagedCursor compactionCursor = ml.openCursor(subscriptionName);
-        ManagedCursor spyCompactionCursor = spy(compactionCursor);
-        CountDownLatch delayCloseCursorSignal = new CountDownLatch(1);
-        Answer answer = new Answer() {
-            @Override
-            public Object answer(InvocationOnMock invocationOnMock) throws Throwable {
-                delayCloseCursorSignal.await();
-                return invocationOnMock.callRealMethod();
-            }
-        };
-        doAnswer(answer).when(spyCompactionCursor).asyncClose(any(AsyncCallbacks.CloseCallback.class), any());
-        ml.getCursors().removeCursor(subscriptionName);
-        ml.getCursors().add(spyCompactionCursor, ml.getLastConfirmedEntry());
-
-        // Unload topic after reader is connected.
-        // The topic state comes to "fenced", then RawReader will get a ServiceNotReady error,
-        CompletableFuture<RawMessage> msgFuture = reader.readNextAsync();
-        CompletableFuture<Void> unloadFuture = admin.topics().unloadAsync(topic);
-        Awaitility.await().untilAsserted(() -> {
-            Assert.assertTrue(persistentTopic.isFenced());
-        });
-
-        // Verify: RasReader reconnected after that the unloading is finished, and it can consume successfully.
-        delayCloseCursorSignal.countDown();
-        unloadFuture.get();
-        MessageIdImpl msgIdSent = (MessageIdImpl) producer.send("msg");
-        RawMessage rawMessage = msgFuture.get();
-        Assert.assertNotNull(rawMessage);
-        MessageIdImpl msgIdReceived = (MessageIdImpl) rawMessage.getMessageId();
-        Assert.assertEquals(msgIdSent.getLedgerId(), msgIdReceived.getLedgerId());
-        Assert.assertEquals(msgIdSent.getEntryId(), msgIdReceived.getEntryId());
-
-        // cleanup.
-        rawMessage.close();;
-        producer.close();
-        reader.closeAsync().get();
-        admin.topics().delete(topic, false);
-    }
 }
