@@ -36,8 +36,6 @@ import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.TimeoutException;
-
 import lombok.Cleanup;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.pulsar.broker.auth.MockedPulsarServiceBaseTest;
@@ -46,11 +44,9 @@ import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.Message;
 import org.apache.pulsar.client.api.MessageId;
-import org.apache.pulsar.client.api.MessageIdAdv;
 import org.apache.pulsar.client.api.MessageRoutingMode;
 import org.apache.pulsar.client.api.Producer;
 import org.apache.pulsar.client.api.ProducerBuilder;
-import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Range;
 import org.apache.pulsar.client.api.Reader;
@@ -68,9 +64,8 @@ import org.apache.pulsar.common.util.Murmur3_32Hash;
 import org.apache.pulsar.schema.Schemas;
 import org.awaitility.Awaitility;
 import org.testng.Assert;
-import org.testng.annotations.AfterClass;
-import org.testng.annotations.BeforeClass;
-import org.testng.annotations.DataProvider;
+import org.testng.annotations.AfterMethod;
+import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
 
 @Slf4j
@@ -79,7 +74,7 @@ public class ReaderTest extends MockedPulsarServiceBaseTest {
 
     private static final String subscription = "reader-sub";
 
-    @BeforeClass(alwaysRun = true)
+    @BeforeMethod
     @Override
     protected void setup() throws Exception {
         super.internalSetup();
@@ -91,7 +86,7 @@ public class ReaderTest extends MockedPulsarServiceBaseTest {
         admin.namespaces().createNamespace("my-property/my-ns", Sets.newHashSet("test"));
     }
 
-    @AfterClass(alwaysRun = true)
+    @AfterMethod(alwaysRun = true)
     @Override
     protected void cleanup() throws Exception {
         super.internalCleanup();
@@ -150,41 +145,21 @@ public class ReaderTest extends MockedPulsarServiceBaseTest {
         testReadMessages(topic, true);
     }
 
-    @DataProvider
-    public static Object[][] seekBeforeHasMessageAvailable() {
-        return new Object[][] { { true }, { false } };
-    }
-
-    @Test(timeOut = 20000, dataProvider = "seekBeforeHasMessageAvailable")
-    public void testReadMessageWithBatchingWithMessageInclusive(boolean seekBeforeHasMessageAvailable)
-            throws Exception {
+    @Test
+    public void testReadMessageWithBatchingWithMessageInclusive() throws Exception {
         String topic = "persistent://my-property/my-ns/my-reader-topic-with-batching-inclusive";
         Set<String> keys = publishMessages(topic, 10, true);
 
         Reader<byte[]> reader = pulsarClient.newReader().topic(topic).startMessageId(MessageId.latest)
                                             .startMessageIdInclusive().readerName(subscription).create();
 
-        if (seekBeforeHasMessageAvailable) {
-            reader.seek(0L); // it should seek to the earliest
+        while (reader.hasMessageAvailable()) {
+            Assert.assertTrue(keys.remove(reader.readNext().getKey()));
         }
-
-        assertTrue(reader.hasMessageAvailable());
-        final Message<byte[]> msg = reader.readNext();
-        assertTrue(keys.remove(msg.getKey()));
         // start from latest with start message inclusive should only read the last message in batch
         assertEquals(keys.size(), 9);
-
-        final MessageIdAdv msgId = (MessageIdAdv) msg.getMessageId();
-        if (seekBeforeHasMessageAvailable) {
-            assertEquals(msgId.getBatchIndex(), 0);
-            assertFalse(keys.contains("key0"));
-            assertTrue(reader.hasMessageAvailable());
-        } else {
-            assertEquals(msgId.getBatchIndex(), 9);
-            assertFalse(reader.hasMessageAvailable());
-            assertFalse(keys.contains("key9"));
-            assertFalse(reader.hasMessageAvailable());
-        }
+        Assert.assertFalse(keys.contains("key9"));
+        Assert.assertFalse(reader.hasMessageAvailable());
     }
 
     private void testReadMessages(String topic, boolean enableBatch) throws Exception {
@@ -282,7 +257,7 @@ public class ReaderTest extends MockedPulsarServiceBaseTest {
     @Test
     public void testReaderWithTimeLong() throws Exception {
         String ns = "my-property/my-ns";
-        String topic = "persistent://" + ns + "/testReaderWithTimeLong";
+        String topic = "persistent://" + ns + "/testReadFromPartition";
         RetentionPolicies retention = new RetentionPolicies(-1, -1);
         admin.namespaces().setRetention(ns, retention);
 
@@ -625,7 +600,7 @@ public class ReaderTest extends MockedPulsarServiceBaseTest {
                 .until(() -> {
             TopicStats topicStats = admin.topics().getStats(topic);
             System.out.println("subscriptions size: " + topicStats.getSubscriptions().size());
-            return topicStats.getSubscriptions().size() == 1;
+            return topicStats.getSubscriptions().size() == 0;
         });
     }
 
@@ -758,165 +733,4 @@ public class ReaderTest extends MockedPulsarServiceBaseTest {
         admin.topics().deletePartitionedTopic(partitionedTopic);
     }
 
-    @Test
-    public void testReaderReconnectedFromNextEntry() throws Exception {
-        final String topic = "persistent://my-property/my-ns/testReaderReconnectedFromNextEntry";
-        Reader<String> reader = pulsarClient.newReader(Schema.STRING).topic(topic).receiverQueueSize(1)
-                .startMessageId(MessageId.earliest).create();
-        Producer<String> producer = pulsarClient.newProducer(Schema.STRING).topic(topic).create();
-
-        // Send 3 and consume 1.
-        producer.send("1");
-        producer.send("2");
-        producer.send("3");
-        Message<String> msg1 = reader.readNext(2, TimeUnit.SECONDS);
-        assertEquals(msg1.getValue(), "1");
-
-        // Trigger reader reconnect.
-        admin.topics().unload(topic);
-
-        // For non-durable we are going to restart from the next entry.
-        Message<String> msg2 = reader.readNext(2, TimeUnit.SECONDS);
-        assertEquals(msg2.getValue(), "2");
-        Message<String> msg3 = reader.readNext(2, TimeUnit.SECONDS);
-        assertEquals(msg3.getValue(), "3");
-
-        // cleanup.
-        reader.close();
-        producer.close();
-        admin.topics().delete(topic, false);
-    }
-
-    @DataProvider
-    public static Object[][] initializeLastMessageIdInBroker() {
-        return new Object[][] { { true }, { false } };
-    }
-
-    @Test(dataProvider = "initializeLastMessageIdInBroker")
-    public void testHasMessageAvailableAfterSeek(boolean initializeLastMessageIdInBroker) throws Exception {
-        final String topic = "persistent://my-property/my-ns/test-has-message-available-after-seek";
-        @Cleanup Reader<String> reader = pulsarClient.newReader(Schema.STRING).topic(topic).receiverQueueSize(1)
-                .startMessageId(MessageId.earliest).create();
-
-        @Cleanup Producer<String> producer = pulsarClient.newProducer(Schema.STRING).topic(topic).create();
-        producer.send("msg");
-
-        if (initializeLastMessageIdInBroker) {
-            assertTrue(reader.hasMessageAvailable());
-        } // else: lastMessageIdInBroker is earliest
-
-        reader.seek(MessageId.latest);
-        // lastMessageIdInBroker is the last message ID, while startMessageId is still earliest
-        assertFalse(reader.hasMessageAvailable());
-
-        producer.send("msg");
-        assertTrue(reader.hasMessageAvailable());
-    }
-
-    @Test(dataProvider = "initializeLastMessageIdInBroker")
-    public void testHasMessageAvailableAfterSeekTimestamp(boolean initializeLastMessageIdInBroker) throws Exception {
-        final String topic = "persistent://my-property/my-ns/test-has-message-available-after-seek-timestamp";
-
-        @Cleanup Producer<String> producer = pulsarClient.newProducer(Schema.STRING).topic(topic).create();
-        final long timestampBeforeSend = System.currentTimeMillis();
-        final MessageId sentMsgId = producer.send("msg");
-
-        final List<MessageId> messageIds = new ArrayList<>();
-        messageIds.add(MessageId.earliest);
-        messageIds.add(sentMsgId);
-        messageIds.add(MessageId.latest);
-
-        for (MessageId messageId : messageIds) {
-            @Cleanup Reader<String> reader = pulsarClient.newReader(Schema.STRING).topic(topic).receiverQueueSize(1)
-                    .startMessageId(messageId).create();
-            if (initializeLastMessageIdInBroker) {
-                if (messageId == MessageId.earliest) {
-                    assertTrue(reader.hasMessageAvailable());
-                } else {
-                    assertFalse(reader.hasMessageAvailable());
-                }
-            } // else: lastMessageIdInBroker is earliest
-            reader.seek(System.currentTimeMillis());
-            assertFalse(reader.hasMessageAvailable());
-        }
-
-        for (MessageId messageId : messageIds) {
-            @Cleanup Reader<String> reader = pulsarClient.newReader(Schema.STRING).topic(topic).receiverQueueSize(1)
-                    .startMessageId(messageId).create();
-            if (initializeLastMessageIdInBroker) {
-                if (messageId == MessageId.earliest) {
-                    assertTrue(reader.hasMessageAvailable());
-                } else {
-                    assertFalse(reader.hasMessageAvailable());
-                }
-            } // else: lastMessageIdInBroker is earliest
-            reader.seek(timestampBeforeSend);
-            assertTrue(reader.hasMessageAvailable());
-        }
-    }
-
-    @Test
-    public void testHasMessageAvailableAfterSeekTimestampWithMessageIdInclusive() throws Exception {
-        final String topic = "persistent://my-property/my-ns/" +
-                "testHasMessageAvailableAfterSeekTimestampWithMessageInclusive";
-
-        @Cleanup
-        Producer<String> producer = pulsarClient.newProducer(Schema.STRING).topic(topic).create();
-        final long timestampBeforeSend = System.currentTimeMillis();
-        final MessageId sentMsgId = producer.send("msg");
-
-        final List<MessageId> messageIds = new ArrayList<>();
-        messageIds.add(MessageId.earliest);
-        messageIds.add(sentMsgId);
-        messageIds.add(MessageId.latest);
-
-        for (MessageId messageId : messageIds) {
-            @Cleanup
-            Reader<String> reader = pulsarClient.newReader(Schema.STRING).topic(topic).receiverQueueSize(1)
-                    .startMessageIdInclusive()
-                    .startMessageId(messageId).create();
-            assertTrue(reader.hasMessageAvailable());
-
-            reader.seek(System.currentTimeMillis());
-            assertFalse(reader.hasMessageAvailable());
-            Message<String> message = reader.readNext(10, TimeUnit.SECONDS);
-            assertNull(message);
-        }
-
-        for (MessageId messageId : messageIds) {
-            @Cleanup
-            Reader<String> reader = pulsarClient.newReader(Schema.STRING).topic(topic).receiverQueueSize(1)
-                    .startMessageIdInclusive()
-                    .startMessageId(messageId).create();
-            assertTrue(reader.hasMessageAvailable());
-
-            reader.seek(timestampBeforeSend);
-            assertTrue(reader.hasMessageAvailable());
-        }
-    }
-
-    @Test
-    public void testReaderBuilderStateOnRetryFailure() throws Exception {
-        String ns = "my-property/my-ns";
-        String topic = "persistent://" + ns + "/testRetryReader";
-        RetentionPolicies retention = new RetentionPolicies(-1, -1);
-        admin.namespaces().setRetention(ns, retention);
-        String badUrl = "pulsar://bad-host:8080";
-
-        @Cleanup
-        PulsarClient client = PulsarClient.builder().serviceUrl(badUrl).build();
-
-        ReaderBuilder<byte[]> readerBuilder = client.newReader().topic(topic).startMessageFromRollbackDuration(100,
-                TimeUnit.SECONDS);
-
-        for (int i = 0; i < 3; i++) {
-            try {
-                readerBuilder.createAsync().get(1, TimeUnit.SECONDS);
-            } catch (TimeoutException e) {
-                log.info("It should time out due to invalid url");
-            } catch (IllegalArgumentException e) {
-                fail("It should not fail with corrupt reader state");
-            }
-        }
-    }
 }
