@@ -47,7 +47,6 @@ import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.mledger.Position;
 import org.apache.bookkeeper.mledger.impl.ManagedLedgerImpl;
 import org.apache.bookkeeper.mledger.impl.PositionImpl;
-import org.apache.pulsar.broker.BrokerTestUtil;
 import org.apache.pulsar.broker.TransactionMetadataStoreService;
 import org.apache.pulsar.broker.service.BrokerService;
 import org.apache.pulsar.broker.service.Topic;
@@ -97,7 +96,7 @@ public class TransactionEndToEndTest extends TransactionTestBase {
     protected static final String TOPIC_OUTPUT = NAMESPACE1 + "/output";
     protected static final String TOPIC_MESSAGE_ACK_TEST = NAMESPACE1 + "/message-ack-test";
     protected static final int NUM_PARTITIONS = 16;
-    @BeforeClass(alwaysRun = true)
+    @BeforeClass
     protected void setup() throws Exception {
         conf.setAcknowledgmentAtBatchIndexLevelEnabled(true);
         setUpBase(1, NUM_PARTITIONS, TOPIC_OUTPUT, TOPIC_PARTITION);
@@ -255,65 +254,6 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         Assert.assertEquals(receiveCounter, count / 2);
     }
 
-    @Test
-    private void testMsgsInPendingAckStateWouldNotGetTheConsumerStuck() throws Exception {
-        final String topicName = NAMESPACE1 + "/testMsgsInPendingAckStateWouldNotGetTheConsumerStuck";
-        final String subscription = "test";
-
-        @Cleanup
-        Producer<Integer> producer = pulsarClient.newProducer(Schema.INT32)
-                .topic(topicName)
-                .create();
-        @Cleanup
-        Consumer<Integer> consumer = pulsarClient.newConsumer(Schema.INT32)
-                .topic(topicName)
-                .subscriptionName(subscription)
-                .subscriptionType(SubscriptionType.Shared)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        int numStep1Receive = 2, numStep2Receive = 2, numStep3Receive = 2;
-        int numTotalMessage = numStep1Receive + numStep2Receive + numStep3Receive;
-
-        for (int i = 0; i < numTotalMessage; i++) {
-            producer.send(i);
-        }
-
-        Transaction step1Txn = getTxn();
-        Transaction step2Txn = getTxn();
-
-        // Step 1, try to consume some messages but do not commit the transaction
-        for (int i = 0; i < numStep1Receive; i++) {
-            consumer.acknowledgeAsync(consumer.receive().getMessageId(), step1Txn).get();
-        }
-
-        // Step 2, try to consume some messages and commit the transaction
-        for (int i = 0; i < numStep2Receive; i++) {
-            consumer.acknowledgeAsync(consumer.receive().getMessageId(), step2Txn).get();
-        }
-
-        // commit step2Txn
-        step2Txn.commit().get();
-
-        // close and re-create consumer
-        consumer.close();
-        @Cleanup
-        Consumer<Integer> consumer2 = pulsarClient.newConsumer(Schema.INT32)
-                .topic(topicName)
-                .receiverQueueSize(numStep3Receive)
-                .subscriptionName(subscription)
-                .subscriptionType(SubscriptionType.Shared)
-                .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest)
-                .subscribe();
-
-        // Step 3, try to consume the rest messages and should receive all of them
-        for (int i = 0; i < numStep3Receive; i++) {
-            // should get the message instead of timeout
-            Message<Integer> msg = consumer2.receive(3, TimeUnit.SECONDS);
-            Assert.assertEquals(msg.getValue(), numStep1Receive + numStep2Receive + i);
-        }
-    }
-
     @Test(dataProvider="enableBatch")
     private void produceCommitTest(boolean enableBatch) throws Exception {
         @Cleanup
@@ -460,76 +400,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         log.info("finished test partitionAbortTest");
     }
 
-    @DataProvider
-    public Object[][] unackMessagesCountParams() {
-        return new Object[][] {
-                // batchSend, batchAck, asyncAck
-                {Boolean.TRUE, Boolean.TRUE, Boolean.TRUE},
-                {Boolean.TRUE, Boolean.TRUE, Boolean.FALSE},
-                {Boolean.TRUE, Boolean.FALSE, Boolean.TRUE},
-                {Boolean.TRUE, Boolean.FALSE, Boolean.FALSE},
-                {Boolean.FALSE, Boolean.TRUE, Boolean.TRUE},
-                {Boolean.FALSE, Boolean.TRUE, Boolean.FALSE},
-                {Boolean.FALSE, Boolean.FALSE, Boolean.TRUE},
-                {Boolean.FALSE, Boolean.FALSE, Boolean.FALSE}
-        };
-    }
-
-    @Test(dataProvider= "unackMessagesCountParams")
-    public void testUnackMessageAfterAckAllMessages(boolean batchSend, boolean batchAck, boolean asyncAck)
-            throws Exception {
-        final int messageCount = 50;
-        final String subName = "s1";
-        final String topicName = BrokerTestUtil.newUniqueName(NAMESPACE1 + "/tp");
-        // Create producer and consumer.
-        Consumer<byte[]> consumer = pulsarClient.newConsumer().topic(topicName).subscriptionName(subName)
-                .subscriptionType(SubscriptionType.Shared).isAckReceiptEnabled(true).subscribe();
-        Awaitility.await().until(consumer::isConnected);
-        Producer<byte[]> producer = pulsarClient.newProducer().topic(topicName).enableBatching(batchSend)
-                .batchingMaxMessages(10).create();
-        Awaitility.await().until(producer::isConnected);
-        // Publish messages.
-        CompletableFuture<MessageId> latestSendFuture = null;
-        for (int i = 0; i < messageCount; i++) {
-            latestSendFuture = producer.sendAsync((i + "").getBytes());
-        }
-        latestSendFuture.join();
-
-        // Ack messages with TXN.
-        Transaction txn = getTxn();
-        List<MessageId> msgIds = new ArrayList<>();
-        for (int i = 0; i < messageCount; i++) {
-            Message<byte[]> message = consumer.receive(5, TimeUnit.SECONDS);
-            msgIds.add(message.getMessageId());
-        }
-        if (batchAck) {
-            consumer.acknowledgeAsync(msgIds, txn).get();
-        } else if (asyncAck) {
-            CompletableFuture<Void> latestAckFuture = null;
-            for (MessageId msgId : msgIds) {
-                latestAckFuture = consumer.acknowledgeAsync(msgId, txn);
-            }
-            latestAckFuture.join();
-        } else {
-            for (MessageId msgId : msgIds) {
-                consumer.acknowledgeAsync(msgId, txn).get();
-            }
-        }
-        txn.commit().get();
-
-        // Verify: the quantity of un-ack messages is 0.
-        int unAckMsgs = admin.topics().getStats(topicName).getSubscriptions().get(subName).getConsumers().get(0)
-                .getUnackedMessages();
-        assertEquals(unAckMsgs, 0);
-
-        // cleanup.
-        producer.close();
-        consumer.close();
-        admin.topics().delete(topicName, true);
-    }
-
     @Test(dataProvider="enableBatch")
-    public void testAckWithTransactionReduceUnAckMessageCount(boolean enableBatch) throws Exception {
+    private void testAckWithTransactionReduceUnAckMessageCount(boolean enableBatch) throws Exception {
 
         final int messageCount = 50;
         final String subName = "testAckWithTransactionReduceUnAckMessageCount";
@@ -1577,7 +1449,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
 
         @Cleanup
         Consumer<byte[]> deadLetterConsumer = pulsarClient.newConsumer()
-                .topic(RetryMessageUtil.getDLQTopic(topic, subName))
+                .topic(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                        topic, subName))
                 .subscriptionType(SubscriptionType.Shared)
                 .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(1).build())
                 .subscriptionName("test")
@@ -1612,7 +1485,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         consumer.close();
         deadLetterConsumer.close();
         producer.close();
-        admin.topics().delete(RetryMessageUtil.getDLQTopic(topic, subName), true);
+        admin.topics().delete(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                topic, subName), true);
         admin.topics().delete(topic, true);
     }
 
@@ -1639,7 +1513,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
 
         @Cleanup
         Consumer<byte[]> deadLetterConsumer = pulsarClient.newConsumer()
-                .topic(RetryMessageUtil.getDLQTopic(topic, subName))
+                .topic(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                        topic, subName))
                 .subscriptionType(SubscriptionType.Shared)
                 .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(1).build())
                 .subscriptionName("test")
@@ -1685,11 +1560,12 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         consumer.close();
         deadLetterConsumer.close();
         producer.close();
-        admin.topics().delete(RetryMessageUtil.getDLQTopic(topic, subName), true);
+        admin.topics().delete(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                topic, subName), true);
         admin.topics().delete(topic, true);
     }
 
-    @Test(groups = "flaky")
+    @Test
     public void testDelayedTransactionMessages() throws Exception {
         String topic = NAMESPACE1 + "/testDelayedTransactionMessages";
 
