@@ -34,7 +34,6 @@ import java.lang.reflect.Method;
 import java.net.URL;
 import java.util.Collections;
 import java.util.HashSet;
-import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Properties;
@@ -216,26 +215,11 @@ public class PulsarFunctionE2ESecurityTest {
     void shutdown() throws Exception {
         try {
             log.info("--- Shutting down ---");
-            if (pulsarClient != null) {
-                pulsarClient.close();
-                pulsarClient = null;
-            }
-            if (superUserAdmin != null) {
-                superUserAdmin.close();
-                superUserAdmin = null;
-            }
-            if (functionsWorkerService != null) {
-                functionsWorkerService.stop();
-                functionsWorkerService = null;
-            }
-            if (pulsar != null) {
-                pulsar.close();
-                pulsar = null;
-            }
-            if (bkEnsemble != null) {
-                bkEnsemble.stop();
-                bkEnsemble = null;
-            }
+            pulsarClient.close();
+            superUserAdmin.close();
+            functionsWorkerService.stop();
+            pulsar.close();
+            bkEnsemble.stop();
         } finally {
             if (tempDirectory != null) {
                 tempDirectory.delete();
@@ -283,12 +267,8 @@ public class PulsarFunctionE2ESecurityTest {
         workerConfig.setAuthorizationEnabled(config.isAuthorizationEnabled());
         workerConfig.setAuthorizationProvider(config.getAuthorizationProvider());
 
-        List<String> urlPatterns =
-                List.of(getPulsarApiExamplesJar().getParentFile().toURI() + ".*", "http://127\\.0\\.0\\.1:.*");
-        workerConfig.setAdditionalEnabledConnectorUrlPatterns(urlPatterns);
-        workerConfig.setAdditionalEnabledFunctionsUrlPatterns(urlPatterns);
-
         PulsarWorkerService workerService = new PulsarWorkerService();
+        workerService.init(workerConfig, null, false);
         return workerService;
     }
 
