@@ -218,26 +218,11 @@ public class PulsarFunctionPublishTest {
     void shutdown() throws Exception {
         try {
             log.info("--- Shutting down ---");
-            if (pulsarClient != null) {
-                pulsarClient.close();
-                pulsarClient = null;
-            }
-            if (admin != null) {
-                admin.close();
-                admin = null;
-            }
-            if (functionsWorkerService != null) {
-                functionsWorkerService.stop();
-                functionsWorkerService = null;
-            }
-            if (pulsar != null) {
-                pulsar.close();
-                pulsar = null;
-            }
-            if (bkEnsemble != null) {
-                bkEnsemble.stop();
-                bkEnsemble = null;
-            }
+            pulsarClient.close();
+            admin.close();
+            functionsWorkerService.stop();
+            pulsar.close();
+            bkEnsemble.stop();
         } finally {
             if (tempDirectory != null) {
                 tempDirectory.delete();
@@ -283,11 +268,8 @@ public class PulsarFunctionPublishTest {
         workerConfig.setAuthenticationEnabled(true);
         workerConfig.setAuthorizationEnabled(true);
 
-        List<String> urlPatterns = List.of(getPulsarApiExamplesJar().getParentFile().toURI() + ".*");
-        workerConfig.setAdditionalEnabledConnectorUrlPatterns(urlPatterns);
-        workerConfig.setAdditionalEnabledFunctionsUrlPatterns(urlPatterns);
-
         PulsarWorkerService workerService = new PulsarWorkerService();
+        workerService.init(workerConfig, null, false);
         return workerService;
     }
 
