@@ -19,21 +19,17 @@
 package org.apache.pulsar.client.impl;
 
 import static java.lang.String.format;
-import static org.apache.pulsar.client.api.PulsarClientException.FailedFeatureCheck.SupportsGetPartitionedMetadataWithoutAutoCreation;
 import io.netty.buffer.ByteBuf;
-import io.netty.util.concurrent.DefaultThreadFactory;
 import java.net.InetSocketAddress;
 import java.net.URI;
-import java.util.Objects;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
-import org.apache.commons.lang3.mutable.MutableObject;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.SchemaSerializationException;
@@ -47,8 +43,6 @@ import org.apache.pulsar.common.partition.PartitionedTopicMetadata;
 import org.apache.pulsar.common.protocol.Commands;
 import org.apache.pulsar.common.protocol.schema.BytesSchemaVersion;
 import org.apache.pulsar.common.schema.SchemaInfo;
-import org.apache.pulsar.common.util.Backoff;
-import org.apache.pulsar.common.util.BackoffBuilder;
 import org.apache.pulsar.common.util.FutureUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -58,68 +52,31 @@ public class BinaryProtoLookupService implements LookupService {
     private final PulsarClientImpl client;
     private final ServiceNameResolver serviceNameResolver;
     private final boolean useTls;
-    private final ExecutorService scheduleExecutor;
+    private final ExecutorService executor;
     private final String listenerName;
     private final int maxLookupRedirects;
-    private final ExecutorService lookupPinnedExecutor;
-    private final boolean createdLookupPinnedExecutor;
 
-    private final ConcurrentHashMap<TopicName, CompletableFuture<Pair<InetSocketAddress, InetSocketAddress>>>
-            lookupInProgress = new ConcurrentHashMap<>();
-
-    private final ConcurrentHashMap<PartitionedTopicMetadataKey, CompletableFuture<PartitionedTopicMetadata>>
-            partitionedMetadataInProgress = new ConcurrentHashMap<>();
-
-    /**
-     * @deprecated use {@link
-     * #BinaryProtoLookupService(PulsarClientImpl, String, String, boolean, ExecutorService, ExecutorService)} instead.
-     */
-    @Deprecated
     public BinaryProtoLookupService(PulsarClientImpl client,
                                     String serviceUrl,
                                     boolean useTls,
-                                    ExecutorService scheduleExecutor)
+                                    ExecutorService executor)
             throws PulsarClientException {
-        this(client, serviceUrl, null, useTls, scheduleExecutor);
-    }
-
-    /**
-     * @deprecated use {@link
-     * #BinaryProtoLookupService(PulsarClientImpl, String, String, boolean, ExecutorService, ExecutorService)} instead.
-     */
-    @Deprecated
-    public BinaryProtoLookupService(PulsarClientImpl client,
-                                    String serviceUrl,
-                                    String listenerName,
-                                    boolean useTls,
-                                    ExecutorService scheduleExecutor)
-            throws PulsarClientException {
-        this(client, serviceUrl, listenerName, useTls, scheduleExecutor, null);
+        this(client, serviceUrl, null, useTls, executor);
     }
 
     public BinaryProtoLookupService(PulsarClientImpl client,
                                     String serviceUrl,
                                     String listenerName,
                                     boolean useTls,
-                                    ExecutorService scheduleExecutor,
-                                    ExecutorService lookupPinnedExecutor)
+                                    ExecutorService executor)
             throws PulsarClientException {
         this.client = client;
         this.useTls = useTls;
-        this.scheduleExecutor = scheduleExecutor;
+        this.executor = executor;
         this.maxLookupRedirects = client.getConfiguration().getMaxLookupRedirects();
         this.serviceNameResolver = new PulsarServiceNameResolver();
         this.listenerName = listenerName;
         updateServiceUrl(serviceUrl);
-
-        if (lookupPinnedExecutor == null) {
-            this.createdLookupPinnedExecutor = true;
-            this.lookupPinnedExecutor =
-                    Executors.newSingleThreadExecutor(new DefaultThreadFactory("pulsar-client-binary-proto-lookup"));
-        } else {
-            this.createdLookupPinnedExecutor = false;
-            this.lookupPinnedExecutor = lookupPinnedExecutor;
-        }
     }
 
     @Override
@@ -135,48 +92,15 @@ public class BinaryProtoLookupService implements LookupService {
      * @return broker-socket-address that serves given topic
      */
     public CompletableFuture<Pair<InetSocketAddress, InetSocketAddress>> getBroker(TopicName topicName) {
-        final MutableObject<CompletableFuture> newFutureCreated = new MutableObject<>();
-        try {
-            return lookupInProgress.computeIfAbsent(topicName, tpName -> {
-                CompletableFuture<Pair<InetSocketAddress, InetSocketAddress>> newFuture =
-                        findBroker(serviceNameResolver.resolveHost(), false, topicName, 0);
-                newFutureCreated.setValue(newFuture);
-                return newFuture;
-            });
-        } finally {
-            if (newFutureCreated.getValue() != null) {
-                newFutureCreated.getValue().whenComplete((v, ex) -> {
-                    lookupInProgress.remove(topicName, newFutureCreated.getValue());
-                });
-            }
-        }
+        return findBroker(serviceNameResolver.resolveHost(), false, topicName, 0);
     }
 
     /**
      * calls broker binaryProto-lookup api to get metadata of partitioned-topic.
      *
      */
-    @Override
-    public CompletableFuture<PartitionedTopicMetadata> getPartitionedTopicMetadata(
-            TopicName topicName, boolean metadataAutoCreationEnabled, boolean useFallbackForNonPIP344Brokers) {
-        final MutableObject<CompletableFuture> newFutureCreated = new MutableObject<>();
-        final PartitionedTopicMetadataKey key = new PartitionedTopicMetadataKey(
-                topicName, metadataAutoCreationEnabled, useFallbackForNonPIP344Brokers);
-        try {
-            return partitionedMetadataInProgress.computeIfAbsent(key, k -> {
-                CompletableFuture<PartitionedTopicMetadata> newFuture = getPartitionedTopicMetadata(
-                        serviceNameResolver.resolveHost(), topicName, metadataAutoCreationEnabled,
-                        useFallbackForNonPIP344Brokers);
-                newFutureCreated.setValue(newFuture);
-                return newFuture;
-            });
-        } finally {
-            if (newFutureCreated.getValue() != null) {
-                newFutureCreated.getValue().whenComplete((v, ex) -> {
-                    partitionedMetadataInProgress.remove(key, newFutureCreated.getValue());
-                });
-            }
-        }
+    public CompletableFuture<PartitionedTopicMetadata> getPartitionedTopicMetadata(TopicName topicName) {
+        return getPartitionedTopicMetadata(serviceNameResolver.resolveHost(), topicName);
     }
 
     private CompletableFuture<Pair<InetSocketAddress, InetSocketAddress>> findBroker(InetSocketAddress socketAddress,
@@ -189,7 +113,7 @@ public class BinaryProtoLookupService implements LookupService {
             return addressFuture;
         }
 
-        client.getCnxPool().getConnection(socketAddress).thenAcceptAsync(clientCnx -> {
+        client.getCnxPool().getConnection(socketAddress).thenAccept(clientCnx -> {
             long requestId = client.newRequestId();
             ByteBuf request = Commands.newLookup(topicName.toString(), listenerName, authoritative, requestId);
             clientCnx.newLookup(request, requestId).whenComplete((r, t) -> {
@@ -254,7 +178,7 @@ public class BinaryProtoLookupService implements LookupService {
                 }
                 client.getCnxPool().releaseConnection(clientCnx);
             });
-        }, lookupPinnedExecutor).exceptionally(connectionException -> {
+        }).exceptionally(connectionException -> {
             addressFuture.completeExceptionally(FutureUtil.unwrapCompletionException(connectionException));
             return null;
         });
@@ -262,31 +186,13 @@ public class BinaryProtoLookupService implements LookupService {
     }
 
     private CompletableFuture<PartitionedTopicMetadata> getPartitionedTopicMetadata(InetSocketAddress socketAddress,
-            TopicName topicName, boolean metadataAutoCreationEnabled, boolean useFallbackForNonPIP344Brokers) {
+            TopicName topicName) {
 
         CompletableFuture<PartitionedTopicMetadata> partitionFuture = new CompletableFuture<>();
 
-        client.getCnxPool().getConnection(socketAddress).thenAcceptAsync(clientCnx -> {
-            boolean finalAutoCreationEnabled = metadataAutoCreationEnabled;
-            if (!metadataAutoCreationEnabled && !clientCnx.isSupportsGetPartitionedMetadataWithoutAutoCreation()) {
-                if (useFallbackForNonPIP344Brokers) {
-                    log.info("[{}] Using original behavior of getPartitionedTopicMetadata(topic) in "
-                            + "getPartitionedTopicMetadata(topic, false) "
-                            + "since the target broker does not support PIP-344 and fallback is enabled.", topicName);
-                    finalAutoCreationEnabled = true;
-                } else {
-                    partitionFuture.completeExceptionally(
-                            new PulsarClientException.FeatureNotSupportedException("The feature of "
-                                    + "getting partitions without auto-creation is not supported by the broker. "
-                                    + "Please upgrade the broker to version that supports PIP-344 to resolve this "
-                                    + "issue.",
-                                    SupportsGetPartitionedMetadataWithoutAutoCreation));
-                    return;
-                }
-            }
+        client.getCnxPool().getConnection(socketAddress).thenAccept(clientCnx -> {
             long requestId = client.newRequestId();
-            ByteBuf request = Commands.newPartitionMetadataRequest(topicName.toString(), requestId,
-                    finalAutoCreationEnabled);
+            ByteBuf request = Commands.newPartitionMetadataRequest(topicName.toString(), requestId);
             clientCnx.newLookup(request, requestId).whenComplete((r, t) -> {
                 if (t != null) {
                     log.warn("[{}] failed to get Partitioned metadata : {}", topicName,
@@ -305,7 +211,7 @@ public class BinaryProtoLookupService implements LookupService {
                 }
                 client.getCnxPool().releaseConnection(clientCnx);
             });
-        }, lookupPinnedExecutor).exceptionally(connectionException -> {
+        }).exceptionally(connectionException -> {
             partitionFuture.completeExceptionally(FutureUtil.unwrapCompletionException(connectionException));
             return null;
         });
@@ -327,7 +233,7 @@ public class BinaryProtoLookupService implements LookupService {
             return schemaFuture;
         }
         InetSocketAddress socketAddress = serviceNameResolver.resolveHost();
-        client.getCnxPool().getConnection(socketAddress).thenAcceptAsync(clientCnx -> {
+        client.getCnxPool().getConnection(socketAddress).thenAccept(clientCnx -> {
             long requestId = client.newRequestId();
             ByteBuf request = Commands.newGetSchema(requestId, topicName.toString(),
                 Optional.ofNullable(BytesSchemaVersion.of(version)));
@@ -341,7 +247,7 @@ public class BinaryProtoLookupService implements LookupService {
                 }
                 client.getCnxPool().releaseConnection(clientCnx);
             });
-        }, lookupPinnedExecutor).exceptionally(ex -> {
+        }).exceptionally(ex -> {
             schemaFuture.completeExceptionally(FutureUtil.unwrapCompletionException(ex));
             return null;
         });
@@ -384,7 +290,7 @@ public class BinaryProtoLookupService implements LookupService {
                                          Mode mode,
                                          String topicsPattern,
                                          String topicsHash) {
-        client.getCnxPool().getConnection(socketAddress).thenAcceptAsync(clientCnx -> {
+        client.getCnxPool().getConnection(socketAddress).thenAccept(clientCnx -> {
             long requestId = client.newRequestId();
             ByteBuf request = Commands.newGetTopicsOfNamespaceRequest(
                 namespace.toString(), requestId, mode, topicsPattern, topicsHash);
@@ -397,11 +303,21 @@ public class BinaryProtoLookupService implements LookupService {
                         log.debug("[namespace: {}] Success get topics list in request: {}",
                                 namespace, requestId);
                     }
-                    getTopicsResultFuture.complete(r);
+                    // do not keep partition part of topic name
+                    List<String> result = new ArrayList<>();
+                    r.getTopics().forEach(topic -> {
+                        String filtered = TopicName.get(topic).getPartitionedTopicName();
+                        if (!result.contains(filtered)) {
+                            result.add(filtered);
+                        }
+                    });
+
+                    getTopicsResultFuture.complete(new GetTopicsResult(result, r.getTopicsHash(),
+                            r.isFiltered(), r.isChanged()));
                 }
                 client.getCnxPool().releaseConnection(clientCnx);
             });
-        }, lookupPinnedExecutor).exceptionally((e) -> {
+        }).exceptionally((e) -> {
             long nextDelay = Math.min(backoff.next(), remainingTime.get());
             if (nextDelay <= 0) {
                 getTopicsResultFuture.completeExceptionally(
@@ -411,7 +327,7 @@ public class BinaryProtoLookupService implements LookupService {
                 return null;
             }
 
-            ((ScheduledExecutorService) scheduleExecutor).schedule(() -> {
+            ((ScheduledExecutorService) executor).schedule(() -> {
                 log.warn("[namespace: {}] Could not get connection while getTopicsUnderNamespace -- Will try again in"
                                 + " {} ms", namespace, nextDelay);
                 remainingTime.addAndGet(-nextDelay);
@@ -425,9 +341,7 @@ public class BinaryProtoLookupService implements LookupService {
 
     @Override
     public void close() throws Exception {
-        if (createdLookupPinnedExecutor && lookupPinnedExecutor != null && !lookupPinnedExecutor.isShutdown()) {
-            lookupPinnedExecutor.shutdown();
-        }
+        // no-op
     }
 
     public static class LookupDataResult {
@@ -460,48 +374,5 @@ public class BinaryProtoLookupService implements LookupService {
 
     }
 
-    private static final class PartitionedTopicMetadataKey {
-        private final TopicName topicName;
-        private final boolean metadataAutoCreationEnabled;
-        private final boolean useFallbackForNonPIP344Brokers;
-
-        PartitionedTopicMetadataKey(TopicName topicName,
-                               boolean metadataAutoCreationEnabled,
-                               boolean useFallbackForNonPIP344Brokers) {
-            this.topicName = topicName;
-            this.metadataAutoCreationEnabled = metadataAutoCreationEnabled;
-            this.useFallbackForNonPIP344Brokers = useFallbackForNonPIP344Brokers;
-        }
-
-        @Override
-        public boolean equals(Object o) {
-            if (this == o) {
-                return true;
-            }
-            if (o == null || getClass() != o.getClass()) {
-                return false;
-            }
-            PartitionedTopicMetadataKey that = (PartitionedTopicMetadataKey) o;
-            return metadataAutoCreationEnabled == that.metadataAutoCreationEnabled
-                    && useFallbackForNonPIP344Brokers == that.useFallbackForNonPIP344Brokers
-                    && Objects.equals(topicName, that.topicName);
-        }
-
-        @Override
-        public int hashCode() {
-            return Objects.hash(topicName, metadataAutoCreationEnabled, useFallbackForNonPIP344Brokers);
-        }
-
-        @Override
-        public String toString() {
-            return "PartitionedTopicMetadataKey{"
-                    + "topicName=" + topicName
-                    + ", metadataAutoCreationEnabled=" + metadataAutoCreationEnabled
-                    + ", useFallbackForNonPIP344Brokers=" + useFallbackForNonPIP344Brokers
-                    + '}';
-        }
-    }
-
-
     private static final Logger log = LoggerFactory.getLogger(BinaryProtoLookupService.class);
 }
