@@ -29,6 +29,7 @@ import io.netty.channel.Channel;
 import io.netty.channel.ChannelHandlerContext;
 import io.netty.channel.EventLoopGroup;
 import io.netty.channel.unix.Errors.NativeIoException;
+import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
 import io.netty.util.concurrent.Promise;
 import java.net.InetSocketAddress;
 import java.net.SocketAddress;
@@ -95,7 +96,6 @@ import org.apache.pulsar.common.api.proto.CommandWatchTopicUpdate;
 import org.apache.pulsar.common.api.proto.ServerError;
 import org.apache.pulsar.common.lookup.GetTopicsResult;
 import org.apache.pulsar.common.protocol.Commands;
-import org.apache.pulsar.common.protocol.FrameDecoderUtil;
 import org.apache.pulsar.common.protocol.PulsarHandler;
 import org.apache.pulsar.common.protocol.schema.SchemaVersion;
 import org.apache.pulsar.common.schema.SchemaInfo;
@@ -116,12 +116,10 @@ public class ClientCnx extends PulsarHandler {
     protected final Authentication authentication;
     protected State state;
 
-    @VisibleForTesting
-    protected AtomicLong duplicatedResponseCounter = new AtomicLong(0);
+    private AtomicLong duplicatedResponseCounter = new AtomicLong(0);
 
-    @VisibleForTesting
     @Getter
-    protected final ConcurrentLongHashMap<TimedCompletableFuture<? extends Object>> pendingRequests =
+    private final ConcurrentLongHashMap<TimedCompletableFuture<? extends Object>> pendingRequests =
             ConcurrentLongHashMap.<TimedCompletableFuture<? extends Object>>newBuilder()
                     .expectedItems(16)
                     .concurrencyLevel(1)
@@ -168,7 +166,8 @@ public class ClientCnx extends PulsarHandler {
     private volatile int numberOfRejectRequests = 0;
 
     @Getter
-    private int maxMessageSize = Commands.DEFAULT_MAX_MESSAGE_SIZE;
+    private static int maxMessageSize = Commands.DEFAULT_MAX_MESSAGE_SIZE;
+
     private final int maxNumberOfRejectedRequestPerConnection;
     private final int rejectedRequestResetTimeSec = 60;
     protected final int protocolVersion;
@@ -187,10 +186,6 @@ public class ClientCnx extends PulsarHandler {
     protected AuthenticationDataProvider authenticationDataProvider;
     private TransactionBufferHandler transactionBufferHandler;
     private boolean supportsTopicWatchers;
-    @Getter
-    private boolean supportsGetPartitionedMetadataWithoutAutoCreation;
-    @Getter
-    private boolean brokerSupportsReplDedupByLidAndEid;
 
     /** Idle stat. **/
     @Getter
@@ -199,7 +194,7 @@ public class ClientCnx extends PulsarHandler {
     @Getter
     private long lastDisconnectedTimestamp;
 
-    protected final String clientVersion;
+    private final String clientVersion;
 
     protected enum State {
         None, SentConnectFrame, Ready, Failed, Connecting
@@ -378,7 +373,8 @@ public class ClientCnx extends PulsarHandler {
                           + "server frame size {}", ctx.channel(), connected.getMaxMessageSize());
             }
             maxMessageSize = connected.getMaxMessageSize();
-            FrameDecoderUtil.replaceFrameDecoder(ctx.pipeline(), connected.getMaxMessageSize());
+            ctx.pipeline().replace("frameDecoder", "newFrameDecoder", new LengthFieldBasedFrameDecoder(
+                connected.getMaxMessageSize() + Commands.MESSAGE_SIZE_FRAME_PADDING, 0, 4, 0, 4));
         }
         if (log.isDebugEnabled()) {
             log.debug("{} Connection is ready", ctx.channel());
@@ -386,11 +382,6 @@ public class ClientCnx extends PulsarHandler {
 
         supportsTopicWatchers =
             connected.hasFeatureFlags() && connected.getFeatureFlags().isSupportsTopicWatchers();
-        supportsGetPartitionedMetadataWithoutAutoCreation =
-            connected.hasFeatureFlags()
-                    && connected.getFeatureFlags().isSupportsGetPartitionedMetadataWithoutAutoCreation();
-        brokerSupportsReplDedupByLidAndEid =
-            connected.hasFeatureFlags() && connected.getFeatureFlags().isSupportsReplDedupByLidAndEid();
 
         // set remote protocol version to the correct version before we complete the connection future
         setRemoteEndpointProtocolVersion(connected.getProtocolVersion());
@@ -460,23 +451,18 @@ public class ClientCnx extends PulsarHandler {
             ledgerId = sendReceipt.getMessageId().getLedgerId();
             entryId = sendReceipt.getMessageId().getEntryId();
         }
-        ProducerImpl<?> producer = producers.get(producerId);
-        if (ledgerId == -1 && entryId == -1) {
-            if (producer == null) {
-                log.warn("{} Message with sequence-id {}-{} published by producer [id:{}, name:{}] has been dropped",
-                        ctx.channel(), sequenceId, highestSequenceId, producerId, "null");
-            } else {
-                producer.printWarnLogWhenCanNotDetermineDeduplication(ctx.channel(), sequenceId, highestSequenceId);
-            }
 
-        } else {
-            if (log.isDebugEnabled()) {
-                log.debug("{} Got receipt for producer: [id:{}, name:{}] -- sequence-id: {}-{} -- entry-id: {}:{}",
-                        ctx.channel(), producerId, producer.getProducerName(), sequenceId, highestSequenceId,
-                        ledgerId, entryId);
-            }
+        if (ledgerId == -1 && entryId == -1) {
+            log.warn("{} Message with sequence-id {} published by producer {} has been dropped", ctx.channel(),
+                    sequenceId, producerId);
         }
 
+        if (log.isDebugEnabled()) {
+            log.debug("{} Got receipt for producer: {} -- msg: {} -- id: {}:{}", ctx.channel(), producerId, sequenceId,
+                    ledgerId, entryId);
+        }
+
+        ProducerImpl<?> producer = producers.get(producerId);
         if (producer != null) {
             producer.ackReceived(this, sequenceId, highestSequenceId, ledgerId, entryId);
         } else {
@@ -628,12 +614,10 @@ public class ClientCnx extends PulsarHandler {
             if (!lookupResult.hasResponse()
                     || CommandLookupTopicResponse.LookupType.Failed.equals(lookupResult.getResponse())) {
                 if (lookupResult.hasError()) {
-                    checkServerError(lookupResult.getError(),
-                            lookupResult.hasMessage() ? lookupResult.getMessage() : lookupResult.getError().name());
+                    checkServerError(lookupResult.getError(), lookupResult.getMessage());
                     requestFuture.completeExceptionally(
                             getPulsarClientException(lookupResult.getError(),
-                                    buildError(lookupResult.getRequestId(),
-                                            lookupResult.hasMessage() ? lookupResult.getMessage() : null)));
+                                    buildError(lookupResult.getRequestId(), lookupResult.getMessage())));
                 } else {
                     requestFuture
                             .completeExceptionally(new PulsarClientException.LookupException("Empty lookup response"));
@@ -765,21 +749,16 @@ public class ClientCnx extends PulsarHandler {
         long producerId = sendError.getProducerId();
         long sequenceId = sendError.getSequenceId();
 
-        ProducerImpl<?> producer = producers.get(producerId);
-        if (producer == null) {
-            log.warn("{} Producer with id {} not found while handling send error", ctx.channel(), producerId);
-            return;
-        }
-
         switch (sendError.getError()) {
         case ChecksumError:
-            producer.recoverChecksumError(this, sequenceId);
+            producers.get(producerId).recoverChecksumError(this, sequenceId);
             break;
+
         case TopicTerminatedError:
-            producer.terminated(this);
+            producers.get(producerId).terminated(this);
             break;
         case NotAllowedError:
-            producer.recoverNotAllowedError(sequenceId, sendError.getMessage());
+            producers.get(producerId).recoverNotAllowedError(sequenceId, sendError.getMessage());
             break;
 
         default:
@@ -875,7 +854,6 @@ public class ClientCnx extends PulsarHandler {
             if (maxLookupRequestSemaphore.tryAcquire()) {
                 waitingLookupRequests.add(Pair.of(requestId, Pair.of(request, future)));
             } else {
-                request.release();
                 if (log.isDebugEnabled()) {
                     log.debug("{} Failed to add lookup-request into waiting queue", requestId);
                 }
@@ -966,8 +944,7 @@ public class ClientCnx extends PulsarHandler {
         return ctx;
     }
 
-    @VisibleForTesting
-    protected Channel channel() {
+    Channel channel() {
         return ctx.channel();
     }
 
@@ -1177,7 +1154,7 @@ public class ClientCnx extends PulsarHandler {
         CompletableFuture<CommandWatchTopicListSuccess> requestFuture =
                 (CompletableFuture<CommandWatchTopicListSuccess>) pendingRequests.remove(requestId);
         if (requestFuture != null) {
-            requestFuture.complete(new CommandWatchTopicListSuccess().copyFrom(commandWatchTopicListSuccess));
+            requestFuture.complete(commandWatchTopicListSuccess);
         } else {
             duplicatedResponseCounter.incrementAndGet();
             log.warn("{} Received unknown request id from server: {}",
@@ -1301,7 +1278,7 @@ public class ClientCnx extends PulsarHandler {
         case PersistenceError:
             return new PulsarClientException.BrokerPersistenceException(errorMsg);
         case ServiceNotReady:
-            return new PulsarClientException.ServiceNotReadyException(errorMsg);
+            return new PulsarClientException.LookupException(errorMsg);
         case TooManyRequests:
             return new PulsarClientException.TooManyRequestsException(errorMsg);
         case ProducerBlockedQuotaExceededError:
@@ -1314,8 +1291,6 @@ public class ClientCnx extends PulsarHandler {
             return new PulsarClientException.IncompatibleSchemaException(errorMsg);
         case TopicNotFound:
             return new PulsarClientException.TopicDoesNotExistException(errorMsg);
-        case SubscriptionNotFound:
-            return new PulsarClientException.SubscriptionNotFoundException(errorMsg);
         case ConsumerAssignError:
             return new PulsarClientException.ConsumerAssignException(errorMsg);
         case NotAllowedError:
@@ -1330,47 +1305,6 @@ public class ClientCnx extends PulsarHandler {
         }
     }
 
-    public static ServerError revertClientExToErrorCode(PulsarClientException ex) {
-        if (ex instanceof PulsarClientException.AuthenticationException) {
-            return ServerError.AuthenticationError;
-        } else if (ex instanceof PulsarClientException.AuthorizationException) {
-            return ServerError.AuthorizationError;
-        } else if (ex instanceof PulsarClientException.ProducerBusyException) {
-            return ServerError.ProducerBusy;
-        } else if (ex instanceof PulsarClientException.ConsumerBusyException) {
-            return ServerError.ConsumerBusy;
-        } else if (ex instanceof PulsarClientException.BrokerMetadataException) {
-            return ServerError.MetadataError;
-        } else if (ex instanceof PulsarClientException.BrokerPersistenceException) {
-            return ServerError.PersistenceError;
-        } else if (ex instanceof PulsarClientException.TooManyRequestsException) {
-            return ServerError.TooManyRequests;
-        } else if (ex instanceof PulsarClientException.LookupException) {
-            return ServerError.ServiceNotReady;
-        } else if (ex instanceof PulsarClientException.ProducerBlockedQuotaExceededError) {
-            return ServerError.ProducerBlockedQuotaExceededError;
-        } else if (ex instanceof PulsarClientException.ProducerBlockedQuotaExceededException) {
-            return ServerError.ProducerBlockedQuotaExceededException;
-        } else if (ex instanceof PulsarClientException.TopicTerminatedException) {
-            return ServerError.TopicTerminatedError;
-        } else if (ex instanceof PulsarClientException.IncompatibleSchemaException) {
-            return ServerError.IncompatibleSchema;
-        } else if (ex instanceof PulsarClientException.TopicDoesNotExistException) {
-            return ServerError.TopicNotFound;
-        } else if (ex instanceof PulsarClientException.SubscriptionNotFoundException) {
-            return ServerError.SubscriptionNotFound;
-        } else if (ex instanceof PulsarClientException.ConsumerAssignException) {
-            return ServerError.ConsumerAssignError;
-        } else if (ex instanceof PulsarClientException.NotAllowedException) {
-            return ServerError.NotAllowedError;
-        } else if (ex instanceof PulsarClientException.TransactionConflictException) {
-            return ServerError.TransactionConflict;
-        } else if (ex instanceof PulsarClientException.ProducerFencedException) {
-            return ServerError.ProducerFenced;
-        }
-        return ServerError.UnknownError;
-    }
-
     public void close() {
        if (ctx != null) {
            ctx.close();
@@ -1437,9 +1371,6 @@ public class ClientCnx extends PulsarHandler {
         if (!transactionMetaStoreHandlers.isEmpty()) {
             return false;
         }
-        if (!topicListWatchers.isEmpty()) {
-            return false;
-        }
         return true;
     }
 }
