@@ -18,19 +18,14 @@
  */
 package org.apache.pulsar.client.impl;
 
-import com.google.common.annotations.VisibleForTesting;
 import java.net.InetSocketAddress;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicLongFieldUpdater;
 import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
-import lombok.Getter;
-import lombok.Setter;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.impl.HandlerState.State;
-import org.apache.pulsar.common.protocol.Commands;
-import org.apache.pulsar.common.util.Backoff;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -39,10 +34,6 @@ public class ConnectionHandler {
             AtomicReferenceFieldUpdater.newUpdater(ConnectionHandler.class, ClientCnx.class, "clientCnx");
     @SuppressWarnings("unused")
     private volatile ClientCnx clientCnx = null;
-    @Getter
-    @Setter
-    // Since the `clientCnx` variable will be set to null at some times, it is necessary to save this value here.
-    private volatile int maxMessageSize = Commands.DEFAULT_MAX_MESSAGE_SIZE;
 
     protected final HandlerState state;
     protected final Backoff backoff;
@@ -52,7 +43,6 @@ public class ConnectionHandler {
     private volatile long epoch = -1L;
     protected volatile long lastConnectionClosedTimestamp = 0L;
     private final AtomicBoolean duringConnect = new AtomicBoolean(false);
-    protected final int randomKeyForSelectConnection;
 
     interface Connection {
 
@@ -60,15 +50,7 @@ public class ConnectionHandler {
          * @apiNote If the returned future is completed exceptionally, reconnectLater will be called.
          */
         CompletableFuture<Void> connectionOpened(ClientCnx cnx);
-
-        /**
-         *
-         * @param e What error happened when tries to get a connection
-         * @return If "true", the connection handler will retry to get a connection, otherwise, it stops to get a new
-         * connection. If it returns "false", you should release resources that consumers/producers occupied.
-         */
-        default boolean connectionFailed(PulsarClientException e) {
-            return true;
+        default void connectionFailed(PulsarClientException e) {
         }
     }
 
@@ -76,7 +58,6 @@ public class ConnectionHandler {
 
     protected ConnectionHandler(HandlerState state, Backoff backoff, Connection connection) {
         this.state = state;
-        this.randomKeyForSelectConnection = state.client.getCnxPool().genRandomKeyToSelectCon();
         this.connection = connection;
         this.backoff = backoff;
         CLIENT_CNX_UPDATER.set(this, null);
@@ -105,20 +86,13 @@ public class ConnectionHandler {
         try {
             CompletableFuture<ClientCnx> cnxFuture;
             if (state.redirectedClusterURI != null) {
-                if (state.topic == null) {
-                    InetSocketAddress address = InetSocketAddress.createUnresolved(state.redirectedClusterURI.getHost(),
-                            state.redirectedClusterURI.getPort());
-                    cnxFuture = state.client.getConnection(address, address, randomKeyForSelectConnection);
-                } else {
-                    // once, client receives redirection url, client has to perform lookup on migrated
-                    // cluster to find the broker that owns the topic and then create connection.
-                    // below method, performs the lookup for a given topic and then creates connection
-                    cnxFuture = state.client.getConnection(state.topic, (state.redirectedClusterURI.toString()));
-                }
+                InetSocketAddress address = InetSocketAddress.createUnresolved(state.redirectedClusterURI.getHost(),
+                        state.redirectedClusterURI.getPort());
+                cnxFuture = state.client.getConnection(address, address);
             } else if (state.topic == null) {
                 cnxFuture = state.client.getConnectionToServiceUrl();
             } else {
-                cnxFuture = state.client.getConnection(state.topic, randomKeyForSelectConnection);
+                cnxFuture = state.client.getConnection(state.topic); //
             }
             cnxFuture.thenCompose(cnx -> connection.connectionOpened(cnx))
                     .thenAccept(__ -> duringConnect.set(false))
@@ -130,24 +104,22 @@ public class ConnectionHandler {
     }
 
     private Void handleConnectionError(Throwable exception) {
-        boolean toRetry = true;
         try {
             log.warn("[{}] [{}] Error connecting to broker: {}",
                     state.topic, state.getHandlerName(), exception.getMessage());
             if (exception instanceof PulsarClientException) {
-                toRetry = connection.connectionFailed((PulsarClientException) exception);
+                connection.connectionFailed((PulsarClientException) exception);
             } else if (exception.getCause() instanceof PulsarClientException) {
-                toRetry = connection.connectionFailed((PulsarClientException) exception.getCause());
+                connection.connectionFailed((PulsarClientException) exception.getCause());
             } else {
-                toRetry = connection.connectionFailed(new PulsarClientException(exception));
+                connection.connectionFailed(new PulsarClientException(exception));
             }
         } catch (Throwable throwable) {
             log.error("[{}] [{}] Unexpected exception after the connection",
                     state.topic, state.getHandlerName(), throwable);
         }
-        if (toRetry) {
-            reconnectLater(exception);
-        }
+
+        reconnectLater(exception);
         return null;
     }
 
@@ -179,12 +151,13 @@ public class ConnectionHandler {
         duringConnect.set(false);
         state.client.getCnxPool().releaseConnection(cnx);
         if (CLIENT_CNX_UPDATER.compareAndSet(this, cnx, null)) {
-            if (!state.changeToConnecting()) {
+            if (!isValidStateForReconnection()) {
                 log.info("[{}] [{}] Ignoring reconnection request (state: {})",
                         state.topic, state.getHandlerName(), state.getState());
                 return;
             }
             long delayMs = backoff.next();
+            state.setState(State.Connecting);
             log.info("[{}] [{}] Closed connection {} -- Will try again in {} s",
                     state.topic, state.getHandlerName(), cnx.channel(),
                     delayMs / 1000.0);
@@ -218,8 +191,7 @@ public class ConnectionHandler {
         return EPOCH_UPDATER.incrementAndGet(this);
     }
 
-    @VisibleForTesting
-    public boolean isValidStateForReconnection() {
+    private boolean isValidStateForReconnection() {
         State state = this.state.getState();
         switch (state) {
             case Uninitialized:
