@@ -61,7 +61,7 @@ import org.slf4j.LoggerFactory;
 
 public class ConnectionPool implements AutoCloseable {
 
-    public static final int IDLE_DETECTION_INTERVAL_SECONDS_MIN = 15;
+    public static final int IDLE_DETECTION_INTERVAL_SECONDS_MIN = 60;
 
     protected final ConcurrentHashMap<InetSocketAddress, ConcurrentMap<Integer, CompletableFuture<ClientCnx>>> pool;
 
@@ -150,9 +150,7 @@ public class ConnectionPool implements AutoCloseable {
     private static AddressResolver<InetSocketAddress> createAddressResolver(ClientConfigurationData conf,
                                                                             EventLoopGroup eventLoopGroup) {
         DnsNameResolverBuilder dnsNameResolverBuilder = new DnsNameResolverBuilder()
-                .traceEnabled(true)
-                .channelType(EventLoopUtil.getDatagramChannelClass(eventLoopGroup))
-                .socketChannelType(EventLoopUtil.getClientSocketChannelClass(eventLoopGroup), true);
+                .traceEnabled(true).channelType(EventLoopUtil.getDatagramChannelClass(eventLoopGroup));
         if (conf.getDnsLookupBindAddress() != null) {
             InetSocketAddress addr = new InetSocketAddress(conf.getDnsLookupBindAddress(),
                     conf.getDnsLookupBindPort());
@@ -167,18 +165,8 @@ public class ConnectionPool implements AutoCloseable {
 
     private static final Random random = new Random();
 
-    public int genRandomKeyToSelectCon() {
-        if (maxConnectionsPerHosts == 0) {
-            return -1;
-        }
-        return signSafeMod(random.nextInt(), maxConnectionsPerHosts);
-    }
-
     public CompletableFuture<ClientCnx> getConnection(final InetSocketAddress address) {
-        if (maxConnectionsPerHosts == 0) {
-            return getConnection(address, address, -1);
-        }
-        return getConnection(address, address, signSafeMod(random.nextInt(), maxConnectionsPerHosts));
+        return getConnection(address, address);
     }
 
     void closeAllConnections() {
@@ -216,12 +204,14 @@ public class ConnectionPool implements AutoCloseable {
      * @return a future that will produce the ClientCnx object
      */
     public CompletableFuture<ClientCnx> getConnection(InetSocketAddress logicalAddress,
-            InetSocketAddress physicalAddress, final int randomKey) {
+            InetSocketAddress physicalAddress) {
         if (maxConnectionsPerHosts == 0) {
             // Disable pooling
             return createConnection(logicalAddress, physicalAddress, -1);
         }
 
+        final int randomKey = signSafeMod(random.nextInt(), maxConnectionsPerHosts);
+
         final ConcurrentMap<Integer, CompletableFuture<ClientCnx>> innerPool =
                 pool.computeIfAbsent(logicalAddress, a -> new ConcurrentHashMap<>());
         CompletableFuture<ClientCnx> completableFuture = innerPool
