@@ -19,7 +19,6 @@
 package org.apache.pulsar.client.impl;
 
 import static org.apache.pulsar.common.util.Runnables.catchingAndLoggingThrowables;
-import com.google.common.annotations.VisibleForTesting;
 import io.netty.buffer.ByteBuf;
 import io.netty.channel.EventLoopGroup;
 import io.netty.util.concurrent.FastThreadLocal;
@@ -27,12 +26,13 @@ import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.Collections;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ConcurrentSkipListMap;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.ScheduledFuture;
 import java.util.concurrent.TimeUnit;
@@ -52,7 +52,7 @@ import org.apache.pulsar.common.api.proto.CommandAck.AckType;
 import org.apache.pulsar.common.protocol.Commands;
 import org.apache.pulsar.common.util.FutureUtil;
 import org.apache.pulsar.common.util.collections.BitSetRecyclable;
-import org.apache.pulsar.common.util.collections.ConcurrentBitSet;
+import org.apache.pulsar.common.util.collections.ConcurrentBitSetRecyclable;
 
 /**
  * Group the acknowledgements for a certain time and then sends them out in a single protobuf command.
@@ -82,8 +82,7 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
      * broker.
      */
     private final ConcurrentSkipListSet<MessageIdAdv> pendingIndividualAcks;
-    @VisibleForTesting
-    final ConcurrentSkipListMap<MessageIdAdv, ConcurrentBitSet> pendingIndividualBatchIndexAcks;
+    private final ConcurrentHashMap<MessageIdAdv, ConcurrentBitSetRecyclable> pendingIndividualBatchIndexAcks;
 
     private final ScheduledFuture<?> scheduledTask;
     private final boolean batchIndexAckEnabled;
@@ -93,7 +92,7 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
                                                     EventLoopGroup eventLoopGroup) {
         this.consumer = consumer;
         this.pendingIndividualAcks = new ConcurrentSkipListSet<>();
-        this.pendingIndividualBatchIndexAcks = new ConcurrentSkipListMap<>();
+        this.pendingIndividualBatchIndexAcks = new ConcurrentHashMap<>();
         this.acknowledgementGroupTimeMicros = conf.getAcknowledgementsGroupTimeMicros();
         this.maxAckGroupSize = conf.getMaxAcknowledgmentGroupSize();
         this.batchIndexAckEnabled = conf.isBatchIndexAckEnabled();
@@ -125,18 +124,7 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
             // Already included in a cumulative ack
             return true;
         } else {
-            // If "batchIndexAckEnabled" is false, the batched messages acknowledgment will be traced by
-            // pendingIndividualAcks. So no matter what type the message ID is, check with "pendingIndividualAcks"
-            // first.
-            MessageIdAdv key = MessageIdAdvUtils.discardBatch(messageIdAdv);
-            if (pendingIndividualAcks.contains(key)) {
-                return true;
-            }
-            if (messageIdAdv.getBatchIndex() >= 0) {
-                ConcurrentBitSet bitSet = pendingIndividualBatchIndexAcks.get(key);
-                return bitSet != null && !bitSet.get(messageIdAdv.getBatchIndex());
-            }
-            return false;
+            return pendingIndividualAcks.contains(MessageIdAdvUtils.discardBatch(messageIdAdv));
         }
     }
 
@@ -313,36 +301,22 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
             Optional<Lock> readLock = acquireReadLock();
             try {
                 doCumulativeAckAsync(messageId, bitSet);
-                return readLock.map(__ -> {
-                    if (consumer.isAckReceiptEnabled() && lastCumulativeAck.compareTo(messageId) == 0) {
-                        return CompletableFuture.<Void>completedFuture(null);
-                    }
-                    return currentCumulativeAckFuture;
-                }).orElse(CompletableFuture.completedFuture(null));
+                return readLock.map(__ -> currentCumulativeAckFuture).orElse(CompletableFuture.completedFuture(null));
             } finally {
                 readLock.ifPresent(Lock::unlock);
             }
         }
     }
 
-    @VisibleForTesting
-    CompletableFuture<Void> doIndividualBatchAckAsync(MessageIdAdv msgId) {
-        ConcurrentBitSet bitSet = pendingIndividualBatchIndexAcks.computeIfAbsent(
+    private CompletableFuture<Void> doIndividualBatchAckAsync(MessageIdAdv msgId) {
+        ConcurrentBitSetRecyclable bitSet = pendingIndividualBatchIndexAcks.computeIfAbsent(
                 MessageIdAdvUtils.discardBatch(msgId), __ -> {
                     final BitSet ackSet = msgId.getAckSet();
-                    final ConcurrentBitSet value;
-                    if (ackSet != null) {
-                        synchronized (ackSet) {
-                            if (!ackSet.isEmpty()) {
-                                value =  new ConcurrentBitSet();
-                                value.or(ackSet);
-                            } else {
-                                value = new ConcurrentBitSet();
-                                value.set(0, msgId.getBatchSize());
-                            }
-                        }
+                    final ConcurrentBitSetRecyclable value;
+                    if (ackSet != null && !ackSet.isEmpty()) {
+                        value = ConcurrentBitSetRecyclable.create(ackSet);
                     } else {
-                        value = new ConcurrentBitSet();
+                        value = ConcurrentBitSetRecyclable.create();
                         value.set(0, msgId.getBatchSize());
                     }
                     return value;
@@ -389,11 +363,8 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
                     .ConnectException("Consumer connect fail! consumer state:" + consumer.getState()));
         }
         BitSetRecyclable bitSet;
-        BitSet ackSetFromMsgId = msgId.getAckSet();
-        if (ackSetFromMsgId != null) {
-            synchronized (ackSetFromMsgId) {
-                bitSet = BitSetRecyclable.valueOf(ackSetFromMsgId.toLongArray());
-            }
+        if (msgId.getAckSet() != null) {
+            bitSet = BitSetRecyclable.valueOf(msgId.getAckSet().toLongArray());
         } else {
             bitSet = BitSetRecyclable.create();
             bitSet.set(0, batchSize);
@@ -446,7 +417,7 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
         }
 
         // Flush all individual acks
-        List<Triple<Long, Long, ConcurrentBitSet>> entriesToAck =
+        List<Triple<Long, Long, ConcurrentBitSetRecyclable>> entriesToAck =
                 new ArrayList<>(pendingIndividualAcks.size() + pendingIndividualBatchIndexAcks.size());
         if (!pendingIndividualAcks.isEmpty()) {
             if (Commands.peerSupportsMultiMessageAcknowledgment(cnx.getRemoteEndpointProtocolVersion())) {
@@ -487,15 +458,16 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
             }
         }
 
-        while (true) {
-            Map.Entry<MessageIdAdv, ConcurrentBitSet> entry =
-                    pendingIndividualBatchIndexAcks.pollFirstEntry();
-            if (entry == null) {
-                // The entry has been removed in a different thread
-                break;
+        if (!pendingIndividualBatchIndexAcks.isEmpty()) {
+            Iterator<Map.Entry<MessageIdAdv, ConcurrentBitSetRecyclable>> iterator =
+                    pendingIndividualBatchIndexAcks.entrySet().iterator();
+
+            while (iterator.hasNext()) {
+                Map.Entry<MessageIdAdv, ConcurrentBitSetRecyclable> entry = iterator.next();
+                entriesToAck.add(Triple.of(
+                        entry.getKey().getLedgerId(), entry.getKey().getEntryId(), entry.getValue()));
+                iterator.remove();
             }
-            entriesToAck.add(Triple.of(
-                    entry.getKey().getLedgerId(), entry.getKey().getEntryId(), entry.getValue()));
         }
 
         if (entriesToAck.size() > 0) {
@@ -540,7 +512,7 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
         // cumulative ack chunk by the last messageId
         if (chunkMsgIds != null &&  ackType != AckType.Cumulative) {
             if (Commands.peerSupportsMultiMessageAcknowledgment(cnx.getRemoteEndpointProtocolVersion())) {
-                List<Triple<Long, Long, ConcurrentBitSet>> entriesToAck = new ArrayList<>(chunkMsgIds.length);
+                List<Triple<Long, Long, ConcurrentBitSetRecyclable>> entriesToAck = new ArrayList<>(chunkMsgIds.length);
                 for (MessageIdImpl cMsgId : chunkMsgIds) {
                     if (cMsgId != null && chunkMsgIds.length > 1) {
                         entriesToAck.add(Triple.of(cMsgId.getLedgerId(), cMsgId.getEntryId(), null));
@@ -569,7 +541,7 @@ public class PersistentAcknowledgmentsGroupingTracker implements Acknowledgments
             long entryId, BitSetRecyclable ackSet, AckType ackType,
             Map<String, Long> properties, boolean flush,
             TimedCompletableFuture<Void> timedCompletableFuture,
-            List<Triple<Long, Long, ConcurrentBitSet>> entriesToAck) {
+            List<Triple<Long, Long, ConcurrentBitSetRecyclable>> entriesToAck) {
         if (consumer.isAckReceiptEnabled()) {
             final long requestId = consumer.getClient().newRequestId();
             final ByteBuf cmd;
