@@ -32,9 +32,7 @@ import java.time.Clock;
 import java.time.Duration;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
@@ -48,10 +46,8 @@ import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicReference;
-import javax.annotation.Nullable;
 import lombok.Builder;
 import lombok.Getter;
-import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.api.Authentication;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.ConsumerBuilder;
@@ -88,8 +84,6 @@ import org.apache.pulsar.common.partition.PartitionedTopicMetadata;
 import org.apache.pulsar.common.schema.SchemaInfo;
 import org.apache.pulsar.common.schema.SchemaType;
 import org.apache.pulsar.common.topics.TopicList;
-import org.apache.pulsar.common.util.Backoff;
-import org.apache.pulsar.common.util.BackoffBuilder;
 import org.apache.pulsar.common.util.FutureUtil;
 import org.apache.pulsar.common.util.netty.EventLoopUtil;
 import org.slf4j.Logger;
@@ -109,16 +103,13 @@ public class PulsarClientImpl implements PulsarClient {
     private final boolean createdExecutorProviders;
 
     private final boolean createdScheduledProviders;
-    private final boolean createdLookupProviders;
-    private volatile LookupService lookup;
-    private Map<String, LookupService> urlLookupMap = new ConcurrentHashMap<>();
+    private LookupService lookup;
     private final ConnectionPool cnxPool;
     @Getter
     private final Timer timer;
     private boolean needStopTimer;
     private final ExecutorProvider externalExecutorProvider;
     private final ExecutorProvider internalExecutorProvider;
-    private final ExecutorProvider lookupExecutorProvider;
 
     private final ScheduledExecutorProvider scheduledExecutorProvider;
     private final boolean createdEventLoopGroup;
@@ -159,47 +150,32 @@ public class PulsarClientImpl implements PulsarClient {
     private TransactionCoordinatorClientImpl tcClient;
 
     public PulsarClientImpl(ClientConfigurationData conf) throws PulsarClientException {
-        this(conf, null, null, null, null, null, null, null);
+        this(conf, null, null, null, null, null, null);
     }
 
     public PulsarClientImpl(ClientConfigurationData conf, EventLoopGroup eventLoopGroup) throws PulsarClientException {
-        this(conf, eventLoopGroup, null, null, null, null, null, null);
+        this(conf, eventLoopGroup, null, null, null, null, null);
     }
 
     public PulsarClientImpl(ClientConfigurationData conf, EventLoopGroup eventLoopGroup, ConnectionPool cnxPool)
             throws PulsarClientException {
-        this(conf, eventLoopGroup, cnxPool, null, null, null, null, null);
+        this(conf, eventLoopGroup, cnxPool, null, null, null, null);
     }
 
     public PulsarClientImpl(ClientConfigurationData conf, EventLoopGroup eventLoopGroup, ConnectionPool cnxPool,
                             Timer timer)
             throws PulsarClientException {
-        this(conf, eventLoopGroup, cnxPool, timer, null, null, null, null);
-    }
-
-    public PulsarClientImpl(ClientConfigurationData conf, EventLoopGroup eventLoopGroup, ConnectionPool connectionPool,
-                            Timer timer, ExecutorProvider externalExecutorProvider,
-                            ExecutorProvider internalExecutorProvider,
-                            ScheduledExecutorProvider scheduledExecutorProvider)
-            throws PulsarClientException {
-        this(conf, eventLoopGroup, connectionPool, timer, externalExecutorProvider, internalExecutorProvider,
-                scheduledExecutorProvider, null);
+        this(conf, eventLoopGroup, cnxPool, timer, null, null, null);
     }
 
     @Builder(builderClassName = "PulsarClientImplBuilder")
     private PulsarClientImpl(ClientConfigurationData conf, EventLoopGroup eventLoopGroup, ConnectionPool connectionPool,
                              Timer timer, ExecutorProvider externalExecutorProvider,
                              ExecutorProvider internalExecutorProvider,
-                             ScheduledExecutorProvider scheduledExecutorProvider,
-                             ExecutorProvider lookupExecutorProvider) throws PulsarClientException {
-
+                             ScheduledExecutorProvider scheduledExecutorProvider) throws PulsarClientException {
         EventLoopGroup eventLoopGroupReference = null;
         ConnectionPool connectionPoolReference = null;
         try {
-            if (conf == null || isBlank(conf.getServiceUrl())) {
-                throw new PulsarClientException.InvalidConfigurationException("Invalid client configuration");
-            }
-            this.conf = conf;
             this.createdEventLoopGroup = eventLoopGroup == null;
             this.createdCnxPool = connectionPool == null;
             if ((externalExecutorProvider == null) != (internalExecutorProvider == null)) {
@@ -208,9 +184,12 @@ public class PulsarClientImpl implements PulsarClient {
             }
             this.createdExecutorProviders = externalExecutorProvider == null;
             this.createdScheduledProviders = scheduledExecutorProvider == null;
-            this.createdLookupProviders = lookupExecutorProvider == null;
             eventLoopGroupReference = eventLoopGroup != null ? eventLoopGroup : getEventLoopGroup(conf);
             this.eventLoopGroup = eventLoopGroupReference;
+            if (conf == null || isBlank(conf.getServiceUrl())) {
+                throw new PulsarClientException.InvalidConfigurationException("Invalid client configuration");
+            }
+            this.conf = conf;
             clientClock = conf.getClock();
             conf.getAuthentication().start();
             connectionPoolReference =
@@ -220,16 +199,13 @@ public class PulsarClientImpl implements PulsarClient {
                     new ExecutorProvider(conf.getNumListenerThreads(), "pulsar-external-listener");
             this.internalExecutorProvider = internalExecutorProvider != null ? internalExecutorProvider :
                     new ExecutorProvider(conf.getNumIoThreads(), "pulsar-client-internal");
-            this.lookupExecutorProvider = lookupExecutorProvider != null ? lookupExecutorProvider :
-                    new ExecutorProvider(1, "pulsar-client-lookup");
             this.scheduledExecutorProvider = scheduledExecutorProvider != null ? scheduledExecutorProvider :
                     new ScheduledExecutorProvider(conf.getNumIoThreads(), "pulsar-client-scheduled");
             if (conf.getServiceUrl().startsWith("http")) {
                 lookup = new HttpLookupService(conf, this.eventLoopGroup);
             } else {
                 lookup = new BinaryProtoLookupService(this, conf.getServiceUrl(), conf.getListenerName(),
-                        conf.isUseTls(), this.scheduledExecutorProvider.getExecutor(),
-                        this.lookupExecutorProvider.getExecutor());
+                        conf.isUseTls(), this.scheduledExecutorProvider.getExecutor());
             }
             if (timer == null) {
                 this.timer = new HashedWheelTimer(getThreadFactory("pulsar-timer"), 1, TimeUnit.MILLISECONDS);
@@ -238,10 +214,6 @@ public class PulsarClientImpl implements PulsarClient {
                 this.timer = timer;
             }
 
-            if (conf.getServiceUrlProvider() != null) {
-                conf.getServiceUrlProvider().initialize(this);
-            }
-
             if (conf.isEnableTransaction()) {
                 tcClient = new TransactionCoordinatorClientImpl(this);
                 try {
@@ -257,9 +229,6 @@ public class PulsarClientImpl implements PulsarClient {
                     this::reduceConsumerReceiverQueueSize);
             state.set(State.Open);
         } catch (Throwable t) {
-            // Log the exception first, or it could be missed if there are any subsequent exceptions in the
-            // shutdown sequence
-            log.error("Failed to create Pulsar client instance.", t);
             shutdown();
             shutdownEventLoopGroup(eventLoopGroupReference);
             closeCnxPool(connectionPoolReference);
@@ -377,7 +346,7 @@ public class PulsarClientImpl implements PulsarClient {
 
         if (schema instanceof AutoProduceBytesSchema) {
             AutoProduceBytesSchema autoProduceBytesSchema = (AutoProduceBytesSchema) schema;
-            if (autoProduceBytesSchema.hasUserProvidedSchema()) {
+            if (autoProduceBytesSchema.schemaInitialized()) {
                 return createProducerAsync(topic, conf, schema, interceptors);
             }
             return lookup.getSchema(TopicName.get(conf.getTopicName()))
@@ -400,55 +369,26 @@ public class PulsarClientImpl implements PulsarClient {
 
     }
 
-    private CompletableFuture<Integer> checkPartitions(String topic, boolean forceNoPartitioned,
-                                                       @Nullable String producerNameForLog) {
-        CompletableFuture<Integer> checkPartitions = new CompletableFuture<>();
-        getPartitionedTopicMetadata(topic, !forceNoPartitioned, true).thenAccept(metadata -> {
-            if (forceNoPartitioned && metadata.partitions > 0) {
-                String errorMsg = String.format("Can not create the producer[%s] for the topic[%s] that contains %s"
-                                + " partitions b,ut the producer does not support for a partitioned topic.",
-                        producerNameForLog, topic, metadata.partitions);
-                log.error(errorMsg);
-                checkPartitions.completeExceptionally(
-                        new PulsarClientException.LookupException(errorMsg));
-            } else {
-                checkPartitions.complete(metadata.partitions);
-            }
-        }).exceptionally(ex -> {
-            Throwable actEx = FutureUtil.unwrapCompletionException(ex);
-            if (forceNoPartitioned && (actEx instanceof PulsarClientException.NotFoundException
-                    || actEx instanceof PulsarClientException.TopicDoesNotExistException
-                    || actEx instanceof PulsarAdminException.NotFoundException)) {
-                checkPartitions.complete(0);
-            } else {
-                checkPartitions.completeExceptionally(ex);
-            }
-            return null;
-        });
-        return checkPartitions;
-    }
-
     private <T> CompletableFuture<Producer<T>> createProducerAsync(String topic,
                                                                    ProducerConfigurationData conf,
                                                                    Schema<T> schema,
                                                                    ProducerInterceptors interceptors) {
         CompletableFuture<Producer<T>> producerCreatedFuture = new CompletableFuture<>();
 
-
-
-        checkPartitions(topic, conf.isNonPartitionedTopicExpected(), conf.getProducerName()).thenAccept(partitions -> {
+        getPartitionedTopicMetadata(topic).thenAccept(metadata -> {
             if (log.isDebugEnabled()) {
-                log.debug("[{}] Received topic metadata. partitions: {}", topic, partitions);
+                log.debug("[{}] Received topic metadata. partitions: {}", topic, metadata.partitions);
             }
 
             ProducerBase<T> producer;
-            if (partitions > 0) {
+            if (metadata.partitions > 0) {
                 producer = newPartitionedProducerImpl(topic, conf, schema, interceptors, producerCreatedFuture,
-                        partitions);
+                        metadata);
             } else {
                 producer = newProducerImpl(topic, -1, conf, schema, interceptors, producerCreatedFuture,
                         Optional.empty());
             }
+
             producers.add(producer);
         }).exceptionally(ex -> {
             log.warn("[{}] Failed to get partitioned topic metadata: {}", topic, ex.getMessage());
@@ -469,6 +409,7 @@ public class PulsarClientImpl implements PulsarClient {
      * @param schema topic schema
      * @param interceptors producer interceptors
      * @param producerCreatedFuture future for signaling completion of async producer creation
+     * @param metadata partitioned topic metadata
      * @param <T> message type class
      * @return new PartitionedProducerImpl instance
      */
@@ -478,8 +419,8 @@ public class PulsarClientImpl implements PulsarClient {
                                                                         ProducerInterceptors interceptors,
                                                                         CompletableFuture<Producer<T>>
                                                                                 producerCreatedFuture,
-                                                                        int partitions) {
-        return new PartitionedProducerImpl<>(PulsarClientImpl.this, topic, conf, partitions,
+                                                                        PartitionedTopicMetadata metadata) {
+        return new PartitionedProducerImpl<>(PulsarClientImpl.this, topic, conf, metadata.partitions,
                 producerCreatedFuture, schema, interceptors);
     }
 
@@ -504,10 +445,6 @@ public class PulsarClientImpl implements PulsarClient {
                                                   ProducerInterceptors interceptors,
                                                   CompletableFuture<Producer<T>> producerCreatedFuture,
                                                   Optional<String> overrideProducerName) {
-        if (conf.isReplProducer()) {
-            return new GeoReplicationProducerImpl(PulsarClientImpl.this, topic, conf, producerCreatedFuture,
-                    partitionIndex, schema, interceptors, overrideProducerName);
-        }
         return new ProducerImpl<>(PulsarClientImpl.this, topic, conf, producerCreatedFuture, partitionIndex, schema,
                 interceptors, overrideProducerName);
     }
@@ -580,7 +517,7 @@ public class PulsarClientImpl implements PulsarClient {
 
         String topic = conf.getSingleTopic();
 
-        getPartitionedTopicMetadata(topic, true, false).thenAccept(metadata -> {
+        getPartitionedTopicMetadata(topic).thenAccept(metadata -> {
             if (log.isDebugEnabled()) {
                 log.debug("[{}] Received topic metadata. partitions: {}", topic, metadata.partitions);
             }
@@ -634,13 +571,12 @@ public class PulsarClientImpl implements PulsarClient {
         lookup.getTopicsUnderNamespace(namespaceName, subscriptionMode, regex, null)
             .thenAccept(getTopicsResult -> {
                 if (log.isDebugEnabled()) {
-                    log.debug("Pattern consumer [{}] get topics under namespace {}, topics.size: {},"
-                                    + " topicsHash: {}, changed: {}, filtered: {}", conf.getSubscriptionName(),
+                    log.debug("Get topics under namespace {}, topics.size: {},"
+                                    + " topicsHash: {}, changed: {}, filtered: {}",
                             namespaceName, getTopicsResult.getTopics().size(), getTopicsResult.getTopicsHash(),
                             getTopicsResult.isChanged(), getTopicsResult.isFiltered());
                     getTopicsResult.getTopics().forEach(topicName ->
-                        log.debug("Pattern consumer [{}] get topics under namespace {}, topic: {}",
-                                conf.getSubscriptionName(), namespaceName, topicName));
+                        log.debug("Get topics under namespace {}, topic: {}", namespaceName, topicName));
                 }
 
                 List<String> topicsList = getTopicsResult.getTopics();
@@ -648,14 +584,6 @@ public class PulsarClientImpl implements PulsarClient {
                    topicsList = TopicList.filterTopics(getTopicsResult.getTopics(), conf.getTopicsPattern());
                 }
                 conf.getTopicNames().addAll(topicsList);
-
-                if (log.isDebugEnabled()) {
-                    log.debug("Pattern consumer [{}] initialize topics. {}", conf.getSubscriptionName(),
-                            getTopicsResult.getNonPartitionedOrPartitionTopics());
-                }
-
-                // Pattern consumer has his unique check mechanism, so do not need the feature "autoUpdatePartitions".
-                conf.setAutoUpdatePartitions(false);
                 ConsumerBase<T> consumer = new PatternMultiTopicsConsumerImpl<>(conf.getTopicsPattern(),
                         getTopicsResult.getTopicsHash(),
                         PulsarClientImpl.this,
@@ -729,7 +657,7 @@ public class PulsarClientImpl implements PulsarClient {
 
         CompletableFuture<Reader<T>> readerFuture = new CompletableFuture<>();
 
-        getPartitionedTopicMetadata(topic, true, false).thenAccept(metadata -> {
+        getPartitionedTopicMetadata(topic).thenAccept(metadata -> {
             if (log.isDebugEnabled()) {
                 log.debug("[{}] Received topic metadata. partitions: {}", topic, metadata.partitions);
             }
@@ -803,21 +731,6 @@ public class PulsarClientImpl implements PulsarClient {
         }
     }
 
-    private void closeUrlLookupMap() {
-        Map<String, LookupService> closedUrlLookupServices = new HashMap(urlLookupMap.size());
-        urlLookupMap.entrySet().forEach(e -> {
-            try {
-                e.getValue().close();
-            } catch (Exception ex) {
-                log.error("Error closing lookup service {}", e.getKey(), ex);
-            }
-            closedUrlLookupServices.put(e.getKey(), e.getValue());
-        });
-        closedUrlLookupServices.entrySet().forEach(e -> {
-            urlLookupMap.remove(e.getKey(), e.getValue());
-        });
-    }
-
     @Override
     public CompletableFuture<Void> closeAsync() {
         log.info("Client closing. URL: {}", lookup.getServiceUrl());
@@ -828,8 +741,6 @@ public class PulsarClientImpl implements PulsarClient {
         final CompletableFuture<Void> closeFuture = new CompletableFuture<>();
         List<CompletableFuture<Void>> futures = new ArrayList<>();
 
-        closeUrlLookupMap();
-
         producers.forEach(p -> futures.add(p.closeAsync().handle((__, t) -> {
             if (t != null) {
                 log.error("Error closing producer {}", p, t);
@@ -993,16 +904,6 @@ public class PulsarClientImpl implements PulsarClient {
                 pulsarClientException = PulsarClientException.unwrap(t);
             }
         }
-
-        if (createdLookupProviders && lookupExecutorProvider != null && !lookupExecutorProvider.isShutdown()) {
-            try {
-                lookupExecutorProvider.shutdownNow();
-            } catch (Throwable t) {
-                log.warn("Failed to shutdown lookupExecutorProvider", t);
-                pulsarClientException = PulsarClientException.unwrap(t);
-            }
-        }
-
         if (pulsarClientException != null) {
             throw pulsarClientException;
         }
@@ -1043,41 +944,10 @@ public class PulsarClientImpl implements PulsarClient {
         conf.setTlsTrustStorePassword(tlsTrustStorePassword);
     }
 
-    public CompletableFuture<ClientCnx> getConnection(final String topic, int randomKeyForSelectConnection) {
-        TopicName topicName = TopicName.get(topic);
-        return lookup.getBroker(topicName)
-                .thenCompose(pair -> getConnection(pair.getLeft(), pair.getRight(), randomKeyForSelectConnection));
-    }
-
-    /**
-     * Only for test.
-     */
-    @VisibleForTesting
     public CompletableFuture<ClientCnx> getConnection(final String topic) {
         TopicName topicName = TopicName.get(topic);
         return lookup.getBroker(topicName)
-                .thenCompose(pair -> getConnection(pair.getLeft(), pair.getRight(), cnxPool.genRandomKeyToSelectCon()));
-    }
-
-    public CompletableFuture<ClientCnx> getConnection(final String topic, final String url) {
-        TopicName topicName = TopicName.get(topic);
-        return getLookup(url).getBroker(topicName)
-                .thenCompose(pair -> getConnection(pair.getLeft(), pair.getRight(), cnxPool.genRandomKeyToSelectCon()));
-    }
-
-    public LookupService getLookup(String serviceUrl) {
-        return urlLookupMap.computeIfAbsent(serviceUrl, url -> {
-            if (isClosed()) {
-                throw new IllegalStateException("Pulsar client has been closed, can not build LookupService when"
-                        + " calling get lookup with an url");
-            }
-            try {
-                return createLookup(serviceUrl);
-            } catch (PulsarClientException e) {
-                log.warn("Failed to update url to lookup service {}, {}", url, e.getMessage());
-                throw new IllegalStateException("Failed to update url " + url);
-            }
-        });
+                .thenCompose(pair -> getConnection(pair.getLeft(), pair.getRight()));
     }
 
     public CompletableFuture<ClientCnx> getConnectionToServiceUrl() {
@@ -1086,13 +956,12 @@ public class PulsarClientImpl implements PulsarClient {
                     "Can't get client connection to HTTP service URL", null));
         }
         InetSocketAddress address = lookup.resolveHost();
-        return getConnection(address, address, cnxPool.genRandomKeyToSelectCon());
+        return getConnection(address, address);
     }
 
     public CompletableFuture<ClientCnx> getConnection(final InetSocketAddress logicalAddress,
-                                                      final InetSocketAddress physicalAddress,
-                                                      final int randomKeyForSelectConnection) {
-        return cnxPool.getConnection(logicalAddress, physicalAddress, randomKeyForSelectConnection);
+                                                      final InetSocketAddress physicalAddress) {
+        return cnxPool.getConnection(logicalAddress, physicalAddress);
     }
 
     /** visible for pulsar-functions. **/
@@ -1138,36 +1007,19 @@ public class PulsarClientImpl implements PulsarClient {
     }
 
     public void reloadLookUp() throws PulsarClientException {
-        LookupService previousLookup = lookup;
-        lookup = createLookup(conf.getServiceUrl());
-        // close the previous lookup after the new lookup is created successfully
-        if (previousLookup != null && previousLookup != lookup) {
-            try {
-                previousLookup.close();
-            } catch (Exception e) {
-                log.warn("Failed to close previous lookup service", e);
-            }
-        }
-    }
-
-    public LookupService createLookup(String url) throws PulsarClientException {
-        if (url.startsWith("http")) {
-            return new HttpLookupService(conf, eventLoopGroup);
+        if (conf.getServiceUrl().startsWith("http")) {
+            lookup = new HttpLookupService(conf, eventLoopGroup);
         } else {
-            return new BinaryProtoLookupService(this, url, conf.getListenerName(), conf.isUseTls(),
+            lookup = new BinaryProtoLookupService(this, conf.getServiceUrl(), conf.getListenerName(), conf.isUseTls(),
                     externalExecutorProvider.getExecutor());
         }
     }
 
-    /**
-     * @param useFallbackForNonPIP344Brokers <p>If true, fallback to the prior behavior of the method
-     *                                       getPartitionedTopicMetadata if the broker does not support the PIP-344
-     *                                       feature 'supports_get_partitioned_metadata_without_auto_creation'. This
-     *                                       parameter only affects the behavior when
-     *                                       {@param metadataAutoCreationEnabled} is false.</p>
-     */
-    public CompletableFuture<PartitionedTopicMetadata> getPartitionedTopicMetadata(
-            String topic, boolean metadataAutoCreationEnabled, boolean useFallbackForNonPIP344Brokers) {
+    public CompletableFuture<Integer> getNumberOfPartitions(String topic) {
+        return getPartitionedTopicMetadata(topic).thenApply(metadata -> metadata.partitions);
+    }
+
+    public CompletableFuture<PartitionedTopicMetadata> getPartitionedTopicMetadata(String topic) {
 
         CompletableFuture<PartitionedTopicMetadata> metadataFuture = new CompletableFuture<>();
 
@@ -1179,8 +1031,8 @@ public class PulsarClientImpl implements PulsarClient {
                     .setMandatoryStop(opTimeoutMs.get() * 2, TimeUnit.MILLISECONDS)
                     .setMax(conf.getMaxBackoffIntervalNanos(), TimeUnit.NANOSECONDS)
                     .create();
-            getPartitionedTopicMetadata(topicName, backoff, opTimeoutMs, metadataFuture, new ArrayList<>(),
-                    metadataAutoCreationEnabled, useFallbackForNonPIP344Brokers);
+            getPartitionedTopicMetadata(topicName, backoff, opTimeoutMs,
+                                        metadataFuture, new ArrayList<>());
         } catch (IllegalArgumentException e) {
             return FutureUtil.failedFuture(new PulsarClientException.InvalidConfigurationException(e.getMessage()));
         }
@@ -1191,20 +1043,15 @@ public class PulsarClientImpl implements PulsarClient {
                                              Backoff backoff,
                                              AtomicLong remainingTime,
                                              CompletableFuture<PartitionedTopicMetadata> future,
-                                             List<Throwable> previousExceptions,
-                                             boolean metadataAutoCreationEnabled,
-                                             boolean useFallbackForNonPIP344Brokers) {
+                                             List<Throwable> previousExceptions) {
         long startTime = System.nanoTime();
-        CompletableFuture<PartitionedTopicMetadata> queryFuture = lookup.getPartitionedTopicMetadata(topicName,
-                metadataAutoCreationEnabled, useFallbackForNonPIP344Brokers);
-        queryFuture.thenAccept(future::complete).exceptionally(e -> {
+        lookup.getPartitionedTopicMetadata(topicName).thenAccept(future::complete).exceptionally(e -> {
             remainingTime.addAndGet(-1 * TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - startTime));
             long nextDelay = Math.min(backoff.next(), remainingTime.get());
             // skip retry scheduler when set lookup throttle in client or server side which will lead to
             // `TooManyRequestsException`
             boolean isLookupThrottling = !PulsarClientException.isRetriableError(e.getCause())
-                || e.getCause() instanceof PulsarClientException.AuthenticationException
-                || e.getCause() instanceof PulsarClientException.NotFoundException;
+                || e.getCause() instanceof PulsarClientException.AuthenticationException;
             if (nextDelay <= 0 || isLookupThrottling) {
                 PulsarClientException.setPreviousExceptions(e, previousExceptions);
                 future.completeExceptionally(e);
@@ -1216,16 +1063,15 @@ public class PulsarClientImpl implements PulsarClient {
                 log.warn("[topic: {}] Could not get connection while getPartitionedTopicMetadata -- "
                         + "Will try again in {} ms", topicName, nextDelay);
                 remainingTime.addAndGet(-nextDelay);
-                getPartitionedTopicMetadata(topicName, backoff, remainingTime, future, previousExceptions,
-                        metadataAutoCreationEnabled, useFallbackForNonPIP344Brokers);
+                getPartitionedTopicMetadata(topicName, backoff, remainingTime, future, previousExceptions);
             }, nextDelay, TimeUnit.MILLISECONDS);
             return null;
         });
     }
 
     @Override
-    public CompletableFuture<List<String>> getPartitionsForTopic(String topic, boolean metadataAutoCreationEnabled) {
-        return getPartitionedTopicMetadata(topic, metadataAutoCreationEnabled, true).thenApply(metadata -> {
+    public CompletableFuture<List<String>> getPartitionsForTopic(String topic) {
+        return getPartitionedTopicMetadata(topic).thenApply(metadata -> {
             if (metadata.partitions > 0) {
                 TopicName topicName = TopicName.get(topic);
                 List<String> partitions = new ArrayList<>(metadata.partitions);
@@ -1297,11 +1143,10 @@ public class PulsarClientImpl implements PulsarClient {
                                                                       String topicName) {
         if (schema != null && schema.supportSchemaVersioning()) {
             final SchemaInfoProvider schemaInfoProvider;
-            String schemaTopicName = TopicName.getPartitionedTopicName(topicName).toString();
             try {
-                schemaInfoProvider = pulsarClientImpl.getSchemaProviderLoadingCache().get(schemaTopicName);
+                schemaInfoProvider = pulsarClientImpl.getSchemaProviderLoadingCache().get(topicName);
             } catch (ExecutionException e) {
-                log.error("Failed to load schema info provider for topic {}", schemaTopicName, e);
+                log.error("Failed to load schema info provider for topic {}", topicName, e);
                 return FutureUtil.failedFuture(e.getCause());
             }
             schema = schema.clone();
