@@ -18,7 +18,6 @@
  */
 package org.apache.pulsar.client.impl;
 
-import static org.mockito.Mockito.doAnswer;
 import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.spy;
@@ -31,21 +30,22 @@ import io.netty.buffer.ByteBuf;
 import io.netty.channel.ChannelHandlerContext;
 import io.netty.channel.EventLoopGroup;
 import io.netty.channel.nio.NioEventLoopGroup;
-import java.util.ArrayList;
+import java.lang.reflect.Field;
+import java.lang.reflect.Method;
 import java.util.BitSet;
 import java.util.Collections;
-import java.util.List;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
-import lombok.var;
+import org.apache.pulsar.client.api.MessageId;
 import org.apache.pulsar.client.api.MessageIdAdv;
 import org.apache.pulsar.client.impl.conf.ClientConfigurationData;
 import org.apache.pulsar.client.impl.conf.ConsumerConfigurationData;
 import org.apache.pulsar.client.util.TimedCompletableFuture;
 import org.apache.pulsar.common.api.proto.CommandAck.AckType;
-import org.apache.pulsar.common.api.proto.ProtocolVersion;
+import org.apache.pulsar.common.util.collections.ConcurrentBitSetRecyclable;
 import org.apache.pulsar.common.util.collections.ConcurrentOpenHashMap;
+import org.apache.pulsar.common.api.proto.ProtocolVersion;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.BeforeClass;
 import org.testng.annotations.DataProvider;
@@ -56,7 +56,6 @@ public class AcknowledgementsGroupingTrackerTest {
     private ClientCnx cnx;
     private ConsumerImpl<?> consumer;
     private EventLoopGroup eventLoopGroup;
-    private AtomicBoolean returnCnx = new AtomicBoolean(true);
 
     @BeforeClass
     public void setup() throws NoSuchFieldException, IllegalAccessException {
@@ -66,15 +65,13 @@ public class AcknowledgementsGroupingTrackerTest {
                 ConcurrentOpenHashMap.<MessageIdAdv, MessageIdImpl[]>newBuilder().build();
         cnx = spy(new ClientCnxTest(new ClientConfigurationData(), eventLoopGroup));
         PulsarClientImpl client = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(client.getCnxPool()).thenReturn(connectionPool);
         doReturn(client).when(consumer).getClient();
+        doReturn(cnx).when(consumer).getClientCnx();
         doReturn(new ConsumerStatsRecorderImpl()).when(consumer).getStats();
         doReturn(new UnAckedMessageTracker().UNACKED_MESSAGE_TRACKER_DISABLED)
                 .when(consumer).getUnAckedMessageTracker();
-        ChannelHandlerContext ctx = ClientTestFixtures.mockChannelHandlerContext();
-        doAnswer(invocation -> returnCnx.get() ? cnx : null).when(consumer).getClientCnx();
-        doReturn(ctx).when(cnx).ctx();
+        ChannelHandlerContext ctx = mock(ChannelHandlerContext.class);
+        when(cnx.ctx()).thenReturn(ctx);
     }
 
     @DataProvider(name = "isNeedReceipt")
@@ -132,6 +129,8 @@ public class AcknowledgementsGroupingTrackerTest {
         tracker.addAcknowledgment(msg6, AckType.Individual, Collections.emptyMap());
         assertTrue(tracker.isDuplicate(msg6));
 
+        when(consumer.getClientCnx()).thenReturn(cnx);
+
         tracker.flush();
 
         assertTrue(tracker.isDuplicate(msg1));
@@ -190,6 +189,8 @@ public class AcknowledgementsGroupingTrackerTest {
         tracker.addListAcknowledgment(Collections.singletonList(msg6), AckType.Individual, Collections.emptyMap());
         assertTrue(tracker.isDuplicate(msg6));
 
+        when(consumer.getClientCnx()).thenReturn(cnx);
+
         tracker.flush();
 
         assertTrue(tracker.isDuplicate(msg1));
@@ -216,13 +217,12 @@ public class AcknowledgementsGroupingTrackerTest {
 
         assertFalse(tracker.isDuplicate(msg1));
 
-        returnCnx.set(false);
-        try {
-            tracker.addAcknowledgment(msg1, AckType.Individual, Collections.emptyMap());
-            assertFalse(tracker.isDuplicate(msg1));
-        } finally {
-            returnCnx.set(true);
-        }
+        when(consumer.getClientCnx()).thenReturn(null);
+
+        tracker.addAcknowledgment(msg1, AckType.Individual, Collections.emptyMap());
+        assertFalse(tracker.isDuplicate(msg1));
+
+        when(consumer.getClientCnx()).thenReturn(cnx);
 
         tracker.flush();
         assertFalse(tracker.isDuplicate(msg1));
@@ -246,13 +246,12 @@ public class AcknowledgementsGroupingTrackerTest {
 
         assertFalse(tracker.isDuplicate(msg1));
 
-        returnCnx.set(false);
-        try {
-            tracker.addListAcknowledgment(Collections.singletonList(msg1), AckType.Individual, Collections.emptyMap());
-            assertTrue(tracker.isDuplicate(msg1));
-        } finally {
-            returnCnx.set(true);
-        }
+        when(consumer.getClientCnx()).thenReturn(null);
+
+        tracker.addListAcknowledgment(Collections.singletonList(msg1), AckType.Individual, Collections.emptyMap());
+        assertTrue(tracker.isDuplicate(msg1));
+
+        when(consumer.getClientCnx()).thenReturn(cnx);
 
         tracker.flush();
         assertFalse(tracker.isDuplicate(msg1));
@@ -312,6 +311,8 @@ public class AcknowledgementsGroupingTrackerTest {
         tracker.addAcknowledgment(msg6, AckType.Individual, Collections.emptyMap());
         assertTrue(tracker.isDuplicate(msg6));
 
+        when(consumer.getClientCnx()).thenReturn(cnx);
+
         tracker.flush();
 
         assertTrue(tracker.isDuplicate(msg1));
@@ -372,6 +373,8 @@ public class AcknowledgementsGroupingTrackerTest {
         tracker.addListAcknowledgment(Collections.singletonList(msg6), AckType.Individual, Collections.emptyMap());
         assertTrue(tracker.isDuplicate(msg6));
 
+        when(consumer.getClientCnx()).thenReturn(cnx);
+
         tracker.flush();
 
         assertTrue(tracker.isDuplicate(msg1));
@@ -386,18 +389,24 @@ public class AcknowledgementsGroupingTrackerTest {
     }
 
     @Test
-    public void testDoIndividualBatchAckAsync() {
+    public void testDoIndividualBatchAckAsync() throws Exception{
         ConsumerConfigurationData<?> conf = new ConsumerConfigurationData<>();
-        var tracker = new PersistentAcknowledgmentsGroupingTracker(consumer, conf, eventLoopGroup);
-        var messageId1 = new BatchMessageIdImpl(5, 1, 0, 3, 10, null);
+        AcknowledgmentsGroupingTracker tracker = new PersistentAcknowledgmentsGroupingTracker(consumer, conf, eventLoopGroup);
+        MessageId messageId1 = new BatchMessageIdImpl(5, 1, 0, 3, 10, null);
         BitSet bitSet = new BitSet(20);
-        for (int i = 0; i < 20; i++) {
+        for(int i = 0; i < 20; i ++) {
             bitSet.set(i, true);
         }
-        var messageId2 = new BatchMessageIdImpl(3, 2, 0, 5, 20, bitSet);
-        tracker.doIndividualBatchAckAsync(messageId1);
-        tracker.doIndividualBatchAckAsync(messageId2);
-        var batchIndexAcks = tracker.pendingIndividualBatchIndexAcks;
+        MessageId messageId2 = new BatchMessageIdImpl(3, 2, 0, 5, 20, bitSet);
+        Method doIndividualBatchAckAsync = PersistentAcknowledgmentsGroupingTracker.class
+                .getDeclaredMethod("doIndividualBatchAckAsync", MessageIdAdv.class);
+        doIndividualBatchAckAsync.setAccessible(true);
+        doIndividualBatchAckAsync.invoke(tracker, messageId1);
+        doIndividualBatchAckAsync.invoke(tracker, messageId2);
+        Field pendingIndividualBatchIndexAcks = PersistentAcknowledgmentsGroupingTracker.class.getDeclaredField("pendingIndividualBatchIndexAcks");
+        pendingIndividualBatchIndexAcks.setAccessible(true);
+        ConcurrentHashMap<MessageIdAdv, ConcurrentBitSetRecyclable> batchIndexAcks =
+                (ConcurrentHashMap<MessageIdAdv, ConcurrentBitSetRecyclable>) pendingIndividualBatchIndexAcks.get(tracker);
         MessageIdImpl position1 = new MessageIdImpl(5, 1, 0);
         MessageIdImpl position2 = new MessageIdImpl(3, 2, 0);
         assertTrue(batchIndexAcks.containsKey(position1));
@@ -409,54 +418,6 @@ public class AcknowledgementsGroupingTrackerTest {
         tracker.close();
     }
 
-    @Test
-    public void testDoIndividualBatchAckNeverAffectIsDuplicate() throws Exception {
-        ConsumerConfigurationData<?> conf = new ConsumerConfigurationData<>();
-        conf.setMaxAcknowledgmentGroupSize(1);
-        PersistentAcknowledgmentsGroupingTracker tracker =
-                new PersistentAcknowledgmentsGroupingTracker(consumer, conf, eventLoopGroup);
-
-        BatchMessageIdImpl batchMessageId0 = new BatchMessageIdImpl(5, 1, 0, 0, 10, null);
-        BatchMessageIdImpl batchMessageId1 = new BatchMessageIdImpl(5, 1, 0, 1, 10, null);
-
-        int loops = 10000;
-        int addAcknowledgmentThreadCount = 10;
-        List<Thread> addAcknowledgmentThreads = new ArrayList<>(addAcknowledgmentThreadCount);
-        for (int i = 0; i < addAcknowledgmentThreadCount; i++) {
-            Thread addAcknowledgmentThread = new Thread(() -> {
-                for (int j = 0; j < loops; j++) {
-                    tracker.addAcknowledgment(batchMessageId0, AckType.Individual, Collections.emptyMap());
-                }
-            }, "doIndividualBatchAck-thread-" + i);
-            addAcknowledgmentThread.start();
-            addAcknowledgmentThreads.add(addAcknowledgmentThread);
-        }
-
-        int isDuplicateThreadCount = 10;
-        AtomicBoolean assertResult = new AtomicBoolean();
-        List<Thread> isDuplicateThreads = new ArrayList<>(isDuplicateThreadCount);
-        for (int i = 0; i < isDuplicateThreadCount; i++) {
-            Thread isDuplicateThread = new Thread(() -> {
-                for (int j = 0; j < loops; j++) {
-                    boolean duplicate = tracker.isDuplicate(batchMessageId1);
-                    assertResult.set(assertResult.get() || duplicate);
-                }
-            }, "isDuplicate-thread-" + i);
-            isDuplicateThread.start();
-            isDuplicateThreads.add(isDuplicateThread);
-        }
-
-        for (Thread addAcknowledgmentThread : addAcknowledgmentThreads) {
-            addAcknowledgmentThread.join();
-        }
-
-        for (Thread isDuplicateThread : isDuplicateThreads) {
-            isDuplicateThread.join();
-        }
-
-        assertFalse(assertResult.get());
-    }
-
     public class ClientCnxTest extends ClientCnx {
 
         public ClientCnxTest(ClientConfigurationData conf, EventLoopGroup eventLoopGroup) {
