@@ -19,10 +19,10 @@
 package org.apache.pulsar.client.impl;
 
 import static org.mockito.Mockito.mock;
-import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertTrue;
+import java.io.IOException;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
@@ -30,6 +30,7 @@ import java.util.concurrent.TimeUnit;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.pulsar.client.api.Authentication;
 import org.apache.pulsar.client.api.AuthenticationFactory;
+import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.ServiceUrlProvider;
 import org.apache.pulsar.client.impl.conf.ClientConfigurationData;
 import org.awaitility.Awaitility;
@@ -40,7 +41,7 @@ import org.testng.annotations.Test;
 @Slf4j
 public class AutoClusterFailoverTest {
     @Test
-    public void testBuildAutoClusterFailoverInstance() throws Exception {
+    public void testBuildAutoClusterFailoverInstance() throws PulsarClientException {
         String primary = "pulsar://localhost:6650";
         String secondary = "pulsar://localhost:6651";
         long failoverDelay = 30;
@@ -101,7 +102,7 @@ public class AutoClusterFailoverTest {
     }
 
     @Test
-    public void testInitialize() throws Exception {
+    public void testInitialize() {
         String primary = "pulsar://localhost:6650";
         String secondary = "pulsar://localhost:6651";
         long failoverDelay = 10;
@@ -120,8 +121,6 @@ public class AutoClusterFailoverTest {
 
         AutoClusterFailover autoClusterFailover = Mockito.spy((AutoClusterFailover) provider);
         PulsarClientImpl pulsarClient = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(pulsarClient.getCnxPool()).thenReturn(connectionPool);
         Mockito.doReturn(false).when(autoClusterFailover).probeAvailable(primary);
         Mockito.doReturn(true).when(autoClusterFailover).probeAvailable(secondary);
         Mockito.doReturn(configurationData).when(pulsarClient).getConfiguration();
@@ -145,7 +144,7 @@ public class AutoClusterFailoverTest {
     }
 
     @Test
-    public void testAutoClusterFailoverSwitchWithoutAuthentication() throws Exception {
+    public void testAutoClusterFailoverSwitchWithoutAuthentication() {
         String primary = "pulsar://localhost:6650";
         String secondary = "pulsar://localhost:6651";
         long failoverDelay = 1;
@@ -164,8 +163,6 @@ public class AutoClusterFailoverTest {
 
         AutoClusterFailover autoClusterFailover = Mockito.spy((AutoClusterFailover) provider);
         PulsarClientImpl pulsarClient = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(pulsarClient.getCnxPool()).thenReturn(connectionPool);
         Mockito.doReturn(false).when(autoClusterFailover).probeAvailable(primary);
         Mockito.doReturn(true).when(autoClusterFailover).probeAvailable(secondary);
         Mockito.doReturn(configurationData).when(pulsarClient).getConfiguration();
@@ -180,7 +177,7 @@ public class AutoClusterFailoverTest {
     }
 
     @Test
-    public void testAutoClusterFailoverSwitchWithAuthentication() throws Exception {
+    public void testAutoClusterFailoverSwitchWithAuthentication() throws IOException {
         String primary = "pulsar+ssl://localhost:6651";
         String secondary = "pulsar+ssl://localhost:6661";
         long failoverDelay = 1;
@@ -220,8 +217,6 @@ public class AutoClusterFailoverTest {
 
         AutoClusterFailover autoClusterFailover = Mockito.spy((AutoClusterFailover) provider);
         PulsarClientImpl pulsarClient = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(pulsarClient.getCnxPool()).thenReturn(connectionPool);
         Mockito.doReturn(false).when(autoClusterFailover).probeAvailable(primary);
         Mockito.doReturn(true).when(autoClusterFailover).probeAvailable(secondary);
         Mockito.doReturn(configurationData).when(pulsarClient).getConfiguration();
@@ -243,7 +238,7 @@ public class AutoClusterFailoverTest {
     }
 
     @Test
-    public void testAutoClusterFailoverSwitchTlsTrustStore() throws Exception {
+    public void testAutoClusterFailoverSwitchTlsTrustStore() throws IOException {
         String primary = "pulsar+ssl://localhost:6651";
         String secondary = "pulsar+ssl://localhost:6661";
         long failoverDelay = 1;
@@ -275,8 +270,6 @@ public class AutoClusterFailoverTest {
 
         AutoClusterFailover autoClusterFailover = Mockito.spy((AutoClusterFailover) provider);
         PulsarClientImpl pulsarClient = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(pulsarClient.getCnxPool()).thenReturn(connectionPool);
         Mockito.doReturn(false).when(autoClusterFailover).probeAvailable(primary);
         Mockito.doReturn(true).when(autoClusterFailover).probeAvailable(secondary);
         Mockito.doReturn(configurationData).when(pulsarClient).getConfiguration();
