@@ -76,8 +76,6 @@ public class ConsumerBuilderImplTest {
     @BeforeMethod(alwaysRun = true)
     public void setup() {
         PulsarClientImpl client = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(client.getCnxPool()).thenReturn(connectionPool);
         ConsumerConfigurationData consumerConfigurationData = mock(ConsumerConfigurationData.class);
         when(consumerConfigurationData.getTopicsPattern()).thenReturn(Pattern.compile("\\w+"));
         when(consumerConfigurationData.getSubscriptionName()).thenReturn("testSubscriptionName");
@@ -106,8 +104,6 @@ public class ConsumerBuilderImplTest {
     @Test(expectedExceptions = IllegalArgumentException.class)
     public void testConsumerBuilderImplWhenSchemaIsNull() {
         PulsarClientImpl client = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(client.getCnxPool()).thenReturn(connectionPool);
         ConsumerConfigurationData consumerConfigurationData = mock(ConsumerConfigurationData.class);
         new ConsumerBuilderImpl(client, consumerConfigurationData, null);
     }
@@ -501,7 +497,7 @@ public class ConsumerBuilderImplTest {
         assertTrue(configurationData.isRetryEnable());
         assertFalse(configurationData.isAutoUpdatePartitions());
         assertEquals(configurationData.getAutoUpdatePartitionsIntervalSeconds(), 2);
-        assertEquals(configurationData.getReplicateSubscriptionState(), Boolean.TRUE);
+        assertTrue(configurationData.isReplicateSubscriptionState());
         assertTrue(configurationData.isResetIncludeHead());
         assertTrue(configurationData.isBatchIndexAckEnabled());
         assertTrue(configurationData.isAckReceiptEnabled());
@@ -561,7 +557,7 @@ public class ConsumerBuilderImplTest {
         assertFalse(configurationData.isRetryEnable());
         assertTrue(configurationData.isAutoUpdatePartitions());
         assertEquals(configurationData.getAutoUpdatePartitionsIntervalSeconds(), 60);
-        assertNull(configurationData.getReplicateSubscriptionState());
+        assertFalse(configurationData.isReplicateSubscriptionState());
         assertFalse(configurationData.isResetIncludeHead());
         assertFalse(configurationData.isBatchIndexAckEnabled());
         assertFalse(configurationData.isAckReceiptEnabled());
@@ -581,38 +577,6 @@ public class ConsumerBuilderImplTest {
         assertNull(configurationData.getPayloadProcessor());
     }
 
-    @Test
-    public void testReplicateSubscriptionState() {
-        ConsumerBuilderImpl<byte[]> consumerBuilder = createConsumerBuilder();
-        assertNull(consumerBuilder.getConf().getReplicateSubscriptionState());
-
-        consumerBuilder.replicateSubscriptionState(true);
-        assertEquals(consumerBuilder.getConf().getReplicateSubscriptionState(), Boolean.TRUE);
-
-        consumerBuilder.replicateSubscriptionState(false);
-        assertEquals(consumerBuilder.getConf().getReplicateSubscriptionState(), Boolean.FALSE);
-
-        Map<String, Object> conf = new HashMap<>();
-        consumerBuilder = createConsumerBuilder();
-        consumerBuilder.loadConf(conf);
-        assertNull(consumerBuilder.getConf().getReplicateSubscriptionState());
-
-        conf.put("replicateSubscriptionState", true);
-        consumerBuilder = createConsumerBuilder();
-        consumerBuilder.loadConf(conf);
-        assertEquals(consumerBuilder.getConf().getReplicateSubscriptionState(), Boolean.TRUE);
-
-        conf.put("replicateSubscriptionState", false);
-        consumerBuilder = createConsumerBuilder();
-        consumerBuilder.loadConf(conf);
-        assertEquals(consumerBuilder.getConf().getReplicateSubscriptionState(), Boolean.FALSE);
-
-        conf.put("replicateSubscriptionState", null);
-        consumerBuilder = createConsumerBuilder();
-        consumerBuilder.loadConf(conf);
-        assertNull(consumerBuilder.getConf().getReplicateSubscriptionState());
-    }
-
     private ConsumerBuilderImpl<byte[]> createConsumerBuilder() {
         ConsumerBuilderImpl<byte[]> consumerBuilder = new ConsumerBuilderImpl<>(null, Schema.BYTES);
         Map<String, String> properties = new HashMap<>();
