@@ -18,6 +18,7 @@
  */
 package org.apache.pulsar.client.impl;
 
+import java.io.IOException;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.concurrent.TimeUnit;
@@ -30,12 +31,11 @@ import org.testng.Assert;
 import org.testng.annotations.Test;
 
 import static org.mockito.Mockito.mock;
-import static org.mockito.Mockito.when;
 
 @Test(groups = "broker-impl")
 public class ControlledClusterFailoverTest {
     @Test
-    public void testBuildControlledClusterFailoverInstance() throws Exception {
+    public void testBuildControlledClusterFailoverInstance() throws IOException {
         String defaultServiceUrl = "pulsar://localhost:6650";
         String urlProvider = "http://localhost:8080/test";
         String keyA = "key-a";
@@ -64,7 +64,7 @@ public class ControlledClusterFailoverTest {
     }
 
     @Test
-    public void testControlledClusterFailoverSwitch() throws Exception {
+    public void testControlledClusterFailoverSwitch() throws IOException {
         String defaultServiceUrl = "pulsar+ssl://localhost:6651";
         String backupServiceUrl = "pulsar+ssl://localhost:6661";
         String urlProvider = "http://localhost:8080";
@@ -88,8 +88,6 @@ public class ControlledClusterFailoverTest {
 
         ControlledClusterFailover controlledClusterFailover = Mockito.spy((ControlledClusterFailover) provider);
         PulsarClientImpl pulsarClient = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(pulsarClient.getCnxPool()).thenReturn(connectionPool);
 
         controlledClusterFailover.initialize(pulsarClient);
 
