@@ -29,10 +29,10 @@ import static org.testng.Assert.assertNotEquals;
 import static org.testng.Assert.assertNotNull;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
-import com.google.api.client.util.Lists;
 import io.netty.channel.EventLoopGroup;
 import io.netty.util.Timer;
 import io.netty.util.concurrent.DefaultThreadFactory;
+import com.google.api.client.util.Lists;
 import java.lang.reflect.Field;
 import java.nio.ByteBuffer;
 import java.util.List;
@@ -40,7 +40,6 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ThreadFactory;
-import lombok.Cleanup;
 import org.apache.pulsar.client.api.Message;
 import org.apache.pulsar.client.api.MessageRouter;
 import org.apache.pulsar.client.api.MessageRoutingMode;
@@ -71,8 +70,6 @@ public class PartitionedProducerImplTest {
     @BeforeMethod(alwaysRun = true)
     public void setup() {
         client = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(client.getCnxPool()).thenReturn(connectionPool);
         schema = mock(Schema.class);
         producerInterceptors = mock(ProducerInterceptors.class);
         producerCreatedFuture = new CompletableFuture<>();
@@ -267,41 +264,4 @@ public class PartitionedProducerImplTest {
         assertEquals(producerImpl.getNumOfPartitions(), 0);
     }
 
-    @Test
-    public void testMaxPendingQueueSize() throws Exception {
-        String topicName = "test-max-pending-queue-size";
-        ClientConfigurationData conf = new ClientConfigurationData();
-        conf.setServiceUrl("pulsar://localhost:6650");
-        conf.setStatsIntervalSeconds(100);
-
-        ThreadFactory threadFactory = new DefaultThreadFactory("client-test-stats", Thread.currentThread().isDaemon());
-        @Cleanup("shutdownGracefully")
-        EventLoopGroup eventLoopGroup = EventLoopUtil.newEventLoopGroup(conf.getNumIoThreads(), false, threadFactory);
-
-        @Cleanup
-        PulsarClientImpl clientImpl = new PulsarClientImpl(conf, eventLoopGroup);
-
-        // Test set maxPendingMessage to 10
-        ProducerConfigurationData producerConfData = new ProducerConfigurationData();
-        producerConfData.setMessageRoutingMode(MessageRoutingMode.CustomPartition);
-        producerConfData.setCustomMessageRouter(new CustomMessageRouter());
-        producerConfData.setMaxPendingMessages(10);
-        PartitionedProducerImpl partitionedProducerImpl = new PartitionedProducerImpl(
-                clientImpl, topicName, producerConfData, 1, null, null, null);
-        assertEquals(partitionedProducerImpl.getConfiguration().getMaxPendingMessages(), 10);
-
-        // Test set MaxPendingMessagesAcrossPartitions=5
-        producerConfData.setMaxPendingMessages(ProducerConfigurationData.DEFAULT_MAX_PENDING_MESSAGES);
-        producerConfData.setMaxPendingMessagesAcrossPartitions(5);
-        partitionedProducerImpl = new PartitionedProducerImpl(
-                clientImpl, topicName, producerConfData, 1, null, null, null);
-        assertEquals(partitionedProducerImpl.getConfiguration().getMaxPendingMessages(), 5);
-
-        // Test set maxPendingMessage=10 and MaxPendingMessagesAcrossPartitions=10 with 2 partitions
-        producerConfData.setMaxPendingMessages(10);
-        producerConfData.setMaxPendingMessagesAcrossPartitions(10);
-        partitionedProducerImpl = new PartitionedProducerImpl(
-                clientImpl, topicName, producerConfData, 2, null, null, null);
-        assertEquals(partitionedProducerImpl.getConfiguration().getMaxPendingMessages(), 5);
-    }
 }
