@@ -40,8 +40,6 @@ public class ProducerStatsRecorderImplTest {
         ClientConfigurationData conf = new ClientConfigurationData();
         conf.setStatsIntervalSeconds(1);
         PulsarClientImpl client = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(client.getCnxPool()).thenReturn(connectionPool);
         when(client.getConfiguration()).thenReturn(conf);
         Timer timer = new HashedWheelTimer();
         when(client.timer()).thenReturn(timer);
@@ -62,8 +60,6 @@ public class ProducerStatsRecorderImplTest {
         ClientConfigurationData conf = new ClientConfigurationData();
         conf.setStatsIntervalSeconds(60);
         PulsarClientImpl client = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(client.getCnxPool()).thenReturn(connectionPool);
         when(client.getConfiguration()).thenReturn(conf);
         Timer timer = new HashedWheelTimer();
         when(client.timer()).thenReturn(timer);
@@ -89,29 +85,4 @@ public class ProducerStatsRecorderImplTest {
         assertTrue(recorder2.getSendBytesRate() > 0);
         assertTrue(recorder2.getSendMsgsRate() > 0);
     }
-
-    @Test
-    public void testPartitionedTopicProducerStatsPendingQueueSizeDoesntNPE() {
-        PartitionedTopicProducerStatsRecorderImpl recorder = new PartitionedTopicProducerStatsRecorderImpl();
-        assertEquals(recorder.getPendingQueueSize(), 0);
-    }
-
-    @Test
-    public void testProducerStatsPendingQueueSizeDoesntNPE() {
-        ProducerStatsRecorderImpl recorder = new ProducerStatsRecorderImpl();
-        assertEquals(recorder.getPendingQueueSize(), 0);
-    }
-
-    @Test
-    public void testPartitionedTopicProducerStatsPendingQueueSizeAggregated() {
-        PartitionedTopicProducerStatsRecorderImpl recorder = new PartitionedTopicProducerStatsRecorderImpl();
-
-        ProducerStatsRecorderImpl individualStats = spy(new ProducerStatsRecorderImpl());
-        when(individualStats.getPendingQueueSize()).thenReturn(1);
-        recorder.updateCumulativeStats("1", individualStats);
-        recorder.updateCumulativeStats("2", individualStats);
-        recorder.updateCumulativeStats("3", individualStats);
-
-        assertEquals(recorder.getPendingQueueSize(), 3);
-    }
 }
