@@ -21,7 +21,6 @@ package org.apache.pulsar.client.impl;
 import io.netty.channel.ChannelHandlerContext;
 import io.netty.util.HashedWheelTimer;
 import io.netty.util.Timer;
-import lombok.Cleanup;
 import org.apache.pulsar.client.impl.PatternMultiTopicsConsumerImpl.TopicsChangedListener;
 import org.apache.pulsar.client.impl.conf.ClientConfigurationData;
 import org.apache.pulsar.common.api.proto.BaseCommand;
@@ -29,10 +28,7 @@ import org.apache.pulsar.common.api.proto.CommandWatchTopicListSuccess;
 import org.apache.pulsar.common.api.proto.CommandWatchTopicUpdate;
 import org.apache.pulsar.common.naming.NamespaceName;
 import static org.mockito.ArgumentMatchers.any;
-import static org.mockito.ArgumentMatchers.anyCollection;
-import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.ArgumentMatchers.anyLong;
-import static org.mockito.ArgumentMatchers.anyString;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
@@ -56,38 +52,22 @@ public class TopicListWatcherTest {
     public void setup() {
         listener = mock(TopicsChangedListener.class);
         client = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(client.getCnxPool()).thenReturn(connectionPool);
-        when(connectionPool.genRandomKeyToSelectCon()).thenReturn(0);
         when(client.getConfiguration()).thenReturn(new ClientConfigurationData());
         clientCnxFuture = new CompletableFuture<>();
         when(client.getConnectionToServiceUrl()).thenReturn(clientCnxFuture);
-        @Cleanup("stop")
         Timer timer = new HashedWheelTimer();
         when(client.timer()).thenReturn(timer);
         String topic = "persistent://tenant/ns/topic\\d+";
         when(client.getConnection(topic)).thenReturn(clientCnxFuture);
-        when(client.getConnection(topic, 0)).thenReturn(clientCnxFuture);
-        when(client.getConnection(any(), any(), anyInt())).thenReturn(clientCnxFuture);
-        when(connectionPool.getConnection(any(), any(), anyInt())).thenReturn(clientCnxFuture);
-
-        CompletableFuture<Void> completedFuture = CompletableFuture.completedFuture(null);
-        PatternMultiTopicsConsumerImpl patternConsumer = mock(PatternMultiTopicsConsumerImpl.class);
-        when(patternConsumer.getSubscribeFuture()).thenReturn(completedFuture);
-        when(patternConsumer.recheckTopicsChange()).thenReturn(completedFuture);
-        when(listener.onTopicsAdded(anyCollection())).thenReturn(completedFuture);
-        when(listener.onTopicsRemoved(anyCollection())).thenReturn(completedFuture);
-        PatternConsumerUpdateQueue queue = new PatternConsumerUpdateQueue(patternConsumer, listener);
-
         watcherFuture = new CompletableFuture<>();
-        watcher = new TopicListWatcher(queue, client,
+        watcher = new TopicListWatcher(listener, client,
                 Pattern.compile(topic), 7,
-                NamespaceName.get("tenant/ns"), null, watcherFuture, () -> {});
+                NamespaceName.get("tenant/ns"), null, watcherFuture);
     }
 
     @Test
     public void testWatcherGrabsConnection() {
-        verify(client).getConnection(anyString(), anyInt());
+        verify(client).getConnection(any());
     }
 
     @Test
