@@ -18,7 +18,6 @@
  */
 package org.apache.pulsar.client.impl.schema.generic;
 
-import com.google.protobuf.Descriptors;
 import com.google.protobuf.DynamicMessage;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.pulsar.client.api.schema.GenericRecord;
@@ -30,7 +29,6 @@ import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
 
 import static org.testng.Assert.assertEquals;
-import static org.testng.Assert.assertTrue;
 
 @Slf4j
 public class GenericProtobufNativeReaderTest {
@@ -81,12 +79,6 @@ public class GenericProtobufNativeReaderTest {
         assertEquals(nativeRecord.getField(nativeRecord.getDescriptorForType().findFieldByName("doubleField")), DOUBLE_FIELD_VLUE);
     }
 
-    @Test
-    public void testGetNativeSchema() {
-        assertTrue(genericProtobufNativeSchema.getNativeSchema().isPresent());
-        assertTrue(genericProtobufNativeSchema.getNativeSchema().get() instanceof Descriptors.Descriptor);
-    }
-
     private static final String STRING_FIELD_VLUE = "stringFieldValue";
     private static final double DOUBLE_FIELD_VLUE = 0.2D;
 
