@@ -30,7 +30,6 @@ import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
@@ -87,7 +86,6 @@ public class OffloadPoliciesImpl implements Serializable, OffloadPolicies {
     public static final Long DEFAULT_OFFLOAD_THRESHOLD_IN_BYTES = null;
     public static final Long DEFAULT_OFFLOAD_THRESHOLD_IN_SECONDS = null;
     public static final Long DEFAULT_OFFLOAD_DELETION_LAG_IN_MILLIS = null;
-    public static final String EXTRA_CONFIG_PREFIX = "managedLedgerOffloadExtraConfig";
 
     public static final String OFFLOAD_THRESHOLD_NAME_IN_CONF_FILE =
             "managedLedgerOffloadAutoTriggerSizeThresholdBytes";
@@ -123,7 +121,8 @@ public class OffloadPoliciesImpl implements Serializable, OffloadPolicies {
     private OffloadedReadPriority managedLedgerOffloadedReadPriority = DEFAULT_OFFLOADED_READ_PRIORITY;
     @Configuration
     @JsonProperty(access = JsonProperty.Access.READ_WRITE)
-    private Map<String, String> managedLedgerExtraConfigurations = new HashMap<>();
+    private Map<String, String> managedLedgerExtraConfigurations = null;
+
     // s3 config, set by service configuration or cli
     @Configuration
     @JsonProperty(access = JsonProperty.Access.READ_WRITE)
@@ -249,7 +248,8 @@ public class OffloadPoliciesImpl implements Serializable, OffloadPolicies {
 
     public static OffloadPoliciesImpl create(Properties properties) {
         OffloadPoliciesImpl data = new OffloadPoliciesImpl();
-        for (Field f : CONFIGURATION_FIELDS) {
+        Field[] fields = OffloadPoliciesImpl.class.getDeclaredFields();
+        Arrays.stream(fields).forEach(f -> {
             if (properties.containsKey(f.getName())) {
                 try {
                     f.setAccessible(true);
@@ -260,15 +260,14 @@ public class OffloadPoliciesImpl implements Serializable, OffloadPolicies {
                                     f.getName(), properties.get(f.getName())), e);
                 }
             }
-        }
-
+        });
         Map<String, String> extraConfigurations = properties.entrySet().stream()
-                .filter(entry -> entry.getKey().toString().startsWith(EXTRA_CONFIG_PREFIX))
-                .collect(Collectors.toMap(
-                        entry -> entry.getKey().toString().replaceFirst(EXTRA_CONFIG_PREFIX, ""),
-                        entry -> entry.getValue().toString()));
+            .filter(entry -> entry.getKey().toString().startsWith("managedLedgerOffloadExtraConfig"))
+            .collect(Collectors.toMap(
+                entry -> entry.getKey().toString().replaceFirst("managedLedgerOffloadExtraConfig", ""),
+                entry -> entry.getValue().toString()));
 
-        data.getManagedLedgerExtraConfigurations().putAll(extraConfigurations);
+        data.setManagedLedgerExtraConfigurations(extraConfigurations);
 
         data.compatibleWithBrokerConfigFile(properties);
         return data;
@@ -347,21 +346,66 @@ public class OffloadPoliciesImpl implements Serializable, OffloadPolicies {
 
     public Properties toProperties() {
         Properties properties = new Properties();
-        for (Field f : CONFIGURATION_FIELDS) {
-            try {
-                f.setAccessible(true);
-                if ("managedLedgerExtraConfigurations".equals(f.getName())) {
-                    Map<String, String> extraConfig = (Map<String, String>) f.get(this);
-                    extraConfig.forEach((key, value) -> {
-                        setProperty(properties, EXTRA_CONFIG_PREFIX + key, value);
-                    });
-                } else {
-                    setProperty(properties, f.getName(), f.get(this));
-                }
-            } catch (Exception e) {
-                throw new IllegalArgumentException("An error occurred while processing the field: " + f.getName(), e);
-            }
+        setProperty(properties, "managedLedgerOffloadedReadPriority", this.getManagedLedgerOffloadedReadPriority());
+        setProperty(properties, "offloadersDirectory", this.getOffloadersDirectory());
+        setProperty(properties, "managedLedgerOffloadDriver", this.getManagedLedgerOffloadDriver());
+        setProperty(properties, "managedLedgerOffloadMaxThreads",
+                this.getManagedLedgerOffloadMaxThreads());
+        setProperty(properties, "managedLedgerOffloadPrefetchRounds",
+                this.getManagedLedgerOffloadPrefetchRounds());
+        setProperty(properties, "managedLedgerOffloadThresholdInBytes",
+                this.getManagedLedgerOffloadThresholdInBytes());
+        setProperty(properties, "managedLedgerOffloadThresholdInSeconds",
+                this.getManagedLedgerOffloadThresholdInSeconds());
+        setProperty(properties, "managedLedgerOffloadDeletionLagInMillis",
+                this.getManagedLedgerOffloadDeletionLagInMillis());
+        setProperty(properties, "managedLedgerOffloadExtraConfigurations",
+                this.getManagedLedgerExtraConfigurations());
+
+        if (this.isS3Driver()) {
+            setProperty(properties, "s3ManagedLedgerOffloadRegion",
+                    this.getS3ManagedLedgerOffloadRegion());
+            setProperty(properties, "s3ManagedLedgerOffloadBucket",
+                    this.getS3ManagedLedgerOffloadBucket());
+            setProperty(properties, "s3ManagedLedgerOffloadServiceEndpoint",
+                    this.getS3ManagedLedgerOffloadServiceEndpoint());
+            setProperty(properties, "s3ManagedLedgerOffloadMaxBlockSizeInBytes",
+                    this.getS3ManagedLedgerOffloadMaxBlockSizeInBytes());
+            setProperty(properties, "s3ManagedLedgerOffloadCredentialId",
+                    this.getS3ManagedLedgerOffloadCredentialId());
+            setProperty(properties, "s3ManagedLedgerOffloadCredentialSecret",
+                    this.getS3ManagedLedgerOffloadCredentialSecret());
+            setProperty(properties, "s3ManagedLedgerOffloadRole",
+                    this.getS3ManagedLedgerOffloadRole());
+            setProperty(properties, "s3ManagedLedgerOffloadRoleSessionName",
+                    this.getS3ManagedLedgerOffloadRoleSessionName());
+            setProperty(properties, "s3ManagedLedgerOffloadReadBufferSizeInBytes",
+                    this.getS3ManagedLedgerOffloadReadBufferSizeInBytes());
+        } else if (this.isGcsDriver()) {
+            setProperty(properties, "gcsManagedLedgerOffloadRegion",
+                    this.getGcsManagedLedgerOffloadRegion());
+            setProperty(properties, "gcsManagedLedgerOffloadBucket",
+                    this.getGcsManagedLedgerOffloadBucket());
+            setProperty(properties, "gcsManagedLedgerOffloadMaxBlockSizeInBytes",
+                    this.getGcsManagedLedgerOffloadMaxBlockSizeInBytes());
+            setProperty(properties, "gcsManagedLedgerOffloadReadBufferSizeInBytes",
+                    this.getGcsManagedLedgerOffloadReadBufferSizeInBytes());
+            setProperty(properties, "gcsManagedLedgerOffloadServiceAccountKeyFile",
+                    this.getGcsManagedLedgerOffloadServiceAccountKeyFile());
+        } else if (this.isFileSystemDriver()) {
+            setProperty(properties, "fileSystemProfilePath", this.getFileSystemProfilePath());
+            setProperty(properties, "fileSystemURI", this.getFileSystemURI());
         }
+
+        setProperty(properties, "managedLedgerOffloadBucket", this.getManagedLedgerOffloadBucket());
+        setProperty(properties, "managedLedgerOffloadRegion", this.getManagedLedgerOffloadRegion());
+        setProperty(properties, "managedLedgerOffloadServiceEndpoint",
+                this.getManagedLedgerOffloadServiceEndpoint());
+        setProperty(properties, "managedLedgerOffloadMaxBlockSizeInBytes",
+                this.getManagedLedgerOffloadMaxBlockSizeInBytes());
+        setProperty(properties, "managedLedgerOffloadReadBufferSizeInBytes",
+                this.getManagedLedgerOffloadReadBufferSizeInBytes());
+
         return properties;
     }
 
