@@ -306,17 +306,16 @@ public class ConcurrentLongHashMap<V> {
         }
 
         V get(long key, int keyHash) {
+            int bucket = keyHash;
+
             long stamp = tryOptimisticRead();
             boolean acquiredLock = false;
 
-            // add local variable here, so OutOfBound won't happen
-            long[] keys = this.keys;
-            V[] values = this.values;
-            // calculate table.length as capacity to avoid rehash changing capacity
-            int bucket = signSafeMod(keyHash, values.length);
-
             try {
                 while (true) {
+                    int capacity = this.capacity;
+                    bucket = signSafeMod(bucket, capacity);
+
                     // First try optimistic locking
                     long storedKey = keys[bucket];
                     V storedValue = values[bucket];
@@ -334,15 +333,16 @@ public class ConcurrentLongHashMap<V> {
                         if (!acquiredLock) {
                             stamp = readLock();
                             acquiredLock = true;
-
-                            // update local variable
-                            keys = this.keys;
-                            values = this.values;
-                            bucket = signSafeMod(keyHash, values.length);
                             storedKey = keys[bucket];
                             storedValue = values[bucket];
                         }
 
+                        if (capacity != this.capacity) {
+                            // There has been a rehashing. We need to restart the search
+                            bucket = keyHash;
+                            continue;
+                        }
+
                         if (storedKey == key) {
                             return storedValue != DeletedValue ? storedValue : null;
                         } else if (storedValue == EmptyValue) {
@@ -350,7 +350,8 @@ public class ConcurrentLongHashMap<V> {
                             return null;
                         }
                     }
-                    bucket = (bucket + 1) & (values.length - 1);
+
+                    ++bucket;
                 }
             } finally {
                 if (acquiredLock) {
