@@ -284,9 +284,6 @@ public class ConcurrentLongLongPairHashMap {
     // A section is a portion of the hash map that is covered by a single
     @SuppressWarnings("serial")
     private static final class Section extends StampedLock {
-        // Each item take up 4 continuous array space.
-        private static final int ITEM_SIZE = 4;
-
         // Keys and values are stored interleaved in the table array
         private volatile long[] table;
 
@@ -309,7 +306,7 @@ public class ConcurrentLongLongPairHashMap {
                 float expandFactor, float shrinkFactor) {
             this.capacity = alignToPowerOfTwo(capacity);
             this.initCapacity = this.capacity;
-            this.table = new long[ITEM_SIZE * this.capacity];
+            this.table = new long[4 * this.capacity];
             this.size = 0;
             this.usedBuckets = 0;
             this.autoShrink = autoShrink;
@@ -325,10 +322,7 @@ public class ConcurrentLongLongPairHashMap {
         LongPair get(long key1, long key2, int keyHash) {
             long stamp = tryOptimisticRead();
             boolean acquiredLock = false;
-            // add local variable here, so OutOfBound won't happen
-            long[] table = this.table;
-            // calculate table.length / 4 as capacity to avoid rehash changing capacity
-            int bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+            int bucket = signSafeMod(keyHash, capacity);
 
             try {
                 while (true) {
@@ -351,9 +345,8 @@ public class ConcurrentLongLongPairHashMap {
                         if (!acquiredLock) {
                             stamp = readLock();
                             acquiredLock = true;
-                            // update local variable
-                            table = this.table;
-                            bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+
+                            bucket = signSafeMod(keyHash, capacity);
                             storedKey1 = table[bucket];
                             storedKey2 = table[bucket + 1];
                             storedValue1 = table[bucket + 2];
@@ -368,7 +361,7 @@ public class ConcurrentLongLongPairHashMap {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 4) & (table.length - 1);
                 }
             } finally {
                 if (acquiredLock) {
@@ -420,7 +413,7 @@ public class ConcurrentLongLongPairHashMap {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 4) & (table.length - 1);
                 }
             } finally {
                 if (usedBuckets > resizeThresholdUp) {
@@ -461,7 +454,7 @@ public class ConcurrentLongLongPairHashMap {
                         return false;
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 4) & (table.length - 1);
                 }
 
             } finally {
@@ -487,7 +480,7 @@ public class ConcurrentLongLongPairHashMap {
         }
 
         private void cleanBucket(int bucket) {
-            int nextInArray = (bucket + ITEM_SIZE) & (table.length - 1);
+            int nextInArray = (bucket + 4) & (table.length - 1);
             if (table[nextInArray] == EmptyKey) {
                 table[bucket] = EmptyKey;
                 table[bucket + 1] = EmptyKey;
@@ -496,7 +489,7 @@ public class ConcurrentLongLongPairHashMap {
                 --usedBuckets;
 
                 // Cleanup all the buckets that were in `DeletedKey` state, so that we can reduce unnecessary expansions
-                bucket = (bucket - ITEM_SIZE) & (table.length - 1);
+                bucket = (bucket - 4) & (table.length - 1);
                 while (table[bucket] == DeletedKey) {
                     table[bucket] = EmptyKey;
                     table[bucket + 1] = EmptyKey;
@@ -504,7 +497,7 @@ public class ConcurrentLongLongPairHashMap {
                     table[bucket + 3] = ValueNotFound;
                     --usedBuckets;
 
-                    bucket = (bucket - ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket - 4) & (table.length - 1);
                 }
             } else {
                 table[bucket] = DeletedKey;
@@ -547,7 +540,7 @@ public class ConcurrentLongLongPairHashMap {
                 }
 
                 // Go through all the buckets for this section
-                for (int bucket = 0; bucket < table.length; bucket += ITEM_SIZE) {
+                for (int bucket = 0; bucket < table.length; bucket += 4) {
                     long storedKey1 = table[bucket];
                     long storedKey2 = table[bucket + 1];
                     long storedValue1 = table[bucket + 2];
@@ -576,11 +569,11 @@ public class ConcurrentLongLongPairHashMap {
         }
 
         private void rehash(int newCapacity) {
-            long[] newTable = new long[ITEM_SIZE * newCapacity];
+            long[] newTable = new long[4 * newCapacity];
             Arrays.fill(newTable, EmptyKey);
 
             // Re-hash table
-            for (int i = 0; i < table.length; i += ITEM_SIZE) {
+            for (int i = 0; i < table.length; i += 4) {
                 long storedKey1 = table[i];
                 long storedKey2 = table[i + 1];
                 long storedValue1 = table[i + 2];
@@ -600,7 +593,7 @@ public class ConcurrentLongLongPairHashMap {
         }
 
         private void shrinkToInitCapacity() {
-            long[] newTable = new long[ITEM_SIZE * initCapacity];
+            long[] newTable = new long[4 * initCapacity];
             Arrays.fill(newTable, EmptyKey);
 
             table = newTable;
@@ -629,7 +622,7 @@ public class ConcurrentLongLongPairHashMap {
                     return;
                 }
 
-                bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                bucket = (bucket + 4) & (table.length - 1);
             }
         }
     }
@@ -648,8 +641,6 @@ public class ConcurrentLongLongPairHashMap {
     }
 
     static final int signSafeMod(long n, int max) {
-        // as the ITEM_SIZE of Section is 4, so the index is the multiple of 4
-        // that is to left shift 2 bits
         return (int) (n & (max - 1)) << 2;
     }
 
