@@ -296,9 +296,6 @@ public class ConcurrentOpenHashMap<K, V> {
     // A section is a portion of the hash map that is covered by a single
     @SuppressWarnings("serial")
     private static final class Section<K, V> extends StampedLock {
-        // Each item take up 2 continuous array space.
-        private static final int ITEM_SIZE = 2;
-
         // Keys and values are stored interleaved in the table array
         private volatile Object[] table;
 
@@ -320,7 +317,7 @@ public class ConcurrentOpenHashMap<K, V> {
                 float expandFactor, float shrinkFactor) {
             this.capacity = alignToPowerOfTwo(capacity);
             this.initCapacity = this.capacity;
-            this.table = new Object[ITEM_SIZE * this.capacity];
+            this.table = new Object[2 * this.capacity];
             this.size = 0;
             this.usedBuckets = 0;
             this.autoShrink = autoShrink;
@@ -335,11 +332,7 @@ public class ConcurrentOpenHashMap<K, V> {
         V get(K key, int keyHash) {
             long stamp = tryOptimisticRead();
             boolean acquiredLock = false;
-
-            // add local variable here, so OutOfBound won't happen
-            Object[] table = this.table;
-            // calculate table.length / 2 as capacity to avoid rehash changing capacity
-            int bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+            int bucket = signSafeMod(keyHash, capacity);
 
             try {
                 while (true) {
@@ -361,9 +354,7 @@ public class ConcurrentOpenHashMap<K, V> {
                             stamp = readLock();
                             acquiredLock = true;
 
-                            // update local variable
-                            table = this.table;
-                            bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+                            bucket = signSafeMod(keyHash, capacity);
                             storedKey = (K) table[bucket];
                             storedValue = (V) table[bucket + 1];
                         }
@@ -376,7 +367,7 @@ public class ConcurrentOpenHashMap<K, V> {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 2) & (table.length - 1);
                 }
             } finally {
                 if (acquiredLock) {
@@ -429,7 +420,7 @@ public class ConcurrentOpenHashMap<K, V> {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 2) & (table.length - 1);
                 }
             } finally {
                 if (usedBuckets > resizeThresholdUp) {
@@ -458,7 +449,7 @@ public class ConcurrentOpenHashMap<K, V> {
                         if (value == null || value.equals(storedValue)) {
                             SIZE_UPDATER.decrementAndGet(this);
 
-                            int nextInArray = (bucket + ITEM_SIZE) & (table.length - 1);
+                            int nextInArray = (bucket + 2) & (table.length - 1);
                             if (table[nextInArray] == EmptyKey) {
                                 table[bucket] = EmptyKey;
                                 table[bucket + 1] = null;
@@ -466,13 +457,13 @@ public class ConcurrentOpenHashMap<K, V> {
 
                                 // Cleanup all the buckets that were in `DeletedKey` state,
                                 // so that we can reduce unnecessary expansions
-                                int lastBucket = (bucket - ITEM_SIZE) & (table.length - 1);
+                                int lastBucket = (bucket - 2) & (table.length - 1);
                                 while (table[lastBucket] == DeletedKey) {
                                     table[lastBucket] = EmptyKey;
                                     table[lastBucket + 1] = null;
                                     --usedBuckets;
 
-                                    lastBucket = (lastBucket - ITEM_SIZE) & (table.length - 1);
+                                    lastBucket = (lastBucket - 2) & (table.length - 1);
                                 }
                             } else {
                                 table[bucket] = DeletedKey;
@@ -488,7 +479,7 @@ public class ConcurrentOpenHashMap<K, V> {
                         return null;
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 2) & (table.length - 1);
                 }
 
             } finally {
@@ -537,7 +528,7 @@ public class ConcurrentOpenHashMap<K, V> {
             // Go through all the buckets for this section. We try to renew the stamp only after a validation
             // error, otherwise we keep going with the same.
             long stamp = 0;
-            for (int bucket = 0; bucket < table.length; bucket += ITEM_SIZE) {
+            for (int bucket = 0; bucket < table.length; bucket += 2) {
                 if (stamp == 0) {
                     stamp = tryOptimisticRead();
                 }
@@ -567,10 +558,10 @@ public class ConcurrentOpenHashMap<K, V> {
 
         private void rehash(int newCapacity) {
             // Expand the hashmap
-            Object[] newTable = new Object[ITEM_SIZE * newCapacity];
+            Object[] newTable = new Object[2 * newCapacity];
 
             // Re-hash table
-            for (int i = 0; i < table.length; i += ITEM_SIZE) {
+            for (int i = 0; i < table.length; i += 2) {
                 K storedKey = (K) table[i];
                 V storedValue = (V) table[i + 1];
                 if (storedKey != EmptyKey && storedKey != DeletedKey) {
@@ -586,7 +577,7 @@ public class ConcurrentOpenHashMap<K, V> {
         }
 
         private void shrinkToInitCapacity() {
-            Object[] newTable = new Object[ITEM_SIZE * initCapacity];
+            Object[] newTable = new Object[2 * initCapacity];
 
             table = newTable;
             size = 0;
@@ -611,7 +602,7 @@ public class ConcurrentOpenHashMap<K, V> {
                     return;
                 }
 
-                bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                bucket = (bucket + 2) & (table.length - 1);
             }
         }
     }
@@ -627,8 +618,6 @@ public class ConcurrentOpenHashMap<K, V> {
     }
 
     static final int signSafeMod(long n, int max) {
-        // as the ITEM_SIZE of Section is 2, so the index is the multiple of 2
-        // that is to left shift 1 bit
         return (int) (n & (max - 1)) << 1;
     }
 
