@@ -30,11 +30,9 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Random;
 import java.util.Set;
-import java.util.concurrent.CyclicBarrier;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
-import java.util.concurrent.atomic.AtomicReference;
 
 import lombok.Cleanup;
 import org.apache.pulsar.common.util.collections.ConcurrentLongPairSet.LongPair;
@@ -213,68 +211,6 @@ public class ConcurrentLongPairSetTest {
         assertTrue(map.capacity() == initCapacity);
     }
 
-    @Test
-    public void testConcurrentExpandAndShrinkAndGet()  throws Throwable {
-        ConcurrentLongPairSet set = ConcurrentLongPairSet.newBuilder()
-                .expectedItems(2)
-                .concurrencyLevel(1)
-                .autoShrink(true)
-                .mapIdleFactor(0.25f)
-                .build();
-        assertEquals(set.capacity(), 4);
-
-        ExecutorService executor = Executors.newCachedThreadPool();
-        final int readThreads = 16;
-        final int writeThreads = 1;
-        final int n = 1_000;
-        CyclicBarrier barrier = new CyclicBarrier(writeThreads + readThreads);
-        Future<?> future = null;
-        AtomicReference<Exception> ex = new AtomicReference<>();
-
-        for (int i = 0; i < readThreads; i++) {
-            executor.submit(() -> {
-                try {
-                    barrier.await();
-                } catch (Exception e) {
-                    throw new RuntimeException(e);
-                }
-                while (true) {
-                    try {
-                        set.contains(1, 1);
-                    } catch (Exception e) {
-                        ex.set(e);
-                    }
-                }
-            });
-        }
-
-        assertTrue(set.add(1, 1));
-        future = executor.submit(() -> {
-            try {
-                barrier.await();
-            } catch (Exception e) {
-                throw new RuntimeException(e);
-            }
-
-            for (int i = 0; i < n; i++) {
-                // expand hashmap
-                assertTrue(set.add(2, 2));
-                assertTrue(set.add(3, 3));
-                assertEquals(set.capacity(), 8);
-
-                // shrink hashmap
-                assertTrue(set.remove(2, 2));
-                assertTrue(set.remove(3, 3));
-                assertEquals(set.capacity(), 4);
-            }
-        });
-
-        future.get();
-        assertTrue(ex.get() == null);
-        // shut down pool
-        executor.shutdown();
-    }
-
 
     @Test
     public void testRemove() {
