@@ -18,18 +18,17 @@
  */
 package org.apache.pulsar.functions.sink;
 
-import static org.apache.pulsar.functions.instance.ProducerCache.CacheArea.SINK_RECORD_CACHE;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyBoolean;
 import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.ArgumentMatchers.anyList;
 import static org.mockito.ArgumentMatchers.anyLong;
 import static org.mockito.ArgumentMatchers.anyString;
+import static org.mockito.ArgumentMatchers.argThat;
 import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
-import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertNull;
@@ -37,6 +36,7 @@ import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
 import java.io.IOException;
 import java.util.HashMap;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
 import lombok.Getter;
@@ -56,7 +56,6 @@ import org.apache.pulsar.client.api.schema.GenericRecordBuilder;
 import org.apache.pulsar.client.api.schema.GenericSchema;
 import org.apache.pulsar.client.api.schema.RecordSchemaBuilder;
 import org.apache.pulsar.client.api.schema.SchemaBuilder;
-import org.apache.pulsar.client.impl.ConnectionPool;
 import org.apache.pulsar.client.impl.PulsarClientImpl;
 import org.apache.pulsar.client.impl.schema.AutoConsumeSchema;
 import org.apache.pulsar.common.functions.FunctionConfig;
@@ -64,14 +63,12 @@ import org.apache.pulsar.common.functions.FunctionConfig.ProcessingGuarantees;
 import org.apache.pulsar.common.schema.SchemaType;
 import org.apache.pulsar.functions.api.Record;
 import org.apache.pulsar.functions.api.SerDe;
-import org.apache.pulsar.functions.instance.ProducerCache;
 import org.apache.pulsar.functions.instance.SinkRecord;
 import org.apache.pulsar.functions.instance.stats.ComponentStatsManager;
 import org.apache.pulsar.functions.sink.PulsarSink.PulsarSinkProcessorBase;
 import org.apache.pulsar.functions.source.TopicSchema;
 import org.apache.pulsar.io.core.SinkContext;
 import org.testng.Assert;
-import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
 
@@ -98,8 +95,6 @@ public class PulsarSinkTest {
      */
     private static PulsarClientImpl getPulsarClient() throws PulsarClientException {
         PulsarClientImpl pulsarClient = mock(PulsarClientImpl.class);
-        ConnectionPool connectionPool = mock(ConnectionPool.class);
-        when(pulsarClient.getCnxPool()).thenReturn(connectionPool);
         ConsumerBuilder consumerBuilder = mock(ConsumerBuilder.class);
         doReturn(consumerBuilder).when(consumerBuilder).topics(anyList());
         doReturn(consumerBuilder).when(consumerBuilder).subscriptionName(anyString());
@@ -133,7 +128,6 @@ public class PulsarSinkTest {
         doReturn(producer).when(producerBuilder).create();
         doReturn(typedMessageBuilder).when(producer).newMessage();
         doReturn(typedMessageBuilder).when(producer).newMessage(any(Schema.class));
-        doReturn(CompletableFuture.completedFuture(null)).when(producer).flushAsync();
 
         doReturn(producerBuilder).when(pulsarClient).newProducer();
         doReturn(producerBuilder).when(pulsarClient).newProducer(any());
@@ -141,17 +135,9 @@ public class PulsarSinkTest {
         return pulsarClient;
     }
 
-    ProducerCache producerCache;
-
-    @BeforeMethod(alwaysRun = true)
+    @BeforeMethod
     public void setup() {
-        producerCache = new ProducerCache();
-    }
 
-    @AfterMethod(alwaysRun = true)
-    public void tearDown() {
-        producerCache.close();
-        producerCache = null;
     }
 
     private static PulsarSinkConfig getPulsarConfigs() {
@@ -192,7 +178,7 @@ public class PulsarSinkTest {
         pulsarConfig.setTypeClassName(Void.class.getName());
         PulsarSink pulsarSink =
                 new PulsarSink(getPulsarClient(), pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                        Thread.currentThread().getContextClassLoader(), producerCache);
+                        Thread.currentThread().getContextClassLoader());
 
         try {
             Schema schema = pulsarSink.initializeSchema();
@@ -212,7 +198,7 @@ public class PulsarSinkTest {
         pulsarConfig.setSerdeClassName(TestSerDe.class.getName());
         PulsarSink pulsarSink =
                 new PulsarSink(getPulsarClient(), pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                        Thread.currentThread().getContextClassLoader(), producerCache);
+                        Thread.currentThread().getContextClassLoader());
         try {
             pulsarSink.initializeSchema();
             fail("Should fail constructing java instance if function type is inconsistent with serde type");
@@ -237,7 +223,7 @@ public class PulsarSinkTest {
         pulsarConfig.setTypeClassName(String.class.getName());
         PulsarSink pulsarSink =
                 new PulsarSink(getPulsarClient(), pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                        Thread.currentThread().getContextClassLoader(), producerCache);
+                        Thread.currentThread().getContextClassLoader());
 
         try {
             pulsarSink.initializeSchema();
@@ -258,7 +244,7 @@ public class PulsarSinkTest {
         pulsarConfig.setSerdeClassName(TopicSchema.DEFAULT_SERDE);
         PulsarSink pulsarSink =
                 new PulsarSink(getPulsarClient(), pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                        Thread.currentThread().getContextClassLoader(), producerCache);
+                        Thread.currentThread().getContextClassLoader());
 
         try {
             pulsarSink.initializeSchema();
@@ -276,7 +262,7 @@ public class PulsarSinkTest {
         pulsarConfig.setSerdeClassName(ComplexSerDe.class.getName());
         PulsarSink pulsarSink =
                 new PulsarSink(getPulsarClient(), pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                        Thread.currentThread().getContextClassLoader(), producerCache);
+                        Thread.currentThread().getContextClassLoader());
 
         try {
             pulsarSink.initializeSchema();
@@ -296,7 +282,7 @@ public class PulsarSinkTest {
         pulsarSinkConfig.setTypeClassName(GenericRecord.class.getName());
         PulsarSink sink = new PulsarSink(
             pulsarClient, pulsarSinkConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-            Thread.currentThread().getContextClassLoader(), producerCache);
+            Thread.currentThread().getContextClassLoader());
         Schema<?> schema = sink.initializeSchema();
         assertTrue(schema instanceof AutoConsumeSchema);
 
@@ -305,7 +291,7 @@ public class PulsarSinkTest {
         pulsarSinkConfig.setTypeClassName(GenericRecord.class.getName());
         sink = new PulsarSink(
             pulsarClient, pulsarSinkConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-            Thread.currentThread().getContextClassLoader(), producerCache);
+            Thread.currentThread().getContextClassLoader());
         schema = sink.initializeSchema();
         assertTrue(schema instanceof AutoConsumeSchema);
 
@@ -316,7 +302,7 @@ public class PulsarSinkTest {
         pulsarSinkConfig.setTypeClassName(GenericRecord.class.getName());
         sink = new PulsarSink(
             pulsarClient, pulsarSinkConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-            Thread.currentThread().getContextClassLoader(), producerCache);
+            Thread.currentThread().getContextClassLoader());
         schema = sink.initializeSchema();
         assertTrue(schema instanceof AutoConsumeSchema);
 
@@ -327,7 +313,7 @@ public class PulsarSinkTest {
         pulsarSinkConfig.setTypeClassName(GenericRecord.class.getName());
         sink = new PulsarSink(
             pulsarClient, pulsarSinkConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-            Thread.currentThread().getContextClassLoader(), producerCache);
+            Thread.currentThread().getContextClassLoader());
         schema = sink.initializeSchema();
         assertTrue(schema instanceof AutoConsumeSchema);
 
@@ -337,7 +323,7 @@ public class PulsarSinkTest {
         pulsarSinkConfig.setTypeClassName(GenericRecord.class.getName());
         sink = new PulsarSink(
             pulsarClient, pulsarSinkConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-            Thread.currentThread().getContextClassLoader(), producerCache);
+            Thread.currentThread().getContextClassLoader());
         schema = sink.initializeSchema();
         assertTrue(schema instanceof AutoConsumeSchema);
     }
@@ -354,12 +340,9 @@ public class PulsarSinkTest {
         /** test MANUAL **/
         pulsarClient = getPulsarClient();
         pulsarConfig.setProcessingGuarantees(ProcessingGuarantees.MANUAL);
-        PulsarSink pulsarSink =
-                new PulsarSink(pulsarClient, pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                        Thread.currentThread().getContextClassLoader(), producerCache);
+        PulsarSink pulsarSink = new PulsarSink(pulsarClient, pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class), Thread.currentThread().getContextClassLoader());
 
         pulsarSink.open(new HashMap<>(), mock(SinkContext.class));
-        verify(pulsarClient.newProducer(), times(1)).topic(defaultTopic);
 
         for (String topic : topics) {
 
@@ -383,19 +366,23 @@ public class PulsarSinkTest {
             PulsarSink.PulsarSinkManualProcessor pulsarSinkManualProcessor
                     = (PulsarSink.PulsarSinkManualProcessor) pulsarSink.pulsarSinkProcessor;
             if (topic != null) {
-                Assert.assertTrue(producerCache.containsKey(SINK_RECORD_CACHE, topic));
+                Assert.assertTrue(pulsarSinkManualProcessor.publishProducers.containsKey(topic));
             } else {
-                Assert.assertTrue(producerCache.containsKey(SINK_RECORD_CACHE, defaultTopic));
+                Assert.assertTrue(pulsarSinkManualProcessor.publishProducers.containsKey(defaultTopic));
             }
-            String actualTopic = topic != null ? topic : defaultTopic;
-            verify(pulsarClient.newProducer(), times(1)).topic(actualTopic);
+            verify(pulsarClient.newProducer(), times(1)).topic(argThat(otherTopic -> {
+                if (topic != null) {
+                    return topic.equals(otherTopic);
+                } else {
+                    return defaultTopic.equals(otherTopic);
+                }
+            }));
         }
 
         /** test At-least-once **/
         pulsarClient = getPulsarClient();
         pulsarConfig.setProcessingGuarantees(ProcessingGuarantees.ATLEAST_ONCE);
-        pulsarSink = new PulsarSink(pulsarClient, pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                Thread.currentThread().getContextClassLoader(), producerCache);
+        pulsarSink = new PulsarSink(pulsarClient, pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class), Thread.currentThread().getContextClassLoader());
 
         pulsarSink.open(new HashMap<>(), mock(SinkContext.class));
 
@@ -419,17 +406,24 @@ public class PulsarSinkTest {
             PulsarSink.PulsarSinkAtLeastOnceProcessor pulsarSinkAtLeastOnceProcessor
                     = (PulsarSink.PulsarSinkAtLeastOnceProcessor) pulsarSink.pulsarSinkProcessor;
             if (topic != null) {
-                Assert.assertTrue(producerCache.containsKey(SINK_RECORD_CACHE, topic));
+                Assert.assertTrue(pulsarSinkAtLeastOnceProcessor.publishProducers.containsKey(topic));
             } else {
-                Assert.assertTrue(producerCache.containsKey(SINK_RECORD_CACHE, defaultTopic));
+                Assert.assertTrue(pulsarSinkAtLeastOnceProcessor.publishProducers.containsKey(defaultTopic));
             }
+            verify(pulsarClient.newProducer(), times(1)).topic(argThat(otherTopic -> {
+                if (topic != null) {
+                    return topic.equals(otherTopic);
+                } else {
+                    return defaultTopic.equals(otherTopic);
+                }
+            }));
         }
 
         /** test At-most-once **/
         pulsarClient = getPulsarClient();
         pulsarConfig.setProcessingGuarantees(ProcessingGuarantees.ATMOST_ONCE);
         pulsarSink = new PulsarSink(pulsarClient, pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                Thread.currentThread().getContextClassLoader(), producerCache);
+                Thread.currentThread().getContextClassLoader());
 
         pulsarSink.open(new HashMap<>(), mock(SinkContext.class));
 
@@ -459,17 +453,20 @@ public class PulsarSinkTest {
             PulsarSink.PulsarSinkAtMostOnceProcessor pulsarSinkAtLeastOnceProcessor
                     = (PulsarSink.PulsarSinkAtMostOnceProcessor) pulsarSink.pulsarSinkProcessor;
             if (topic != null) {
-                Assert.assertTrue(producerCache.containsKey(SINK_RECORD_CACHE, topic));
+                Assert.assertTrue(pulsarSinkAtLeastOnceProcessor.publishProducers.containsKey(topic));
             } else {
-                Assert.assertTrue(producerCache.containsKey(SINK_RECORD_CACHE, defaultTopic));
+                Assert.assertTrue(pulsarSinkAtLeastOnceProcessor.publishProducers.containsKey(defaultTopic));
             }
+            verify(pulsarClient.newProducer(), times(1)).topic(argThat(o -> {
+                return getTopicEquals(o, topic, defaultTopic);
+            }));
         }
 
         /** test Effectively-once **/
         pulsarClient = getPulsarClient();
         pulsarConfig.setProcessingGuarantees(FunctionConfig.ProcessingGuarantees.EFFECTIVELY_ONCE);
         pulsarSink = new PulsarSink(pulsarClient, pulsarConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-                Thread.currentThread().getContextClassLoader(), producerCache);
+                Thread.currentThread().getContextClassLoader());
 
         pulsarSink.open(new HashMap<>(), mock(SinkContext.class));
 
@@ -519,19 +516,23 @@ public class PulsarSinkTest {
             PulsarSink.PulsarSinkEffectivelyOnceProcessor pulsarSinkEffectivelyOnceProcessor
                     = (PulsarSink.PulsarSinkEffectivelyOnceProcessor) pulsarSink.pulsarSinkProcessor;
             if (topic != null) {
-                Assert.assertTrue(producerCache
-                        .containsKey(SINK_RECORD_CACHE, topic, String.format("%s-id-1", topic)));
+                Assert.assertTrue(pulsarSinkEffectivelyOnceProcessor.publishProducers
+                        .containsKey(String.format("%s-%s-id-1", topic, topic)));
             } else {
-                Assert.assertTrue(producerCache
-                        .containsKey(SINK_RECORD_CACHE,
-                                defaultTopic, String.format("%s-id-1", defaultTopic)
-                        ));
+                Assert.assertTrue(pulsarSinkEffectivelyOnceProcessor.publishProducers
+                        .containsKey(String.format("%s-%s-id-1", defaultTopic, defaultTopic)));
             }
 
-            String expectedTopicName = topic != null ? topic : defaultTopic;
-            verify(pulsarClient.newProducer(), times(1)).topic(expectedTopicName);
-            String expectedProducerName = String.format("%s-id-1", expectedTopicName);
-            verify(pulsarClient.newProducer(), times(1)).producerName(expectedProducerName);
+            verify(pulsarClient.newProducer(), times(1)).topic(argThat(o -> {
+                return getTopicEquals(o, topic, defaultTopic);
+            }));
+            verify(pulsarClient.newProducer(), times(1)).producerName(argThat(o -> {
+                if (topic != null) {
+                    return String.format("%s-id-1", topic).equals(o);
+                } else {
+                    return String.format("%s-id-1", defaultTopic).equals(o);
+                }
+            }));
         }
     }
 
@@ -561,7 +562,7 @@ public class PulsarSinkTest {
         PulsarClient client = getPulsarClient();
         PulsarSink pulsarSink = new PulsarSink(
             client, sinkConfig, new HashMap<>(), mock(ComponentStatsManager.class),
-            Thread.currentThread().getContextClassLoader(), producerCache);
+            Thread.currentThread().getContextClassLoader());
 
         pulsarSink.open(new HashMap<>(), mock(SinkContext.class));
 
@@ -573,7 +574,7 @@ public class PulsarSinkTest {
             assertTrue(pulsarSink.pulsarSinkProcessor instanceof PulsarSink.PulsarSinkEffectivelyOnceProcessor);
         }
         PulsarSinkProcessorBase processor = (PulsarSinkProcessorBase) pulsarSink.pulsarSinkProcessor;
-        assertFalse(producerCache.containsKey(SINK_RECORD_CACHE, defaultTopic));
+        assertFalse(processor.publishProducers.containsKey(defaultTopic));
 
         String[] topics = {"topic-1", "topic-2", "topic-3"};
         for (String topic : topics) {
@@ -620,15 +621,17 @@ public class PulsarSinkTest {
             pulsarSink.write(record);
 
             if (ProcessingGuarantees.EFFECTIVELY_ONCE == guarantees) {
-                assertTrue(producerCache.containsKey(SINK_RECORD_CACHE,
-                        topic, String.format("%s-id-1", topic)
-                ));
+                assertTrue(processor.publishProducers.containsKey(String.format("%s-%s-id-1", topic, topic)));
             } else {
-                assertTrue(producerCache.containsKey(SINK_RECORD_CACHE, topic));
+                assertTrue(processor.publishProducers.containsKey(topic));
             }
-            String expectedTopicName = topic != null ? topic : defaultTopic;
-            verify(client.newProducer(), times(1)).topic(expectedTopicName);
-            verify(client, times(1)).newProducer(schema);
+            verify(client.newProducer(), times(1))
+                .topic(argThat(
+                    otherTopic -> topic != null ? topic.equals(otherTopic) : defaultTopic.equals(otherTopic)));
+
+            verify(client, times(1))
+                .newProducer(argThat(
+                    otherSchema -> Objects.equals(otherSchema, schema)));
         }
     }
 
@@ -639,4 +642,13 @@ public class PulsarSinkTest {
             return Optional.empty();
         }
     }
+
+    private boolean getTopicEquals(Object o, String topic, String defaultTopic) {
+        if (topic != null) {
+            return topic.equals(o);
+        } else {
+            return defaultTopic.equals(o);
+        }
+    }
+
 }
