@@ -18,11 +18,12 @@
  */
 package org.apache.pulsar.functions.utils;
 
-import static org.apache.commons.lang3.StringUtils.isBlank;
+import static org.apache.commons.lang.StringUtils.isBlank;
+import static org.apache.commons.lang.StringUtils.isNotBlank;
+import static org.apache.commons.lang.StringUtils.isNotEmpty;
 import static org.apache.commons.lang3.StringUtils.isEmpty;
-import static org.apache.commons.lang3.StringUtils.isNotBlank;
-import static org.apache.commons.lang3.StringUtils.isNotEmpty;
 import static org.apache.pulsar.common.functions.Utils.BUILTIN;
+import static org.apache.pulsar.common.util.ClassLoaderUtils.loadJar;
 import static org.apache.pulsar.functions.utils.FunctionCommon.convertFromCompressionType;
 import static org.apache.pulsar.functions.utils.FunctionCommon.convertFromFunctionDetailsCompressionType;
 import static org.apache.pulsar.functions.utils.FunctionCommon.convertFromFunctionDetailsSubscriptionPosition;
@@ -31,7 +32,9 @@ import com.fasterxml.jackson.databind.ObjectMapper;
 import com.google.gson.Gson;
 import com.google.gson.reflect.TypeToken;
 import java.io.File;
+import java.io.IOException;
 import java.lang.reflect.Type;
+import java.net.MalformedURLException;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.LinkedList;
@@ -41,13 +44,10 @@ import lombok.AllArgsConstructor;
 import lombok.Getter;
 import lombok.Setter;
 import lombok.extern.slf4j.Slf4j;
-import net.bytebuddy.description.type.TypeDefinition;
-import net.bytebuddy.pool.TypePool;
-import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang.StringUtils;
 import org.apache.pulsar.client.api.SubscriptionInitialPosition;
 import org.apache.pulsar.common.functions.ConsumerConfig;
 import org.apache.pulsar.common.functions.FunctionConfig;
-import org.apache.pulsar.common.functions.FunctionDefinition;
 import org.apache.pulsar.common.functions.ProducerConfig;
 import org.apache.pulsar.common.functions.Resources;
 import org.apache.pulsar.common.functions.WindowConfig;
@@ -55,6 +55,7 @@ import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.util.ObjectMapperFactory;
 import org.apache.pulsar.functions.proto.Function;
 import org.apache.pulsar.functions.proto.Function.FunctionDetails;
+import org.apache.pulsar.functions.utils.functions.FunctionUtils;
 
 @Slf4j
 public class FunctionConfigUtils {
@@ -73,21 +74,26 @@ public class FunctionConfigUtils {
 
     private static final ObjectMapper OBJECT_MAPPER = ObjectMapperFactory.create();
 
-    public static FunctionDetails convert(FunctionConfig functionConfig) {
-        return convert(functionConfig, (ValidatableFunctionPackage) null);
-    }
-
-    public static FunctionDetails convert(FunctionConfig functionConfig,
-                                          ValidatableFunctionPackage validatableFunctionPackage)
+    public static FunctionDetails convert(FunctionConfig functionConfig, ClassLoader classLoader)
             throws IllegalArgumentException {
-        if (functionConfig == null) {
-            throw new IllegalArgumentException("Function config is not provided");
-        }
-        if (functionConfig.getRuntime() == FunctionConfig.Runtime.JAVA && validatableFunctionPackage != null) {
-            return convert(functionConfig, doJavaChecks(functionConfig, validatableFunctionPackage));
-        } else {
-            return convert(functionConfig, new ExtractedFunctionDetails(functionConfig.getClassName(), null, null));
+
+        if (functionConfig.getRuntime() == FunctionConfig.Runtime.JAVA) {
+            if (classLoader != null) {
+                try {
+                    Class<?>[] typeArgs = FunctionCommon.getFunctionTypes(functionConfig, classLoader);
+                    return convert(
+                            functionConfig,
+                            new ExtractedFunctionDetails(
+                                    functionConfig.getClassName(),
+                                    typeArgs[0].getName(),
+                                    typeArgs[1].getName()));
+                } catch (ClassNotFoundException | NoClassDefFoundError e) {
+                    throw new IllegalArgumentException(
+                            String.format("Function class %s must be in class path", functionConfig.getClassName()), e);
+                }
+            }
         }
+        return convert(functionConfig, new ExtractedFunctionDetails(functionConfig.getClassName(), null, null));
     }
 
     public static FunctionDetails convert(FunctionConfig functionConfig, ExtractedFunctionDetails extractedDetails)
@@ -250,13 +256,29 @@ public class FunctionConfigUtils {
             sinkSpecBuilder.setTypeClassName(functionConfig.getOutputTypeClassName());
         }
         if (functionConfig.getProducerConfig() != null) {
-            sinkSpecBuilder.setProducerSpec(convertProducerConfigToProducerSpec(functionConfig.getProducerConfig()));
-        }
-        if (functionConfig.getBatchBuilder() != null) {
-            Function.ProducerSpec.Builder builder = sinkSpecBuilder.getProducerSpec() != null
-                    ? sinkSpecBuilder.getProducerSpec().toBuilder()
-                    : Function.ProducerSpec.newBuilder();
-            sinkSpecBuilder.setProducerSpec(builder.setBatchBuilder(functionConfig.getBatchBuilder()).build());
+            ProducerConfig producerConf = functionConfig.getProducerConfig();
+            Function.ProducerSpec.Builder pbldr = Function.ProducerSpec.newBuilder();
+            if (producerConf.getMaxPendingMessages() != null) {
+                pbldr.setMaxPendingMessages(producerConf.getMaxPendingMessages());
+            }
+            if (producerConf.getMaxPendingMessagesAcrossPartitions() != null) {
+                pbldr.setMaxPendingMessagesAcrossPartitions(producerConf.getMaxPendingMessagesAcrossPartitions());
+            }
+            if (producerConf.getUseThreadLocalProducers() != null) {
+                pbldr.setUseThreadLocalProducers(producerConf.getUseThreadLocalProducers());
+            }
+            if (producerConf.getCryptoConfig() != null) {
+                pbldr.setCryptoSpec(CryptoUtils.convert(producerConf.getCryptoConfig()));
+            }
+            if (producerConf.getBatchBuilder() != null) {
+                pbldr.setBatchBuilder(producerConf.getBatchBuilder());
+            }
+            if (producerConf.getCompressionType() != null) {
+                pbldr.setCompressionType(convertFromCompressionType(producerConf.getCompressionType()));
+            } else {
+                pbldr.setCompressionType(Function.CompressionType.LZ4);
+            }
+            sinkSpecBuilder.setProducerSpec(pbldr.build());
         }
         functionDetailsBuilder.setSink(sinkSpecBuilder);
 
@@ -441,8 +463,23 @@ public class FunctionConfigUtils {
             functionConfig.setOutputSchemaType(functionDetails.getSink().getSchemaType());
         }
         if (functionDetails.getSink().getProducerSpec() != null) {
-            functionConfig.setProducerConfig(
-                    convertProducerSpecToProducerConfig(functionDetails.getSink().getProducerSpec()));
+            Function.ProducerSpec spec = functionDetails.getSink().getProducerSpec();
+            ProducerConfig producerConfig = new ProducerConfig();
+            if (spec.getMaxPendingMessages() != 0) {
+                producerConfig.setMaxPendingMessages(spec.getMaxPendingMessages());
+            }
+            if (spec.getMaxPendingMessagesAcrossPartitions() != 0) {
+                producerConfig.setMaxPendingMessagesAcrossPartitions(spec.getMaxPendingMessagesAcrossPartitions());
+            }
+            if (spec.hasCryptoSpec()) {
+                producerConfig.setCryptoConfig(CryptoUtils.convertFromSpec(spec.getCryptoSpec()));
+            }
+            if (spec.getBatchBuilder() != null) {
+                producerConfig.setBatchBuilder(spec.getBatchBuilder());
+            }
+            producerConfig.setUseThreadLocalProducers(spec.getUseThreadLocalProducers());
+            producerConfig.setCompressionType(convertFromFunctionDetailsCompressionType(spec.getCompressionType()));
+            functionConfig.setProducerConfig(producerConfig);
         }
         if (!isEmpty(functionDetails.getLogTopic())) {
             functionConfig.setLogTopic(functionDetails.getLogTopic());
@@ -507,50 +544,6 @@ public class FunctionConfigUtils {
         return functionConfig;
     }
 
-    public static Function.ProducerSpec convertProducerConfigToProducerSpec(ProducerConfig producerConf) {
-        Function.ProducerSpec.Builder builder = Function.ProducerSpec.newBuilder();
-        if (producerConf.getMaxPendingMessages() != null) {
-            builder.setMaxPendingMessages(producerConf.getMaxPendingMessages());
-        }
-        if (producerConf.getMaxPendingMessagesAcrossPartitions() != null) {
-            builder.setMaxPendingMessagesAcrossPartitions(producerConf.getMaxPendingMessagesAcrossPartitions());
-        }
-        if (producerConf.getUseThreadLocalProducers() != null) {
-            builder.setUseThreadLocalProducers(producerConf.getUseThreadLocalProducers());
-        }
-        if (producerConf.getCryptoConfig() != null) {
-            builder.setCryptoSpec(CryptoUtils.convert(producerConf.getCryptoConfig()));
-        }
-        if (producerConf.getBatchBuilder() != null) {
-            builder.setBatchBuilder(producerConf.getBatchBuilder());
-        }
-        if (producerConf.getCompressionType() != null) {
-            builder.setCompressionType(convertFromCompressionType(producerConf.getCompressionType()));
-        } else {
-            builder.setCompressionType(Function.CompressionType.LZ4);
-        }
-        return builder.build();
-    }
-
-    public static ProducerConfig convertProducerSpecToProducerConfig(Function.ProducerSpec spec) {
-        ProducerConfig producerConfig = new ProducerConfig();
-        if (spec.getMaxPendingMessages() != 0) {
-            producerConfig.setMaxPendingMessages(spec.getMaxPendingMessages());
-        }
-        if (spec.getMaxPendingMessagesAcrossPartitions() != 0) {
-            producerConfig.setMaxPendingMessagesAcrossPartitions(spec.getMaxPendingMessagesAcrossPartitions());
-        }
-        if (spec.hasCryptoSpec()) {
-            producerConfig.setCryptoConfig(CryptoUtils.convertFromSpec(spec.getCryptoSpec()));
-        }
-        if (spec.getBatchBuilder() != null) {
-            producerConfig.setBatchBuilder(spec.getBatchBuilder());
-        }
-        producerConfig.setUseThreadLocalProducers(spec.getUseThreadLocalProducers());
-        producerConfig.setCompressionType(convertFromFunctionDetailsCompressionType(spec.getCompressionType()));
-        return producerConfig;
-    }
-
     public static void inferMissingArguments(FunctionConfig functionConfig,
                                              boolean forwardSourceMessagePropertyEnabled) {
         if (StringUtils.isEmpty(functionConfig.getName())) {
@@ -594,49 +587,48 @@ public class FunctionConfigUtils {
         }
     }
 
-    public static ExtractedFunctionDetails doJavaChecks(FunctionConfig functionConfig,
-                                                        ValidatableFunctionPackage validatableFunctionPackage) {
+    public static ExtractedFunctionDetails doJavaChecks(FunctionConfig functionConfig, ClassLoader clsLoader) {
 
-        String functionClassName = StringUtils.trimToNull(functionConfig.getClassName());
-        TypeDefinition functionClass;
+        String functionClassName = functionConfig.getClassName();
+        Class functionClass;
         try {
             // if class name in function config is not set, this should be a built-in function
             // thus we should try to find its class name in the NAR service definition
             if (functionClassName == null) {
-                FunctionDefinition functionDefinition =
-                        validatableFunctionPackage.getFunctionMetaData(FunctionDefinition.class);
-                if (functionDefinition == null) {
-                    throw new IllegalArgumentException("Function class name is not provided.");
-                }
-                functionClassName = functionDefinition.getFunctionClass();
-                if (functionClassName == null) {
-                    throw new IllegalArgumentException("Function class name is not provided.");
+                try {
+                    functionClassName = FunctionUtils.getFunctionClass(clsLoader);
+                } catch (IOException e) {
+                    throw new IllegalArgumentException("Failed to extract source class from archive", e);
                 }
             }
-            functionClass = validatableFunctionPackage.resolveType(functionClassName);
+            functionClass = clsLoader.loadClass(functionClassName);
 
-            if (!functionClass.asErasure().isAssignableTo(org.apache.pulsar.functions.api.Function.class)
-                    && !functionClass.asErasure().isAssignableTo(java.util.function.Function.class)
-                    && !functionClass.asErasure()
-                    .isAssignableTo(org.apache.pulsar.functions.api.WindowFunction.class)) {
+            if (!org.apache.pulsar.functions.api.Function.class.isAssignableFrom(functionClass)
+                    && !java.util.function.Function.class.isAssignableFrom(functionClass)
+                    && !org.apache.pulsar.functions.api.WindowFunction.class.isAssignableFrom(functionClass)) {
                 throw new IllegalArgumentException(
                         String.format("Function class %s does not implement the correct interface",
-                                functionClassName));
+                                functionClass.getName()));
             }
-        } catch (TypePool.Resolution.NoSuchTypeException e) {
+        } catch (ClassNotFoundException | NoClassDefFoundError e) {
             throw new IllegalArgumentException(
-                    String.format("Function class %s must be in class path", functionClassName), e);
+                    String.format("Function class %s must be in class path", functionConfig.getClassName()), e);
         }
 
-        TypeDefinition[] typeArgs = FunctionCommon.getFunctionTypes(functionConfig, functionClass);
+        Class<?>[] typeArgs;
+        try {
+            typeArgs = FunctionCommon.getFunctionTypes(functionConfig, functionClass);
+        } catch (ClassNotFoundException | NoClassDefFoundError e) {
+            throw new IllegalArgumentException(
+                    String.format("Function class %s must be in class path", functionConfig.getClassName()), e);
+        }
         // inputs use default schema, so there is no check needed there
 
         // Check if the Input serialization/deserialization class exists in jar or already loaded and that it
         // implements SerDe class
         if (functionConfig.getCustomSerdeInputs() != null) {
             functionConfig.getCustomSerdeInputs().forEach((topicName, inputSerializer) -> {
-                ValidatorUtils.validateSerde(inputSerializer, typeArgs[0], validatableFunctionPackage.getTypePool(),
-                        true);
+                ValidatorUtils.validateSerde(inputSerializer, typeArgs[0], clsLoader, true);
             });
         }
 
@@ -651,8 +643,8 @@ public class FunctionConfigUtils {
                     throw new IllegalArgumentException(
                             String.format("Topic %s has an incorrect schema Info", topicName));
                 }
-                ValidatorUtils.validateSchema(consumerConfig.getSchemaType(), typeArgs[0],
-                        validatableFunctionPackage.getTypePool(), true);
+                ValidatorUtils.validateSchema(consumerConfig.getSchemaType(), typeArgs[0], clsLoader, true);
+
             });
         }
 
@@ -667,16 +659,13 @@ public class FunctionConfigUtils {
                         "Only one of schemaType or serdeClassName should be set in inputSpec");
                 }
                 if (!isEmpty(conf.getSerdeClassName())) {
-                    ValidatorUtils.validateSerde(conf.getSerdeClassName(), typeArgs[0],
-                            validatableFunctionPackage.getTypePool(), true);
+                    ValidatorUtils.validateSerde(conf.getSerdeClassName(), typeArgs[0], clsLoader, true);
                 }
                 if (!isEmpty(conf.getSchemaType())) {
-                    ValidatorUtils.validateSchema(conf.getSchemaType(), typeArgs[0],
-                            validatableFunctionPackage.getTypePool(), true);
+                    ValidatorUtils.validateSchema(conf.getSchemaType(), typeArgs[0], clsLoader, true);
                 }
                 if (conf.getCryptoConfig() != null) {
-                    ValidatorUtils.validateCryptoKeyReader(conf.getCryptoConfig(),
-                            validatableFunctionPackage.getTypePool(), false);
+                    ValidatorUtils.validateCryptoKeyReader(conf.getCryptoConfig(), clsLoader, false);
                 }
             });
         }
@@ -684,8 +673,8 @@ public class FunctionConfigUtils {
         if (Void.class.equals(typeArgs[1])) {
             return new FunctionConfigUtils.ExtractedFunctionDetails(
                     functionClassName,
-                    typeArgs[0].asErasure().getTypeName(),
-                    typeArgs[1].asErasure().getTypeName());
+                    typeArgs[0].getName(),
+                    typeArgs[1].getName());
         }
 
         // One and only one of outputSchemaType and outputSerdeClassName should be set
@@ -695,25 +684,22 @@ public class FunctionConfigUtils {
         }
 
         if (!isEmpty(functionConfig.getOutputSchemaType())) {
-            ValidatorUtils.validateSchema(functionConfig.getOutputSchemaType(), typeArgs[1],
-                    validatableFunctionPackage.getTypePool(), false);
+            ValidatorUtils.validateSchema(functionConfig.getOutputSchemaType(), typeArgs[1], clsLoader, false);
         }
 
         if (!isEmpty(functionConfig.getOutputSerdeClassName())) {
-            ValidatorUtils.validateSerde(functionConfig.getOutputSerdeClassName(), typeArgs[1],
-                    validatableFunctionPackage.getTypePool(), false);
+            ValidatorUtils.validateSerde(functionConfig.getOutputSerdeClassName(), typeArgs[1], clsLoader, false);
         }
 
         if (functionConfig.getProducerConfig() != null
                 && functionConfig.getProducerConfig().getCryptoConfig() != null) {
             ValidatorUtils
-                    .validateCryptoKeyReader(functionConfig.getProducerConfig().getCryptoConfig(),
-                            validatableFunctionPackage.getTypePool(), true);
+                    .validateCryptoKeyReader(functionConfig.getProducerConfig().getCryptoConfig(), clsLoader, true);
         }
         return new FunctionConfigUtils.ExtractedFunctionDetails(
                 functionClassName,
-                typeArgs[0].asErasure().getTypeName(),
-                typeArgs[1].asErasure().getTypeName());
+                typeArgs[0].getName(),
+                typeArgs[1].getName());
     }
 
     private static void doPythonChecks(FunctionConfig functionConfig) {
@@ -728,10 +714,6 @@ public class FunctionConfigUtils {
         if (functionConfig.getMaxMessageRetries() != null && functionConfig.getMaxMessageRetries() >= 0) {
             throw new IllegalArgumentException("Message retries not yet supported in python");
         }
-
-        if (functionConfig.getRetainKeyOrdering() != null && functionConfig.getRetainKeyOrdering()) {
-            throw new IllegalArgumentException("Retain Key Orderering not yet supported in python");
-        }
     }
 
     private static void doGolangChecks(FunctionConfig functionConfig) {
@@ -864,24 +846,14 @@ public class FunctionConfigUtils {
         if (!isEmpty(functionConfig.getPy()) && !org.apache.pulsar.common.functions.Utils
                 .isFunctionPackageUrlSupported(functionConfig.getPy())
                 && functionConfig.getPy().startsWith(BUILTIN)) {
-            String filename = functionConfig.getPy();
-            if (filename.contains("..")) {
-                throw new IllegalArgumentException("Invalid filename: " + filename);
-            }
-
-            if (!new File(filename).exists()) {
+            if (!new File(functionConfig.getPy()).exists()) {
                 throw new IllegalArgumentException("The supplied python file does not exist");
             }
         }
         if (!isEmpty(functionConfig.getGo()) && !org.apache.pulsar.common.functions.Utils
                 .isFunctionPackageUrlSupported(functionConfig.getGo())
                 && functionConfig.getGo().startsWith(BUILTIN)) {
-            String filename = functionConfig.getGo();
-            if (filename.contains("..")) {
-                throw new IllegalArgumentException("Invalid filename: " + filename);
-            }
-
-            if (!new File(filename).exists()) {
+            if (!new File(functionConfig.getGo()).exists()) {
                 throw new IllegalArgumentException("The supplied go file does not exist");
             }
         }
@@ -934,21 +906,47 @@ public class FunctionConfigUtils {
         return retval;
     }
 
-    public static void validateNonJavaFunction(FunctionConfig functionConfig) {
+    public static ClassLoader validate(FunctionConfig functionConfig, File functionPackageFile) {
         doCommonChecks(functionConfig);
-        if (functionConfig.getRuntime() == FunctionConfig.Runtime.GO) {
+        if (functionConfig.getRuntime() == FunctionConfig.Runtime.JAVA) {
+            ClassLoader classLoader;
+            if (functionPackageFile != null) {
+                try {
+                    classLoader = loadJar(functionPackageFile);
+                } catch (MalformedURLException e) {
+                    throw new IllegalArgumentException("Corrupted Jar File", e);
+                }
+            } else if (!isEmpty(functionConfig.getJar())) {
+                File jarFile = new File(functionConfig.getJar());
+                if (!jarFile.exists()) {
+                    throw new IllegalArgumentException("Jar file does not exist");
+                }
+                try {
+                    classLoader = loadJar(jarFile);
+                } catch (Exception e) {
+                    throw new IllegalArgumentException("Corrupted Jar File", e);
+                }
+            } else {
+                throw new IllegalArgumentException("Function Package is not provided");
+            }
+
+            doJavaChecks(functionConfig, classLoader);
+            return classLoader;
+        } else if (functionConfig.getRuntime() == FunctionConfig.Runtime.GO) {
             doGolangChecks(functionConfig);
+            return null;
         } else if (functionConfig.getRuntime() == FunctionConfig.Runtime.PYTHON) {
             doPythonChecks(functionConfig);
+            return null;
         } else {
             throw new IllegalArgumentException("Function language runtime is either not set or cannot be determined");
         }
     }
 
     public static ExtractedFunctionDetails validateJavaFunction(FunctionConfig functionConfig,
-                                                                ValidatableFunctionPackage validatableFunctionPackage) {
+                                                                ClassLoader classLoader) {
         doCommonChecks(functionConfig);
-        return doJavaChecks(functionConfig, validatableFunctionPackage);
+        return doJavaChecks(functionConfig, classLoader);
     }
 
     public static FunctionConfig validateUpdate(FunctionConfig existingConfig, FunctionConfig newConfig) {
@@ -1090,9 +1088,6 @@ public class FunctionConfigUtils {
         if (!StringUtils.isEmpty(newConfig.getCustomRuntimeOptions())) {
             mergedConfig.setCustomRuntimeOptions(newConfig.getCustomRuntimeOptions());
         }
-        if (newConfig.getProducerConfig() != null) {
-            mergedConfig.setProducerConfig(newConfig.getProducerConfig());
-        }
         return mergedConfig;
     }
 }
