@@ -41,23 +41,23 @@ import lombok.Getter;
 import lombok.Setter;
 import lombok.SneakyThrows;
 import lombok.extern.slf4j.Slf4j;
-import net.bytebuddy.description.type.TypeDefinition;
-import net.bytebuddy.pool.TypePool;
-import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang.StringUtils;
 import org.apache.pulsar.client.api.SubscriptionInitialPosition;
 import org.apache.pulsar.common.functions.ConsumerConfig;
 import org.apache.pulsar.common.functions.FunctionConfig;
-import org.apache.pulsar.common.functions.FunctionDefinition;
 import org.apache.pulsar.common.functions.Resources;
 import org.apache.pulsar.common.io.ConnectorDefinition;
 import org.apache.pulsar.common.io.SinkConfig;
 import org.apache.pulsar.common.naming.TopicName;
+import org.apache.pulsar.common.nar.NarClassLoader;
 import org.apache.pulsar.common.util.ObjectMapperFactory;
 import org.apache.pulsar.config.validation.ConfigValidation;
 import org.apache.pulsar.functions.api.Record;
 import org.apache.pulsar.functions.api.utils.IdentityFunction;
 import org.apache.pulsar.functions.proto.Function;
 import org.apache.pulsar.functions.proto.Function.FunctionDetails;
+import org.apache.pulsar.functions.utils.functions.FunctionUtils;
+import org.apache.pulsar.functions.utils.io.ConnectorUtils;
 
 @Slf4j
 public class SinkConfigUtils {
@@ -202,12 +202,6 @@ public class SinkConfigUtils {
             sourceSpecBuilder.setNegativeAckRedeliveryDelayMs(sinkConfig.getNegativeAckRedeliveryDelayMs());
         }
 
-        if (sinkConfig.getCleanupSubscription() != null) {
-            sourceSpecBuilder.setCleanupSubscription(sinkConfig.getCleanupSubscription());
-        } else {
-            sourceSpecBuilder.setCleanupSubscription(true);
-        }
-
         if (sinkConfig.getSourceSubscriptionPosition() == SubscriptionInitialPosition.Earliest) {
             sourceSpecBuilder.setSubscriptionPosition(Function.SubscriptionPosition.EARLIEST);
         } else {
@@ -216,13 +210,6 @@ public class SinkConfigUtils {
 
         functionDetailsBuilder.setSource(sourceSpecBuilder);
 
-        if (sinkConfig.getRetainKeyOrdering() != null) {
-            functionDetailsBuilder.setRetainKeyOrdering(sinkConfig.getRetainKeyOrdering());
-        }
-        if (sinkConfig.getRetainOrdering() != null) {
-            functionDetailsBuilder.setRetainOrdering(sinkConfig.getRetainOrdering());
-        }
-
         if (sinkConfig.getMaxMessageRetries() != null && sinkConfig.getMaxMessageRetries() > 0) {
             Function.RetryDetails.Builder retryDetails = Function.RetryDetails.newBuilder();
             retryDetails.setMaxMessageRetries(sinkConfig.getMaxMessageRetries());
@@ -336,7 +323,6 @@ public class SinkConfigUtils {
         // Set subscription position
         sinkConfig.setSourceSubscriptionPosition(
                 convertFromFunctionDetailsSubscriptionPosition(functionDetails.getSource().getSubscriptionPosition()));
-        sinkConfig.setCleanupSubscription(functionDetails.getSource().getCleanupSubscription());
 
         if (functionDetails.getSource().getTimeoutMs() != 0) {
             sinkConfig.setTimeoutMs(functionDetails.getSource().getTimeoutMs());
@@ -409,8 +395,8 @@ public class SinkConfigUtils {
     }
 
     public static ExtractedSinkDetails validateAndExtractDetails(SinkConfig sinkConfig,
-                                                                 ValidatableFunctionPackage sinkFunction,
-                                                                 ValidatableFunctionPackage transformFunction,
+                                                                 ClassLoader sinkClassLoader,
+                                                                 ClassLoader functionClassLoader,
                                                                  boolean validateConnectorConfig) {
         if (isEmpty(sinkConfig.getTenant())) {
             throw new IllegalArgumentException("Sink tenant cannot be null");
@@ -450,72 +436,63 @@ public class SinkConfigUtils {
         // if class name in sink config is not set, this should be a built-in sink
         // thus we should try to find it class name in the NAR service definition
         if (sinkClassName == null) {
-            ConnectorDefinition connectorDefinition = sinkFunction.getFunctionMetaData(ConnectorDefinition.class);
-            if (connectorDefinition == null) {
-                throw new IllegalArgumentException(
-                        "Sink package doesn't contain the META-INF/services/pulsar-io.yaml file.");
-            }
-            sinkClassName = connectorDefinition.getSinkClass();
-            if (sinkClassName == null) {
-                throw new IllegalArgumentException("Failed to extract sink class from archive");
+            try {
+                sinkClassName = ConnectorUtils.getIOSinkClass((NarClassLoader) sinkClassLoader);
+            } catch (IOException e) {
+                throw new IllegalArgumentException("Failed to extract sink class from archive", e);
             }
         }
 
         // check if sink implements the correct interfaces
-        TypeDefinition sinkClass;
+        Class sinkClass;
         try {
-            sinkClass = sinkFunction.resolveType(sinkClassName);
-        } catch (TypePool.Resolution.NoSuchTypeException e) {
+            sinkClass = sinkClassLoader.loadClass(sinkClassName);
+        } catch (ClassNotFoundException e) {
             throw new IllegalArgumentException(
-                    String.format("Sink class %s not found", sinkClassName), e);
+                    String.format("Sink class %s not found in class loader", sinkClassName), e);
         }
 
         String functionClassName = sinkConfig.getTransformFunctionClassName();
-        TypeDefinition typeArg;
-        ValidatableFunctionPackage inputFunction;
-        if (transformFunction != null) {
+        Class<?> typeArg;
+        ClassLoader inputClassLoader;
+        if (functionClassLoader != null) {
             // if function class name in sink config is not set, this should be a built-in function
             // thus we should try to find it class name in the NAR service definition
             if (functionClassName == null) {
-                FunctionDefinition functionDefinition =
-                        transformFunction.getFunctionMetaData(FunctionDefinition.class);
-                if (functionDefinition == null) {
-                    throw new IllegalArgumentException(
-                            "Function package doesn't contain the META-INF/services/pulsar-io.yaml file.");
-                }
-                functionClassName = functionDefinition.getFunctionClass();
-                if (functionClassName == null) {
-                    throw new IllegalArgumentException("Transform function class name must be set");
+                try {
+                    functionClassName = FunctionUtils.getFunctionClass(functionClassLoader);
+                } catch (IOException e) {
+                    throw new IllegalArgumentException("Failed to extract function class from archive", e);
                 }
             }
-            TypeDefinition functionClass;
+            Class functionClass;
             try {
-                functionClass = transformFunction.resolveType(functionClassName);
-            } catch (TypePool.Resolution.NoSuchTypeException e) {
+                functionClass = functionClassLoader.loadClass(functionClassName);
+            } catch (ClassNotFoundException e) {
                 throw new IllegalArgumentException(
-                        String.format("Function class %s not found", functionClassName), e);
+                        String.format("Function class %s not found in class loader", functionClassName), e);
             }
             // extract type from transform function class
-            if (!getRawFunctionTypes(functionClass, false)[1].asErasure().isAssignableTo(Record.class)) {
+            if (!getRawFunctionTypes(functionClass, false)[1].equals(Record.class)) {
                 throw new IllegalArgumentException("Sink transform function output must be of type Record");
             }
             typeArg = getFunctionTypes(functionClass, false)[0];
-            inputFunction = transformFunction;
+            inputClassLoader = functionClassLoader;
         } else {
             // extract type from sink class
             typeArg = getSinkType(sinkClass);
-            inputFunction = sinkFunction;
+            inputClassLoader = sinkClassLoader;
         }
 
         if (sinkConfig.getTopicToSerdeClassName() != null) {
            for (String serdeClassName : sinkConfig.getTopicToSerdeClassName().values()) {
-               ValidatorUtils.validateSerde(serdeClassName, typeArg, inputFunction.getTypePool(), true);
+               ValidatorUtils.validateSerde(serdeClassName, typeArg, inputClassLoader, true);
            }
         }
 
         if (sinkConfig.getTopicToSchemaType() != null) {
             for (String schemaType : sinkConfig.getTopicToSchemaType().values()) {
-                ValidatorUtils.validateSchema(schemaType, typeArg, inputFunction.getTypePool(), true);
+                ValidatorUtils.validateSchema(schemaType, typeArg, inputClassLoader, true);
             }
         }
 
@@ -528,43 +505,23 @@ public class SinkConfigUtils {
                     throw new IllegalArgumentException("Only one of serdeClassName or schemaType should be set");
                 }
                 if (!isEmpty(consumerSpec.getSerdeClassName())) {
-                    ValidatorUtils.validateSerde(consumerSpec.getSerdeClassName(), typeArg,
-                            inputFunction.getTypePool(), true);
+                    ValidatorUtils.validateSerde(consumerSpec.getSerdeClassName(), typeArg, inputClassLoader, true);
                 }
                 if (!isEmpty(consumerSpec.getSchemaType())) {
-                    ValidatorUtils.validateSchema(consumerSpec.getSchemaType(), typeArg,
-                            inputFunction.getTypePool(), true);
+                    ValidatorUtils.validateSchema(consumerSpec.getSchemaType(), typeArg, inputClassLoader, true);
                 }
                 if (consumerSpec.getCryptoConfig() != null) {
-                    ValidatorUtils.validateCryptoKeyReader(consumerSpec.getCryptoConfig(),
-                            inputFunction.getTypePool(), false);
+                    ValidatorUtils.validateCryptoKeyReader(consumerSpec.getCryptoConfig(), inputClassLoader, false);
                 }
             }
         }
 
-        if (sinkConfig.getRetainKeyOrdering() != null
-                && sinkConfig.getRetainKeyOrdering()
-                && sinkConfig.getProcessingGuarantees() != null
-                && sinkConfig.getProcessingGuarantees() == FunctionConfig.ProcessingGuarantees.EFFECTIVELY_ONCE) {
-            throw new IllegalArgumentException(
-                    "When effectively once processing guarantee is specified, retain Key ordering cannot be set");
+        // validate user defined config if enabled and sink is loaded from NAR
+        if (validateConnectorConfig && sinkClassLoader instanceof NarClassLoader) {
+            validateSinkConfig(sinkConfig, (NarClassLoader) sinkClassLoader);
         }
 
-        if (sinkConfig.getRetainKeyOrdering() != null && sinkConfig.getRetainKeyOrdering()
-                && sinkConfig.getRetainOrdering() != null && sinkConfig.getRetainOrdering()) {
-            throw new IllegalArgumentException("Only one of retain ordering or retain key ordering can be set");
-        }
-
-        // validate user defined config if enabled and classloading is enabled
-        if (validateConnectorConfig) {
-            if (sinkFunction.isEnableClassloading()) {
-                validateSinkConfig(sinkConfig, sinkFunction);
-            } else {
-                log.warn("Skipping annotation based validation of sink config as classloading is disabled");
-            }
-        }
-
-        return new ExtractedSinkDetails(sinkClassName, typeArg.asErasure().getTypeName(), functionClassName);
+        return new ExtractedSinkDetails(sinkClassName, typeArg.getName(), functionClassName);
     }
 
     public static Collection<String> collectAllInputTopics(SinkConfig sinkConfig) {
@@ -707,9 +664,6 @@ public class SinkConfigUtils {
         if (!StringUtils.isEmpty(newConfig.getCustomRuntimeOptions())) {
             mergedConfig.setCustomRuntimeOptions(newConfig.getCustomRuntimeOptions());
         }
-        if (newConfig.getCleanupSubscription() != null) {
-            mergedConfig.setCleanupSubscription(newConfig.getCleanupSubscription());
-        }
         if (newConfig.getTransformFunction() != null) {
             mergedConfig.setTransformFunction(newConfig.getTransformFunction());
         }
@@ -719,19 +673,33 @@ public class SinkConfigUtils {
         if (newConfig.getTransformFunctionConfig() != null) {
             mergedConfig.setTransformFunctionConfig(newConfig.getTransformFunctionConfig());
         }
-        if (newConfig.getSourceSubscriptionPosition() != null) {
-            mergedConfig.setSourceSubscriptionPosition(newConfig.getSourceSubscriptionPosition());
-        }
+
         return mergedConfig;
     }
 
-    public static void validateSinkConfig(SinkConfig sinkConfig, ValidatableFunctionPackage sinkFunction) {
+    public static void validateSinkConfig(SinkConfig sinkConfig, NarClassLoader narClassLoader) {
+
+        if (sinkConfig.getRetainKeyOrdering() != null
+                && sinkConfig.getRetainKeyOrdering()
+                && sinkConfig.getProcessingGuarantees() != null
+                && sinkConfig.getProcessingGuarantees() == FunctionConfig.ProcessingGuarantees.EFFECTIVELY_ONCE) {
+            throw new IllegalArgumentException(
+                    "When effectively once processing guarantee is specified, retain Key ordering cannot be set");
+        }
+
+        if (sinkConfig.getRetainKeyOrdering() != null && sinkConfig.getRetainKeyOrdering()
+                && sinkConfig.getRetainOrdering() != null && sinkConfig.getRetainOrdering()) {
+            throw new IllegalArgumentException("Only one of retain ordering or retain key ordering can be set");
+        }
+
         try {
-            ConnectorDefinition defn = sinkFunction.getFunctionMetaData(ConnectorDefinition.class);
-            if (defn != null && defn.getSinkConfigClass() != null) {
-                Class configClass = Class.forName(defn.getSinkConfigClass(), true, sinkFunction.getClassLoader());
+            ConnectorDefinition defn = ConnectorUtils.getConnectorDefinition(narClassLoader);
+            if (defn.getSinkConfigClass() != null) {
+                Class configClass = Class.forName(defn.getSinkConfigClass(), true, narClassLoader);
                 validateSinkConfig(sinkConfig, configClass);
             }
+        } catch (IOException e) {
+            throw new IllegalArgumentException("Error validating sink config", e);
         } catch (ClassNotFoundException e) {
             throw new IllegalArgumentException("Could not find sink config class", e);
         }
