@@ -19,10 +19,10 @@
 package org.apache.pulsar.functions.utils;
 
 import static org.apache.commons.lang3.StringUtils.isEmpty;
+import static org.apache.pulsar.functions.utils.FunctionCommon.convertFromCompressionType;
+import static org.apache.pulsar.functions.utils.FunctionCommon.convertFromFunctionDetailsCompressionType;
 import static org.apache.pulsar.functions.utils.FunctionCommon.convertProcessingGuarantee;
 import static org.apache.pulsar.functions.utils.FunctionCommon.getSourceType;
-import static org.apache.pulsar.functions.utils.FunctionConfigUtils.convertProducerConfigToProducerSpec;
-import static org.apache.pulsar.functions.utils.FunctionConfigUtils.convertProducerSpecToProducerConfig;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.google.gson.Gson;
 import com.google.gson.reflect.TypeToken;
@@ -35,21 +35,22 @@ import lombok.Getter;
 import lombok.Setter;
 import lombok.SneakyThrows;
 import lombok.extern.slf4j.Slf4j;
-import net.bytebuddy.description.type.TypeDefinition;
-import net.bytebuddy.description.type.TypeDescription;
-import net.bytebuddy.pool.TypePool;
+import net.jodah.typetools.TypeResolver;
 import org.apache.commons.lang3.StringUtils;
+import org.apache.pulsar.common.functions.ProducerConfig;
 import org.apache.pulsar.common.functions.Resources;
 import org.apache.pulsar.common.io.BatchSourceConfig;
 import org.apache.pulsar.common.io.ConnectorDefinition;
 import org.apache.pulsar.common.io.SourceConfig;
 import org.apache.pulsar.common.naming.TopicDomain;
 import org.apache.pulsar.common.naming.TopicName;
+import org.apache.pulsar.common.nar.NarClassLoader;
 import org.apache.pulsar.common.util.ObjectMapperFactory;
 import org.apache.pulsar.config.validation.ConfigValidation;
 import org.apache.pulsar.functions.api.utils.IdentityFunction;
 import org.apache.pulsar.functions.proto.Function;
 import org.apache.pulsar.functions.proto.Function.FunctionDetails;
+import org.apache.pulsar.functions.utils.io.ConnectorUtils;
 import org.apache.pulsar.io.core.BatchSource;
 import org.apache.pulsar.io.core.Source;
 
@@ -148,7 +149,29 @@ public class SourceConfigUtils {
         }
 
         if (sourceConfig.getProducerConfig() != null) {
-            sinkSpecBuilder.setProducerSpec(convertProducerConfigToProducerSpec(sourceConfig.getProducerConfig()));
+            ProducerConfig conf = sourceConfig.getProducerConfig();
+            Function.ProducerSpec.Builder pbldr = Function.ProducerSpec.newBuilder();
+            if (conf.getMaxPendingMessages() != null) {
+                pbldr.setMaxPendingMessages(conf.getMaxPendingMessages());
+            }
+            if (conf.getMaxPendingMessagesAcrossPartitions() != null) {
+                pbldr.setMaxPendingMessagesAcrossPartitions(conf.getMaxPendingMessagesAcrossPartitions());
+            }
+            if (conf.getUseThreadLocalProducers() != null) {
+                pbldr.setUseThreadLocalProducers(conf.getUseThreadLocalProducers());
+            }
+            if (conf.getCryptoConfig() != null) {
+                pbldr.setCryptoSpec(CryptoUtils.convert(conf.getCryptoConfig()));
+            }
+            if (conf.getBatchBuilder() != null) {
+                pbldr.setBatchBuilder(conf.getBatchBuilder());
+            }
+            if (conf.getCompressionType() != null) {
+                pbldr.setCompressionType(convertFromCompressionType(conf.getCompressionType()));
+            } else {
+                pbldr.setCompressionType(Function.CompressionType.LZ4);
+            }
+            sinkSpecBuilder.setProducerSpec(pbldr.build());
         }
 
         if (sourceConfig.getBatchBuilder() != null) {
@@ -233,7 +256,23 @@ public class SourceConfigUtils {
             sourceConfig.setSerdeClassName(sinkSpec.getSerDeClassName());
         }
         if (sinkSpec.getProducerSpec() != null) {
-            sourceConfig.setProducerConfig(convertProducerSpecToProducerConfig(sinkSpec.getProducerSpec()));
+            Function.ProducerSpec spec = sinkSpec.getProducerSpec();
+            ProducerConfig producerConfig = new ProducerConfig();
+            if (spec.getMaxPendingMessages() != 0) {
+                producerConfig.setMaxPendingMessages(spec.getMaxPendingMessages());
+            }
+            if (spec.getMaxPendingMessagesAcrossPartitions() != 0) {
+                producerConfig.setMaxPendingMessagesAcrossPartitions(spec.getMaxPendingMessagesAcrossPartitions());
+            }
+            if (spec.hasCryptoSpec()) {
+                producerConfig.setCryptoConfig(CryptoUtils.convertFromSpec(spec.getCryptoSpec()));
+            }
+            if (spec.getBatchBuilder() != null) {
+                producerConfig.setBatchBuilder(spec.getBatchBuilder());
+            }
+            producerConfig.setUseThreadLocalProducers(spec.getUseThreadLocalProducers());
+            producerConfig.setCompressionType(convertFromFunctionDetailsCompressionType(spec.getCompressionType()));
+            sourceConfig.setProducerConfig(producerConfig);
         }
         if (functionDetails.hasResources()) {
             Resources resources = new Resources();
@@ -255,7 +294,7 @@ public class SourceConfigUtils {
     }
 
     public static ExtractedSourceDetails validateAndExtractDetails(SourceConfig sourceConfig,
-                                                                   ValidatableFunctionPackage sourceFunction,
+                                                                   ClassLoader sourceClassLoader,
                                                                    boolean validateConnectorConfig) {
         if (isEmpty(sourceConfig.getTenant())) {
             throw new IllegalArgumentException("Source tenant cannot be null");
@@ -280,34 +319,29 @@ public class SourceConfigUtils {
         // if class name in source config is not set, this should be a built-in source
         // thus we should try to find it class name in the NAR service definition
         if (sourceClassName == null) {
-            ConnectorDefinition connectorDefinition = sourceFunction.getFunctionMetaData(ConnectorDefinition.class);
-            if (connectorDefinition == null) {
-                throw new IllegalArgumentException(
-                        "Source package doesn't contain the META-INF/services/pulsar-io.yaml file.");
-            }
-            sourceClassName = connectorDefinition.getSourceClass();
-            if (sourceClassName == null) {
-                throw new IllegalArgumentException("Failed to extract source class from archive");
+            try {
+                sourceClassName = ConnectorUtils.getIOSourceClass((NarClassLoader) sourceClassLoader);
+            } catch (IOException e) {
+                throw new IllegalArgumentException("Failed to extract source class from archive", e);
             }
         }
 
         // check if source implements the correct interfaces
-        TypeDescription sourceClass;
+        Class sourceClass;
         try {
-            sourceClass = sourceFunction.resolveType(sourceClassName);
-        } catch (TypePool.Resolution.NoSuchTypeException e) {
+            sourceClass = sourceClassLoader.loadClass(sourceClassName);
+        } catch (ClassNotFoundException e) {
             throw new IllegalArgumentException(
               String.format("Source class %s not found in class loader", sourceClassName), e);
         }
 
-        if (!(sourceClass.asErasure().isAssignableTo(Source.class) || sourceClass.asErasure()
-                .isAssignableTo(BatchSource.class))) {
+        if (!Source.class.isAssignableFrom(sourceClass) && !BatchSource.class.isAssignableFrom(sourceClass)) {
             throw new IllegalArgumentException(
-                    String.format("Source class %s does not implement the correct interface",
-                            sourceClass.getName()));
+              String.format("Source class %s does not implement the correct interface",
+                sourceClass.getName()));
         }
 
-        if (sourceClass.asErasure().isAssignableTo(BatchSource.class)) {
+        if (BatchSource.class.isAssignableFrom(sourceClass)) {
             if (sourceConfig.getBatchSourceConfig() != null) {
                 validateBatchSourceConfig(sourceConfig.getBatchSourceConfig());
             } else {
@@ -318,14 +352,7 @@ public class SourceConfigUtils {
         }
 
         // extract type from source class
-        TypeDefinition typeArg;
-
-        try {
-            typeArg = getSourceType(sourceClass);
-        } catch (Exception e) {
-            throw new IllegalArgumentException(
-                    String.format("Failed to resolve type for Source class %s", sourceClassName), e);
-        }
+        Class<?> typeArg = getSourceType(sourceClass);
 
         // Only one of serdeClassName or schemaType should be set
         if (!StringUtils.isEmpty(sourceConfig.getSerdeClassName()) && !StringUtils
@@ -334,30 +361,29 @@ public class SourceConfigUtils {
         }
 
         if (!StringUtils.isEmpty(sourceConfig.getSerdeClassName())) {
-            ValidatorUtils.validateSerde(sourceConfig.getSerdeClassName(), typeArg, sourceFunction.getTypePool(),
-                    false);
+            ValidatorUtils.validateSerde(sourceConfig.getSerdeClassName(), typeArg, sourceClassLoader, false);
         }
         if (!StringUtils.isEmpty(sourceConfig.getSchemaType())) {
-            ValidatorUtils.validateSchema(sourceConfig.getSchemaType(), typeArg, sourceFunction.getTypePool(),
-                    false);
+            ValidatorUtils.validateSchema(sourceConfig.getSchemaType(), typeArg, sourceClassLoader, false);
         }
 
         if (sourceConfig.getProducerConfig() != null && sourceConfig.getProducerConfig().getCryptoConfig() != null) {
             ValidatorUtils
-                    .validateCryptoKeyReader(sourceConfig.getProducerConfig().getCryptoConfig(),
-                            sourceFunction.getTypePool(), true);
+                    .validateCryptoKeyReader(sourceConfig.getProducerConfig().getCryptoConfig(), sourceClassLoader,
+                            true);
         }
 
-        // validate user defined config if enabled and classloading is enabled
-        if (validateConnectorConfig) {
-            if (sourceFunction.isEnableClassloading()) {
-                validateSourceConfig(sourceConfig, sourceFunction);
-            } else {
-                log.warn("Skipping annotation based validation of sink config as classloading is disabled");
-            }
+        if (typeArg.equals(TypeResolver.Unknown.class)) {
+            throw new IllegalArgumentException(
+              String.format("Failed to resolve type for Source class %s", sourceClassName));
         }
 
-        return new ExtractedSourceDetails(sourceClassName, typeArg.asErasure().getTypeName());
+        // validate user defined config if enabled and source is loaded from NAR
+        if (validateConnectorConfig && sourceClassLoader instanceof NarClassLoader) {
+            validateSourceConfig(sourceConfig, (NarClassLoader) sourceClassLoader);
+        }
+
+        return new ExtractedSourceDetails(sourceClassName, typeArg.getName());
     }
 
     @SneakyThrows
@@ -422,9 +448,6 @@ public class SourceConfigUtils {
             validateBatchSourceConfigUpdate(existingConfig.getBatchSourceConfig(), newConfig.getBatchSourceConfig());
             mergedConfig.setBatchSourceConfig(newConfig.getBatchSourceConfig());
         }
-        if (newConfig.getProducerConfig() != null) {
-            mergedConfig.setProducerConfig(newConfig.getProducerConfig());
-        }
         return mergedConfig;
     }
 
@@ -498,14 +521,15 @@ public class SourceConfigUtils {
         }
     }
 
-    public static void validateSourceConfig(SourceConfig sourceConfig, ValidatableFunctionPackage sourceFunction) {
+    public static void validateSourceConfig(SourceConfig sourceConfig, NarClassLoader narClassLoader) {
         try {
-            ConnectorDefinition defn = sourceFunction.getFunctionMetaData(ConnectorDefinition.class);
-            if (defn != null && defn.getSourceConfigClass() != null) {
-                Class configClass =
-                        Class.forName(defn.getSourceConfigClass(), true, sourceFunction.getClassLoader());
+            ConnectorDefinition defn = ConnectorUtils.getConnectorDefinition(narClassLoader);
+            if (defn.getSourceConfigClass() != null) {
+                Class configClass = Class.forName(defn.getSourceConfigClass(), true, narClassLoader);
                 validateSourceConfig(sourceConfig, configClass);
             }
+        } catch (IOException e) {
+            throw new IllegalArgumentException("Error validating source config", e);
         } catch (ClassNotFoundException e) {
             throw new IllegalArgumentException("Could not find source config class");
         }
