@@ -285,30 +285,6 @@ public class SourceConfigUtilsTest {
         );
     }
 
-    @Test
-    public void testMergeDifferentProducerConfig() {
-        SourceConfig sourceConfig = createSourceConfig();
-
-        ProducerConfig producerConfig = new ProducerConfig();
-        producerConfig.setMaxPendingMessages(100);
-        producerConfig.setMaxPendingMessagesAcrossPartitions(1000);
-        producerConfig.setUseThreadLocalProducers(true);
-        producerConfig.setBatchBuilder("DEFAULT");
-        producerConfig.setCompressionType(CompressionType.ZLIB);
-        SourceConfig newSourceConfig = createUpdatedSourceConfig("producerConfig", producerConfig);
-
-        SourceConfig mergedConfig = SourceConfigUtils.validateUpdate(sourceConfig, newSourceConfig);
-        assertEquals(
-                mergedConfig.getProducerConfig(),
-                producerConfig
-        );
-        mergedConfig.setProducerConfig(sourceConfig.getProducerConfig());
-        assertEquals(
-                new Gson().toJson(sourceConfig),
-                new Gson().toJson(mergedConfig)
-        );
-    }
-
     @Test
     public void testValidateConfig() {
         SourceConfig sourceConfig = createSourceConfig();
