@@ -70,7 +70,6 @@ import org.apache.pulsar.functions.runtime.RuntimeSpawner;
 import org.apache.pulsar.functions.utils.Actions;
 import org.apache.pulsar.functions.utils.FunctionCommon;
 import org.apache.pulsar.functions.utils.SourceConfigUtils;
-import org.apache.pulsar.functions.utils.ValidatableFunctionPackage;
 import org.apache.pulsar.functions.utils.io.Connector;
 
 @Data
@@ -83,21 +82,18 @@ public class FunctionActioner {
     private final ConnectorsManager connectorsManager;
     private final FunctionsManager functionsManager;
     private final PulsarAdmin pulsarAdmin;
-    private final PackageUrlValidator packageUrlValidator;
 
     public FunctionActioner(WorkerConfig workerConfig,
                             RuntimeFactory runtimeFactory,
                             Namespace dlogNamespace,
                             ConnectorsManager connectorsManager,
-                            FunctionsManager functionsManager, PulsarAdmin pulsarAdmin,
-                            PackageUrlValidator packageUrlValidator) {
+                            FunctionsManager functionsManager, PulsarAdmin pulsarAdmin) {
         this.workerConfig = workerConfig;
         this.runtimeFactory = runtimeFactory;
         this.dlogNamespace = dlogNamespace;
         this.connectorsManager = connectorsManager;
         this.functionsManager = functionsManager;
         this.pulsarAdmin = pulsarAdmin;
-        this.packageUrlValidator = packageUrlValidator;
     }
 
 
@@ -155,9 +151,6 @@ public class FunctionActioner {
         boolean isPkgUrlProvided = isFunctionPackageUrlSupported(packagePath);
         String packageFile;
         if (isPkgUrlProvided && packagePath.startsWith(FILE)) {
-            if (!packageUrlValidator.isValidPackageUrl(componentType, packagePath)) {
-                throw new IllegalArgumentException("Package URL " + packagePath + " is not valid");
-            }
             URL url = new URL(packagePath);
             File pkgFile = new File(url.toURI());
             packageFile = pkgFile.getAbsolutePath();
@@ -174,7 +167,7 @@ public class FunctionActioner {
                     pkgDir,
                     new File(getDownloadFileName(functionMetaData.getFunctionDetails(),
                             pkgLocation)).getName());
-            downloadFile(pkgFile, isPkgUrlProvided, functionMetaData, instanceId, pkgLocation, componentType);
+            downloadFile(pkgFile, isPkgUrlProvided, functionMetaData, instanceId, pkgLocation);
             packageFile = pkgFile.getAbsolutePath();
         }
         return packageFile;
@@ -228,12 +221,12 @@ public class FunctionActioner {
         if (workerConfig.getAdditionalJavaRuntimeArguments() != null) {
             instanceConfig.setAdditionalJavaRuntimeArguments(workerConfig.getAdditionalJavaRuntimeArguments());
         }
+        instanceConfig.setIgnoreUnknownConfigFields(workerConfig.isIgnoreUnknownConfigFields());
         return instanceConfig;
     }
 
     private void downloadFile(File pkgFile, boolean isPkgUrlProvided, FunctionMetaData functionMetaData,
-                              int instanceId, Function.PackageLocationMetaData pkgLocation,
-                              FunctionDetails.ComponentType componentType)
+                              int instanceId, Function.PackageLocationMetaData pkgLocation)
             throws IOException, PulsarAdminException {
 
         FunctionDetails details = functionMetaData.getFunctionDetails();
@@ -258,18 +251,17 @@ public class FunctionActioner {
                 downloadFromHttp ? pkgLocationPath : pkgLocation);
 
         if (downloadFromHttp) {
-            if (!packageUrlValidator.isValidPackageUrl(componentType, pkgLocationPath)) {
-                throw new IllegalArgumentException("Package URL " + pkgLocationPath + " is not valid");
-            }
             FunctionCommon.downloadFromHttpUrl(pkgLocationPath, tempPkgFile);
         } else if (downloadFromPackageManagementService) {
             getPulsarAdmin().packages().download(pkgLocationPath, tempPkgFile.getPath());
         } else {
-            try (FileOutputStream tempPkgFos = new FileOutputStream(tempPkgFile)) {
-                WorkerUtils.downloadFromBookkeeper(
-                        dlogNamespace,
-                        tempPkgFos,
-                        pkgLocationPath);
+            FileOutputStream tempPkgFos = new FileOutputStream(tempPkgFile);
+            WorkerUtils.downloadFromBookkeeper(
+                    dlogNamespace,
+                    tempPkgFos,
+                    pkgLocationPath);
+            if (tempPkgFos != null) {
+                tempPkgFos.close();
             }
         }
 
@@ -535,7 +527,7 @@ public class FunctionActioner {
                 builder.setClassName(sourceClass);
                 functionDetails.setSource(builder);
 
-                fillSourceTypeClass(functionDetails, connector.getConnectorFunctionPackage(), sourceClass);
+                fillSourceTypeClass(functionDetails, connector.getClassLoader(), sourceClass);
                 return archive;
             }
         }
@@ -551,7 +543,7 @@ public class FunctionActioner {
                 builder.setClassName(sinkClass);
                 functionDetails.setSink(builder);
 
-                fillSinkTypeClass(functionDetails, connector.getConnectorFunctionPackage(), sinkClass);
+                fillSinkTypeClass(functionDetails, connector.getClassLoader(), sinkClass);
                 return archive;
             }
         }
@@ -565,8 +557,8 @@ public class FunctionActioner {
     }
 
     private void fillSourceTypeClass(FunctionDetails.Builder functionDetails,
-                                     ValidatableFunctionPackage functionPackage, String className) {
-        String typeArg = getSourceType(className, functionPackage.getTypePool()).asErasure().getName();
+                                     ClassLoader narClassLoader, String className) throws ClassNotFoundException {
+        String typeArg = getSourceType(className, narClassLoader).getName();
 
         SourceSpec.Builder sourceBuilder = SourceSpec.newBuilder(functionDetails.getSource());
         sourceBuilder.setTypeClassName(typeArg);
@@ -581,8 +573,8 @@ public class FunctionActioner {
     }
 
     private void fillSinkTypeClass(FunctionDetails.Builder functionDetails,
-                                   ValidatableFunctionPackage functionPackage, String className) {
-        String typeArg = getSinkType(className, functionPackage.getTypePool()).asErasure().getName();
+                                   ClassLoader narClassLoader, String className) throws ClassNotFoundException {
+        String typeArg = getSinkType(className, narClassLoader).getName();
 
         SinkSpec.Builder sinkBuilder = SinkSpec.newBuilder(functionDetails.getSink());
         sinkBuilder.setTypeClassName(typeArg);
@@ -598,7 +590,7 @@ public class FunctionActioner {
 
     private static String getDownloadFileName(FunctionDetails functionDetails,
                                              Function.PackageLocationMetaData packageLocation) {
-        if (!org.apache.commons.lang3.StringUtils.isEmpty(packageLocation.getOriginalFileName())) {
+        if (!org.apache.commons.lang.StringUtils.isEmpty(packageLocation.getOriginalFileName())) {
             return packageLocation.getOriginalFileName();
         }
         String[] hierarchy = functionDetails.getClassName().split("\\.");
