@@ -20,6 +20,7 @@ package org.apache.pulsar.io.kafka;
 
 import java.io.IOException;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Properties;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.commons.lang3.StringUtils;
@@ -43,7 +44,7 @@ import org.apache.pulsar.io.core.SinkContext;
 public abstract class KafkaAbstractSink<K, V> implements Sink<byte[]> {
 
     private Producer<K, V> producer;
-    private Properties props = new Properties();
+    private final Properties props = new Properties();
     private KafkaSinkConfig kafkaSinkConfig;
 
     @Override
@@ -78,14 +79,17 @@ public abstract class KafkaAbstractSink<K, V> implements Sink<byte[]> {
 
     @Override
     public void open(Map<String, Object> config, SinkContext sinkContext) throws Exception {
-        kafkaSinkConfig = KafkaSinkConfig.load(config, sinkContext);
+        kafkaSinkConfig = KafkaSinkConfig.load(config);
+        Objects.requireNonNull(kafkaSinkConfig.getTopic(), "Kafka topic is not set");
+        Objects.requireNonNull(kafkaSinkConfig.getBootstrapServers(), "Kafka bootstrapServers is not set");
+        Objects.requireNonNull(kafkaSinkConfig.getAcks(), "Kafka acks mode is not set");
         if (kafkaSinkConfig.getBatchSize() <= 0) {
             throw new IllegalArgumentException("Invalid Kafka Producer batchSize : "
-                    + kafkaSinkConfig.getBatchSize());
+                + kafkaSinkConfig.getBatchSize());
         }
         if (kafkaSinkConfig.getMaxRequestSize() <= 0) {
             throw new IllegalArgumentException("Invalid Kafka Producer maxRequestSize : "
-                    + kafkaSinkConfig.getMaxRequestSize());
+                + kafkaSinkConfig.getMaxRequestSize());
         }
         if (kafkaSinkConfig.getProducerConfigProperties() != null) {
             props.putAll(kafkaSinkConfig.getProducerConfigProperties());
@@ -122,7 +126,7 @@ public abstract class KafkaAbstractSink<K, V> implements Sink<byte[]> {
 
         producer = new KafkaProducer<>(beforeCreateProducer(props));
 
-        log.info("Kafka sink started : {}.", props.getOrDefault(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, ""));
+        log.info("Kafka sink started : {}.", props);
     }
 
     public abstract KeyValue<K, V> extractKeyValue(Record<byte[]> message);
