@@ -19,8 +19,6 @@
 package org.apache.pulsar.io.solr;
 
 import com.google.common.collect.Lists;
-import org.apache.pulsar.io.core.SinkContext;
-import org.mockito.Mockito;
 import org.testng.annotations.Test;
 
 import java.io.File;
@@ -63,31 +61,7 @@ public class SolrSinkConfigTest {
         map.put("username", "fakeuser");
         map.put("password", "fake@123");
 
-        SinkContext sinkContext = Mockito.mock(SinkContext.class);
-        SolrSinkConfig config = SolrSinkConfig.load(map, sinkContext);
-        assertNotNull(config);
-        assertEquals(config.getSolrUrl(), "localhost:2181,localhost:2182/chroot");
-        assertEquals(config.getSolrMode(), "SolrCloud");
-        assertEquals(config.getSolrCollection(), "techproducts");
-        assertEquals(config.getSolrCommitWithinMs(), Integer.parseInt("100"));
-        assertEquals(config.getUsername(), "fakeuser");
-        assertEquals(config.getPassword(), "fake@123");
-    }
-
-    @Test
-    public final void loadFromMapCredentialsFromSecretTest() throws IOException {
-        Map<String, Object> map = new HashMap<>();
-        map.put("solrUrl", "localhost:2181,localhost:2182/chroot");
-        map.put("solrMode", "SolrCloud");
-        map.put("solrCollection", "techproducts");
-        map.put("solrCommitWithinMs", "100");
-
-        SinkContext sinkContext = Mockito.mock(SinkContext.class);
-        Mockito.when(sinkContext.getSecret("username"))
-                .thenReturn("fakeuser");
-        Mockito.when(sinkContext.getSecret("password"))
-                .thenReturn("fake@123");
-        SolrSinkConfig config = SolrSinkConfig.load(map, sinkContext);
+        SolrSinkConfig config = SolrSinkConfig.load(map);
         assertNotNull(config);
         assertEquals(config.getSolrUrl(), "localhost:2181,localhost:2182/chroot");
         assertEquals(config.getSolrMode(), "SolrCloud");
@@ -107,13 +81,12 @@ public class SolrSinkConfigTest {
         map.put("username", "fakeuser");
         map.put("password", "fake@123");
 
-        SinkContext sinkContext = Mockito.mock(SinkContext.class);
-        SolrSinkConfig config = SolrSinkConfig.load(map, sinkContext);
+        SolrSinkConfig config = SolrSinkConfig.load(map);
         config.validate();
     }
 
-    @Test(expectedExceptions = IllegalArgumentException.class,
-        expectedExceptionsMessageRegExp = "solrUrl cannot be null")
+    @Test(expectedExceptions = NullPointerException.class,
+        expectedExceptionsMessageRegExp = "solrUrl property not set.")
     public final void missingValidValidateSolrModeTest() throws IOException {
         Map<String, Object> map = new HashMap<>();
         map.put("solrMode", "SolrCloud");
@@ -122,8 +95,7 @@ public class SolrSinkConfigTest {
         map.put("username", "fakeuser");
         map.put("password", "fake@123");
 
-        SinkContext sinkContext = Mockito.mock(SinkContext.class);
-        SolrSinkConfig config = SolrSinkConfig.load(map, sinkContext);
+        SolrSinkConfig config = SolrSinkConfig.load(map);
         config.validate();
     }
 
@@ -138,8 +110,7 @@ public class SolrSinkConfigTest {
         map.put("username", "fakeuser");
         map.put("password", "fake@123");
 
-        SinkContext sinkContext = Mockito.mock(SinkContext.class);
-        SolrSinkConfig config = SolrSinkConfig.load(map, sinkContext);
+        SolrSinkConfig config = SolrSinkConfig.load(map);
         config.validate();
     }
 
@@ -154,8 +125,7 @@ public class SolrSinkConfigTest {
         map.put("username", "fakeuser");
         map.put("password", "fake@123");
 
-        SinkContext sinkContext = Mockito.mock(SinkContext.class);
-        SolrSinkConfig config = SolrSinkConfig.load(map, sinkContext);
+        SolrSinkConfig config = SolrSinkConfig.load(map);
         config.validate();
 
         SolrAbstractSink.SolrMode.valueOf(config.getSolrMode().toUpperCase());
@@ -171,8 +141,7 @@ public class SolrSinkConfigTest {
         map.put("username", "fakeuser");
         map.put("password", "fake@123");
 
-        SinkContext sinkContext = Mockito.mock(SinkContext.class);
-        SolrSinkConfig config = SolrSinkConfig.load(map, sinkContext);
+        SolrSinkConfig config = SolrSinkConfig.load(map);
         config.validate();
 
         String url = config.getSolrUrl();
