@@ -206,8 +206,8 @@ public class LocalMemoryMetadataStoreTest {
         }
 
         @Override
-        public CompletableFuture<Void> closeAsync() {
-            return CompletableFuture.completedFuture(null);
+        public void close() {
+            // No-op
         }
 
     }
