@@ -29,19 +29,20 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.Objects;
 import java.util.Set;
+import java.util.concurrent.Executor;
 import javax.net.ssl.SSLContext;
+import javax.servlet.ServletConfig;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-import org.apache.pulsar.broker.PulsarServerException;
 import org.apache.pulsar.broker.web.AuthenticationFilter;
 import org.apache.pulsar.client.api.Authentication;
 import org.apache.pulsar.client.api.AuthenticationDataProvider;
-import org.apache.pulsar.client.api.KeyStoreParams;
+import org.apache.pulsar.client.api.AuthenticationFactory;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.common.util.SecurityUtility;
-import org.apache.pulsar.common.util.keystoretls.KeyStoreSSLContext;
 import org.apache.pulsar.policies.data.loadbalancer.ServiceLookupData;
 import org.eclipse.jetty.client.HttpClient;
 import org.eclipse.jetty.client.HttpRequest;
@@ -52,7 +53,9 @@ import org.eclipse.jetty.client.api.Request;
 import org.eclipse.jetty.client.http.HttpClientTransportOverHTTP;
 import org.eclipse.jetty.http.HttpHeader;
 import org.eclipse.jetty.proxy.ProxyServlet;
+import org.eclipse.jetty.util.HttpCookieStore;
 import org.eclipse.jetty.util.ssl.SslContextFactory;
+import org.eclipse.jetty.util.thread.QueuedThreadPool;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -82,40 +85,91 @@ class AdminProxyHandler extends ProxyServlet {
 
     private final ProxyConfiguration config;
     private final BrokerDiscoveryProvider discoveryProvider;
-    private final Authentication proxyClientAuthentication;
     private final String brokerWebServiceUrl;
     private final String functionWorkerWebServiceUrl;
 
-    AdminProxyHandler(ProxyConfiguration config, BrokerDiscoveryProvider discoveryProvider,
-                      Authentication proxyClientAuthentication) {
+    AdminProxyHandler(ProxyConfiguration config, BrokerDiscoveryProvider discoveryProvider) {
         this.config = config;
         this.discoveryProvider = discoveryProvider;
-        this.proxyClientAuthentication = proxyClientAuthentication;
         this.brokerWebServiceUrl = config.isTlsEnabledWithBroker() ? config.getBrokerWebServiceURLTLS()
                 : config.getBrokerWebServiceURL();
         this.functionWorkerWebServiceUrl = config.isTlsEnabledWithBroker() ? config.getFunctionWorkerWebServiceURLTLS()
                 : config.getFunctionWorkerWebServiceURL();
 
+        super.setTimeout(config.getHttpProxyTimeout());
     }
 
     @Override
     protected HttpClient createHttpClient() throws ServletException {
-        HttpClient httpClient = super.createHttpClient();
-        customizeHttpClient(httpClient);
-        return httpClient;
-    }
+        ServletConfig config = getServletConfig();
 
-    protected void customizeHttpClient(HttpClient httpClient) {
-        httpClient.setFollowRedirects(true);
+        HttpClient client = newHttpClient();
 
-        ProtocolHandlers protocolHandlers = httpClient.getProtocolHandlers();
-        if (protocolHandlers != null) {
-            protocolHandlers.put(new RedirectProtocolHandler(httpClient));
+        client.setFollowRedirects(true);
+
+        // Must not store cookies, otherwise cookies of different clients will mix.
+        client.setCookieStore(new HttpCookieStore.Empty());
+
+        Executor executor;
+        String value = config.getInitParameter("maxThreads");
+        if (value == null || "-".equals(value)) {
+            executor = (Executor) getServletContext().getAttribute("org.eclipse.jetty.server.Executor");
+            if (executor == null) {
+                throw new IllegalStateException("No server executor for proxy");
+            }
+        } else {
+            QueuedThreadPool qtp = new QueuedThreadPool(Integer.parseInt(value));
+            String servletName = config.getServletName();
+            int dot = servletName.lastIndexOf('.');
+            if (dot >= 0) {
+                servletName = servletName.substring(dot + 1);
+            }
+            qtp.setName(servletName);
+            executor = qtp;
         }
 
-        setTimeout(config.getHttpProxyTimeout());
+        client.setExecutor(executor);
+
+        value = config.getInitParameter("maxConnections");
+        if (value == null) {
+            value = "256";
+        }
+        client.setMaxConnectionsPerDestination(Integer.parseInt(value));
+
+        value = config.getInitParameter("idleTimeout");
+        if (value == null) {
+            value = "30000";
+        }
+        client.setIdleTimeout(Long.parseLong(value));
+
+        value = config.getInitParameter(INIT_PARAM_REQUEST_BUFFER_SIZE);
+        if (value != null) {
+            client.setRequestBufferSize(Integer.parseInt(value));
+        }
+
+        value = config.getInitParameter("responseBufferSize");
+        if (value != null){
+            client.setResponseBufferSize(Integer.parseInt(value));
+        }
+
+        try {
+            client.start();
+
+            // Content must not be decoded, otherwise the client gets confused.
+            client.getContentDecoderFactories().clear();
+
+            // Pass traffic to the client, only intercept what's necessary.
+            ProtocolHandlers protocolHandlers = client.getProtocolHandlers();
+            protocolHandlers.clear();
+            protocolHandlers.put(new RedirectProtocolHandler(client));
+
+            return client;
+        } catch (Exception x) {
+            throw new ServletException(x);
+        }
     }
 
+
     // This class allows the request body to be replayed, the default implementation
     // does not
     protected class ReplayableProxyContentProvider extends ProxyInputStreamContentProvider {
@@ -199,43 +253,36 @@ class AdminProxyHandler extends ProxyServlet {
     @Override
     protected HttpClient newHttpClient() {
         try {
+            Authentication auth = AuthenticationFactory.create(
+                config.getBrokerClientAuthenticationPlugin(),
+                config.getBrokerClientAuthenticationParameters()
+            );
+
+            Objects.requireNonNull(auth, "No supported auth found for proxy");
+
+            auth.start();
+
             if (config.isTlsEnabledWithBroker()) {
                 try {
                     X509Certificate[] trustCertificates = SecurityUtility
                         .loadCertificatesFromPemFile(config.getBrokerClientTrustCertsFilePath());
 
                     SSLContext sslCtx;
-                    AuthenticationDataProvider authData =
-                            proxyClientAuthentication.getAuthData(URI.create(getWebServiceUrl()).getHost());
-                    if (config.isBrokerClientTlsEnabledWithKeyStore()) {
-                        KeyStoreParams params = authData.hasDataForTls() ? authData.getTlsKeyStoreParams() : null;
-                        sslCtx = KeyStoreSSLContext.createClientSslContext(
-                                config.getBrokerClientSslProvider(),
-                                params != null ? params.getKeyStoreType() : null,
-                                params != null ? params.getKeyStorePath() : null,
-                                params != null ? params.getKeyStorePassword() : null,
+                    AuthenticationDataProvider authData = auth.getAuthData();
+                    if (authData.hasDataForTls()) {
+                        sslCtx = SecurityUtility.createSslContext(
                                 config.isTlsAllowInsecureConnection(),
-                                config.getBrokerClientTlsTrustStoreType(),
-                                config.getBrokerClientTlsTrustStore(),
-                                config.getBrokerClientTlsTrustStorePassword(),
-                                config.getBrokerClientTlsCiphers(),
-                                config.getBrokerClientTlsProtocols());
+                                trustCertificates,
+                                authData.getTlsCertificates(),
+                                authData.getTlsPrivateKey(),
+                                config.getBrokerClientSslProvider()
+                        );
                     } else {
-                        if (authData.hasDataForTls()) {
-                            sslCtx = SecurityUtility.createSslContext(
-                                    config.isTlsAllowInsecureConnection(),
-                                    trustCertificates,
-                                    authData.getTlsCertificates(),
-                                    authData.getTlsPrivateKey(),
-                                    config.getBrokerClientSslProvider()
-                            );
-                        } else {
-                            sslCtx = SecurityUtility.createSslContext(
-                                    config.isTlsAllowInsecureConnection(),
-                                    trustCertificates,
-                                    config.getBrokerClientSslProvider()
-                            );
-                        }
+                        sslCtx = SecurityUtility.createSslContext(
+                                config.isTlsAllowInsecureConnection(),
+                                trustCertificates,
+                                config.getBrokerClientSslProvider()
+                        );
                     }
 
                     SslContextFactory contextFactory = new SslContextFactory.Client();
@@ -246,6 +293,11 @@ class AdminProxyHandler extends ProxyServlet {
                     return new JettyHttpClient(contextFactory);
                 } catch (Exception e) {
                     LOG.error("new jetty http client exception ", e);
+                    try {
+                        auth.close();
+                    } catch (IOException ioe) {
+                        LOG.error("Failed to close the authentication service", ioe);
+                    }
                     throw new PulsarClientException.InvalidConfigurationException(e.getMessage());
                 }
             }
@@ -257,19 +309,6 @@ class AdminProxyHandler extends ProxyServlet {
         return new JettyHttpClient();
     }
 
-    private String getWebServiceUrl() throws PulsarServerException {
-        if (isBlank(brokerWebServiceUrl)) {
-            ServiceLookupData availableBroker = discoveryProvider.nextBroker();
-            if (config.isTlsEnabledWithBroker()) {
-                return availableBroker.getWebServiceUrlTls();
-            } else {
-                return availableBroker.getWebServiceUrl();
-            }
-        } else {
-            return brokerWebServiceUrl;
-        }
-    }
-
     @Override
     protected String rewriteTarget(HttpServletRequest request) {
         StringBuilder url = new StringBuilder();
@@ -285,10 +324,17 @@ class AdminProxyHandler extends ProxyServlet {
 
         if (isFunctionsRestRequest && !isBlank(functionWorkerWebServiceUrl)) {
             url.append(functionWorkerWebServiceUrl);
-        } else {
+        } else if (isBlank(brokerWebServiceUrl)) {
             try {
-                url.append(getWebServiceUrl());
-                if (LOG.isDebugEnabled() && isBlank(brokerWebServiceUrl)) {
+                ServiceLookupData availableBroker = discoveryProvider.nextBroker();
+
+                if (config.isTlsEnabledWithBroker()) {
+                    url.append(availableBroker.getWebServiceUrlTls());
+                } else {
+                    url.append(availableBroker.getWebServiceUrl());
+                }
+
+                if (LOG.isDebugEnabled()) {
                     LOG.debug("[{}:{}] Selected active broker is {}", request.getRemoteAddr(), request.getRemotePort(),
                             url);
                 }
@@ -297,6 +343,8 @@ class AdminProxyHandler extends ProxyServlet {
                         request.getRemotePort(), e.getMessage(), e);
                 return null;
             }
+        } else {
+            url.append(brokerWebServiceUrl);
         }
 
         if (url.lastIndexOf("/") == url.length() - 1) {
