@@ -26,19 +26,15 @@ import static org.apache.pulsar.common.stats.JvmMetrics.getJvmDirectMemoryUsed;
 import com.beust.jcommander.JCommander;
 import com.beust.jcommander.Parameter;
 import com.google.common.annotations.VisibleForTesting;
-import io.prometheus.client.Collector;
 import io.prometheus.client.CollectorRegistry;
 import io.prometheus.client.Gauge;
 import io.prometheus.client.Gauge.Child;
 import io.prometheus.client.hotspot.DefaultExports;
-import java.io.IOException;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Date;
-import java.util.Objects;
-import java.util.function.Consumer;
 import lombok.Getter;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.core.util.datetime.FixedDateFormat;
@@ -47,16 +43,12 @@ import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.broker.authentication.AuthenticationService;
 import org.apache.pulsar.broker.stats.prometheus.PrometheusMetricsServlet;
 import org.apache.pulsar.broker.web.plugin.servlet.AdditionalServletWithClassLoader;
-import org.apache.pulsar.client.api.Authentication;
-import org.apache.pulsar.client.api.AuthenticationFactory;
-import org.apache.pulsar.client.api.PulsarClientException;
-import org.apache.pulsar.client.impl.auth.AuthenticationDisabled;
 import org.apache.pulsar.common.configuration.PulsarConfigurationLoader;
 import org.apache.pulsar.common.configuration.VipStatus;
 import org.apache.pulsar.common.policies.data.ClusterData;
-import org.apache.pulsar.common.util.CmdGenerateDocs;
 import org.apache.pulsar.common.util.DirectMemoryUtils;
 import org.apache.pulsar.common.util.ShutdownUtil;
+import org.apache.pulsar.docs.tools.CmdGenerateDocs;
 import org.apache.pulsar.proxy.stats.ProxyStats;
 import org.apache.pulsar.websocket.WebSocketConsumerServlet;
 import org.apache.pulsar.websocket.WebSocketProducerServlet;
@@ -68,10 +60,8 @@ import org.eclipse.jetty.websocket.servlet.WebSocketServlet;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-
 /**
  * Starts an instance of the Pulsar ProxyService.
- *
  */
 public class ProxyServiceStarter {
 
@@ -107,23 +97,13 @@ public class ProxyServiceStarter {
 
     private ProxyConfiguration config;
 
-    @Getter
-    private Authentication proxyClientAuthentication;
-
     @Getter
     private ProxyService proxyService;
 
     private WebServer server;
     private static boolean metricsInitialized;
-    private boolean embeddedMode;
 
     public ProxyServiceStarter(String[] args) throws Exception {
-        this(args, null, false);
-    }
-
-    public ProxyServiceStarter(String[] args, Consumer<ProxyConfiguration> proxyConfigurationCustomizer,
-                               boolean embeddedMode) throws Exception {
-        this.embeddedMode = embeddedMode;
         try {
             DateFormat dateFormat = new SimpleDateFormat(
                 FixedDateFormat.FixedFormat.ISO8601_OFFSET_DATE_TIME_HHMM.getPattern());
@@ -146,26 +126,15 @@ public class ProxyServiceStarter {
                     CmdGenerateDocs cmd = new CmdGenerateDocs("pulsar");
                     cmd.addCommand("proxy", this);
                     cmd.run(null);
-                    if (embeddedMode) {
-                        return;
-                    } else {
-                        System.exit(0);
-                    }
+                    System.exit(0);
                 }
             } catch (Exception e) {
                 jcommander.usage();
-                if (embeddedMode) {
-                    return;
-                } else {
-                    System.exit(1);
-                }
+                System.exit(1);
             }
 
             // load config file
             config = PulsarConfigurationLoader.create(configFile, ProxyConfiguration.class);
-            if (proxyConfigurationCustomizer != null) {
-                proxyConfigurationCustomizer.accept(config);
-            }
 
             if (!isBlank(zookeeperServers)) {
                 // Use zookeeperServers from command line
@@ -190,28 +159,11 @@ public class ProxyServiceStarter {
             if (isNotBlank(config.getBrokerServiceURL())) {
                 checkArgument(config.getBrokerServiceURL().startsWith("pulsar://"),
                         "brokerServiceURL must start with pulsar://");
-                ensureUrlNotContainsComma("brokerServiceURL", config.getBrokerServiceURL());
             }
+
             if (isNotBlank(config.getBrokerServiceURLTLS())) {
                 checkArgument(config.getBrokerServiceURLTLS().startsWith("pulsar+ssl://"),
                         "brokerServiceURLTLS must start with pulsar+ssl://");
-                ensureUrlNotContainsComma("brokerServiceURLTLS", config.getBrokerServiceURLTLS());
-            }
-
-            if (isNotBlank(config.getBrokerWebServiceURL())) {
-                ensureUrlNotContainsComma("brokerWebServiceURL", config.getBrokerWebServiceURL());
-            }
-            if (isNotBlank(config.getBrokerWebServiceURLTLS())) {
-                ensureUrlNotContainsComma("brokerWebServiceURLTLS", config.getBrokerWebServiceURLTLS());
-            }
-
-            if (isNotBlank(config.getFunctionWorkerWebServiceURL())) {
-                ensureUrlNotContainsComma("functionWorkerWebServiceURLTLS",
-                        config.getFunctionWorkerWebServiceURL());
-            }
-            if (isNotBlank(config.getFunctionWorkerWebServiceURLTLS())) {
-                ensureUrlNotContainsComma("functionWorkerWebServiceURLTLS",
-                        config.getFunctionWorkerWebServiceURLTLS());
             }
 
             if ((isBlank(config.getBrokerServiceURL()) && isBlank(config.getBrokerServiceURLTLS()))
@@ -232,11 +184,6 @@ public class ProxyServiceStarter {
         }
     }
 
-    private void ensureUrlNotContainsComma(String paramName, String paramValue) {
-        checkArgument(!paramValue.contains(","), paramName + " does not support multi urls yet,"
-                + " it should point to the discovery service provider.");
-    }
-
     public static void main(String[] args) throws Exception {
         ProxyServiceStarter serviceStarter = new ProxyServiceStarter(args);
         try {
@@ -250,75 +197,38 @@ public class ProxyServiceStarter {
     public void start() throws Exception {
         AuthenticationService authenticationService = new AuthenticationService(
                 PulsarConfigurationLoader.convertFrom(config));
-
-        if (config.getBrokerClientAuthenticationPlugin() != null) {
-            proxyClientAuthentication = AuthenticationFactory.create(config.getBrokerClientAuthenticationPlugin(),
-                    config.getBrokerClientAuthenticationParameters());
-            Objects.requireNonNull(proxyClientAuthentication, "No supported auth found for proxy");
-            try {
-                proxyClientAuthentication.start();
-            } catch (Exception e) {
-                try {
-                    proxyClientAuthentication.close();
-                } catch (IOException ioe) {
-                    log.error("Failed to close the authentication service", ioe);
-                }
-                throw new PulsarClientException.InvalidConfigurationException(e.getMessage());
-            }
-        } else {
-            proxyClientAuthentication = AuthenticationDisabled.INSTANCE;
-        }
-
         // create proxy service
-        proxyService = new ProxyService(config, authenticationService, proxyClientAuthentication);
+        proxyService = new ProxyService(config, authenticationService);
         // create a web-service
         server = new WebServer(config, authenticationService);
 
-        if (!embeddedMode) {
-            Runtime.getRuntime().addShutdownHook(new Thread(this::close));
-        }
+        Runtime.getRuntime().addShutdownHook(new Thread(this::close));
 
         proxyService.start();
 
         if (!metricsInitialized) {
             // Setup metrics
             DefaultExports.initialize();
-            CollectorRegistry registry = CollectorRegistry.defaultRegistry;
 
             // Report direct memory from Netty counters
-            Collector jvmMemoryDirectBytesUsed =
-                    Gauge.build("jvm_memory_direct_bytes_used", "-").create().setChild(new Child() {
-                        @Override
-                        public double get() {
-                            return getJvmDirectMemoryUsed();
-                        }
-                    });
-            try {
-                registry.register(jvmMemoryDirectBytesUsed);
-            } catch (IllegalArgumentException e) {
-                // workaround issue in tests where the metric is already registered
-                log.debug("Failed to register jvm_memory_direct_bytes_used metric: {}", e.getMessage());
-            }
+            Gauge.build("jvm_memory_direct_bytes_used", "-").create().setChild(new Child() {
+                @Override
+                public double get() {
+                    return getJvmDirectMemoryUsed();
+                }
+            }).register(CollectorRegistry.defaultRegistry);
 
-            Collector jvmMemoryDirectBytesMax =
-                    Gauge.build("jvm_memory_direct_bytes_max", "-").create().setChild(new Child() {
-                        @Override
-                        public double get() {
-                            return DirectMemoryUtils.jvmMaxDirectMemory();
-                        }
-                    });
-            try {
-                registry.register(jvmMemoryDirectBytesMax);
-            } catch (IllegalArgumentException e) {
-                // workaround issue in tests where the metric is already registered
-                log.debug("Failed to register jvm_memory_direct_bytes_max metric: {}", e.getMessage());
-            }
+            Gauge.build("jvm_memory_direct_bytes_max", "-").create().setChild(new Child() {
+                @Override
+                public double get() {
+                    return DirectMemoryUtils.jvmMaxDirectMemory();
+                }
+            }).register(CollectorRegistry.defaultRegistry);
 
             metricsInitialized = true;
         }
 
-        addWebServerHandlers(server, config, proxyService, proxyService.getDiscoveryProvider(),
-                proxyClientAuthentication);
+        addWebServerHandlers(server, config, proxyService, proxyService.getDiscoveryProvider());
 
         // start web-service
         server.start();
@@ -332,41 +242,28 @@ public class ProxyServiceStarter {
             if (server != null) {
                 server.stop();
             }
-            if (proxyClientAuthentication != null) {
-                proxyClientAuthentication.close();
-            }
         } catch (Exception e) {
             log.warn("server couldn't stop gracefully {}", e.getMessage(), e);
         } finally {
-            if (!embeddedMode) {
-                LogManager.shutdown();
-            }
+            LogManager.shutdown();
         }
     }
 
     public static void addWebServerHandlers(WebServer server,
-                                            ProxyConfiguration config,
-                                            ProxyService service,
-                                            BrokerDiscoveryProvider discoveryProvider,
-                                            Authentication proxyClientAuthentication) throws Exception {
-        // We can make 'status.html' publicly accessible without authentication since
-        // it does not contain any sensitive data.
-        server.addRestResource("/", VipStatus.ATTRIBUTE_STATUS_FILE_PATH, config.getStatusFilePath(),
-                VipStatus.class, false);
-        if (config.isEnableProxyStatsEndpoints()) {
-            server.addRestResource("/proxy-stats", ProxyStats.ATTRIBUTE_PULSAR_PROXY_NAME, service,
-                    ProxyStats.class);
-            if (service != null) {
-                PrometheusMetricsServlet metricsServlet = service.getMetricsServlet();
-                if (metricsServlet != null) {
-                    server.addServlet("/metrics", new ServletHolder(metricsServlet),
-                            Collections.emptyList(), config.isAuthenticateMetricsEndpoint());
-                }
+                                     ProxyConfiguration config,
+                                     ProxyService service,
+                                     BrokerDiscoveryProvider discoveryProvider) throws Exception {
+        if (service != null) {
+            PrometheusMetricsServlet metricsServlet = service.getMetricsServlet();
+            if (metricsServlet != null) {
+                server.addServlet("/metrics", new ServletHolder(metricsServlet),
+                        Collections.emptyList(), config.isAuthenticateMetricsEndpoint());
             }
         }
+        server.addRestResource("/", VipStatus.ATTRIBUTE_STATUS_FILE_PATH, config.getStatusFilePath(), VipStatus.class);
+        server.addRestResource("/proxy-stats", ProxyStats.ATTRIBUTE_PULSAR_PROXY_NAME, service, ProxyStats.class);
 
-        AdminProxyHandler adminProxyHandler = new AdminProxyHandler(config, discoveryProvider,
-                proxyClientAuthentication);
+        AdminProxyHandler adminProxyHandler = new AdminProxyHandler(config, discoveryProvider);
         ServletHolder servletHolder = new ServletHolder(adminProxyHandler);
         server.addServlet("/admin", servletHolder);
         server.addServlet("/lookup", servletHolder);
