@@ -30,8 +30,6 @@ import lombok.Cleanup;
 
 import org.apache.pulsar.broker.authentication.AuthenticationService;
 import org.apache.pulsar.client.admin.PulsarAdmin;
-import org.apache.pulsar.client.api.Authentication;
-import org.apache.pulsar.client.api.AuthenticationFactory;
 import org.apache.pulsar.client.api.ProducerConsumerBase;
 import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
@@ -55,7 +53,7 @@ public class ProxyForwardAuthDataTest extends ProducerConsumerBase {
         conf.setAuthenticationEnabled(true);
         conf.setAuthorizationEnabled(true);
         conf.setBrokerClientAuthenticationPlugin(BasicAuthentication.class.getName());
-        conf.setBrokerClientAuthenticationParameters("authParam:admin");
+        conf.setBrokerClientAuthenticationParameters("authParam:broker");
         conf.setAuthenticateOriginalAuthData(true);
 
         Set<String> superUserRoles = new HashSet<String>();
@@ -118,11 +116,7 @@ public class ProxyForwardAuthDataTest extends ProducerConsumerBase {
 
         AuthenticationService authenticationService = new AuthenticationService(
                 PulsarConfigurationLoader.convertFrom(proxyConfig));
-        @Cleanup
-        final Authentication proxyClientAuthentication = AuthenticationFactory.create(proxyConfig.getBrokerClientAuthenticationPlugin(),
-                proxyConfig.getBrokerClientAuthenticationParameters());
-        proxyClientAuthentication.start();
-        try (ProxyService proxyService = new ProxyService(proxyConfig, authenticationService, proxyClientAuthentication)) {
+        try (ProxyService proxyService = new ProxyService(proxyConfig, authenticationService)) {
             proxyService.start();
             try (PulsarClient proxyClient = createPulsarClient(proxyService.getServiceUrl(), clientAuthParams)) {
                 proxyClient.newConsumer().topic(topicName).subscriptionName(subscriptionName).subscribe();
@@ -138,7 +132,7 @@ public class ProxyForwardAuthDataTest extends ProducerConsumerBase {
                 PulsarConfigurationLoader.convertFrom(proxyConfig));
 
         @Cleanup
-        ProxyService proxyService = new ProxyService(proxyConfig, authenticationService, proxyClientAuthentication);
+        ProxyService proxyService = new ProxyService(proxyConfig, authenticationService);
         proxyService.start();
 
         @Cleanup
