@@ -29,6 +29,7 @@ import static java.lang.String.format;
 import static java.util.Objects.requireNonNull;
 import com.fasterxml.jackson.databind.JsonNode;
 import com.fasterxml.jackson.databind.node.ArrayNode;
+import com.fasterxml.jackson.databind.node.DecimalNode;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.google.common.collect.ImmutableList;
 import io.airlift.log.Logger;
@@ -57,13 +58,11 @@ import io.trino.spi.type.TimestampType;
 import io.trino.spi.type.Timestamps;
 import io.trino.spi.type.TinyintType;
 import io.trino.spi.type.Type;
-import io.trino.spi.type.UuidType;
 import io.trino.spi.type.VarbinaryType;
 import io.trino.spi.type.VarcharType;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.UUID;
 import org.apache.commons.lang3.tuple.Pair;
 
 /**
@@ -127,8 +126,7 @@ public class PulsarJsonFieldDecoder
                 TimestampType.TIMESTAMP_MILLIS,
                 DateType.DATE,
                 TimeType.TIME_MILLIS,
-                RealType.REAL,
-                UuidType.UUID
+                RealType.REAL
         ).contains(type)) {
             return true;
         }
@@ -276,10 +274,6 @@ public class PulsarJsonFieldDecoder
         private static Slice getSlice(JsonNode value, Type type, String columnName) {
             String textValue = value.isValueNode() ? value.asText() : value.toString();
 
-            if (type instanceof UuidType) {
-                return UuidType.javaUuidToTrinoUuid(UUID.fromString(textValue));
-            }
-
             Slice slice = utf8Slice(textValue);
             if (type instanceof VarcharType) {
                 slice = truncateToLength(slice, type);
@@ -340,21 +334,12 @@ public class PulsarJsonFieldDecoder
             } else {
                 blockBuilder = type.createBlockBuilder(null, 1);
             }
-            if (value == null) {
-                return null;
-            }
-            JsonNode jsonNode;
-            if (value instanceof JsonNode) {
-                jsonNode = (JsonNode) value;
-            } else {
-                throw new TrinoException(DECODER_CONVERSION_NOT_SUPPORTED,
-                        format("decimal object of '%s' as '%s' for column '%s' cannot be converted to JsonNode",
-                                value.getClass(), type, columnName));
-            }
-            String textValue = jsonNode.isValueNode() ? jsonNode.asText() : jsonNode.toString();
+
+            assert value instanceof DecimalNode;
+            final DecimalNode node = (DecimalNode) value;
             // For decimalType, need to eliminate the decimal point,
             // and give it to trino to set the decimal point
-            type.writeObject(blockBuilder, Int128.valueOf(textValue.replace(".", "")));
+            type.writeObject(blockBuilder, Int128.valueOf(node.asText().replace(".", "")));
 
             if (parentBlockBuilder == null) {
                 return blockBuilder.getSingleValueBlock(0);
