@@ -19,7 +19,6 @@
 package org.apache.pulsar.sql.presto.decoder;
 
 import java.math.BigDecimal;
-import java.util.UUID;
 import lombok.Data;
 
 import java.util.List;
@@ -56,9 +55,6 @@ public class DecoderTestMessage {
     public Map<String, Long> mapField;
     public CompositeRow compositeRow;
 
-    @org.apache.avro.reflect.AvroSchema("{\"type\":\"string\",\"logicalType\":\"uuid\"}")
-    public UUID uuidField;
-
     public static class TestRow {
         public String stringField;
         public int intField;
