@@ -274,7 +274,7 @@ public class MockManagedLedger implements ManagedLedger {
 
     @Override
     public ManagedLedgerConfig getConfig() {
-        return new ManagedLedgerConfig();
+        return null;
     }
 
     @Override
@@ -372,8 +372,8 @@ public class MockManagedLedger implements ManagedLedger {
     }
 
     @Override
-    public boolean checkInactiveLedgerAndRollOver() {
-        return false;
+    public void checkInactiveLedgerAndRollOver() {
+
     }
 
     @Override
